#!/bin/sh
######################################################################
# appreqts.sh

# Supplement to setup script to define additional requirements for
# service

######################################################################
# Copyright 2015 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
CheckDataDirectories()
{
    # Check that the three data directories exist with correct
    # permissions
    Template="drwxr-sr-x $WebServerUser $WebServerGroup"
    for Dir in fullsize webpics albums
    do
        DirPath=$ServiceRootPath/$Dir
	if [ -d $DirPath ]
	then
	    Perms=$(ls -ld $DirPath | awk '{print $1,$3,$4}');
	    if [ "$Perms" != "$Template" ]
	    then
		BadDataFolders=True
	    fi
	else
	    BadDataFolders=True
	fi
    done
    return
}

DescribeAppReqts()
{
    # Edit this function to describe to a human user any additional
    # requirements for the service. The output of this function
    # appears in the help screen.

    cat <<EOF

  o The system administrator has set up GraphicsMagick.

  o The system administrator has created three subdirectories:
    fullsize, webpics, and albums under PhotoTrove's service
    root. They should be owned by the web server user $WebServerUser,
    group $WebServerGroup, with permissions 2755 (rwxr-sr-x), for
    example:

    drwxr-sr-x    2 $WebServerUser $WebServerGroup \
     4096 Nov 24 20:07 albums/

EOF
    return
}

ReportAppReqtStatus()
{
    # Edit this function to report any unmet requirements.
    # Use ReportStatus (arguments are Description, is/are,
    # FailedFlag), or write your own messages

    ReportStatus "three data folders" "are" $BadDataFolders
    return
}

TestAppReqts()
{
    # Edit this function to verify whether any additional requirements
    # for the service are met. Set flags for any failed tests; unset
    # flags if tests pass.

    # Check that the three data directories exist with correct
    # permissions
    CheckDataDirectories

    # Don't remove this part! Just edit to match the flags.
    # Set AppReqtFailed if any requirement failed 
    Comp=${BadDataFolders}
    if [ "z$Comp" != "z" ]
    then
	AppReqtFailed=True
    fi
    return
}
