#!bin/sh
######################################################################
# clonealbum.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

clonealbum.sh                    Copyright 2021 Theodore B. Ruegsegger

Creates an identical copy (with a different name) of a specified photo
album.

Usage:
    clonealbum.sh [ -h ] database albumname newalbumname

EOF
    return  
}

AlbumExists()
{
    # Return 1 if album name $1 exists in album table, 0 if not
    Name=$1
    Q="SELECT COUNT(*) FROM album WHERE name = '$Name'"
    psql -t -d $Database -U $DbUser -c "$Q"
    return
}

######################################################################
DbUser="trove"
Usage="Usage: $0 [ -h ] database albumname newalbumname"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

if [ $# -lt 3 ]
then
    echo $Usage
    exit
fi
Database=$1
AlbumName=$2
NewAlbumName=$3

Error=

Exists=$(AlbumExists $AlbumName)
if [ $Exists -eq 0 ]
then
    echo Cannot find album \"$AlbumName\".
    Error=True;
fi
Exists=$(AlbumExists $NewAlbumName)
if [ $Exists -eq 1 ]
then
    echo Cannot create album \"$NewAlbumName\" since it already exists.
    Error=True;
fi

if [ $Error ]
then
    echo Aborting.
    exit
fi

psql -q -d $Database -U $DbUser <<EOF
-- Start an atomic transaction:
BEGIN;
-- Copy the album tuple:
INSERT INTO album
(name, title, pubdate, bgcolor, textcolor, vidbgcolor, author, url,
 notes)
SELECT '$NewAlbumName', title, pubdate, bgcolor, textcolor,
 vidbgcolor, author, url, notes
FROM album WHERE name = '$AlbumName';
-- Now copy all the album pages:
INSERT INTO albumpage
(albumsn, page, heading, titlepicord, titlepiccaption, pagetext)
SELECT a2.albumsn, page, heading, titlepicord, titlepiccaption,
 pagetext
FROM album a2, album a1, albumpage pg1
WHERE  a2.name = '$NewAlbumName' AND a1.name = '$AlbumName'
AND pg1.albumsn = a1.albumsn;
-- Copy all the albumphotos:
INSERT INTO albumphoto
(albumpagesn, photosn, ordinal, caption)
SELECT pg2.albumpagesn, photosn, ordinal, caption
FROM album a2, albumpage pg2, album a1, albumpage pg1, albumphoto ph1
WHERE a2.name = '$NewAlbumName' AND pg2.albumsn = a2.albumsn
AND a1.name = '$AlbumName' AND pg1.albumsn = a1.albumsn
AND pg2.page = pg1.page AND ph1.albumpagesn = pg1.albumpagesn;
-- Commit the transaction:
COMMIT;
EOF

echo Cloned album \"$AlbumName\" as new album \"$NewAlbumName\".
