#!/bin/sh
######################################################################
# compareoggvideos.sh

# Report sizes of original videos and their Ogg equivalents
######################################################################
# Copyright 2014 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
# For each existing video, reports the size of the original file and
# its Ogg (Theora/Vorbis) file, if one exists.

# Usage:
#     compareoggvideos.sh database serviceroot
# Example:
#     sh compareoggvideos.sh trove /home/web/webroot-ssl/trove \
#         | tee /home/public/compareoggvideos.log
######################################################################
if [ $# -lt 2 ]
then
    echo Usage: compareoggvideos.sh database serviceroot
    exit
fi
Database=$1
Root=$2
Query="psql -qtd $Database -U trove"

Q="SELECT foldername || '/' || vidfilename"
Q="$Q FROM video v, photo p, photoset s"
Q="$Q WHERE p.photosn = v.stillsn AND s.photosetsn = p.photosetsn"
Q="$Q ORDER BY foldername, vidfilename"

VidFiles=$($Query -c "$Q")
for Path in $VidFiles
do
    Folder=$(echo $Path | cut -d / -f 1)
    VidFile=$(echo $Path | cut -d / -f 2)
    BaseName=$(echo $VidFile | sed -e "s/\.[^.]*$//")
    OggFile=${BaseName}.ogg
    FlvFile=${BaseName}.flv
    WebFolder=$Root/webpics/$Folder

    OrigFolder=$Root/fullsize/$Folder
    OrigSize=$(ls -s $OrigFolder/$VidFile | awk '{print $1}')

    if [ -e $WebFolder/$OggFile ]
    then
	OggSize=$(ls -s $WebFolder/$OggFile | awk '{print $1}')
    else
	OggSize="[TBD]"
    fi

    if [ -e $WebFolder/$FlvFile ]
    then
	FlvSize=$(ls -s $WebFolder/$FlvFile | awk '{print $1}')
    else
	FlvSize="[TBD]"
    fi

    echo "$Folder\t$VidFile\t$OrigSize\t$OggSize\t$FlvSize"
done
