#!/bin/sh
######################################################################
# convertfoto2trove.sh

# Import data from a Photo Album Manager database to the PhotoTrove
# database.
######################################################################
# Copyright 2011 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################

# Start with a dump of the Photo Album Manager production database
# "foto"

# Usage:

#     convertfoto2trove.sh trovedatabase fotodumpfile


######################################################################
# Conversions for tables we will ignore

Convert_report()
{
    Q="INSERT INTO report(name, dispord, description)"
    Q="$Q SELECT name, dispord, description FROM zzreport"
    $QueryTrove -c "$Q"
    return
}

Convert_report_priv()
{
    Q="INSERT INTO report_priv(reportsn, privilegesn)"
    Q="$Q SELECT r.reportsn, p.privilegesn"
    Q="$Q FROM zzreport_priv z, report r, privilege p"
    Q="$Q WHERE r.name = z.rptname AND p.privname = z.privname"
    Q="$Q AND NOT EXISTS"
    Q="$Q (SELECT * FROM report_priv t"
    Q="$Q WHERE t.reportsn = r.reportsn"
    Q="$Q AND t.privilegesn = p.privilegesn)"
    $QueryTrove -c "$Q"
    return
}

Convert_tool()
{
    Q="UPDATE tool t SET dispord ="
    Q="$Q (SELECT dispord FROM zztool z WHERE z.name = t.name)"
    $QueryTrove -c "$Q"
    Q="INSERT INTO tool(name, dispord, description)"
    Q="$Q SELECT name, dispord, description FROM zztool z"
    Q="$Q WHERE NOT EXISTS"
    Q="$Q (SELECT * FROM tool t WHERE t.name = z.name)"
    $QueryTrove -c "$Q"
    return
}

Convert_tool_priv()
{
    Q="INSERT INTO tool_priv(toolsn, privilegesn)"
    Q="$Q SELECT tl.toolsn, p.privilegesn"
    Q="$Q FROM zztool_priv z, tool tl, privilege p"
    Q="$Q WHERE tl.name = z.toolname AND p.privname = z.privname"
    Q="$Q AND NOT EXISTS"
    Q="$Q (SELECT * FROM tool_priv t"
    Q="$Q WHERE t.toolsn = tl.toolsn"
    Q="$Q AND t.privilegesn = p.privilegesn)"
    $QueryTrove -c "$Q"
    return
}

######################################################################
Convert_album()
{
    Q="INSERT INTO album(name, title, pubdate, bgcolor, textcolor,"
    Q="$Q vidbgcolor, author, url, notes)"
    Q="$Q SELECT name, title, date_trunc('day',"
    Q="$Q timestamp with time zone 'epoch' + MAX(time) * INTERVAL"
    Q="$Q '1 second') AS latest, bgcolor,"
    Q="$Q textcolor, vidbgcolor, author, email, notes"
    Q="$Q FROM zzalbum za, zzalbumphoto zaph, zzphoto zp"
    Q="$Q WHERE zaph.albumname = za.name"
    Q="$Q AND zp.foldername = zaph.foldername"
    Q="$Q AND zp.filename = zaph.filename"
    Q="$Q GROUP BY name, title, bgcolor, textcolor, vidbgcolor,"
    Q="$Q author, email, notes"
    $QueryTrove -c "$Q"
    Q="UPDATE album SET title = name"
    Q="$Q WHERE title IS NULL OR title = ''"
    $QueryTrove -c "$Q"
    return
}

Convert_albumpage()
{
    Q="INSERT INTO albumpage(albumsn, page, heading, titlepicord,"
    Q="$Q titlepiccaption, pagetext)"
    Q="$Q SELECT a.albumsn, page, heading, titlepicord,"
    Q="$Q titlepiccaption, pagetext FROM zzalbumpage z, album a"
    Q="$Q WHERE a.name = z.albumname"
    $QueryTrove -c "$Q"
    return
}

Convert_albumphoto()
{
    Q="INSERT INTO albumphoto(albumphotosn, albumpagesn, photosn,"
    Q="$Q ordinal, caption)"
    Q="$Q SELECT albumphotosn, ap.albumpagesn, p.photosn, ordinal,"
    Q="$Q caption FROM zzalbumphoto z, album a, albumpage ap,"
    Q="$Q photoset s, photo p"
    Q="$Q WHERE a.name = z.albumname AND ap.albumsn = a.albumsn"
    Q="$Q AND ap.page = z.page AND s.foldername = z.foldername"
    Q="$Q AND p.photosetsn = s.photosetsn AND p.filename = z.filename"
    $QueryTrove -c "$Q"
    Q="SELECT setval('albumphoto_albumphotosn_seq',"
    Q="$Q max(albumphotosn)) FROM albumphoto"
    $QueryTrove -c "$Q" >/dev/null
    return
}

Convert_altauth()
{
    Q="INSERT INTO altauth(regusersn, challenge1, response1,"
    Q="$Q challenge2, response2)"
    Q="$Q SELECT r.regusersn, challenge1, response1, challenge2,"
    Q="$Q response2 FROM zzaltauth z, reguser r"
    Q="$Q WHERE r.userid = z.userid AND NOT EXISTS"
    Q="$Q (SELECT * FROM altauth t WHERE t.regusersn = r.regusersn)"
    $QueryTrove -c "$Q"
    return
}

Convert_archivedalbum()
{
    Q="INSERT INTO archivedalbum(albumsn)"
    Q="$Q SELECT a.albumsn FROM zzarchivedalbum z, album a"
    Q="$Q WHERE a.name = z.albumname"
    $QueryTrove -c "$Q"
    return
}

Convert_derived_photo()
{
    Q="INSERT INTO derived_photo(childphotosn, parentphotosn)"
    Q="$Q SELECT c.photosn, p.photosn"
    Q="$Q FROM zzderived_photo z, photoset s, photo c, photo p"
    Q="$Q WHERE s.foldername = z.foldername"
    Q="$Q AND c.photosetsn = s.photosetsn"
    Q="$Q AND p.photosetsn = s.photosetsn AND p.filename = z.parent"
    Q="$Q AND c.filename = z.child"
    $QueryTrove -c "$Q"
    return
}

Convert_location()
{
    Q="INSERT INTO location(locationsn, name, last_used)"
    Q="$Q SELECT locationsn, name, timestamp with time zone 'epoch'"
    Q="$Q + last_used * INTERVAL '1 second' FROM zzlocation"
    $QueryTrove -c "$Q"
    Q="SELECT setval('location_locationsn_seq', max(locationsn))"
    Q="$Q FROM location"
    $QueryTrove -c "$Q" >/dev/null
    return
}

Convert_person()
{
    Q="INSERT INTO person(lastname, firstname)"
    Q="$Q SELECT lastname, firstname FROM zzperson z"
    Q="$Q WHERE NOT EXISTS"
    Q="$Q (SELECT * FROM person t WHERE t.lastname = z.lastname"
    Q="$Q  AND t.firstname = z.firstname)"
    $QueryTrove -c "$Q"
    return
}

Convert_photo()
{
    Q="INSERT INTO photo(photosetsn, filename, locationsn,"
    Q="$Q description, width, height, thumbw, thumbh, time, aperture,"
    Q="$Q shutterspeed, focallength)"
    Q="$Q SELECT s.photosetsn, filename, locationsn, description,"
    Q="$Q width, height, thumbw, thumbh, timestamp with time zone"
    Q="$Q 'epoch' + time * INTERVAL '1 second', aperture,"
    Q="$Q shutterspeed, focallength FROM zzphoto z, photoset s"
    Q="$Q WHERE s.foldername = z.foldername"
    $QueryTrove -c "$Q"
    return
}

Convert_photoset()
{
    Q="INSERT INTO photoset(foldername, notes, nickname)"
    Q="$Q SELECT foldername, notes, nickname FROM zzphotoset"
    $QueryTrove -c "$Q"
    return
}

Convert_privilege()
{
    Q="INSERT INTO privilege(privname, description)"
    Q="$Q SELECT name, description FROM zzprivilege z"
    Q="$Q WHERE NOT EXISTS"
    Q="$Q (SELECT * FROM privilege t WHERE t.privname = z.name)"
    $QueryTrove -c "$Q"
    return
}

Convert_reguser()
{
    Q="INSERT INTO reguser(userid, personsn, passwd_hash,"
    Q="$Q passwd_changed)"
    Q="$Q SELECT loginid, p.personsn, password, pw_changed"
    Q="$Q FROM zzreguser z, zzperson zp, person p WHERE NOT EXISTS"
    Q="$Q (SELECT * FROM reguser t WHERE t. userid = z.loginid)"
    Q="$Q AND zp.personsn = z.personsn AND p.lastname = zp.lastname"
    Q="$Q AND p.firstname = zp.firstname"
    $QueryTrove -c "$Q"
    return
}

Convert_tag()
{
    Q="INSERT INTO tag(name, notes)"
    Q="$Q SELECT z1.name || '[' || owner || ']' AS name, notes"
    Q="$Q FROM zztag z1 WHERE EXISTS"
    Q="$Q (SELECT * FROM zztag z2"
    Q="$Q  WHERE z2.name = z1.name AND z2.owner != z1.owner)"
    Q="$Q UNION"
    Q="$Q SELECT name, notes FROM zztag z1 WHERE NOT EXISTS"
    Q="$Q (SELECT * FROM zztag z2"
    Q="$Q  WHERE z2.name = z1.name AND z2.owner != z1.owner)"
    $QueryTrove -c "$Q"
    return
}

Convert_tag_photo()
{
    Q="INSERT INTO tag_photo(tagsn, photosn)"
    Q="$Q SELECT t.tagsn, p.photosn"
    Q="$Q FROM zztag_photo z, tag t, photo p, photoset s"
    Q="$Q WHERE (t.name = z.tagname"
    Q="$Q  OR t.name = z.tagname || '[' || z.owner || ']')"
    Q="$Q AND s.foldername = z.foldername"
    Q="$Q AND p.photosetsn = s.photosetsn AND p.filename = z.filename"
    $QueryTrove -c "$Q"
    return
}

Convert_user_metapriv()
{
    # Exclude the admin account
    Q="INSERT INTO user_metapriv(regusersn, privilegesn)"
    Q="$Q SELECT r.regusersn, p.privilegesn"
    Q="$Q FROM zzuser_metapriv z, reguser r, privilege p"
    Q="$Q WHERE r.userid = z.loginid AND p.privname = z.privname"
    Q="$Q AND z.loginid != 'admin'"
    Q="$Q AND NOT EXISTS"
    Q="$Q (SELECT * FROM user_metapriv t"
    Q="$Q WHERE t.regusersn = r.regusersn"
    Q="$Q AND t.privilegesn = p.privilegesn)"
    $QueryTrove -c "$Q"
    return
}

Convert_user_priv()
{
    # Exclude the admin account
    Q="INSERT INTO user_priv(regusersn, privilegesn)"
    Q="$Q SELECT r.regusersn, p.privilegesn"
    Q="$Q FROM zzuser_priv z, reguser r, privilege p"
    Q="$Q WHERE r.userid = z.loginid AND p.privname = z.privname"
    Q="$Q AND z.loginid != 'admin'"
    Q="$Q AND NOT EXISTS"
    Q="$Q (SELECT * FROM user_priv t"
    Q="$Q WHERE t.regusersn = r.regusersn"
    Q="$Q AND t.privilegesn = p.privilegesn)"
    $QueryTrove -c "$Q"
    return
}

Convert_video()
{
    Q="INSERT INTO video(vidfilename, stillsn, width, height)"
    Q="$Q SELECT vidfilename, p.photosn, z.width, z.height"
    Q="$Q FROM zzvideo z, photoset s, photo p"
    Q="$Q WHERE s.foldername = z.foldername"
    Q="$Q AND p.photosetsn = s.photosetsn"
    Q="$Q AND p.filename = z.stillfilename"
    $QueryTrove -c "$Q"
    return
}

CopyAsZzTable()
{
    # Creates a new table from $1 with same content but prefixed with
    # "zz" and without keys or indices or other constraints
    Q="CREATE TABLE zz$1 AS SELECT * FROM $1"
    $QueryFoto -c "$Q" 2>&1 | grep -v NOTICE
    return
}

DropZzTables()
{
    # Prepend each word in $1 with "zz" then drop all
    Q="DROP TABLE"
    for Table in $1
    do
	Q="$Q, zz$Table"
    done
    Q=$(echo "$Q" | sed -e 's/^DROP TABLE,/DROP TABLE/')
    $QueryTrove -c "$Q"
    return
}

IsValueInFoto()
{
    # Return 1 if table $1, column $2 contains value $3, 0 if not
    # Usage: if [ $(IsValueInFoto table column value) -eq 1 ]
    $QueryFoto -t -c "$(QIsValueInTableCol $1 $2 $3)"
    return
}

IsValueInTemplate1()
{
    # Return 1 if table $1, column $2 contains value $3, 0 if not
    # Usage: if [ $(IsValueInTemplate1 table column value) -eq 1 ]
    $QueryTemplate1 -t -c "$(QIsValueInTableCol $1 $2 $3)"
    return
}

IsValueInTrove()
{
    # Return 1 if table $1, column $2 contains value $3, 0 if not
    # Usage: if [ $(IsValueInTrove table column value) -eq 1 ]
    $QueryTrove -t -c "$(QIsValueInTableCol $1 $2 $3)"
    return
}

MakeDbUser()
{
    # Create db user $1 if not already present
    if [ $(IsValueInTemplate1 pg_user usename $1) -eq 0 ]
    then
	echo Creating user $1
	createuser -U postgres --createdb --no-superuser \
	    --no-createrole $1
    else
	echo User $1 already exists
    fi
    return
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

QIsValueInTableCol()
{
    # Construct query for IsValueIn{Foto | Trove}
    # Usage: $Query___ -c "$(IsValueInTableCol table column value)"
    echo "SELECT COUNT(*) FROM $1 WHERE $2 = '$3'"
    return
}

RenameTables()
{
    # Two tables have new names in Trove; rename the ones from foto:
    Q="ALTER TABLE album_page RENAME TO albumpage"
    $QueryFoto -c "$Q" 2>&1 | grep -v NOTICE
    Q="ALTER TABLE album_photo RENAME TO albumphoto"
    $QueryFoto -c "$Q" 2>&1 | grep -v NOTICE
    return
}

######################################################################
QueryFoto="psql -qd fotoxfer -U foto"
QueryTemplate1="psql -qd template1 -U postgres"
FotoTables="
album
albumpage
albumphoto
altauth
archivedalbum
derived_photo
location
person
person_personsn_seq
photo
photoset
privilege
reguser
tag
tag_photo
user_metapriv
user_priv
video
"
IgnoredTables="
report
report_priv
tool
tool_priv
"
######################################################################
if [ $# -lt 2 ]
then
    echo Usage: convertfoto2trove.sh trovedatabase fotodumpfile
    exit
fi
Database=$1
DumpFile=$2

QueryTrove="psql -qd $Database -U trove"

if echo $0 | grep '/' >/dev/null
then
    ThisDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    ThisDir=$(pwd)
fi

MakeDbUser trove

if [ $(IsValueInTemplate1 pg_database datname $Database) -eq 1 ]
then
    echo Database $Database exists!
    PromptYesNo "Continuing will erase it and start over. Is that OK"
    if [ $Response = "N" ]
    then
	echo "Ok, quitting."
	exit
    else
	echo "Ok, erasing current database $Database."
	dropdb -U postgres $Database
    fi
fi
echo Creating database $Database
createdb -U trove $Database
sh $ThisDir/gk/createbasictables.sh $Database
sh $ThisDir/createapptables.sh $Database 2>&1 \
    | grep -v '^psql:trove.sql:.*NOTICE:'

MakeDbUser foto

# Load foto dump into temporary database
if [ $(IsValueInTemplate1 pg_database datname fotoxfer) -eq 1 ]
then
    dropdb -U foto fotoxfer
fi
echo Creating temporary database fotoxfer
createdb -U foto fotoxfer
echo Loading temporary database with foto dump $DumpFile
restoredb.sh $DumpFile fotoxfer foto > /dev/null

# Rename two tables
RenameTables

# Prefix foto tables with "zz" and add them to $Database
for Table in $FotoTables
do
    CopyAsZzTable $Table
done
T="Transferring legacy data to temporary tables in database $Database"
echo $T
pg_dump -U foto -O -t 'zz*' fotoxfer | $QueryTrove 
dropdb -U foto fotoxfer

echo Converting the tables, one-by-one:
#Convert_report
#Convert_tool
echo "   " privilege
Convert_privilege
echo "   " person
Convert_person
echo "   " reguser
Convert_reguser
echo "   " user_metapriv
Convert_user_metapriv
echo "   " user_priv
Convert_user_priv
#Convert_report_priv
#Convert_tool_priv
echo "   " altauth
Convert_altauth

echo "   " location
Convert_location
echo "   " photoset
Convert_photoset
echo "   " album
Convert_album
echo "   " albumpage
Convert_albumpage
echo "   " archivedalbum
Convert_archivedalbum
echo "   " photo
Convert_photo
echo "   " tag
Convert_tag
echo "   " video
Convert_video
echo "   " tag_photo
Convert_tag_photo
echo "   " derived_photo
Convert_derived_photo
echo "   " albumphoto
Convert_albumphoto

echo Dropping the temporary tables
DropZzTables "$FotoTables"

echo Conversion is complete.
