#!bin/sh
######################################################################
# copypagebetweenalbums.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

copypagebetweenalbums.sh         Copyright 2021 Theodore B. Ruegsegger

Copies a specified page from one photo album and appends it as the
last page of another.

Usage:
    copypagebetweenalbums.sh [ -h ] database sourcealbumname page \
	targetalbumname

EOF
    return  
}

AlbumExists()
{
    # Return 1 if album name $1 exists in album table, 0 if not
    Name=$1
    Q="SELECT COUNT(*) FROM album WHERE name = '$Name'"
    psql -t -d $Database -U $DbUser -c "$Q"
    return
}

AlbumPageExists()
{
    # Return 1 if album named $1 has a page numbered $2, 0 if not
    Name=$1
    Pg=$2
    Q="SELECT COUNT(*) FROM album a, albumpage p"
    Q="$Q WHERE a.name = '$Name' AND p.albumsn = a.albumsn"
    Q="$Q  AND p.page = $Pg"
    psql -t -d $Database -U $DbUser -c "$Q"
    return
}

######################################################################
DbUser="trove"
Usage="Usage: $0 [ -h ] database sourcealbumname page targetalbumname"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

if [ $# -lt 4 ]
then
    echo $Usage
    exit
fi
Database=$1
SrcAlbumName=$2
Page=$3
TgtAlbumName=$4

Error=

Exists=$(AlbumExists $SrcAlbumName)
if [ $Exists -eq 0 ]
then
    echo Cannot find album \"$SrcAlbumName\".
    Error=True;
else
    Exists=$(AlbumPageExists $SrcAlbumName $Page)
    if [ $Exists -eq 0 ]
    then
	echo album \"$SrcAlbumName\" has no page numbered $Page.
	Error=True;
    fi
fi
Exists=$(AlbumExists $TgtAlbumName)
if [ $Exists -eq 0 ]
then
    echo Cannot find album \"$TgtAlbumName\".
    Error=True;
fi

if [ $Error ]
then
    echo Aborting.
    exit
fi

psql -q -d $Database -U $DbUser <<EOF
-- Start an atomic transaction:
BEGIN;
-- Copy the album page itself as the last page in the target album:
INSERT INTO albumpage
(albumsn, page, heading, titlepicord, titlepiccaption, pagetext)
SELECT a2.albumsn, (SELECT MAX(page) + 1 FROM albumpage p, album a WHERE a.name = '$TgtAlbumName' AND p.albumsn = a.albumsn), heading, titlepicord, titlepiccaption, pagetext
FROM album a2, album a1, albumpage pg1
WHERE  a2.name = '$TgtAlbumName' AND a1.name = '$SrcAlbumName'
AND pg1.albumsn = a1.albumsn AND pg1.page = $Page;
-- Copy all the albumphotos for the page:
INSERT INTO albumphoto
(albumpagesn, photosn, ordinal, caption)
SELECT pg2.albumpagesn, photosn, ordinal, caption
FROM album a2, albumpage pg2, album a1, albumpage pg1, albumphoto ph1
WHERE a2.name = '$TgtAlbumName' AND pg2.albumsn = a2.albumsn
AND a1.name = '$SrcAlbumName' AND pg1.albumsn = a1.albumsn 
AND pg2.page = (SELECT MAX(page) FROM albumpage p, album a
 WHERE a.name = '$TgtAlbumName' AND p.albumsn = a.albumsn)
AND pg1.page = $Page AND ph1.albumpagesn = pg1.albumpagesn;
-- Commit the transaction:
COMMIT;
EOF

echo Appended page $Page of album \"$SrcAlbumName\" to end of \
    album \"$TgtAlbumName\".
