#!/bin/sh
######################################################################

######################################################################
#
#  WARNING: Needs to be updated for trove!
#  WARNING: Needs to be updated to handle videos!
#
######################################################################


# copytodev.sh

# Copy a photoset's database entries and photos to the dev
# environment, including any related records in these tables:
#     photo
#     photoset
#     album_photo
#     album_page
#     album
#     derived_photo
#     tag_photo

# In order to preserve album integrity, this may necessitate copying
# additional photosets if the specified photoset occurs in an album
# that also includes other photosets.

# Usage:

#     copytodev.sh photoset

# Must run as a user with sufficient sudo privilege to become the
# webserver user "www-data", in order write to the fullsize and
# webpics subfolders.

######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
CopyImages()
{
    Folder=$1
    # Copy fullsize
    if [ -d $FotoRoot/fullsize/$Folder ]
    then
	sudo -u www-data sh -c "cp -R $FotoRoot/fullsize/$Folder \
	    $DevRoot/fullsize/"
    else
	echo "Found no folder named \"$FotoRoot/fullsize/$Folder\""
    fi

    # Copy webpics and thumbnails
    if [ -d $FotoRoot/webpics/$Folder ]
    then
	sudo -u www-data sh -c "cp -R $FotoRoot/webpics/$Folder \
	    $DevRoot/webpics/"
    else
	echo "Found no folder named \"$FotoRoot/webpics/$Folder\""
    fi
}

CreateTempTables()
{
S="
CREATE TABLE folders
(
    foldername text,
    PRIMARY KEY (foldername)
);
CREATE TABLE newfolders
(
    foldername text,
    PRIMARY KEY (foldername)
);
CREATE TABLE albums
(
    albumname text,
    PRIMARY KEY (albumname)
);
CREATE TABLE newalbums
(
    albumname text,
    PRIMARY KEY (albumname)
);
INSERT INTO newfolders VALUES ('$PhotoSet');
"
    $QueryProd -qc "$S" 2>&1 | grep -v NOTICE
    return
}

DropTempTables()
{
    $QueryProd -qc "DROP TABLE folders, albums, newfolders, newalbums"
    return
}

FindNewAlbums()
{
    $QueryProd -q <<EOF
INSERT INTO newalbums
SELECT DISTINCT albumname FROM album_photo ap, newfolders nf
WHERE ap.foldername = nf.foldername AND NOT EXISTS
(SELECT * FROM albums WHERE albumname = ap.albumname);

INSERT INTO folders SELECT foldername FROM newfolders;
DELETE FROM newfolders;
EOF
    return
}

FindNewFolders()
{
    $QueryProd -q <<EOF
INSERT INTO newfolders
SELECT DISTINCT foldername FROM album_photo ap, newalbums na
WHERE ap.albumname = na.albumname AND NOT EXISTS
(SELECT * FROM folders WHERE foldername = ap.foldername);

INSERT INTO albums SELECT albumname FROM newalbums;
DELETE FROM newalbums;
EOF
    return
}

TransferAlbumRecords()
{
    Table=$1
    if [ $Table = 'album' ]
    then
	Key=name
    else
	Key=albumname
    fi
    $QueryProd -q <<-EOF
    CREATE TABLE xfer$Table AS SELECT * FROM $Table t
    WHERE EXISTS
    (SELECT * FROM albums a WHERE a.albumname = t.$Key);
	EOF
    pg_dump -cU $Owner -t xfer$Table $ProdDb | $QueryDev -q 2>&1 \
	| grep -v "does not exist"
    $QueryProd -qc "DROP TABLE xfer$Table" 2>&1
}

TransferPhotoRecords()
{
    Table=$1
    $QueryProd -q <<-EOF
    CREATE TABLE xfer$Table AS SELECT * FROM $Table t
    WHERE EXISTS
    (SELECT * FROM folders f WHERE f.foldername = t.foldername);
	EOF
    pg_dump -cU $Owner -t xfer$Table $ProdDb | $QueryDev -q 2>&1 \
	| grep -v "does not exist"
    $QueryProd -qc "DROP TABLE xfer$Table" 2>&1
}

######################################################################
FotoRoot=/home/web/webroot-ssl/foto
ProdDb=foto
DevDb=foto_dev
Owner=foto
######################################################################
DevRoot=$FotoRoot/dev
QueryProd="psql -d $ProdDb -U $Owner"
QueryDev="psql -d $DevDb -U $Owner"




######################################################################
Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

echo "WARNING: Needs to be updated to handle videos!"
PromptYesNo "Continue anyway"
if [ $Response = "N" ]
then
    echo "Ok, quitting."
    exit
fi


######################################################################



if [ $# -lt 1 ]
then
    echo Usage: copytodev.sh photoset
    exit
fi
PhotoSet=$1

CreateTempTables
NewFolderCount=1
while [ $NewFolderCount -gt 0 ]
do
    FindNewAlbums
    NewAlbumCount=$($QueryProd -tc "SELECT COUNT(*) FROM newalbums")
    if [ $NewAlbumCount -gt 0 ]
    then
	FindNewFolders
    fi
    NewFolderCount=$($QueryProd -tc "SELECT COUNT(*) FROM newfolders")
done

Folders=$($QueryProd -tc "SELECT foldername FROM folders")

for PhotoSet in $Folders
do
    echo Copying images for photo set $PhotoSet
    CopyImages $PhotoSet
done

echo Copying database records
for RelatedTable in photo photoset derived_photo tag_photo
do
    TransferPhotoRecords $RelatedTable
done

for RelatedTable in album_photo album_page album
do
    TransferAlbumRecords $RelatedTable
done

DropTempTables

$QueryDev -q <<EOF
INSERT INTO photoset
SELECT * FROM xferphotoset x WHERE NOT EXISTS
(SELECT * FROM photoset WHERE foldername = x.foldername);

INSERT INTO photo
SELECT * FROM xferphoto x WHERE NOT EXISTS
(SELECT * FROM photo WHERE foldername = x.foldername
 AND filename = x.filename);

INSERT INTO derived_photo
SELECT * FROM xferderived_photo x WHERE NOT EXISTS
(SELECT * FROM derived_photo WHERE foldername = x.foldername
 AND parent = x.parent AND child = x.child);

INSERT INTO tag_photo
SELECT * FROM xfertag_photo x WHERE NOT EXISTS
(SELECT * FROM tag_photo WHERE owner = x.owner
 AND tagname = x.tagname AND foldername = x.foldername
 AND filename = x.filename);

INSERT INTO album
SELECT * FROM xferalbum x WHERE NOT EXISTS
(SELECT * FROM album WHERE name = x.name);

INSERT INTO album_page
SELECT * FROM xferalbum_page x WHERE NOT EXISTS
(SELECT * FROM album_page WHERE albumname = x.albumname
 AND page = x.page);

INSERT INTO album_photo
SELECT * FROM xferalbum_photo x WHERE NOT EXISTS
(SELECT * FROM album_photo WHERE albumname = x.albumname
 AND foldername = x.foldername AND filename = x.filename
 AND page = x.page AND ordinal = x.ordinal);

DROP TABLE xferphotoset, xferphoto, xferderived_photo, xfertag_photo,
 xferalbum, xferalbum_page, xferalbum_photo;
EOF

