/*********************************************************************
createapptables.sql

Creates database tables (beyond the basic ones) for the PhotoTrove
service. Also augments basic tables with additional fields, as needed,
and populates additional tools and privileges.

______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

Usage:
	psql -d database -U dbuser -f createapptables.sql

*********************************************************************/
-- Don't need these fields
ALTER TABLE person DROP COLUMN email;
ALTER TABLE person DROP COLUMN phone;

-- Don't need reports
DELETE FROM tool_priv 
WHERE toolsn = (SELECT toolsn FROM tool WHERE name = 'report.php');
INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'report.php' AND privname = 'mngrpt';

/*********************************************************************
Following table definitions are output from pgModeler, with redundant
ALTER TABLE .. OWNER TO .. lines commented out.
*********************************************************************/
\i trove.sql
/*********************************************************************
End of table definitions imported from pgModeler
*********************************************************************/

GRANT SELECT, INSERT, UPDATE, DELETE ON
    album,
    albumpage,
    albumphoto,
    archivedalbum,
    archivedlocation,
    archivedtag,
    deletedset,
    derived_photo,
    location,
    photo,
    photoset,
    tag,
    tag_photo,
    video
TO "www-data";

GRANT SELECT, UPDATE ON
    album_albumsn_seq,
    albumpage_albumpagesn_seq,
    albumphoto_albumphotosn_seq,
    location_locationsn_seq,
    photo_photosn_seq,
    photoset_photosetsn_seq,
    tag_tagsn_seq,
    video_videosn_seq
TO "www-data";

-- Now populate the initial privileges, tools, and users

INSERT INTO privilege(privname, description)
VALUES ('edit', 'Edit photosets and albums');
INSERT INTO privilege(privname, description)
VALUES ('walbum', 'Write albums to folders for CD');

INSERT INTO tool(name, dispord, description)
VALUES ('viewalbum.php', 1, 'View albums');
INSERT INTO tool(name, dispord, description)
VALUES ('viewset.php', 2, 'View photo sets');
INSERT INTO tool(name, dispord, description)
VALUES ('search.php', 3, 'Search for photos');
INSERT INTO tool(name, dispord, description)
VALUES ('aboutphototrove.php', 4, 'About PhotoTrove');

INSERT INTO tool(name, dispord, description)
VALUES ('importphotoset.php', 1, 'Import a photo set');
INSERT INTO tool(name, dispord, description)
VALUES ('showsetprogress.php', 2, 'Check import progress');
INSERT INTO tool(name, dispord, description)
VALUES ('editset.php', 3, 'Edit photo sets');
INSERT INTO tool(name, dispord, description)
VALUES ('add1pic.php', 4, 'Add single photos/videos');
INSERT INTO tool(name, dispord, description)
VALUES ('editalbum.php', 5, 'Edit albums');
INSERT INTO tool(name, dispord, description)
VALUES ('writealbum.php', 6, 'Publish albums');
INSERT INTO tool(name, dispord, description)
VALUES ('stitchpano.php', 7, 'Assemble a panorama');
INSERT INTO tool(name, dispord, description)
VALUES ('editlocation.php', 8, 'Manage locations');
INSERT INTO tool(name, dispord, description)
VALUES ('edittagged.php', 9, 'Manage tags');
INSERT INTO tool(name, dispord, description)
VALUES ('movepics.php', 10, 'Move photos between sets');
INSERT INTO tool(name, dispord, description)
VALUES ('adjusttimes.php', 11, 'Adjust timestamps');
INSERT INTO tool(name, dispord, description)
VALUES ('applydates.php', 12, 'Apply dates');
INSERT INTO tool(name, dispord, description)
VALUES ('capturegps.php', 13, 'Capture geo-data');
INSERT INTO tool(name, dispord, description)
VALUES ('dumpdb.php', 14, 'Export database');

-- Keep this tool out of menu
/*
INSERT INTO hiddentool(toolsn)
SELECT toolsn FROM tool WHERE name = 'createphotoset.php';
*/

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'viewalbum.php' AND privname = 'public';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'viewset.php' AND privname = 'public';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'search.php' AND privname = 'public';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'aboutphototrove.php' AND privname = 'public';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'importphotoset.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'showsetprogress.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'editset.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'add1pic.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'editalbum.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'writealbum.php' AND privname = 'walbum';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'stitchpano.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'editlocation.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'edittagged.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'movepics.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'adjusttimes.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'applydates.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'capturegps.php' AND privname = 'edit';

INSERT INTO tool_priv(toolsn, privilegesn)
SELECT toolsn, privilegesn FROM tool, privilege
WHERE name = 'dumpdb.php' AND privname = 'edit';

-- Be sure to include this line. Put it after adding any additional
-- privileges
INSERT INTO user_metapriv(regusersn, privilegesn)
SELECT regusersn, privilegesn FROM reguser r, privilege p
WHERE userid = 'admin' AND NOT EXISTS
(SELECT * FROM nogrant WHERE privilegesn = p.privilegesn)
AND NOT EXISTS
(SELECT * FROM user_metapriv WHERE regusersn = r.regusersn
AND privilegesn = p.privilegesn);

-- Add required tags

INSERT INTO tag(name, notes)
VALUES ('panorama-to-do',
'one of a series of images waiting to be stitched into a panorama');

INSERT INTO tag(name, notes)
VALUES ('panorama',
'panoramic view stitched together from a series of images');

INSERT INTO tag(name, notes)
VALUES ('component',
'One of a series of images that are parts of a compound structure, but otherwise not &quot;interesting&quot; in their own right, for example, segments of a panorama');

