#!/bin/sh
######################################################################
# deleteflvvideos.sh

# Update collection to handle videos with HTML5: delete flv files
######################################################################
# Copyright 2013 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
# For each existing video, removes the old flv file.

# Run this script only after all the Ogg files have been created and
# the new HTML5 code is installed and working.

# Usage: Run as web-server user, owner of the files:

#     deleteflvvideos.sh serviceroot
# Example:
#     sudo -u www-data deleteflvvideos.sh \
#         /home/web/webroot-ssl/trove \
#         | tee /home/public/deleteflvvideos.log
######################################################################
if [ $# -lt 1 ]
then
    echo Usage: deleteflvvideos.sh serviceroot
    exit
fi
Root=$1
WebPicsRoot=$Root/webpics

FlvFiles=$(cd $WebPicsRoot ; find . -name *.flv \
    | sed -e 's#^\./##' | sort)
for FlvPath in $FlvFiles
do
    OggPath=$(echo $FlvPath | sed -e "s/\.flv$/.ogg/")
    if [ -e $WebPicsRoot/$OggPath ]
    then
	rm $WebPicsRoot/$FlvPath
	echo Deleted $FlvPath
    else
	echo No $OggPath, so retained $FlvPath
    fi
done
