#!/bin/sh
######################################################################
# createdatabases.sh
######################################################################
# Copyright 2018 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

createdatabases.sh

Sets up PostgreSQL user for specified service, as well as one or more
databases, as specified. Optionally, sets up a PostgreSQL user account
for the web server (not to be confused with the web server's GNU/Unix
user account, though it may have the same name).

Usage:
    createdatabases.sh [ options ] servicename [dbusername]

    If no username supplied, defaults to servicename

Options:
    -h
	Display a help message listing options.

    -3
	Create production, test, and development databases, equivalent
	to -ptd
    -p
	Create a production database named "servicename"
    -t
	Create a test database named "servicename_tst"
    -d
	Create a development database named "servicename_dev"
    -s suffix
	Create a database named "servicename_suffix"
    -w webserverdbuser
	Set up PostgreSQL user named "webserverdbuser"

EOF
return  
}

######################################################################
# Parse command line options
while getopts "h3ptds:w:" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      3 ) Production=True
          Test=True
          Development=True
          ;;
      p ) Production=True
          ;;
      t ) Test=True
          ;;
      d ) Development=True
          ;;
      s ) Suffix=$OPTARG
          ;;
      w ) WebServerDbUser=$OPTARG
          ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -gt 0 ]
then
    DbBaseName=$1
    if [ $# -gt 1 ]
    then
	DbUser=$2
    fi
fi

if [ ! $DbBaseName ]
then
    echo -n "Base name for database? "
    read DbBaseName
fi

if [ ! $DbUser ]
then
    DbUser=$DbBaseName
fi

echo "CREATE ROLE $DbUser CREATEDB NOCREATEROLE LOGIN;" \
    | psql -qd template1 -U postgres

if [ $WebServerDbUser ]
then
    echo "CREATE ROLE \"$WebServerDbUser\" CREATEDB NOCREATEROLE ;" \
	| psql -qd template1 -U postgres 2>&1 \
	| grep -v "ERROR:  role \"$WebServerDbUser\" already exists"
fi

Databases=

if [ $Production ]
then
    Databases=$DbBaseName
fi

if [ $Test ]
then
    Databases=${Databases}' '${DbBaseName}_tst
fi

if [ $Development ]
then
    Databases=${Databases}' '${DbBaseName}_dev
fi

if [ $Suffix ]
then
    Databases=${Databases}' '${DbBaseName}_$Suffix
fi

for Database in $Databases
do
    echo "CREATE DATABASE $Database ;" \
	| psql -qd template1 -U $DbUser
done
