/*********************************************************************
loadgktestdata.sql
____________________________________________________________
Copyright 2010 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
____________________________________________________________

Loads representative values (users) for system testing using Selenium.
In particular, this makes possible the basic GAPP Kit tests. For
application- specific tests, tailor the resettestdb.sh and
resettestdb.sql scripts in the scripts/ folder.

Requires a cyphertext (hashed password) to be supplied as an argument
(see Usage). This will be the password for every user.

Usage (intended to be called from resettestdb.sh):
    psql -d database -U user -f loadgktestdata.sql \
	-v cyphertext=\'XXXXX\'

Where:
    XXXXX is a hashed password in escaped quotes. Since hashes tend to
    	  include special characters, it's probably best to enclose it
    	  in real quotes, too, e.g.:
	      -v cyphertext=\''XXXXX'\'
*********************************************************************/
--SELECT 'Cyphertext is '||:cyphertext;

-- System (not service) administrator:
UPDATE reguser SET passwd_hash = :cyphertext,
passwd_changed = CURRENT_DATE
WHERE userid = 'admin';

-- Users
CREATE TEMPORARY TABLE usermap
(
    lastname text,
    firstname text,
    email text,
    phone text
);

COPY "usermap" (lastname, firstname, email, phone) FROM stdin;
Brewster	Kate	kbrewster@skynet.mil	987-654-3210
Brewster	Robert	rbrewster@skynet.mil	987-654-3210
Buchanan	Matt	mbuchanan@skynet.mil	987-654-3210
Connor	John	jconnor@skynet.mil	987-654-3210
Connor	Sarah	sconnor@skynet.mil	987-654-3210
Dyson	Miles	mdyson@skynet.mil	987-654-3210
Petersen	Scott	spetersen@skynet.mil	987-654-3210
Reese	Kyle	kreese@skynet.mil	987-654-3210
Silberman	Peter	psilberman@skynet.mil	987-654-3210
Traxler	Ed	etraxler@skynet.mil	987-654-3210
Trevaglia	Simon	bofh@skynet.mil	666-666-6666
Ventura	Ginger	gventura@skynet.mil	987-654-3210
\.

/*********************************************************************
For applications that don't include email and phone, comment out the
", email, phone" lines
*********************************************************************/
INSERT INTO person (lastname, firstname
-- , email, phone
) SELECT lastname, firstname
-- , email, phone
FROM usermap;

INSERT INTO reguser (userid, personsn, passwd_hash, passwd_changed)
 SELECT split_part(u.email, '@', 1), p.personsn, :cyphertext,
  CURRENT_DATE
 FROM person p, usermap u WHERE u.email LIKE '%@skynet.mil'
 AND p.lastname = u.lastname AND p.firstname = u.firstname;

DROP TABLE usermap;
