#!/bin/sh
######################################################################
# pgm2gapp.sh
######################################################################
# Copyright 2019 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

**********************************************************************
pgm2gapp.sh

Adjusts pgModeler SQL script output to conform to GAPP Kit coding
standard for SQL. Comments out unwanted statements:

    ALTER TABLE .. OWNER TO ..

Optionally removes CREATE TABLE statements for GAPP Kit basic tables.

Usage: In pgModeler, export the data model as SQL. Then pipe the SQL
through this script, for example:

    pgm2gapp.sh [-s] < pgmoutput.sql > standardscript.sql

Options:
    -h
	Display a help message listing options.
    -s
	Strip GAPP Kit basic tables. Removes CREATE TABLE statements
	for standard tables already created by the GAPP Kit
	createbasictables.sql script (altauth, person, privilege,
	reguser, report, report_priv, tool, tool_priv, user_metapriv,
	user_priv). Often an application's data model will include one
	or more of these tables for clarity, but attempting to create
	them twice will cause an error.

Reference:
    https://pgmodeler.io/

**********************************************************************
EOF
return
}

GAPPTables="
altauth
dbversion
hiddentool
nogrant
person
privilege
reguser
report
report_priv
tool
tool_priv
user_metapriv
user_priv
"

######################################################################
# Parse command line options
while getopts "hs" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      s ) StripGAPPTables=True
          ;;
    esac
done
shift $(( $OPTIND - 1 ))

Strip="cat"
if [ $StripGAPPTables ]
then
    for Table in $GAPPTables
    do
	Strip="$Strip | sed -e '/^CREATE TABLE 'public.$Table'$/,/^$/d'"
    done
fi

sed -e 's/\(ALTER TABLE [^ ][^ ]* OWNER TO .*$\)/-- \1/' \
    | eval "$Strip"

