#!/bin/sh
######################################################################
# resetdb.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
# resetdb.sh

# Resets the specified database to the initial state. For development
# only!

# Usage:

#     resetdb.sh database [ dbuser ]

######################################################################
Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
            Response=Y
            return
            ;;
	N|NO)
            Response=N
            return
            ;;
	*)
            PromptYesNo "$1"
            ;;
    esac
    return
}

RunSqlIfExists()
{
    Script=$1
    if [ -e $ScriptDir/$Script ]
    then
	psql -d $Database -U $DbUser -f $ScriptDir/$Script
    fi
    return
}
######################################################################
ScriptsPre="
"
ScriptsPost="
loadinitialvalues.sql
loadtestdata.sql
"

if [ $# -lt 1 ]
then
    echo Usage: resetdb.sh database [ dbuser ]
    exit
elif [ $# -gt 1 ]
then
    DbUser=$2
else
    # no dbuser supplied, guess from dbname
    DbUser=$(echo $1 | sed -e 's/_.*$//')
fi

Database=$1

echo "This will overwrite most or all of $Database."
PromptYesNo "Are you certain you want to do this"
if [ $Response = "N" ]
then
    echo OK, aborting.
    exit
fi

if echo $0 | grep '/' >/dev/null
then
    GKDir=$(echo $0 | rev | cut -d '/' -f 2- | rev)
else
    GKDir=$(pwd)
fi
TopDir=$(cd $GKDir ; cd ../.. ; pwd)
ScriptDir=$TopDir/scripts

dropdb -U $DbUser $Database
createdb -U $DbUser $Database

sh $GKDir/createbasictables.sh $Database $DbUser

for Script in $ScriptsPre
do
    RunSqlIfExists $Script
done

# There may be a wrapper for createapptables.sql; if so, run that
# instead of running the SQL script directly:
if [ -e $ScriptDir/createapptables.sh ]
then
    sh $ScriptDir/createapptables.sh $Database $DbUser
else
    RunSqlIfExists createapptables.sql
fi

for Script in $ScriptsPost
do
    RunSqlIfExists $Script
done
