#!/bin/sh
######################################################################
# resetpasswords.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################

# Resets all the user passwords in the specified database to the given
# password or the default.

# Usage:
#     resetpasswords.sh database [ password [ dbuser ] ]
# Where:
#     password	is the initial password to be installed for all users
#		(default is xyZZy!42 as in the test scripts)
#     dbuser	is the database user (default is service name, the
#     		base name of the database)
######################################################################
Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo ()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

######################################################################
PassWord='xyZZy!42'

if [ $# -lt 1 ]
then
    echo Usage: resetpasswords.sh database [ password [ dbuser ] ]
    exit
elif [ $# -gt 1 ]
then
    PassWord=$2
fi
Database=$1
if [ $# -gt 2 ]
then
    DbUser=$3
else
    DbUser=$(echo $Database | sed -e 's/_.*$//')
fi

CypherText=$(openssl passwd -1 $PassWord)

T="This will set all passwords to \"$PassWord\" in the $Database"
T="$T database."
echo $T
PromptYesNo "Are you certain you want to do this"
if [ $Response = "N" ]
then
    echo OK, aborting.
    exit
fi

if [ "z$Database" = "z$DbUser" ]
then
    echo ''
    echo '    *************************************************'
    echo '    *                                               *'
    echo '    * Yikes! This looks like a production database! *'
    echo '    *                                               *'
    echo '    *************************************************'
    echo ''
    PromptYesNo "Are you absolutely certain you want to do this"
    if [ $Response = "N" ]
	then
	echo Whew! OK, aborting.
	exit
    fi
fi

psql -qd $Database -U $DbUser <<EOF
UPDATE reguser SET passwd_hash = '$CypherText',
 passwd_changed = CURRENT_DATE;
EOF

echo "OK, password for every account is \"$PassWord\""
