#!/bin/sh
######################################################################
# restoredb.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

restoredb.sh

Restores, with OIDs, the specified database from a full dump generated
by dumpdb.sh.

Usage:
    restoredb.sh [ -h ] dumpfile database dbuser

Example:
    restoredb.sh /home/dumps/foto20041111.dump foto_tst foto

Options:

    -h
        Display a help message listing options.

EOF
return  
}

######################################################################
# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -lt 3 ]
then
    echo "Usage: $0 [ -h ] dumpfile database dbuser"
    exit
else
    DumpFile=$1
    Database=$2
    DbUser=$3
fi

Type=$(file -b $DumpFile | awk '{print $1}')
if [ "z$Type" = "zsymbolic" ]
then
    Dir=$(dirname $DumpFile)
    RealFile=$DumpFile
    while [ "z$Type" = "zsymbolic" ]
    do
	BaseName=$(readlink $RealFile)
	RealFile=$Dir/$BaseName
	Type=$(file -b $RealFile | awk '{print $1}')
    done
fi
if [ "z$Type" = "zgzip" ]
then
    zcat $DumpFile | sed -e '/DROP PROCEDURAL LANGUAGE/i\
\\connect - postgres' | psql -qd $Database -U $DbUser 2>&1 \
	| grep -v "ERROR:  [^ ][^ ]* \".*\" does not exist"
else
    sed -e '/DROP PROCEDURAL LANGUAGE/i\
\\connect - postgres' $DumpFile | psql -qd $Database -U $DbUser 2>&1 \
	| grep -v "ERROR:  [^ ][^ ]* \".*\" does not exist"
fi
