#!/bin/sh
######################################################################
# showdiffs.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
cat <<EOF

showdiffs.sh

Compares files in current directory with their counterparts in
another. Optionally, for those that differ, display them side-by-side.

Usage:
    showdiffs.sh [ -x ] otherdir pattern [ pattern ...]
where:
    otherdir is the path to the other directory
    pattern specifies the files to be compared

Options:
    -h
        Display a help message listing options.

    -x
        Using xless, display the differing files side-by-side.

EOF
return  
}

######################################################################
# Parse command line options
while getopts "hx" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      x ) SideBySide=True
          ;;
    esac
done

shift $(( $OPTIND - 1 ))

if [ $# -lt 2 ]
then
    echo "Usage: $0 [ -h ] [ -x ] otherdir pattern [ pattern ...]"
    exit
fi
OtherDir=$1
shift
Files=$*

for File in $Files
do
    if ! [ -e $OtherDir/$File ]
    then
	echo $File not found in $OtherDir
    elif ! diff $File $OtherDir/$File > /dev/null
    then
	echo $File differs
	if [ $SideBySide ]
	then
	    diff --side-by-side $File $OtherDir/$File \
		| xless -title "$File vs $OtherDir/$File" &
	fi
    fi
done
