#!/bin/sh
######################################################################
# tidyforselenium.sh
######################################################################
# Copyright 2010 Theodore B. Ruegsegger
# This file is part of the GAPP Kit.

# The GAPP Kit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# The GAPP Kit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
######################################################################

# Filter to clean up HTML-formatted Selenium test cases:
#   * Indent two spaces per level
#     --indent auto
#     --indent-spaces 2
#   * Don't wrap
#     --wrap 0
#   * Suppress nonessential output
#     --quiet yes
#   * Omit tidy-mark (meta-element indicating it's tidied)
#     --tidy-mark no
#   * Suppress warnings
#     --show-warnings no
#   * Omit doctype declaration
#     --doctype omit
#   * Don't convert ampersands to &amp;
#     --quote-ampersand no
#   * Strip empty lines
#     (sed filter)

# Other options to consider:
#   * Don't convert quotes to &quot;
#     --quote-marks no (default)

# Usage:

#     tidyforselenium.sh [filename]

#     Uses standard input if filename omitted
#     Writes to standard output

######################################################################
tidy --indent auto \
    --indent-spaces 2 \
    --quiet yes \
    --doctype omit \
    --tidy-mark no \
    --wrap 0 \
    --show-warnings no \
    --quote-ampersand no \
    $1 \
    | sed -e '/^$/d' 
