#!/bin/sh
######################################################################
# oneoggatatime.sh (variant of createoggvideos.sh)
# Exactly like createoggvideos.sh except pauses before each
# conversion, or after some preset number of conversions

# Update collection to handle videos with HTML5: create Ogg files
######################################################################
# Copyright 2014 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
# For each existing video, generates an Ogg (Theora/Vorbis) file
# unless one exists. This should have no impact on operation under the
# old flash-based solution.

# When the ogg videos have all been added, the PHP code can be updated
# to the HTML5 version.

# When all is working, run deleteflvvideos.sh to clear out the
# obsolete flash (flv) files.

# Usage: Run as web-server user, owner of the files:

#     createoggvideos.sh database serviceroot
# Example:
#     sudo -u www-data createoggvideos.sh trove \
#         /home/web/webroot-ssl/trove \
#         | tee /home/public/createoggvideos.log
######################################################################
Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

######################################################################
if [ $# -lt 2 ]
then
    echo Usage: createoggvideos.sh database serviceroot
    exit
fi
Database=$1
Root=$2
Query="psql -qtd $Database -U trove"

Q="SELECT foldername || '/' || vidfilename"
Q="$Q FROM video v, photo p, photoset s"
Q="$Q WHERE p.photosn = v.stillsn AND s.photosetsn = p.photosetsn"
Q="$Q ORDER BY foldername, vidfilename"

Batch=50
Count=0

VidFiles=$($Query -c "$Q")
for Path in $VidFiles
do
    Folder=$(echo $Path | cut -d / -f 1)
    VidFile=$(echo $Path | cut -d / -f 2)
    BaseName=$(echo $VidFile | sed -e "s/\.[^.]*$//")
    OggFile=${BaseName}.ogg
    WebFolder=$Root/webpics/$Folder
    if ! [ -e $WebFolder/$OggFile ]
    then
	OrigFolder=$Root/fullsize/$Folder
	Geometry=$(mplayer -really-quiet -vo null -ao null -identify \
	    -frames 0 $OrigFolder/$VidFile 2>/dev/null | \
	    grep 'ID_VIDEO' | grep 'WIDTH\|HEIGHT')
	Width=$(echo $Geometry | \
	    sed -e 's/^.*ID_VIDEO_WIDTH=\([[:digit:]]*\).*$/\1/')
	Height=$(echo $Geometry | \
	    sed -e 's/^.*ID_VIDEO_HEIGHT=\([[:digit:]]*\).*$/\1/')
	Options="-v 10 -a 10 --optimize"
	if [ $Width -gt 1000 -o $Height -gt 1000 ]
	then
	    Options="$Options --max_size 960x960"
	fi
	ffmpeg2theora -o $WebFolder/$OggFile $Options \
	    $OrigFolder/$VidFile >/dev/null 2>&1
	OrigSize=$(ls -sh $OrigFolder/$VidFile | awk '{print $1}')
	OggSize=$(ls -sh $WebFolder/$OggFile | awk '{print $1}')
	echo Created $OggFile in webpics/$Folder \($OrigSize -\> \
	    $OggSize\)


	Count=$(( $Count + 1 ))
	if [ $Count -ge $Batch ]
	then
	    PromptYesNo "Convert $Batch more files to OGG"
	    if [ $Response != "Y" ]
	    then
		exit
	    else
		echo "OK, continuing"
		Count=0
	    fi
	fi



    fi
done
