#!/bin/sh
######################################################################
# phoneupload.sh
######################################################################
# Copyright 2021 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
#                                                                    #
#     ##########################################################     #
#     # This script is no longer maintained. Use with caution! #     #
#     ##########################################################     #
#                                                                    #
######################################################################
WarnUnmaintained()
{
cat <<EOF
######################################################################
#                                                                    #
#     ##########################################################     #
#     # This script is no longer maintained. Use with caution! #     #
#     ##########################################################     #
#                                                                    #
######################################################################

EOF
return  
}

PrintHelp()
{
    WarnUnmaintained
cat <<EOF

phoneupload.sh Copyright                   2021 Theodore B. Ruegsegger

Mounts any of various memory card types and copies all files to new
subdirectory of $UploadPath. Writes file timestamps to a text file,
phonetimestamps.txt, in the same directory. Optionally, copies (scp)
the new directory and files to a remote host.

Usage:
    phoneupload.sh [ -hs ][ -r remotepath ]

Example:
    phoneupload.sh -r ted@pikachu:/home/public/photos

Options:
    -h
        Display a help message listing options.

    -r remotepath
        After copying files locally, copy the new directory and files
        via scp to remotepath.

    -s
	Display system and configuration requirements.

EOF
return  
}

PrintReqts()
{
cat <<EOF

Requirements:

The -r option uses secure copy (scp) to transfer files, so local host
must have secure shell (ssh) installed.

phoneupload.sh is adapted for cell phone memory cards that contain
images in a single folder under the root.

Currently handles:

  * Some Nokia models: An "Images" subfolder with oddball filenames
    like "Image(029).jpg". The script removes the parentheses and
    captures file timestamps.

  * Some LG models: A "my_pix" subfolder with filenames that are
    timstamps (to nearest minute, with a one-letter suffix for
    additional images within same minute). The script captures file
    timestamps and uses them where they're consistent with the
    filenames; where not (for example, if the image was "moved" within
    the phone's memory architecture, uses the time from the filename.

To handle a different folder name, add it to the list $ ImageFolders
and edit the handler code as needed.

Can handle CompactFlash/MicroDrive in PCMCIA adapter (appears as IDE
drive) as well as various cards in USB devices (appears as SCSI
drive).

The \$Devices variable lists devices to consider, currently:
$Devices
but the script ignores any devices with no corresponding mount point
under the /media/ folder; this avoids conflicts with other devices
like permanently-mounted SCSI disk drives.

Note that all mount points require an entry in /etc/fstab, for
example:

Device:		/dev/sdb1
Mount point:	/media/sdb1
/etc/fstab line:
/dev/sdb1	/media/sdb1	auto	user,noauto	0	0

EOF
return  
}

Prompt()
{
    # Displays a string, waits for a response, and converts it to caps
    echo -n $1' '
    read RawResponse
    Response=$(echo $RawResponse | tr [:lower:] [:upper:])
    return
}

PromptYesNo()
{
    # Displays a string and waits for a Yes or No response
    Prompt "$1"' [yes, no]?'
    case $Response in
	Y|YES)
	    Response=Y
	    return
	    ;;
	N|NO)
	    Response=N
	    return
	    ;;
	*)
	    PromptYesNo "$1"
	    ;;
    esac
    return
}

TimeFromFile ()
{
    # Extracts the time stamp of the specified file in "full-iso"
    # format excluding the fractional seconds, in UTC (GMT).
    # Note: Since the phone's clock is almost certainly on local time,
    # and a GNU platform probably keeps its clock in UTC, copying the
    # files from the phone to GNU will erroneously offset their
    # timestamps. This script undoes that offset.
    Raw=$(ls -l --time-style=full-iso $1 | awk '{print $6,$7}' \
	| sed -e 's/\.[^.][^.]*$//')
    date -u --date="TZ=\"UTC\" $Raw" +"%F %T"
    return
}

TimeFromName ()
{
    # Parses a filename in format MMDDYYmmssX.* where X is an optional
    # letter, e.g., 0922061716b.jpg, to construct a date and time with
    # estimated seconds. Converts to UTC (GMT).
    Filter="s/^\(..\)\(..\)\(..\)\(..\)\(..\).*$/"
    Filter="$Filter$(date +%C)\3-\1-\2 \4:\5/"
    UpToMin=$(echo $1 | sed -e "$Filter")
    Suffix=$(echo $1 | cut -c11)
    case $Suffix in
	'.' ) Sec="00" ;;
	'a' ) Sec="05" ;;
	'b' ) Sec="10" ;;
	'c' ) Sec="15" ;;
	'd' ) Sec="20" ;;
	'e' ) Sec="25" ;;
	'f' ) Sec="30" ;;
	'g' ) Sec="35" ;;
	'h' ) Sec="40" ;;
	'i' ) Sec="45" ;;
	'j' ) Sec="50" ;;
	'k' ) Sec="55" ;;
    esac
    date -u --date="TZ=\"localtime\" $UpToMin:$Sec" +"%F %T"
    return
}

TimeStamp ()
{
    # Compares the time implied by the filename with the actual
    # timestamp of the file. If they're within two minutes, returns
    # the actual timestamp, else returns the implied time with
    # estimated seconds.
    TimeFromName=$(TimeFromName $1)
    TimeFromFile=$(TimeFromFile $1)
    SecsName=$(date -d "$TimeFromName" +%s)
    SecsFile=$(date -d "$TimeFromFile" +%s)
    Diff=$(( $SecsFile - $SecsName ))
    if [ -120 -le $Diff -a $Diff -le 120 ]
    then
	TimeStamp=$TimeFromFile
    else
	TimeStamp=$TimeFromName
    fi
    echo $TimeStamp
    return
}

######################################################################
UploadPath='/home/public/photos'
Devices="
hde1
mmcblk0p1
sda1
sdb1
sdc1
sdd1
sde1
"
ImageFolders="
my_pix
Images
"
Usage="phoneupload.sh [ -hs ][ -r remotepath ]"
######################################################################
WarnUnmaintained

Host=$(hostname)

# Parse command line options
while getopts "hr:s" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      r ) RemotePath=$OPTARG
          ;;
      s ) PrintReqts
          exit
          ;;
      \?) echo $Usage
	  exit
	  ;;
    esac
done

MountPoints=""
for Device in $Devices
do
    if [ -d /media/$Device ]
    then
	if grep /media/$Device /etc/fstab >/dev/null
	then
	    MountPoints="$MountPoints $Device"
	fi
    fi
done
Count=$(echo $MountPoints | wc -w)
if [ $Count -eq 0 ]
then
    echo None of these devices: $Devices
    echo "has both a mount point in the /media/ folder"
    echo "and an entry in the /etc/fstab file."
    echo "Can't continue."
    exit
else
    Devices=$MountPoints
fi

DevicesPresent=""
for Device in $Devices
do
    if [ -b /dev/$Device ]
    then
	DevicesPresent="$DevicesPresent $Device"
    fi
done
Count=$(echo $DevicesPresent | wc -w)
if [ $Count -eq 0 ]
then
    echo Found none of these devices: $Devices
    echo "Can't continue."
    exit
elif [ $Count -gt 1 ]
then
    echo Found more than one device: $DevicesPresent
    echo "Can't continue (yet; in future, I'll ask which you want)."
    exit
else
    Device=$(echo $DevicesPresent)
    if ! (mount -t vfat | grep /media/$Device >/dev/null)
    then
	mount /media/$Device
    fi
    unset -v ImageFolder
    for Folder in $ImageFolders
    do
	if [ -d /media/$Device/$Folder ]
	then
	    ImageFolder=$Folder
	    break
	fi
    done
    if [ ! $ImageFolder ]
    then
	echo "Couldn't find an image folder with a name in this list:"
	for Folder in $ImageFolders
	do
	    echo "    $Folder"
	done
	echo "Might have failed to mount the memory device"
	echo "Can't continue."
	exit
    fi
fi

# If we got this far, then /dev/$Device is mounted as /media/$Device
# and contains an image folder named $ImageFolder

cd $UploadPath
SourceFolder=/media/$Device/$ImageFolder
if [ -d $SourceFolder -a \
    "$(ls -1 $SourceFolder/ | sed -ne '1p')ZZ" != "ZZ" ]
then
    TargetFolder=$(date +%Y%m%d-%H%M%S)
    echo Creating $TargetFolder
    mkdir $TargetFolder
    cp --preserve=timestamps $SourceFolder/*.* $TargetFolder
    sync ; sync
    chmod 664 $TargetFolder/*
    ls -FAl $TargetFolder
    # check if folder contents match
    SrcContents=$(cd $SourceFolder ; ls -l *.*| awk '{print $5,$8}')
    TgtContents=$(cd $TargetFolder ; ls -l *.*| awk '{print $5,$8}')
    if [ "$SrcContents" = "$TgtContents" ]
    then
	Count=$(ls -1 $TargetFolder | wc -l)
	echo "$Count files uploaded to $Host."
	if [ $RemotePath ]
	then
	    echo Copying $TargetFolder to $RemotePath/
	    scp -r $TargetFolder $RemotePath/
	fi
	PromptYesNo "OK to erase all files in $SourceFolder on $Device"
	if [ $Response = "Y" ]
	then
	    echo Erasing files in $SourceFolder
	    rm $SourceFolder/*
	else
	    echo Leaving files in $SourceFolder
	    echo "You'll need to clear it manually"
	fi
	cd $TargetFolder
	if [ "$ImageFolder" = "Images" ]
	then
	    echo Renaming the files to remove parentheses.
	    if [ -e Image.jpg ]
	    then
		mv Image.jpg Image\(000\).jpg
	    fi
	    for File in Image*.jpg
	    do
		New=$(echo $File \
		    | sed -e 's/Image(\([^)][^)]*\))/image\1/')
		mv $File $New
	    done
	fi
	echo Creating timestamp table.
	if [ "$ImageFolder" = "my_pix" ]
	then
	    for File in *.jpg
	    do
		echo $File $(TimeStamp $File) \
		    >> phonetimestamps.txt
	    done
	else
	    for File in *.jpg
	    do
		echo $File $(TimeFromFile $File) \
		    >> phonetimestamps.txt
	    done
	fi
    else
	echo '****************************************'
	echo '*           File Copy Failed!          *'
	echo '****************************************'
	echo
	S="Be sure to correct the problem before erasing removable"
	S="$S storage device $Device"
	echo $S
    fi
fi
sync ; sync
PromptYesNo "Unmount the removable storage device $Device"
if [ $Response = "Y" ]
then
    umount /media/$Device
fi
