#!/bin/sh
######################################################################
# removegps.sh

# Remove embedded GPS (latitude, longitude, altitude) metadata from
# specified photo or video file, if present.
######################################################################
# Copyright 2022 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
PrintHelp()
{
    cat <<EOF

removegps.sh                    Copyright 2022 Theodore B. Ruegsegger

Removes tags related to geolocation from embedded metadata (EXIF in
photos, similar but unnamed format in videos). Preserves original file
but creates a new version with "x" appended to the filename before the
extension.

Usage:

    removegps.sh [ -h ] file

Options:

    -h
        Display this help message listing options.

Example:
    removegps.sh img_0376.mov

EOF
return
}

######################################################################
Usage="Usage: removegps.sh [ -h ] file"
######################################################################

# Parse command line options
while getopts "h" Option
do
    case $Option in
      h ) PrintHelp
          exit
          ;;
      \?) echo $Usage
	  exit 1
	  ;;
    esac
done

if [ $# -lt 1 ]
then
    echo $Usage
    exit
else
    File=$1
fi

Filex=$(echo $File | sed -e 's/\./x./')

MimeType=$(mimetype -b $File)
if [ "$MimeType" = "video/x-theora+ogg" ]
then
    ffmpeg -y -i $File -map 0 -map_metadata -1 -c copy $Filex \
	   2>/dev/null
else
    exiftool -o $Filex -gpsaltitude= -gpslatitude= -gpslongitude= \
	     -gpsposition= -gpscoordinates= $File
fi
