/*********************************************************************
resettestdb.sql

Loads representative values for system testing
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

Requires a cyphertext (hashed password) to be supplied as an argument
(see Usage). This will be the password for every user.

Usage:
    psql -d database -U user -f resettestdb.sql \
	 -v cyphertext=\'XXXXX\'

Where:
    XXXXX is a hashed password in escaped quotes. Since hashes tend to
    	  include special characters, it's probably best to enclose it
    	  in real quotes, too, e.g.:
	      -v cyphertext=\''XXXXX'\'
*********************************************************************/

-- Tailor this script to populate the test database with known data.


-- To make use of the PWK-created users, extract their loginids from
-- their email addresses, for example:

-- SkyNet program staff
/*
INSERT INTO staffmember
 SELECT 'SkyNet', split_part(email, '@', 1)
 FROM person WHERE email LIKE '%@skynet.mil'
 AND email != 'bofh@skynet.mil';
*/

-- Service administrator (adminsvc not yet used in PhotoTrove but
-- does no harm, others useful)
INSERT INTO user_priv(regusersn, privilegesn)
SELECT r.regusersn, p.privilegesn FROM reguser r, privilege p
WHERE userid = 'bofh' AND privname in ('adminsvc', 'mnguser');

INSERT INTO user_metapriv(regusersn, privilegesn)
SELECT r.regusersn, p.privilegesn FROM reguser r, privilege p
WHERE userid = 'bofh' AND privname in ('edit', 'walbum');

-- User with write privileges
INSERT INTO user_priv(regusersn, privilegesn)
SELECT r.regusersn, p.privilegesn FROM reguser r, privilege p
WHERE userid = 'kreese' AND privname in ('edit', 'walbum');

-- User with developer privileges (mainly to use showfunctions.php)
INSERT INTO user_priv(regusersn, privilegesn)
SELECT r.regusersn, p.privilegesn FROM reguser r, privilege p
WHERE userid = 'kreese' AND privname = 'webdev';

-- If you add more users, be sure to set their initial passwords,
-- either upon account creation or in a batch operation, for example:

UPDATE reguser SET passwd_hash = :cyphertext,
passwd_changed = CURRENT_DATE;
