#!/bin/sh
######################################################################
# updatestillfiletimes.sh

# Ensure every video stillfile has a valid timestamp
######################################################################
# Copyright 2014 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################
# For each existing video whose stillfile has a null timestamp,
# extract the timestamp from the original video file and update the
# stillfile's time in the photo table.

# Usage: Run as web-server user, owner of the files:

#     updatestillfiletimes.sh database serviceroot
# Example:
#     sudo -u www-data updatestillfiletimes.sh trove \
#         /home/web/webroot-ssl/trove \
#         | tee /home/public/updatestillfiletimes.log
######################################################################
UpdateTime()
{
    # Usage: UpdateTime $StillSn $VidFile
    # Updates database record for photo $StillSn with timestamp from
    # video $VidFile
    # Assumes camera's timezone was UTC
    TimeStamp=$(VideoTimeStamp $2)
    if [ "z$TimeStamp" != "z" ]
    then
	Q="UPDATE photo SET time = '$TimeStamp UTC'"
	Q="$Q WHERE photosn = $StillSn"
	$Query -c "$Q"
	echo $Folder/$VidFile at $TimeStamp UTC
    else
	echo Still no valid timestamp for $OrigFolder/$VidFile
    fi
    return
}

VideoTimeStamp()
{
    # Usage: VideoTimeStamp $VidFile
    # Return timestamp (as a string) of video $OrigFolder/$VidFile
    T="[[:digit:]][[:digit:]]:[[:digit:]][[:digit:]]:[[:digit:]]"
    T="$T[[:digit:]]"
    S=$(mplayer -really-quiet -vo null -ao null -identify -frames 0 \
	$OrigFolder/$1 2>/dev/null | \
	grep ID_CLIP_INFO_VALUE | \
	grep $T | cut -d = -f 2)
    echo $S
    return
}

######################################################################
if [ $# -lt 2 ]
then
    echo Usage: updatestillfiletimes.sh database serviceroot
    exit
fi
Database=$1
Root=$2
Query="psql -qtd $Database -U trove"

Q="SELECT foldername || '/' || vidfilename || '/' || stillsn"
Q="$Q FROM video v, photo p, photoset s"
Q="$Q WHERE p.photosn = v.stillsn AND p.time IS NULL"
Q="$Q AND s.photosetsn = p.photosetsn"
Q="$Q ORDER BY foldername, vidfilename"

Results=$($Query -c "$Q")
for Result in $Results
do
    Folder=$(echo $Result | cut -d / -f 1)
    VidFile=$(echo $Result | cut -d / -f 2)
    StillSn=$(echo $Result | cut -d / -f 3)
    OrigFolder=$Root/fullsize/$Folder
    UpdateTime $StillSn $VidFile
done
