#!/bin/sh
######################################################################
# videotimestamps.sh

# Extract timestamp(s) from a video file
######################################################################
# Copyright 2018 Theodore B. Ruegsegger
# This file is part of the PhotoTrove web application.

# PhotoTrove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# PhotoTrove is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
######################################################################

# Given a video file, extracts (using mplayer) the metadata, searches
# for ID_CLIP_INFO_VALUE lines with timestamps and displays any
# timestamps found.

# Usage:
#     videotimestamps.sh videofile
# Example:
#     sh videotimestamps.sh img_0011.mov
######################################################################
if [ $# -lt 1 ]
then
    echo Usage: videotimestamps.sh videofile
    exit
else
    VideoFile=$1
fi

TimeStampPattern='[[:digit:]][[:digit:]]:[[:digit:]][[:digit:]]'
TimeStampPattern=${TimeStampPattern}':[[:digit:]][[:digit:]]'

TimeStamps=$(
    mplayer -really-quiet -vo null -ao null -identify -frames 0 \
	    $VideoFile 2>/dev/null \
	| grep ID_CLIP_INFO_VALUE \
	| grep "$TimeStampPattern" \
	| cut -d = -f 2)

for TimeStamp in $TimeStamps
do
    echo $TimeStamp
done

