<?php
/*********************************************************************
adjusttimes.php
Adjust Timestamps Module for PhotoTrove
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require ("lib_inputval.php");
require_once ("lib_layout.php");
require ("lib_tag.php");
require ("lib_photoset.php");
require_once ("local/loc_camera.php");

$T = "COALESCE(to_char(time AT TIME ZONE 'UTC', '";
$T .= DateTimeFormatSQLFull."') || 'UTC', '".NullTimeFlag."')";
define("ClauseUTCTimeForDisplay", $T);
$T = "PhotoTrove saves photo timestamps in Coordinated Univeral";
$T .= " Time (UTC) and displays them in the current local time (";
$T .= LocalTimeZone."). Sometimes photo timestamps are incorrect by";
$T .= " a fixed amount, perhaps because the camera's clock wasn't";
$T .= " set to UTC or because the clock was simply wrong. This tool";
$T .= " adjusts the timestamps for any number of photos in a single";
$T .= " action.<br style=\"margin-bottom: 1em\"/>\n";
$T .= "To specify which photos to adjust, use the &quot;Tag";
$T .= " selected photos&quot; feature on the Search or Edit Photo";
$T .= " Set pages to assign a particular tag to all the photos in";
$T .= " question, and only to those, then select that tag on this";
$T .= " page.";
define("Introduction", $T);
define("IntroStyle", BoxStyle."; width: 50%; text-align: left");

function BoxTagSelect($TagSn)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SearchByTag", True);
    $S .= StrTableBegin("0", "", "", BoxStyle);
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Select tag:&nbsp;"));
    $S .= StrCell(MenuTag($TagSn, "TagSn", True, False));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function CountNullTimePhotos($TagSn)
{
    $Q = " SELECT COUNT(*)";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND time IS NULL";
    return pg_fetch_result(Query($Q, $TagSn), 0, "count");
}

function ErrorSign($Sign)
{
    // Return error bullet if sign isn't one of the two legal values
    if ($Sign == "fwd" or $Sign == "bck")
	$E = "";
    else
    {
	$E = "You must specify whether to move the timestamps";
	$E .= " forward or back";
	$E = BulletItem($E);
    }
    return $E;
}

function ErrorsSetTimeStamps($Sign, $Hr, $Min, $Sec)
{
    $E = ErrorSign($Sign);
    $E .= ErrIfNotNatural("Hours", $Hr, True);
    $E .= ErrIfBeyondRealRange("Minutes", $Min, 0, True, 60, False);
    $E .= ErrIfBeyondRealRange("Seconds", $Sec, 0, True, 60, False);
    $E .= ErrorZero($Hr, $Min, $Sec);
    return $E;
}

function ErrorZero($Hr, $Min, $Sec)
{
    // Return error bullet if adjustment time is zero
    if ($Hr + $Min + $Sec == 0)
    {
	$E = "You must specify a nonzero adjustment amount";
	$E = BulletItem($E);
    } else $E = "";
    return $E;
}

function FormEnterOffset($TagSn, $Sign, $Hr, $Min, $Sec)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= StrTableBegin(0, "", "", BoxStyle);
    $S .= StrRowBegin();
    $S .= StrCell("Adjust all timestamps:");
    $C = "<label>";
    $C .= HtmlRadioButton("Sign", "fwd", ($Sign == "fwd"));
    $C .= " forward</label><br />\n<label>";
    $C .= HtmlRadioButton("Sign", "bck", ($Sign == "bck"));
    $C .= " back</label>";
    $S .= StrCell($C);
    $C = HtmlTextInput("Hr", 1, $Hr, "", "text-align: right");
    $S .= StrCell($C." hours");
    $C = HtmlTextInput("Min", 1, $Min, "", "text-align: right");
    $S .= StrCell($C." minutes");
    $C = HtmlTextInput("Sec", 1, $Sec, "", "text-align: right");
    $S .= StrCell($C." seconds");
    $C = HtmlSubmitButton("Set");
    $S .= StrCell($C);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function FormUntag($TagSn, $TagName)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= StrTableBegin(0, "", "", BoxStyle."; width:50%");
    $S .= StrRowBegin();
    $T = "Now that you've adjusted their timestamps, you may wish to";
    $T .= " remove the &quot;$TagName&quot; tag from all the photos.";
    $S .= StrCell($T);
    $S .= StrCell(HtmlSubmitButton("Untag", "Untag All"));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function HeadingPhotoCount($NumHits, $TagName, $Res, $TagSn)
{
    if ($NumHits > 0)
    {
	$First = pg_fetch_result($Res, 0, "utctimestamp");
	$Plural = ($NumHits > 1) ? "s" : "";
	$S = "$NumHits photo$Plural tagged &quot;$TagName&quot;";
	if ($NumHits > 1)
	{
	    $Last = pg_fetch_result($Res, $NumHits-1, "utctimestamp");
	    $S .= "<br />\nbetween $First and $Last:";
	} else
	{
	    $S .= " at $First:";
	}
    } else $S = "No photos tagged &quot;$TagName&quot;.";
    $S = StrNoticeBox($S);
    $Nulls = CountNullTimePhotos($TagSn);
    if ($Nulls > 0)
    {
	$W = "$Nulls photo".(($Nulls > 1) ? "s are": " is");
	$W .= " tagged &quot;$TagName&quot; but ha";
	$W .= (($Nulls > 1) ? "ve": "s");
	$W .= " no timestamp and will be unaffected by the";
	$W .= " adjustment. Are you sure you meant to include th";
	$W .= (($Nulls > 1) ? "ose": "at one")."?<br/>\n";
	$W .= "If not, use <a href=\"search.php\"";
	$W .= " target=\"_blank\">Search</a> (specifying Date/Time";
	$W .= " as unset) to remove the tag.";
	$S .= StrWarningBox($W, "50%");
    }
    return $S;
}

function PageEnterOffset($TagSn, $IsSet, $Valid, $Sign="", $Hr="0",
			 $Min="0", $Sec="0")
{
    $Res = ResTaggedPhotos($TagSn);
    $NumRows = pg_num_rows($Res);
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $S = HeadingPhotoCount($NumRows, $TagName, $Res, $TagSn);
    if ($NumRows > 0)
    {
	$S .= PhotoStats2TZ($Res, 0, True);
	if ($NumRows > 1)
	    $S .= PhotoStats2TZ($Res, $NumRows - 1, False);
	$S .= FormEnterOffset($TagSn, $Sign, $Hr, $Min, $Sec);
	if ($IsSet and $Valid) $S .= FormUntag($TagSn, $TagName);
    }
    return $S;
}

function PhotoStats2TZ($Res, $Row, $IsFirst)
{
    // Display either the first or the last photo in $Res, with
    // statistics. Shows timestamp in UTC and local time.
    list ($PhotoSn, $FolderName, $FileName, $LocationSn, $Description,
	  $ThumbW, $ThumbH, $UtcTime, $LocName, $VidFileName,
	  $LocalTime) = pg_fetch_array($Res, $Row);
    list($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $Path = "$ThumbFolder/$FileName";
    $V = "";
    $CellStyle = "text-align: right";
    if ($VidFileName > "")
    {
	$V = "(Video) ";
	$CellStyle .= "; background: ".HighlightBg;
    }
    $S = StrTableBegin(0, "", "",
		       BoxStyle."; width: 60%; margin-bottom: 1ex");
    $S .= StrRowBegin();
    $T = ($IsFirst ? "First" : "Last")." photo:<br />\n";
    $T .= "<span style=\"background: yellow\">$UtcTime</span>";
    $T .= "<br />\n($LocalTime)<br />\nFile ".PhotoPointer($PhotoSn);
    $T .= "<br />\nLocation: ${LocName}";
    $T .= HtmlPara($Description);
    $S .= StrCell($T);
    $T = HtmlImage(LatestFile($Path),
		   "$FileName $V$UtcTime ($LocName) $Description",
		   $ThumbH, $ThumbW, 1);
    $S .= StrCell($T, "", "", $CellStyle);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function ResTaggedPhotos($TagSn)
{
    $Q = " SELECT tp.photosn, foldername, filename, p.locationsn,";
    $Q .= " description, thumbw, thumbh, ";
    $Q .= ClauseUTCTimeForDisplay." AS utctimestamp";
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " v.vidfilename, ";
    $Q .= ClauseTimeForDisplay." AS localtimestamp";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND time IS NOT NULL";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY time, filename";
    return Query($Q, $TagSn);
}

function SetTimeStamps($TagSn, $Sign, $Hr, $Min, $Sec, &$Valid)
{
    $Err = ErrorsSetTimeStamps($Sign, $Hr, $Min, $Sec);
    $Valid = ($Err == "");
    if ($Valid)
    {
	$OffsetSec = ($Hr * 60 + $Min) * 60 + $Sec;
	$OffsetSign = ($Sign == "fwd") ? "+" : "-";
	$Q = "UPDATE photo p SET time =";
	$Q .= " time $OffsetSign $OffsetSec * INTERVAL '1 second'";
	$Q .= " WHERE EXISTS (SELECT * FROM tag_photo tp";
	$Q .= " WHERE tagsn = $1 AND tp.photosn = p.photosn)";
	$Q .= " AND time IS NOT NULL";
	Query($Q, $TagSn);
	$S = "Adjusted timestamps ";
	$S .= (($Sign == "fwd") ? "forward" : "back");
	$S .= " by $Hr hr, $Min min, $Sec sec";
	$S = StrWarning($S, NoticeColor);
    } else
    {
	$S = Strong("Can't adjust timestamps:")."\n<ul>$Err\n</ul>";
	$S = StrWarningBox($S, "50%");
    }
    return $S;
}

function UntaggedAll($TagSn)
{
    $Q = "DELETE FROM tag_photo WHERE tagsn = $1";
    Query($Q, $TagSn);
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $S = "Removed tag &quot;$TagName&quot; from all photos.";
    return StrWarning($S, NoticeColor);
}

//********************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "Hr",
    "Min",
    "Sec",
    "Sign",
    "TagSn"
    );
$GetVarList = array(
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (! isset($Sign)) $Sign = False;
if (! isset($TagSn)) $TagSn = False;
if (! isset($Valid)) $Valid = False;
$S = StartOfPage("Adjust Timestamps");
$IsSet = (isset($FormVars["Set"]));
if ($IsSet)
    $S .= SetTimeStamps($TagSn, $Sign, $Hr, $Min, $Sec, $Valid);
elseif (isset($FormVars["Untag"]))
    $S .= UntaggedAll($TagSn);
elseif (isset($TagSn) and ! is_numeric($TagSn))
    $S .= StrWarning("Select a tag");
$S .= BoxTagSelect($TagSn);
if (! isset($Hr)) $Hr = "0";
if (! isset($Min)) $Min = "0";
if (! isset($Sec)) $Sec = "0";
if (is_numeric($TagSn))
    $S .= PageEnterOffset($TagSn, $IsSet, $Valid, $Sign, $Hr, $Min,
			  $Sec);
else $S .= "<div style=\"".IntroStyle."\">".Introduction."</div>\n";
$S .= "</div></body></html>\n";
echo $S;

?>
