<?php
/*********************************************************************
editalbum.php
Edit Album Module for PhotoTrove
_____________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
_____________________________________________________________________

*********************************************************************/
require ("lib_album.php");
require_once ("lib_common.php");
require_once ("lib_layout.php");
require ("lib_filesys.php");
require ("lib_photoset.php");
require ("gk/gk_jscalendar.php");

$T = "<script type=\"text/javascript\">\n";
$T .= "function UpdateBg()\n";
$T .= "{ document.getElementById('Scheme').style.background";
$T .= " = '#'+color1 }\n";
$T .= "function UpdateFg()\n";
$T .= "{ document.getElementById('Scheme').style.color";
$T .= " = '#'+color2\n";
$T .= "  document.getElementById('Highlight').style.color";
$T .= " = '#'+color2 }\n";
$T .= "function UpdateHi()\n";
$T .= "{ document.getElementById('Highlight').style.background";
$T .= " = '#'+color3 }\n";
$T .= "</script>\n";
define ("ColorSwitchers", $T);
$T = "<script type=\"text/javascript\" src=\"jscolor/jscolor.js\">";
$T .= "</script>\n";
define ("JsColorScript", $T);

$T = "Express dates/times in the local time zone (currently ";
$T .= LocalTimeZone."=UTC".LocalTimeZoneOffset.") of this server (";
$T .= $_SERVER["SERVER_NAME"]."), with no time zone suffix";
define ("HintTimeZoneLocal", $T);
$T = "An arbitrary time zone you specify, either by its abbreviation";
$T .= " or by its offset from UTC";
define ("HintTimeZoneOther", $T);
$T = "Express dates/times in Coordinated Universal Time (UTC), aka";
$T .= " Greenwich Mean Time (GMT), with UTC suffix";
define ("HintTimeZoneUTC", $T);

define("TempOffset", 100000); // To avoid collsions in block moves
define ("PromptNewAlbum", "Enter name for new album");
$T = "Browse a photo set in a separate window to add new photos to";
$T .= " the album";
define ("PromptPhotoSet", $T);
define ("PromptSelectAlbum", "Edit an existing album");
define("StyleBtn", "width: 100%");

$T = "<script type=\"text/javascript\">\n";
$T .= "function ReSeq(Sn)\n";
$T .= "{\n";
$T .= "document.getElementById(\"ChangeSeq\").value=Sn;\n";
$T .= "document.getElementById(\"ThumbGallery\").submit();\n";
$T .= "}\n";
$T .= "function EditInSet()\n";
$T .= "{\n";
$T .= "document.getElementById(\"FormEditInSet\").submit();\n";
$T .= "}\n";
$T .= "</script>\n";
define("JavaScriptDefinitions", $T);

function AlbumPageExists($AlbumSn, $Page)
{
    $Q = "SELECT * FROM albumpage";
    $Q .= " WHERE albumsn = $1 AND page = $2";
    $Pg = Query($Q, $AlbumSn, $Page);
    return (pg_num_rows($Pg) > 0);
}

function AlbumPhotoActionOptions()
{
    // Vertical array of actions that may be applied to selected
    // album photos in thumbnail gallery
    $S = LineSelect();
    $S .= StrTableBegin("0");
    $S .= OptionAssignCaption();
    $S .= OptionRevertCaption();
    $S .= OptionAppendToCaption();
    $S .= OptionMove();
    $S .= OptionDrop();
    $S .= StrTableEnd();
    return $S;
}

function AlbumPhotoCaption($AlbumSn, $Page, $Ordinal)
{
    $Q = "SELECT caption FROM albumphoto ap, albumpage pg";
    $Q .= " WHERE albumsn = $1 AND page = $2";
    $Q .= " AND pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND ordinal = $3";
    $Res = Query($Q, $AlbumSn, $Page, $Ordinal);
    return (pg_num_rows($Res) > 0)
        ? pg_fetch_result($Res, 0, "caption") : "";
}

function AlbumPhotoThumbnail($AlbumSn, $Page, $Ordinal)
{
    list($AlbumPhotoSn, $FolderName, $FileName, $Caption, $Width,
         $Height, $ThumbW, $ThumbH, $Time, $LocationSn, $LocName,
         $VidFileName, $VWidth, $VHeight)
        = pg_fetch_array(ResAlbumPhoto($AlbumSn, $Page, $Ordinal));
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $VidBgColor = pg_fetch_result(ResAlbum($AlbumSn), 0,
                                  "vidbgcolor");
    $Path = "$ThumbFolder/$FileName";
    $Label = HtmlCleanTitle($Caption);
    $S = "<img src=\"".LatestFile($Path)."\"";
    $S .= " alt=\"$Label\" title=\"$Label\" width=\"$ThumbW\"";
    $S .= " height=\"$ThumbH\"";
    if ($VidFileName > "")
        $S .= " style=\"border: medium solid $VidBgColor\"";
    $S .= " />";
    return $S;
}

function AppendedToCaption($AlbumSn, $Sort, $NumPics)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    list($FmtDay, $FmtDate, $FmtTime, $TimeZone, $AppendLoc,
         $AppendGPS, $Desc) = FormatsForAppend($FormVars);
    $Count=0;
    for ($Row=0 ; $Row < $NumPics ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            AppendToCaption($FormVars["Sn$Row"], $FmtDay, $FmtDate,
                            $FmtTime, $TimeZone, $AppendLoc,
                            $AppendGPS);
            $Count++;
        }
    }
    if ($Count > 0)
    {
        $W = "";
        $N = "Appended $Desc to caption for $Count photo";
        if ($Count != 1) $N .= "s";
        $N .= ".";
    } else
    {
        $W = "No photo checked, so appended $Desc to no caption.";
        $N = "";
    }
    $S = ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                     False, False, $W, $N);
    return $S;
}

function AppendToCaption($AlbumPhotoSn, $FmtDay, $FmtDate, $FmtTime,
                         $TimeZone, $AppendLoc, $AppendGPS)
{
    $Q = "SELECT ap.caption, l.name";
    $QT = "";
    if ($FmtDay) $QT .= ", $FmtDay";
    if ($FmtDate) $QT .= ", $FmtDate";
    if ($FmtTime) $QT .= ", $FmtTime";
    if ($QT != "")
    {
        $QT = substr($QT, 2);
        $T = ($TimeZone == "UTC")
           ? "time AT TIME ZONE 'UTC'" : "time";
        $Q .= ", COALESCE(to_char($T, '$QT'), '".NullTimeFlag."')";
    } else $Q .= ", time";
    $Q .= " FROM albumphoto ap, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " WHERE albumphotosn = $1 AND p.photosn = ap.photosn";
    list ($Caption, $LocName, $Time)
        = pg_fetch_array(Query($Q, $AlbumPhotoSn), 0);
    $C = "";
    if ($TimeZone == "UTC" and $Time != NullTimeFlag) $Time .= " UTC";
    $Count = 1;
    $Params = array($AlbumPhotoSn);
    if ($QT != "")
    {
        $Count++;
        $Params[] = $Time;
        $C .= " || ', ' || $$Count";
    }
    if ($AppendLoc and $LocName != "")
    {
        $Count++;
        $Params[] = $LocName;
        $C .= " || ', at ' || $$Count";
        $PhotoSn = ColFromSerialNumber("albumphoto", "photosn",
                                       $AlbumPhotoSn);
        $G = GeoCoordinates($PhotoSn);
        if ($G !== False)
        {
            $Count++;
            $Params[] = " ($G)";
            $C .= " || $$Count";
        }
    }
    if ($C != "")
    {
        $C = substr($C, ($Caption == "")? 11 : 4);
        $Q = "UPDATE albumphoto";
        $Q .= " SET caption = COALESCE(caption, '') || $C";
        $Q .= " WHERE albumphotosn = $1";
        QueryArray($Q, $Params);
    }
    return;
}

function AppliedCaption($AlbumSn, $Sort, $NumPics, $Caption)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    $Caption = trim($Caption);
    $Count=0;
    for ($Row=0 ; $Row < $NumPics ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            ApplyCaption($FormVars["Sn$Row"], $Caption);
            $Count++;
        }
    }
    if ($Count > 0)
    {
        $W = "";
        $N = "Applied caption &quot;$Caption&quot; to $Count photo";
        if ($Count != 1) $N .= "s";
        $N .= ".";
    } else
    {
        $W = "Nothing checked, so caption &quot;$Caption&quot; not";
        $W .= " applied.";
        $N = "";
    }
    $S = ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                     False, False, $W, $N);
    return $S;
}

function ApplyCaption($AlbumPhotoSn, $Caption)
{
    $Caption = trim($Caption);
    $Q = "UPDATE albumphoto SET caption = $2";
    $Q .= " WHERE albumphotosn = $1 AND COALESCE(caption, '') != $2";
    Query($Q, $AlbumPhotoSn, $Caption);
    return;
}

function ClauseMoveOnPage($AlbumPageSn, $OldOrdinal, $NewOrdinal)
{
    $TitlePicOrd = TitlePicOrd($AlbumPageSn);
    if ($NewOrdinal < $OldOrdinal)
    {
        // Open a slot for Pic. First move all out of bounds then back
        // in.
        $Q = "UPDATE albumphoto SET ordinal = ordinal+".TempOffset;
        $Q .= " WHERE albumpagesn = $3";
        $Q .= " AND ordinal >= $1";
        $Q .= " AND ordinal < $2";
        Query($Q, $NewOrdinal, $OldOrdinal, $AlbumPageSn);
        $Q = "UPDATE albumphoto SET ordinal = ordinal+1-".TempOffset;
        $Q .= " WHERE albumpagesn = $3";
        $Q .= " AND ordinal >= $1+".TempOffset;
        $Q .= " AND ordinal < $2+".TempOffset;
        Query($Q, $NewOrdinal, $OldOrdinal, $AlbumPageSn);
        // Adjust page's title pic if needed
        if ($TitlePicOrd == $OldOrdinal)
            SetTitlePicOrdinal($AlbumPageSn, $NewOrdinal);
        elseif ($TitlePicOrd >= $NewOrdinal
                and $TitlePicOrd < $OldOrdinal)
            SetTitlePicOrdinal($AlbumPageSn, $TitlePicOrd + 1);
    } elseif ($NewOrdinal > $OldOrdinal)
    {
        // Open a slot for Pic. First move all out of bounds then
        // back in.
        $Q = "UPDATE albumphoto SET ordinal = ordinal+".TempOffset;
        $Q .= " WHERE albumpagesn = $3";
        $Q .= " AND ordinal > $2";
        $Q .= " AND ordinal <= $1";
        Query($Q, $NewOrdinal, $OldOrdinal, $AlbumPageSn);
        $Q = "UPDATE albumphoto SET ordinal = ordinal-1-".TempOffset;
        $Q .= " WHERE albumpagesn = $3";
        $Q .= " AND ordinal > $2+".TempOffset;
        $Q .= " AND ordinal <= $1+".TempOffset;
        Query($Q, $NewOrdinal, $OldOrdinal, $AlbumPageSn);
        // Adjust page's title pic if needed
        if ($TitlePicOrd == $OldOrdinal)
            SetTitlePicOrdinal($AlbumPageSn, $NewOrdinal);
        elseif ($TitlePicOrd > $OldOrdinal
                and $TitlePicOrd <= $NewOrdinal)
            SetTitlePicOrdinal($AlbumPageSn, $TitlePicOrd - 1);
    }
    return "ordinal = $1";
}

function ClauseMoveToPage($AlbumSn, $OldPageSn, $OldOrdinal, $NewPage,
                          $NewOrdinal)

{
    // Ensure new page exists
    if (! AlbumPageExists($AlbumSn, $NewPage))
        CreateAlbumPage($AlbumSn, $NewPage);
    $NewPageSn = pg_fetch_result(
        ResAlbumPage(False, $AlbumSn, $NewPage), 0, "albumpagesn");
    // Close up Pic's slot on old page (Pic has fake ordinal to
    // prevent collisions. First move all out of bounds then back in.
    $Q = "UPDATE albumphoto SET ordinal = ordinal+".TempOffset;
    $Q .= " WHERE albumpagesn = $2 AND ordinal > $1";
    Query($Q, $OldOrdinal, $OldPageSn);
    $Q = "UPDATE albumphoto SET ordinal = ordinal-1-".TempOffset;
    $Q .= " WHERE albumpagesn = $2 AND ordinal > $1+".TempOffset;
    Query($Q, $OldOrdinal, $OldPageSn);
    // Adjust old page's title pic if needed
    $TitlePicOrdOldPage = TitlePicOrd($OldPageSn);
    if ($TitlePicOrdOldPage == $OldOrdinal)
        SetTitlePicOrdinal($OldPageSn, 0);
    elseif ($TitlePicOrdOldPage > $OldOrdinal)
        SetTitlePicOrdinal($OldPageSn, $TitlePicOrdOldPage - 1);
    // Open a slot for Pic in new page. First move all out of bounds
    // then back in.
    $Q = "UPDATE albumphoto SET ordinal = ordinal+".TempOffset;
    $Q .= " WHERE albumpagesn = $2 AND ordinal >= $1";
    Query($Q, $NewOrdinal, $NewPageSn);
    $Q = "UPDATE albumphoto SET ordinal = ordinal+1-".TempOffset;
    $Q .= " WHERE albumpagesn = $2 AND ordinal >= $1+".TempOffset;
    Query($Q, $NewOrdinal, $NewPageSn);
    // Adjust new page's title pic if needed
    $TitlePicOrdNewPage = TitlePicOrd($NewPageSn);
    if ($TitlePicOrdNewPage >= $NewOrdinal)
        SetTitlePicOrdinal($NewPageSn, $TitlePicOrdNewPage + 1);
    return "albumpagesn = $NewPageSn, ordinal = $1";
}

function ClosePageGaps($AlbumSn)
{
    $Q = "SELECT page FROM albumpage";
    $Q .=" WHERE albumsn = $1";
    $Q .=" ORDER BY page";
    $Pgs = Query($Q, $AlbumSn);
    $HoleSize = 0;
    $RangeStart = False;
    $RangeEnd = -1;
    Query("BEGIN");
    for ($Row = 0 ; $Row < pg_num_rows($Pgs) ; $Row++)
    {
        $Page = pg_fetch_result($Pgs, $Row, 0);
        if ($Page > $Row + $HoleSize)
        {
            // Start of a page range to shift (hole-page transition)
            if ($RangeStart)
            {
                // Shift previous range first
                SlideAlbumPages($AlbumSn, $RangeStart, $RangeEnd,
                                -$HoleSize);
            }
            $HoleSize = $HoleSize + $Page - $RangeEnd - 1;
            $RangeEnd = $Page;
            $RangeStart = $Page;
        } else
        {
            // Continuation of a page range
            $RangeEnd = $Page;
        }
    }
    if ($RangeStart)
        SlideAlbumPages($AlbumSn, $RangeStart, $RangeEnd,
                        -$HoleSize);
    Query("COMMIT");
    return;
}

function CompactedPages($AlbumSn)
{
    ClosePageGaps($AlbumSn);
    return ScreenEditTextFormat($AlbumSn);
}

function CountAlbumPhotos($AlbumSn, $ShowPages)
{
    // return number and date range of photos on $ShowPages, where
    // $ShowPages is array(PageNumber)
    $Q = "SELECT COUNT(*), to_char(MIN(time), 'DD Mon YYYY'),";
    $Q .= " to_char(MAX(time), 'DD Mon YYYY')";
    $Q .= " FROM albumphoto ap, photo p, albumpage pg";
    $Q .= " WHERE albumsn = $1 AND pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND p.photosn = ap.photosn AND (FALSE";
    foreach ($ShowPages as $Page) $Q .= " OR page = $Page";
    $Q .= ")";
    return pg_fetch_array(Query($Q, $AlbumSn), 0);
}

function CreatedAlbum($AlbumName, $Sort)
{
    $AlbumName = trim($AlbumName);
    if (! GoodFileName($AlbumName))
    {
        $S = StartOfPage("Album Worksheet");
        $W = "Couldn't create album:<br />\n";
        $W .= WarnBadAlbumName($AlbumName);
        $S .= HdgSelectCreateDelete(False, $Sort, "", $W);
    } else
    {
        $AlbumSn = SerialNumber("album", "name", $AlbumName);
        if ($AlbumSn)
        {
            $W = "Album &quot;$AlbumName&quot; already exists";
            $S = ScreenAlbum($AlbumSn, $Sort,
                             PagesToDisplay($AlbumSn), False, False,
                             $W);
        } else
        {
            $AlbumSn = FinalAlbumSn("newalbum", $AlbumName);
            $N = "Created new album named &quot;$AlbumName&quot;";
            $S = ScreenAlbum($AlbumSn, $Sort,
                             PagesToDisplay($AlbumSn), False, False,
                             "", $N);
        }
    }
    return $S;
}

function DeletedAlbum($AlbumSn, $Sort)
{
    $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
    if ($AlbumName === False)
    {
        $S = StartOfPage("Album Worksheet");
        $W = "No such album exists!";
        $S .= HdgSelectCreateDelete(False, $Sort, "", $W);
    } else
    {
        DeleteAlbum($AlbumSn);
        $S = StartOfPage("Album Worksheet");
        $S .= StrWarning("Deleted album &quot;$AlbumName&quot;",
                         NoticeColor);
        $S .= HdgSelectCreateDelete(False, $Sort);
    }
    return $S;
}

function DeleteAlbum($AlbumSn)
{
    Query("BEGIN");
    $Q = "DELETE FROM albumphoto ap WHERE EXISTS";
    $Q .= " (SELECT TRUE FROM albumpage";
    $Q .= " WHERE albumpagesn = ap.albumpagesn AND albumsn = $1)";
    Query($Q, $AlbumSn);
    Query("DELETE FROM albumpage WHERE albumsn = $1", $AlbumSn);
    Query("DELETE FROM archivedalbum WHERE albumsn = $1", $AlbumSn);
    Query("DELETE FROM album WHERE albumsn = $1", $AlbumSn);
    Query("COMMIT");
    return;
}

function DeletedPage($AlbumSn, $Page)
{
    if ($Page > 0)
    {
        Query("BEGIN");
        // Delete the page itself
        $Q = "DELETE FROM albumpage WHERE albumsn = $1 AND page = $2";
        Query($Q, $AlbumSn, $Page);
        // Move higher pages down
        SlideAlbumPages($AlbumSn, $Page + 1, LastPage($AlbumSn), -1);
        Query("COMMIT");
    }
    return ScreenEditTextFormat($AlbumSn);
}

function DropAlbumPhoto($AlbumPhotoSn)
{
    // Drop photo from album, return array($AlbumPageSn, $Ordinal)
    list($Ordinal, $Z1, $AlbumPageSn)
        = pg_fetch_array(ResAlbumPhotoBySn($AlbumPhotoSn), 0);
    Query("BEGIN");
    $Q = "DELETE FROM albumphoto WHERE albumphotosn = $1";
    Query($Q, $AlbumPhotoSn);
    // Close hole on album page. First move all out of bounds then
    // back in.
    $Q = "UPDATE albumphoto SET ordinal = ordinal+".TempOffset;
    $Q .= " WHERE albumpagesn = $1 AND ordinal > $2";
    Query($Q, $AlbumPageSn, $Ordinal);
    $Q = "UPDATE albumphoto SET ordinal = ordinal-1-".TempOffset;
    $Q .= " WHERE albumpagesn = $1 AND ordinal > $2+".TempOffset;
    Query($Q, $AlbumPageSn, $Ordinal);
    // Check page's title pic, adjust if needed
    $TitlePicOrd = TitlePicOrd($AlbumPageSn);
    if ($TitlePicOrd == $Ordinal)
        SetTitlePicOrdinal($AlbumPageSn, 0);
    elseif ($TitlePicOrd > $Ordinal)
        SetTitlePicOrdinal($AlbumPageSn, $TitlePicOrd - 1);
    Query("COMMIT");
    return array($AlbumPageSn, $Ordinal);
}

function DroppedFromAlbum($AlbumSn, $Sort, $NumPics)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    $DropCount=0;
    for ($Row=0 ; $Row < $NumPics ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            DropAlbumPhoto($FormVars["Sn$Row"]);
            $DropCount++;
        }
    }
    if ($DropCount > 0)
    {
        $W = "";
        $N = "Dropped $DropCount photo";
        if ($DropCount != 1) $N .= "s";
        $N .= " from album.";
    } else
    {
        $W = "Nothing checked, so nothing dropped.";
        $N = "";
    }
    $S = ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                     False, False, $W, $N);
    return $S;
}

function DroppedWebPicFromAlbum($AlbumPhotoSn)
{
    list($AlbumPageSn, $Ordinal) = DropAlbumPhoto($AlbumPhotoSn);
    // Now display whatever photo has dropped into that slot.
    return LargeImage($AlbumPageSn, $Ordinal);
}

function FmtDate($AppendDate, $FmtDate)
{
    if ($AppendDate)
    {
        switch ($FmtDate)
        {
        case "jMY": // 1 Aug 2010
            // $FmtDate = "j M Y";
            $FmtDate = "FMDD FMMonth YYYY";
            break;
        case "MjY": // Aug 1, 2010
            // $FmtDate = "M j,Y";
            $FmtDate = "FMMonth FMDD, YYYY";
            break;
        case "mdY": // 08/01/2010
            // $FmtDate = "%m/%d/%Y";
            $FmtDate = "MM/DD/YYYY";
            break;
        case "ISO": // 2010-08-01
            // $FmtDate = "Y-m-d";
            $FmtDate = "YYYY-MM-DD";
            break;
        default:
            $FmtDate = False;
        }
    } else $FmtDate = False;
    return $FmtDate;
}

function FmtDay($AppendDay, $FmtDay)
{
    if ($AppendDay)
    {
        switch ($FmtDay)
        {
        case "D": // Thu
            // $FmtDay = "D";
            $FmtDay = "Dy";
            break;
        case "l": // Thursday
            // $FmtDay = "l";
            $FmtDay = "FMDay";
            break;
        default:
            $FmtDay = False;
        }
    } else $FmtDay = False;
    return $FmtDay;
}

function FmtTime($AppendTime, $FmtTime)
{
    if ($AppendTime)
    {
        switch ($FmtTime)
        {
        case "hia": // 05:49pm
            // $FmtTime = "h:ia";
            $FmtTime = "HH12:MIam";
            break;
        case "Hi": // 17:49
            // $FmtTime = "H:i";
            $FmtTime = "HH24:MI";
            break;
        default:
            $FmtTime = False;
        }
    } else $FmtTime = False;
    return $FmtTime;
}

function ForcePrefix(&$BgColor, &$TextColor, &$VidBgColor)
{
    // JSColor color picker omits # prefix on color codes; restore it
    if (substr($BgColor, 0, 1) != "#")
        $BgColor = "#$BgColor";
    if (substr($TextColor, 0, 1) != "#")
        $TextColor = "#$TextColor";
    if (substr($VidBgColor, 0, 1) != "#")
        $VidBgColor = "#$VidBgColor";
    return;
}

function FormArchivedAlbumCheckbox($AlbumSn, $ArchivedAlbum, $Sort)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("ToggleArchived", 1);
    $S .= "<label>";
    $S .= HtmlCheckbox("ArchivedAlbum", $ArchivedAlbum,
                       "onchange=\"this.form.submit()\"");
    $S .= "Album is archived</label>";
    $S .= StrFormEnd();
    return $S;
}

function FormatsForAppend($FormVars)
{
    $FmtDay = FmtDay($FormVars["AppendDay"], $FormVars["FmtDay"]);
    $FmtDate = FmtDate($FormVars["AppendDate"], $FormVars["FmtDate"]);
    $FmtTime = FmtTime($FormVars["AppendTime"], $FormVars["FmtTime"]);
    $TimeZone = $FormVars["TimeZone"];
    $AppendLoc = isset($FormVars["AppendLoc"]);
    $AppendGPS = isset($FormVars["AppendGPS"]);
    $Desc = "";
    if ($FmtDay) $Desc .= ", day";
    if ($FmtDate) $Desc .= ", date";
    if ($FmtTime) $Desc .= ", time";
    if ($AppendLoc) $Desc .= ", location";
    $Desc = ($Desc != "") ? substr($Desc, 2)
          : "nothing, since no time or location was checked,";
    return array($FmtDay, $FmtDate, $FmtTime, $TimeZone, $AppendLoc,
                 $AppendGPS, $Desc);
}

function FormBrowsePhotoSet($AlbumSn)
{
    $S = StrFormBegin("editset.php", "", "_blank");
    $S .= MenuPhotoSet("", True, PromptPhotoSet);
    // Include albumsn and set switch even though editset.php doesn't
    // yet handle it
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlHidden("SetAlbumSn", True);
    $S .= StrFormEnd();
    return $S;
}

function FormDeleteAlbumButton($AlbumSn)
{
    $AlbumName = pg_fetch_result(ResAlbum($AlbumSn), 0, "name");
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlSubmitButton("DeleteAlbum",
                           "Delete Album &quot;$AlbumName&quot;", "",
                           "border: medium solid ".WarningColor);
    $S .= StrFormEnd();
    return $S;
}

function FormEditFormatButton($AlbumSn)
{
    $S = StrFormBeginRecurse("", "_blank");
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlSubmitButton("EditTextFormat",
                           "Edit Page Text and Format");
    $S .= StrFormEnd();
    return $S;
}

function FormEditInSet($PhotoSn)
{
    // A form to handle the Edit in Photo Set button
    $S = StrFormBegin("editset.php", "FormEditInSet", "_blank");
    $S .= HtmlHidden("PhotoSn", $PhotoSn);
    $S .= HtmlHidden("EditLargeBySn", 1);
    $S .= StrFormEnd();
    return $S;
}

function FormMetaAlbumPage($AlbumSn, $Page)
{
    list ($AlbumPageSn, $Z1, $Z2, $Heading, $TitlePicOrd,
          $TitlePicCaption, $PageText)
        = pg_fetch_array(ResAlbumPage(False, $AlbumSn, $Page), 0);
    // Precaution; ideally, $TitlePicOrd would adjust as photos are
    // deleted/moved:
    // $TitlePicOrd = BoundedValue($TitlePicOrd,
    //                             PicsOnPage($AlbumSn, $Page), 0);
    $NumPics = PicsOnPage($AlbumSn, $Page);
    $S = StrFormBeginRecurse();
    $S .= StrTableBegin("1", TableColor, "",
                        "border: thin solid; margin-bottom: 3mm");
    $S .= RowPagePics($Page, $NumPics, $TitlePicOrd);
    $S .= RowPageHeading($AlbumSn, $Page, $Heading, $TitlePicOrd,
                         $TitlePicCaption);
    $S .= RowPageText($PageText);
    $S .= StrTableEnd();
    $S .= HtmlHidden("Page", $Page);
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= StrFormEnd();
    return $S;
}

function FormNewAlbumName()
{
    $S = StrFormBeginRecurse();
    $S .= HtmlTextInput("AlbumName", "25", PromptNewAlbum);
    $S .= "&nbsp;";
    $S .= HtmlSubmitButton("Create");
    $S .= StrFormEnd();
    return $S;
}

function FormRenameAlbum($AlbumSn, $Sort, $AlbumName)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlTextInput("AlbumName", "22", $AlbumName, "",
                        "text-align: right");
    $S .= "&nbsp;";
    $S .= HtmlSubmitButton("Rename");
    $S .= StrFormEnd();
    return $S;
}

function FormSelectPages($AlbumSn, $Pages)
{
    list($AllPages, $ShowPages) = $Pages;
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    foreach ($AllPages as $Page => $Heading)
    {
        $T = HtmlCheckbox("Pg$Page", in_array($Page, $ShowPages),
                          SubmitIfChange);
        $S .= "<label class=\"lbl\">$T$Page</label>\n";
    }
    $T = HtmlCheckbox("PgAll", False, SubmitIfChange);
    $S .= "<label class=\"lbl\">${T}All Pages</label>\n";
    $T = HtmlCheckbox("PgNone", False, SubmitIfChange);
    $S .= "<label class=\"lbl\">${T}No Pages</label>\n";
    $T = HtmlCheckbox("PgPop", False, SubmitIfChange);
    $S .= "<label class=\"lbl\">${T}All Photos</label>\n";
    $S .= StrFormEnd();
    return $S;
}

function FormShowArchivedAlbums($AlbumSn, $Sort, $ShowArchived)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("ShowArchivedSwitch", 1);
    $S .= "<label>";
    $S .= HtmlCheckbox("ShowArchived", $ShowArchived,
                       "onchange=\"this.form.submit()\"");
    $S .= "Show <span style=\"background: ".ArchivedAlbumColor;
    $S .= "\">archived</span> albums in menu</label>";
    $S .= StrFormEnd();
    return $S;
}

function FormViewAlbumButton($AlbumSn)
{
    $S = StrFormBegin("viewalbum.php", "", "_blank");
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlSubmitButton("ViewAlbum", "View Album");
    $S .= StrFormEnd();
    return $S;
}

function HdgSelectCreateDelete($ShowArchived, $Sort, $AlbumSn="",
                               $Warning="", $AlbumName="")
{
    // Line of widgets to select, create or delete an album
    $IsAlbumArchived = IsAlbumArchived($AlbumSn);
    $S = ($Warning !== "") ? StrWarningBox($Warning, "50%") : "";
    $S .= StrTableBegin(0);
    $S .= StrRowBegin();
    $S .= StrCell(FormMenuAlbum($Sort, $AlbumSn, True, False,
                                PromptSelectAlbum,
                                ($ShowArchived or $IsAlbumArchived)));
    $S .= StrCell(Strong(" or "));
    $S .= StrCell(FormNewAlbumName());
    if ($AlbumSn != "")
    {
        $S .= StrCell(Strong(" or "));
        $S .= StrCell(FormDeleteAlbumButton($AlbumSn));
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= LineArchiveRename($ShowArchived, $AlbumSn, $IsAlbumArchived,
                            $AlbumName, $Sort);
    return $S;
}

function ImageMap($AlbumPageSn, $Width, $Height, $Ordinal)
{
    // Makes left and right borders open previous/next webpic
    $S = "<map name=\"PicMap\">\n";
    $S .= "<area shape=\"rect\" coords=\"0,0,".PageEdge;
    $S .= ",$Height\" href=\"";
    $S .= WebPicUrl($AlbumPageSn, $Ordinal)."&Backward=1\">\n";
    $S .= "<area shape=\"rect\" coords=\"".($Width - PageEdge);
    $S .= ",0,$Width,$Height\" href=\"";
    $S .= WebPicUrl($AlbumPageSn, $Ordinal)."&Forward=1\">\n";
    $S .= "</map>\n";
    return $S;
}

function IsAlbumArchived($AlbumSn)
{
    $Q = "SELECT * FROM archivedalbum WHERE albumsn = $1";
    return (is_numeric($AlbumSn)
            ? pg_num_rows(Query($Q, $AlbumSn)) > 0
            : False);
}

function LargeImage($AlbumPageSn, $Ordinal)
{
    $Q = "SELECT albumsn, page FROM albumpage WHERE albumpagesn = $1";
    list($AlbumSn, $Page)
        = pg_fetch_array(Query($Q, $AlbumPageSn), 0);
    $NumPhotosOnPage = NumPhotosOnAlbumPage($AlbumPageSn);
    if ($NumPhotosOnPage == 0)
    {
        // Page is empty, try next photo in album
        $NextPhoto = NextPhotoInAlbum($AlbumSn, $Page, 0);
        if ($NextPhoto) list($Page, $Ordinal) = $NextPhoto;
        else
        {
            // Oops, no photo follows, try previous
            $LastPhoto = LastPhotoInAlbum($AlbumSn);
            if ($LastPhoto) list($Page, $Ordinal) = $LastPhoto;
            else
            {
                $S = StartOfPage("Oops!", "", "", True);
                $S .= StrWarningBox("The album has no photos.");
                $S .= StrButton(BtnClose());
                return $S;
            }
        }
    } else $Ordinal = BoundedValue($Ordinal, $NumPhotosOnPage);
    list($AlbumPhotoSn, $FolderName, $FileName, $Caption, $Width,
         $Height, $ThumbW, $ThumbH, $Time, $LocationSn, $LocName,
         $VidFileName, $VWidth, $VHeight, $PhotoSn)
        = pg_fetch_array(ResAlbumPhoto($AlbumSn, $Page, $Ordinal));
    $IsVideo = ($VidFileName > "");
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    list($Z0, $Z1, $BgColor, $TextColor, $VidBgColor)
        = pg_fetch_array(ResAlbum($AlbumSn), 0);
    $T = StyleClassDefinitions($AlbumSn).JavaScriptDefinitions;
    $BodyStyle = "background-color: $BgColor; color: $TextColor";
    $S = StartOfPage("Page $Page, photo $Ordinal: $FileName", $T,
                     $BodyStyle, True);
    $S .= FormEditInSet($PhotoSn);
    if (! $IsVideo)
        $S .= ImageMap($AlbumPageSn, $Width, $Height, $Ordinal);
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumPhotoSn", $AlbumPhotoSn);
    $S .= "<table border=\"0\" cellspacing=\"0\"";
    if ($IsVideo) $S .= " class=\"vid\"";
    $S .= " width=\"".WebPicLength."\" cellpadding=\"4\">\n<tbody>\n";
    if ($IsVideo)
    {
        $S .= RowWebVid($WebFolder, $VidFileName, 3, $VidBgColor);
        $T = " href=\"".WebPicUrl($AlbumPageSn, $Ordinal);
        $HrefBack = "$T&Backward=1\"";
        $T = " href=\"".WebPicUrl($AlbumPageSn, $Ordinal);
        $HrefFwd = "$T&Forward=1\"";
        $S .= RowNavButtons($HrefBack, $HrefFwd, "3", $VidBgColor);
    } else
        $S .= RowWebPic("$WebFolder/$FileName", $Caption, $Height,
                        $Width);
    $S .= RowCaptionAndButtons($Caption, $Page, $Ordinal, $FolderName,
                               $VidFileName);
    $S .= RowPhotoStats($FolderName, $FileName, $Time, $LocName,
                        $VidFileName, $PhotoSn);
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function LastPhotoInAlbum($AlbumSn)
{
    // return array(Page, Ordinal) of last photo in album or, if none,
    // False
    $Q = "SELECT page, ordinal FROM albumphoto ap, albumpage pg";
    $Q .= " WHERE pg.albumpagesn = ap.albumpagesn AND albumsn = $1";
    $Q .= " ORDER BY page DESC, ordinal DESC";
    $Res = Query($Q, $AlbumSn);
    // Query gives all photos in descending order; return first
    return (pg_num_rows($Res) > 0) ? pg_fetch_array($Res, 0) : False;
}

function LineArchiveRename($ShowArchived, $AlbumSn, $ArchivedAlbum,
                           $AlbumName, $Sort)
{
    $S = StrTableBegin("0");
    $S .= StrRowBegin();
    $S .= StrCell(FormShowArchivedAlbums($AlbumSn, $Sort,
                                         $ShowArchived or
                                         $ArchivedAlbum));
    if ($AlbumSn > "")
    {
        $T = FormArchivedAlbumCheckbox($AlbumSn, $ArchivedAlbum,
                                       $Sort);
        $S .= StrCell($T, "", "", "padding-left: 2em");
        $S .= StrCell(FormRenameAlbum($AlbumSn, $Sort, $AlbumName),
                      "", "", "padding-left: 4em");
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function LineDisplay($AlbumSn, $Pages, $NumPics, $MinDate, $MaxDate)
{
    $S = StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $S .= StrCell(FormViewAlbumButton($AlbumSn));
    $T = ($NumPics == 1) ? "" : "s";
    $T = Strong("$NumPics photo{$T} ");
    if ($MinDate != "")
    {
        $T .="$MinDate";
        if ($MaxDate != $MinDate) $T .= "&ndash;$MaxDate";
    }
    $S .= StrCell($T);
    $S .= StrCell("Display&nbsp;pages:", "", "", "text-align: right");
    $S .= StrCell(FormSelectPages($AlbumSn, $Pages));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function LinePhotoSet($AlbumSn)
{
    $S = StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $S .= StrCell(FormEditFormatButton($AlbumSn));
    $S .= StrCell(FormBrowsePhotoSet($AlbumSn));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function LineSelect()
{
    $S = "<table style=\"width: 100%;\" cellpadding=\"0\"";
    $S .= " cellspacing=\"0\">\n";
    $S .= StrRowBegin();
    $S .= StrCell(Strong("For all selected photos:"),
                  "", "", "width: 60%");
    $S .= StrCell(HtmlSubmitButton("SelectAll", "Select All"),
                  "", "", "text-align: right");
    $S .= StrCell(HtmlSubmitButton("SelectRange", "Select Range"),
                  "", "", "text-align: right");
    $S .= StrCell(HtmlSubmitButton("ClearAll", "Clear Selections"),
                  "", "", "text-align: right");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function MenuTitlePicOrd($NumPics, $Selected="")
{
    // Drop-down menu of integers from 0 to $NumPics, representing
    // ordinals of photos on an album page
    $S = HtmlSelectBegin("TitlePicOrd");
    $S .= HtmlSelectOption(0, "none", True);
    for ($I = 1 ; $I <= $NumPics ; $I ++)
        $S .= HtmlSelectOption($I, $I, ($Selected == $I));
    $S .= HtmlSelectEnd();
    return $S;
}

function MetaAlbumPages($AlbumSn)
{
    $Q = "SELECT page FROM albumpage WHERE albumsn = $1";
    $Q .= " ORDER BY page";
    $Res = Query($Q, $AlbumSn);
    $S = "";
    for ($Row = 0 ; $Row < pg_num_rows($Res) ; $Row++)
    {
        $Page = pg_fetch_result($Res, $Row, "page");
        $S .= FormMetaAlbumPage($AlbumSn, $Page);
    }
    return $S;
}

function MetaEntireAlbum($AlbumSn)
{
    list ($AlbumName, $Title, $BgColor, $TextColor, $VidBgColor,
          $Author, $Url, $Notes, $PubDate)
        = pg_fetch_array(ResAlbum($AlbumSn), 0);
    $S = StrTableBegin("0", TableColor, "", "border: thin solid");
    $S .= "<tr><td>\n";
    $S .= "<h4>For entire album:</h4>\n";
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $T = Strong("HTML title (pure text): ")."<br />\n";
    $T .= HtmlTextArea("Title", 85, 2, $Title);
    $T .= "<br />\n".Strong("Publication Date:")."\n";
    $T .= DateInput("PubDate", 10, StrDate($PubDate, PubDateFormat),
                    "", False, CalPubDateFormat)."\n";
    $T .= " (not displayed in actual album)";
    $S .= HtmlPara($T);

    $T = Strong("Color Scheme:");
    $T .= "\nBackground <input class=\"color\" type=\"text\"";
    $T .= " name=\"BgColor\" id=\"BgColor\" size=\"10\"";
    $T .= " onchange=\"color1=this.color.toString(); UpdateBg()\"";
    $T .= " value=\"$BgColor\">";

    $T .= "\nText <input class=\"color\" type=\"text\"";
    $T .= " name=\"TextColor\" id=\"TextColor\" size=\"10\"";
    $T .= " onchange=\"color2=this.color.toString(); UpdateFg()\"";
    $T .= " value=\"$TextColor\">";

    $T .= "\n<span id=\"Scheme\" style=\"color: ".$TextColor;
    $T .= "; background-color: ".$BgColor;
    $T .= "\">&nbsp; Current Scheme &nbsp;</span>";

    $T .= "<br />\nVideo Highlighting: <input class=\"color\"";
    $T .= " type=\"text\" name=\"VidBgColor\" id=\"VidBgColor\"";
    $T .= " size=\"10\" onchange=\"color3=this.color.toString();";
    $T .= " UpdateHi()\" value=\"$VidBgColor\">";

    $T .= "\n<span id=\"Highlight\" style=\"color: ".$TextColor;
    $T .= "; background-color: ".$VidBgColor;
    $T .= "\">&nbsp; Current Highlighting &nbsp;</span>";
    $S .= HtmlPara($T);

    $T = Strong("Author:")."\n";
    $T .= HtmlTextInput("Author", 25, $Author)."\n";
    $T .= Strong("URL:")."\n";
    $T .= HtmlTextInput("Url", 30, $Url);
    $S .= HtmlPara($T);
    $T = Strong("Notes")." (not displayed in actual album):<br />\n";
    $T .= HtmlTextArea("Notes", 85, 4, $Notes);
    $S .= HtmlPara($T);
    $S .= HtmlPara(HtmlSubmitButton("UpdateMeta", "Save Changes"));
    $S .= StrFormEnd();
    $S .= StrBoxEnd();
    return $S;
}

function MoveAlbumPage($AlbumSn, $OldPage, $NewPage)
{
    // Move the original page to an unused region so the others have
    // room to slide without key collisions
    Query("BEGIN");
    $Q = "UPDATE albumpage SET page = $2+".TempOffset;
    $Q .= " WHERE albumsn = $3 AND page = $1";
    Query($Q, $OldPage, $NewPage, $AlbumSn);
    // Shift the other pages and their photos to accommodate the move
    if ($NewPage < $OldPage)
        SlideAlbumPages($AlbumSn, $NewPage, $OldPage - 1, 1);
    elseif ($NewPage > $OldPage)
        SlideAlbumPages($AlbumSn, $OldPage + 1, $NewPage, -1);
    // Move the original page to its final destination, now vacant
    $Q = "UPDATE albumpage SET page = $1";
    $Q .= " WHERE albumsn = $2 AND page = $1+".TempOffset;
    Query($Q, $NewPage, $AlbumSn);
    Query("COMMIT");
    return;
}

function MoveAlbumPhoto($AlbumPhotoSn, $NewPage, $NewOrdinal=False)
{
    // If no $NewOrdinal, appends to end of page
    $Q = "SELECT albumsn, ap.albumpagesn, page, ordinal";
    $Q .= " FROM albumphoto ap, albumpage pg";
    $Q .= " WHERE albumphotosn = $1";
    $Q .= " AND pg.albumpagesn = ap.albumpagesn";
    list($AlbumSn, $AlbumPageSn, $OldPage, $OldOrdinal)
        = pg_fetch_row(Query($Q, $AlbumPhotoSn), 0);
    $MaxOrdinal
        = NumPhotosOnAlbumPage(False, $AlbumSn, $NewPage,
                               $AlbumPhotoSn) + 1;
    $NewOrdinal = $NewOrdinal ?
                BoundedValue($NewOrdinal, $MaxOrdinal) : $MaxOrdinal;
    Query("BEGIN");
    // Set fake ordinal to prevent collisions
    $Q = "UPDATE albumphoto SET ordinal = ".TempOffset;
    $Q .= " WHERE albumphotosn = $1";
    Query($Q, $AlbumPhotoSn);
    $Q = ($NewPage == $OldPage)
       ? ClauseMoveOnPage($AlbumPageSn, $OldOrdinal, $NewOrdinal)
       : ClauseMoveToPage($AlbumSn, $AlbumPageSn, $OldOrdinal,
                          $NewPage, $NewOrdinal);
    if ($Q != "")
    {
        $Q = "UPDATE albumphoto SET ".$Q;
        $Q .= " WHERE albumphotosn = $2";
        Query($Q, $NewOrdinal, $AlbumPhotoSn);
    }
    Query("COMMIT");
    return;
}

function MovedToPage($AlbumSn, $Sort, $NumPics, $TargetPage)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    $MoveCount=0;
    for ($Row=0 ; $Row < $NumPics ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            MoveAlbumPhoto($FormVars["Sn$Row"], $TargetPage);
            $MoveCount++;
        }
    }
    if ($MoveCount > 0)
    {
        $W = "";
        $N = "Moved $MoveCount photo";
        if ($MoveCount != 1) $N .= "s";
        $N .= " to end of page $TargetPage";
    } else
    {
        $W = "Nothing checked, so nothing moved.";
        $N = "";
    }
    $S = ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                     False, False, $W, $N);
    return $S;
}

function NextPhotoInAlbum($AlbumSn, $Page, $Ordinal)
{
    // return ($Page, $Ordinal) of next photo in album after
    // $AlbumPageSn, $Ordinal or, if none, False
    $Q = "SELECT page, ordinal FROM albumphoto ap, albumpage pg";
    $Q .= " WHERE pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND albumsn = $3 AND ((page > $1)";
    $Q .= " OR (page = $1 AND ordinal > $2))";
    $Q .= " ORDER BY page, ordinal";
    $Res = Query($Q, $Page, $Ordinal, $AlbumSn);
    // Query gives all remaining photos; return first
    return (pg_num_rows($Res) > 0) ? pg_fetch_array($Res, 0) : False;
}

function NumPhotosOnAlbumPage($AlbumPageSn, $AlbumSn=False, $Page=0,
                              $ExcludeSn=False)
{
    // Return number of album photos on album page specified either by
    // $AlbumPageSn or by $AlbumSn and $Page. Returns 0 for
    // nonexistent page, as it should.
    $QExcl = ($ExcludeSn) ? " AND albumphotosn != $ExcludeSn" : "";
    $Q = "SELECT count(*) FROM albumphoto WHERE albumpagesn = ";
    if ($AlbumPageSn)
    {
        $Q .= "$1";
        $Res = Query($Q.$QExcl, $AlbumPageSn);
    } else
    {
        $Q .= "(SELECT albumpagesn FROM albumpage";
        $Q .= " WHERE albumsn = $1 AND page = $2)";
        $Res = Query($Q.$QExcl, $AlbumSn, $Page);
    }
    return pg_fetch_result($Res, 0, "count");
}

function OptionAppendToCaption()
{
    $S = StrRowBegin();
    $T = "Append time and/or location where each photo was taken";
    $S .= "<td title=\"$T\">Append to caption:</td>\n";
    $T = StrTableBegin("0");
    $T .= RowAppendDay();
    $T .= RowAppendDate();
    $T .= RowAppendTime();
    $T .= RowAppendTimeZone();
    $T .= RowAppendLocation();
    $T .= StrTableEnd();
    $S .= StrCell($T, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionAssignCaption()
{
    $S = StrRowBegin();
    $S .= StrCell("Assign this caption:");
    $T = StrTableBegin("0", "", "", "width: 100%");
    $T .= StrRowBegin();
    $U = Strong("Caption:")."<br />\n".HtmlTextArea("Caption", 60, 2);
    $T .= StrCell($U);
    $T .= "<td align=\"right\">";
    $T .= HtmlSubmitButton("ApplyCap", "Apply")."</td>\n";
    $T .= StrRowEnd();
    $T .= StrTableEnd();
    $S .= StrCell($T, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionDrop()
{
    $S = StrRowBegin();
    $S .= StrCell("Drop from album:");
    $T = HtmlSubmitButton("Drop", " Drop ", "",
                          "border: medium solid ".WarningColor);
    $S .= StrCell($T, "", "", BoxStyle."; text-align: right");
    $S .= StrRowEnd();
    return $S;
}

function OptionMove()
{
    $S = StrRowBegin();
    $S .= StrCell("Move to:");
    $T = StrTableBegin(0, "", "", "width: 100%");
    $T .= StrRowBegin();
    $U = "end of page ";
    $U .= HtmlTextInput("TargetPage", "2", "0", "",
                        "text-align: right");
    $T .= StrCell($U);
    $T .= StrCell(HtmlSubmitButton("Move", " Move "), "", "",
                  "text-align: right");
    $T .= StrRowEnd();
    $T .= StrTableEnd();
    $S .= StrCell($T, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionRevertCaption()
{
    $S = StrRowBegin();
    $S .= StrCell("Revert caption:");
    $T = StrTableBegin(0, "", "", "width: 100%");
    $T .= StrRowBegin();
    $T .= StrCell("(to original description)");
    $T .= StrCell(HtmlSubmitButton("RevertCaption", " Revert "), "",
                  "", "text-align: right");
    $T .= StrRowEnd();
    $T .= StrTableEnd();
    $S .= StrCell($T, "", "", BoxStyle."; text-align: right");
    $S .= StrRowEnd();
    return $S;
}

function PageConfirmDeleteAlbum($AlbumSn, $Sort)
{
    global $SystemTitle;
    $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
    if ($AlbumName === False)
    {
        $S = StartOfPage("Album Worksheet");
        $S .= HdgSelectCreateDelete(False, $Sort, "",
                                    "No such album exists!");
    } else
    {
        $B = "font-family: helvetica, sans-serif; color: Navy;";
        $B .= " background-color: #ffb062;";
        $B .= " background-image: url(gk/images/flames.jpg)";
        $L = "A:link { color: Blue }\nA:visited { color: Blue }\n";
        $S = HeadAndTitle("", "", $B, $L);
        $S .= "<div align=\"center\">\n";
        $S .= PageTitleWithButton(
            "Delete Album &quot;$AlbumName&quot;?", BtnMainMenu());
        $W = "Deleting an album is complete and irreversible,";
        $W .= " erasing all trace that it ever existed. Be sure!";
        $S .= StrWarningBox($W, "50%");
        $S .= StrFormBeginRecurse();
        $S .= HtmlHidden("AlbumSn", $AlbumSn);
        $S .= HtmlHidden("Sort", $Sort);
        $S .= "<table border=\"0\" cellspacing=\"10\"";
        $S .= " cellpadding=\"4\">\n";
        $S .= StrRowBegin();
        $S .= StrCell(HtmlSubmitButton("Cancel"));
        $S .= StrDeleteButtonCell(
            "ConfirmDeleteAlbum",
            "Delete Album &quot;$AlbumName&quot;");
        $S .= StrRowEnd();
        $S .= StrTableEnd();
        $S .= StrFormEnd();
    }
    return $S;
}

function PagesIndicator($ShowPages)
{
    $S = "";
    foreach ($ShowPages as $Page)
        $S .= HtmlHidden("Pg$Page", "on");
    return $S;
}

function PagesToDisplay($AlbumSn)
{
    // return array($AllPages, $ShowPages) where
    // $AllPages is array(PageNumber => PageHeading) and
    // $ShowPages is array(PageNumber)
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    $ShowAll = isset($FormVars["PgAll"]);
    $ShowNone = isset($FormVars["PgNone"]);
    $ShowPop = isset($FormVars["PgPop"]);
    $AllPop = PopulatedPages($AlbumSn);
    $Q = "SELECT page, heading FROM albumpage";
    $Q .= " WHERE albumsn = $1 ORDER BY page";
    $Res = Query($Q, $AlbumSn);
    $NumRows = pg_num_rows($Res);
    $AllPages = array();
    $ShowPages = array();
    for ($Row = 0 ; $Row < $NumRows ; $Row++)
    {
        list($Page, $Heading) = pg_fetch_array($Res, $Row);
        $AllPages[$Page] = $Heading; 
        if (!$ShowNone and
            ($ShowAll or (!$ShowPop and isset($FormVars["Pg$Page"]))
             or ($ShowPop and in_array($Page, $AllPop))))
            $ShowPages[] = $Page;
    }
    if (!$ShowNone)
    {
        /*
        // if no pages selected at all, then show all populated pages
        if (count($ShowPages) == 0) $ShowPages = $AllPop;
        */
        // if still no pages selected at all, then show just page 0
        if (count($ShowPages) == 0) $ShowPages = array(0);
    }
    return array($AllPages, $ShowPages);
}

function PopulatedPages($AlbumSn)
{
    // return array(PageNumber) with all album pages that actually
    // contain at least one photo
    $Q = "SELECT DISTINCT page FROM albumphoto ap, albumpage pg";
    $Q .= " WHERE pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND albumsn = $1 ORDER BY page";
    $Res = Query($Q, $AlbumSn);
    $NumRows = pg_num_rows($Res);
    $AllPop = array();
    for ($Row = 0 ; $Row < $NumRows ; $Row++)
        $AllPop[] = pg_fetch_result($Res, $Row, "page");
    return $AllPop;
}

function RenameAlbum($AlbumSn, $AlbumName)
{
    $OldName = ColFromSerialNumber("album", "name", $AlbumSn);
    if ($OldName != $AlbumName)
    {
        $Q = "UPDATE album SET name = $1 WHERE albumsn = $2";
        Query($Q, $AlbumName, $AlbumSn);
        $N = "Renamed album &quot;$OldName&quot; to";
        $N .= " &quot;$AlbumName&quot;";
    } else $N = "";
    return $N;
}

function RenamedAlbum($AlbumSn, $Sort, $AlbumName)
{
    $AlbumName = trim($AlbumName);
    if (! GoodFileName($AlbumName))
    {
        $W = "Couldn't rename album:<br />\n";
        $W .= WarnBadAlbumName($AlbumName);
        $S = ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                         False, False, $W);
    } else
    {
        if (IsInDatabase("album", "name", $AlbumName, True, "albumsn",
                         $AlbumSn))
        {
            $W = "Couldn't rename album to &quot;$AlbumName&quot;,";
            $W .= " as another album has that name.";
            $S = ScreenAlbum($AlbumSn, $Sort,
                             PagesToDisplay($AlbumSn), False, False,
                             $W);
        } else
        {
            $S = ScreenAlbum($AlbumSn, $Sort,
                             PagesToDisplay($AlbumSn), False, False,
                             "", RenameAlbum($AlbumSn, $AlbumName));
        }
    }
    return $S;
}

function ReorderPhotoOnPage($PicNum, $FormVars)
{
    $AlbumPhotoSn = $FormVars["Sn$PicNum"];
    $NewOrdinal =  $FormVars["Ord$PicNum"];
    $Page = pg_fetch_result(ResAlbumPhotoBySn($AlbumPhotoSn), 0,
                            "page");
    MoveAlbumPhoto($AlbumPhotoSn, $Page, $NewOrdinal);
    return;
}

function ResAlbumPagePhotos($AlbumSn, $Page)
{
    // return photos on $Page; $ShowPages is array(PageNumber)
    $Q = "SELECT albumphotosn, pg.albumpagesn, ordinal, caption,";
    $Q .= " foldername, filename, thumbw, thumbh, ";
    $Q .= ClauseDateForDisplay;
    $Q .= ", v.vidfilename";
    $Q .= " FROM albumphoto ap, albumpage pg, photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn),";
    $Q .= " photoset s";
    $Q .= " WHERE albumsn = $1 AND page = $2";
    $Q .= " AND ap.albumpagesn = pg.albumpagesn";
    $Q .= " AND p.photosn = ap.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY ordinal";
    return Query($Q, $AlbumSn, $Page);
}

function ResAlbumPhoto($AlbumSn, $Page, $Ordinal)
{
    // Get details for photo on page $Page at position $Ordinal
    $Q = "SELECT albumphotosn, foldername, filename, caption,";
    $Q .= " p.width, p.height, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay;
    $Q .= ", p.locationsn,";
    $Q .= " COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " v.vidfilename, v.width as vwidth, v.height as vheight,";
    $Q .= " p.photosn";
    $Q .= " FROM albumphoto ap, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn),";
    $Q .= " photoset s";
    $Q .= " WHERE albumpagesn = (SELECT albumpagesn FROM albumpage";
    $Q .= " WHERE albumsn = $1 and page = $2)";
    $Q .= " AND ordinal = $3 AND p.photosn = ap.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    return Query($Q, $AlbumSn, $Page, $Ordinal);
}

function ResAlbumPhotoBySn($AlbumPhotoSn)
{
    // Get details for photo with serial number $AlbumPhotoSn
    $Q = "SELECT ordinal, caption, ap.albumpagesn, page, pg.albumsn,";
    $Q .= " name";
    $Q .= " FROM albumphoto ap, albumpage pg, album a";
    $Q .= " WHERE albumphotosn = $1";
    $Q .= " AND pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND a.albumsn = pg.albumsn";
    return Query($Q, $AlbumPhotoSn);
}

function RevertCaption($AlbumPhotoSn)
{
    $Q = "UPDATE albumphoto SET caption = p.description";
    $Q .= " FROM photo p";
    $Q .= " WHERE albumphoto.albumphotosn = $1";
    $Q .= " AND p.photosn = albumphoto.photosn";
    Query($Q, $AlbumPhotoSn);
    return;
}

function RevertedCaption($AlbumSn, $Sort, $NumPics)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    $RevertCount=0;
    for ($Row=0 ; $Row < $NumPics ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            RevertCaption($FormVars["Sn$Row"]);
            $RevertCount++;
        }
    }
    if ($RevertCount > 0)
    {
        $W = "";
        $N = "Reset caption";
        if ($RevertCount != 1) $N .= "s";
        $N .= " to original photo description";
        if ($RevertCount != 1) $N .= "s";
        $N .= " for $RevertCount photo";
        if ($RevertCount != 1) $N .= "s";
        $N .= ".";
    } else
    {
        $W = "Nothing checked, so no caption reverted.";
        $N = "";
    }
    $S = ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                     False, False, $W, $N);
    return $S;
}

function RevertedWebPicCaption($AlbumPhotoSn)
{
    RevertCaption($AlbumPhotoSn);
    list($Ordinal, $Z, $AlbumPageSn)
        = pg_fetch_array(ResAlbumPhotoBySn($AlbumPhotoSn), 0);
    return LargeImage($AlbumPageSn, $Ordinal);
}

function RowAppendDate()
{
    $S = StrRowBegin();
    $T = "<label>".HtmlCheckbox("AppendDate")."Date</label>";
    $S .= StrCell($T);
    $T = "<label>".HtmlRadioButton("FmtDate", "jMY", True);
    $T .= date("j M Y")."</label>\n";
    $T .= "<label>".HtmlRadioButton("FmtDate", "MjY");
    $T .= date("M j, Y")."</label>\n";
    $T .= "<label>".HtmlRadioButton("FmtDate", "mdY");
    $T .= date("m/d/Y")."</label>\n";
    $T .= "<label>".HtmlRadioButton("FmtDate", "ISO");
    $T .= date("Y-m-d")."</label>";
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    return $S;
}

function RowAppendDay()
{
    $S = StrRowBegin();
    $S .= StrCell("<label>".HtmlCheckbox("AppendDay")."Day</label>");
    $T = "<label>".HtmlRadioButton("FmtDay", "D", True);
    $T .= date("D")."</label>\n";
    $T .= "<label>".HtmlRadioButton("FmtDay", "l");
    $T .= date("l")."</label>";
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    return $S;
}

function RowAppendLocation()
{
    $S = StrRowBegin();
    $T = "<label>".HtmlCheckbox("AppendLoc")."Location</label>";
    $S .= StrCell($T);
    $T = "<label title=\"For geotagged photos\">";
    $T .= HtmlCheckbox("AppendGPS");
    $T .= "Include lat/lon/alt (if present)</label>";
    $S .= StrCell($T);
    $T = HtmlSubmitButton("Append", " Append ");
    $S .= StrCell($T, "", "", "text-align: right");
    $S .= StrRowEnd();
    return $S;
}

function RowAppendTime()
{
    $S = StrRowBegin();
    $T = "<label>".HtmlCheckbox("AppendTime")."Time</label>";
    $S .= StrCell($T);
    $T = "<label>".HtmlRadioButton("FmtTime", "hia", True);
    $T .= date("h:ia")."</label>\n";
    $T .= "<label>".HtmlRadioButton("FmtTime", "Hi");
    $T .= date("H:i")."</label>";
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    return $S;
}

function RowAppendTimeZone()
{
    $S = StrRowBegin();
    $S .= StrCell("&nbsp;");
    $T = "Time Zone: <label title=\"".HintTimeZoneLocal."\">";
    $T .= HtmlRadioButton("TimeZone", "Local", True);
    $T .= "local</label> <label title=\"".HintTimeZoneUTC."\">";
    $T .= HtmlRadioButton("TimeZone", "UTC");
    $T .= "UTC</label>";
    /*****************************************************************
    Defer until we find a way to validate time zone inputs:
    $T .= " <label title=\"".HintTimeZoneOther."\">";
    $T .= HtmlRadioButton("TimeZone", "other");
    $T .= "other: </label>";
    $T .= HtmlTextInput("OtherTZ", "3", "", "", "text-align: right");
    *****************************************************************/
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    return $S;
    $U = "end of page ";
    $U .= HtmlTextInput("TargetPage", "3", "", "",
                        "text-align: right");
    $T .= StrCell($U);
}

function RowCaptionAndButtons($Caption, $Page, $Ordinal, $FolderName,
                              $VidFileName)
{
    $S = StrRowBegin();
    $T = HtmlTextArea("Caption", "50", "2", $Caption)."<br />\n";
    $T .= HtmlSubmitButton("ResetCaption", "Reset Caption");
    $T .= " (to original description)\n";
    $S .= StrCell($T);
    $T = StrButton(BtnClose());
    if ($VidFileName > "")
        $T = BtnGetVideo($FolderName, $VidFileName).$T;
    $S .= "<td valign=\"middle\">$T</td>\n";
    $S .= "<td style=\"text-align: right; white-space: nowrap\">";
    $S .= "Page&nbsp;";
    $S .= HtmlTextInput("Page", "2", $Page, "", "text-align: right");
    $S .= "Seq&nbsp;";
    $S .= HtmlTextInput("Ordinal", "2", $Ordinal, "",
                        "text-align: right");
    $S .= HtmlPara(HtmlSubmitButton("Commit", "Save Changes", "",
                                    StyleBtn));
    $T = "<button type=\"button\" style=\"".StyleBtn."\"";
    $T .= " onclick=\"EditInSet()\">Edit in Photo Set</button>";
    $S .= HtmlPara($T);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowPageHeading($AlbumSn, $Page, $Heading, $TitlePicOrd,
                        $TitlePicCaption)
{
    $StyleThumb = "width: ".ThumbnailLength."px; text-align: right";
    $S = StrRowBegin();
    $S .= "<td colspan=\"3\">";
    $S .= StrTableBegin("1");
    if ($TitlePicOrd > 0)
    {
        $S .= StrRowBegin();
        $T = "Title Picture Caption:<br>\n";
        $T .= HtmlTextArea("TitlePicCaption", "55", "2",
                           $TitlePicCaption);
        $S .= StrCell($T);
        $T = AlbumPhotoThumbnail($AlbumSn, $Page, $TitlePicOrd);
        $S .= StrCell($T, "", "3", $StyleThumb);
        $S .= StrRowEnd();
    }
    $S .= StrRowBegin();
    $T = "Page Heading (markup ok):<br>\n";
    $T .= HtmlTextArea("Heading", "55", "2", $Heading);
    $S .= StrCell($T);
    if ($TitlePicOrd == 0)
        $S .= StrCell("&nbsp;", "", "2", $StyleThumb);
    $S .= StrRowEnd();
    return $S;
}

function RowPagePics($Page, $NumPics, $TitlePicOrd)
{
    // Note: Forbids deleting or renumbering Page 0 
    $S = StrRowBegin();
    $C = Strong("Page")." ";
    $C .= ($Page > 0) ? HtmlTextInput("NewPage", "2", $Page) : "0";
    $S .= StrCell($C);
    $C = "$NumPics Pictures on page. ";
    if ($NumPics > 0)
    {
        $C .= "Title picture is number ";
        $C .= MenuTitlePicOrd($NumPics, $TitlePicOrd);
    } elseif ($Page > 0)
          $C .= HtmlSubmitButton("DeletePage", "Delete Empty Page");
    $S .= StrCell($C);
    $S .= "<td style=\"text-align: right\">";
    $S .= HtmlSubmitButton("UpdatePageMeta", "Save Changes");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowPageText($PageText)
{
    $S = StrRowBegin();
    $T = "Page Text:<br>\n";
    $T .= FittedTextArea("PageText", "55", "10", $PageText);
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowPhotoStats($FolderName, $FileName, $Time, $LocName,
                       $VidFileName, $PhotoSn)
{
    $S = StrRowBegin();
    $T = "Image $FileName in photo set $FolderName, taken $Time";
    $T .= " at $LocName";
    $G = GeoCoordinates($PhotoSn);
    if ($G !== False) $T .= " ($G)";
    if ($VidFileName > "") $T .= ". Linked to video $VidFileName.";
    $S .= StrCell($T, "2");
    $S .= "<td align=\"right\" valign=\"bottom\">";
    $Style = StyleBtn."; border: medium solid ".WarningColor;
    $S .= HtmlSubmitButton("DropOne", "Drop from Album", "", $Style);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowWebPic($Path, $Caption, $Height, $Width)
{
    $S = StrRowBegin();
    $S .= "<td colspan=\"3\" align=\"center\">\n";
    $S .= HtmlImage(LatestFile($Path), $Caption, $Height, $Width,
                    "1", "", "#PicMap");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function ScreenAlbum($AlbumSn, $Sort, $Pages, $RangeMarks,
                     $ShowArchived, $Warning="", $Notice="")
{
    // $Pages is array($AllPages, $ShowPages) from PagesToDisplay
    $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
    list($NumPics, $MinDate, $MaxDate)
        = CountAlbumPhotos($AlbumSn, $Pages[1]);
    $G = ThumbGallery($AlbumSn, $Pages, $NumPics, $RangeMarks, $N);
    $T = StyleClassDefinitions($AlbumSn).JavaScriptDefinitions;
    $S = StrToolPageTitle($T, "Album Worksheet");
    if ($Notice != "") $S .= StrWarning($Notice, NoticeColor);
    if ($N != "") $S .= StrWarning($N, NoticeColor);
    $S .= HdgSelectCreateDelete($ShowArchived, $Sort, $AlbumSn,
                                $Warning, $AlbumName);
    $S .= LinePhotoSet($AlbumSn);
    $S .= LineDisplay($AlbumSn, $Pages, $NumPics, $MinDate, $MaxDate);
    $S .= StrFormBeginRecurse("ThumbGallery");
    $S .= AlbumPhotoActionOptions();
    $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= HtmlHidden("Sort", $Sort);
    $S .= HtmlHidden("NumPics", $NumPics);
    $S .= PagesIndicator($Pages[1]);
    $S .= HtmlHidden("ChangeSeq", "");
    $S .= $G;
    $S .= StrFormEnd();
    return $S;
}

function ScreenEditTextFormat($AlbumSn, $Warning="", $Notice="")
{
    $AlbumName = pg_fetch_result(ResAlbum($AlbumSn), 0, "name");
    $S = StartOfPage("Album Worksheet",
                     JsColorScript.ColorSwitchers.CalendarDefinitions,
                     "", False, True);
    if ($Notice != "") $S .= StrWarning($Notice, NoticeColor);
    if ($Warning != "") $S .= StrWarningBox($Warning, "50%");
    $T = "Edit Page Text and Format for Album &quot;$AlbumName&quot;";
    $S .= StrSubTitle($T);
    $S .= MetaEntireAlbum($AlbumSn);
    if (! IsContiguousAlbum($AlbumSn))
    {
        $S .= StrTableBegin("0");
        $S .= StrRowBegin();
        $T = "Press ".Strong("Compact");
        $T .= " to renumber pages, removing &quot;holes&quot;";
        $S .= StrCell($T);
        $T = StrFormBeginRecurse();
        $T .= HtmlHidden("AlbumSn", $AlbumSn);
        $T .= HtmlSubmitButton("Compact");
        $S .= StrCell($T);
        $S .= StrRowEnd();
        $S .= StrTableEnd();
    }
    $S .= MetaAlbumPages($AlbumSn);
    return $S;
}

function SelectRange($AlbumSn, $Sort, $Pages, $NumThumbs, $FormVars)
{
    /*****************************************************************
    If $NumThumbs > 0, builds an ordered array of selected thumbnails
    which specify starting and stopping points of ranges, then calls
    ScreenAlbum to redisplay album page with checkboxes set for all
    images in range(s).
    *****************************************************************/
    $RangeMarks = array();
    for ($Row=0 ; $Row < $NumThumbs ; $Row++ )
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
            $RangeMarks[] = $FormVars["Sn$Row"];
    if (count($RangeMarks) > 0)
    {
        $Plural = (count($RangeMarks) > 2) ? "ranges" : "range";
        // Defer to ThumbGallery to report stats:
        $S = ScreenAlbum($AlbumSn, $Sort, $Pages, $RangeMarks, False);
    } else
        $S = ScreenAlbum($AlbumSn, $Sort, $Pages, False, False,
                         "No selection range specified.");
    return $S;
}

function SetTitlePicOrdinal($AlbumPageSn, $TitlePicOrd)
{
    $Q = "UPDATE albumpage SET titlepicord = $1";
    $Q .= " WHERE albumpagesn = $2";
    Query($Q, $TitlePicOrd, $AlbumPageSn);
    return;
}

function SlideAlbumPages($AlbumSn, $FirstPage, $LastPage, $Incr)
{
    // Slide the given range of pages, and their corresponding photos,
    // by the specified increment (positive or negative)
    // Move in two steps via an unused region to avoid key collisions
    $NetOffset = TempOffset + $Incr;
    $Q = "UPDATE albumpage";
    $Q .= " SET page = $NetOffset + page";
    $Q .= " WHERE albumsn = $3 AND page BETWEEN $1 AND $2";
    Query($Q, $FirstPage, $LastPage, $AlbumSn);
    $TempFirst = $FirstPage + $NetOffset;
    $TempLast = $LastPage + $NetOffset;
    $Q = "UPDATE albumpage";
    $Q .= " SET page = page - ".TempOffset;
    $Q .= " WHERE albumsn = $1";
    $Q .= " AND page BETWEEN $TempFirst AND $TempLast";
    Query($Q, $AlbumSn);
    return;
}

function StyleClassDefinitions($AlbumSn)
{
    list($Z0, $Z1, $BgColor, $TextColor, $VidBgColor)
        = pg_fetch_array(ResAlbum($AlbumSn), 0);
    $S = "<style type=\"text/css\">\n";
    $S .= ".page {padding: 3mm; background-color: $BgColor;";
    $S .= " color: $TextColor}\n";
    $S .= ".lbl {white-space: nowrap}\n";
    $S .= ".vid {background-color: $VidBgColor}";
    $S .= "</style>\n";
    return $S;
}

function ThumbGallery($AlbumSn, $Pages, $NumPics, $RangeMarks,
                      &$N="")
{
    /*****************************************************************
    Display photo thumbnails page-by-page.
    If $RangeMarks is False, clear all thumbnail selection checkboxes.
    If it's True (and Boolean, rather than an array), set them all.
    Otherwise, $RangeMarks is an array of elements ($AlbumPhotoSn),
    each of which starts or ends a range. If a range has been started
    but no marker ends it, it will extend to the last image on the
    page.
    Since a range may span multiple pages, must track current state
    ($RangeMarks, $Selected, $NextMark, $SwitchAfter) between pages.
    *****************************************************************/
    list($AllPages, $ShowPages) = $Pages;
    $S = "";
    $PicCount = 0;
    $IsByRange = is_array($RangeMarks);
    if ($IsByRange)
    {
        $NumRanges = ceil(count($RangeMarks)/2);
        $Selected = False;
        $NextMark = array_shift($RangeMarks);
        $SwitchAfter = False;
        $NumSelected = 0;
    } else $Selected = $RangeMarks;
    foreach ($ShowPages as $Page)
        $S .= ThumbGalleryPage($AlbumSn, $Page, $AllPages[$Page],
                               $PicCount, $RangeMarks, $IsByRange,
                               $Selected, $NextMark, $SwitchAfter,
                               $NumSelected);
    if ($IsByRange)
    {
        // Use the $N argument to return statistics
        $Plural = ($NumRanges > 1) ? "s" : "";
        $N = "Selected $NumSelected photos in $NumRanges specified";
        $N .= " range$Plural.";
    }
    return $S;
}

function ThumbGalleryCell($Res, $Row, $NumPicsOnPage, $PicCount,
                          $Selected)
{
    $S = "";
    if ($Row % 4 == 0)
    {
        if ($Row > 0) $S .= StrRowEnd();
        $S .= StrRowBegin();
    }
    list($AlbumPhotoSn, $AlbumPageSn, $Ordinal, $Caption, $FolderName,
         $FileName, $ThumbW, $ThumbH, $Time, $VidFileName)
        = pg_fetch_array($Res, $Row);
    $Label = HtmlCleanTitle($Caption);
    if ($VidFileName > "") $Label = "(Video) $Label";
    if ($Ordinal != $Row+1) 
    {
        $Ordinal = $Row+1;
        $Q = "UPDATE albumphoto SET ordinal = $1";
        $Q .= " WHERE albumphotosn = $1";
        Query($Q, $Ordinal, $AlbumPhotoSn);
    }
    $T = "?AlbumPageSn=$AlbumPageSn&Ordinal=$Ordinal&ViewLarge=1";
    $Url = ThisScript().$T;
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    $T = HtmlSelectBegin("Ord$PicCount",
                         "onchange=\"ReSeq($PicCount)\"");
    for ($I = 1 ; $I <= $NumPicsOnPage ; $I ++)
        $T .= HtmlSelectOption($I, $I, ($I == $Ordinal));
    $T .= HtmlSelectEnd();
    $T .= " <label>$Time<br />\n$FileName";
    $T .= HtmlCheckbox("Sel$PicCount", $Selected);
    $T .= "</label>".HtmlHidden("Sn$PicCount", $AlbumPhotoSn);
    $T .= "<br />\n<a target=\"_blank\" href=\"$Url\">";
    $Path = "$ThumbFolder/$FileName";
    $T .= HtmlImage(LatestFile($Path), $Label, $ThumbH, $ThumbW);
    $T .= "</a>\n";
    $V = ($VidFileName > "") ? " class=\"vid\"" : "";
    $S .= "<td$V>$T</td>\n";
    return $S;
}

function ThumbGalleryPage($AlbumSn, $Page, $PageHeading, &$PicCount,
                          &$RangeMarks, $IsByRange, &$Selected,
                          &$NextMark, &$SwitchAfter, &$NumSelected)
{
    /*****************************************************************
    Display title and table with thumbnails of all photos on page
    $Page of $AlbumSn, in rows of four.

    If $RangeMarks is False, clear all thumbnail selection checkboxes.
    If it's True (and Boolean, rather than an array), set them all.
    Otherwise, $RangeMarks is an array of elements ($AlbumPhotoSn),
    each of which starts or ends a range. If a range has been started
    but no marker ends it, it will extend to the last image on the
    page.
    Since a range may span multiple pages, must track current state
    ($RangeMarks, $Selected, $NextMark, $SwitchAfter) between pages.
    *****************************************************************/
    $Res = ResAlbumPagePhotos($AlbumSn, $Page);
    $NumPicsOnPage = pg_num_rows($Res);
    $T = ($Page == 0)
       ? "Staging Area (&quot;Page 0&quot;):" : "Page $Page:";
    if ($PageHeading != "") $T .= " &quot;$PageHeading&quot;";
    $T .= " $NumPicsOnPage photo".(($NumPicsOnPage != 1) ? "s" : "");
    $S = StrSubTitle($T);
    if ($NumPicsOnPage > 0)
    {
        $S .= "<table class=\"page\"><tbody>\n";
        for ($Row = 0 ; $Row < $NumPicsOnPage ; $Row ++)
        {
            if ($IsByRange)
            {
                $Sn = pg_fetch_result($Res, $Row, "albumphotosn");
                if ($Sn == $NextMark)
                {
                    if ($Selected) $SwitchAfter = True;
                    else $Selected = True;
                    $NextMark = array_shift($RangeMarks);
                }
            }
            $S .=  ThumbGalleryCell($Res, $Row, $NumPicsOnPage,
                                    $PicCount, $Selected);
            $PicCount ++;
            if ($Selected) $NumSelected ++;
            if ($SwitchAfter) $Selected = False;
            $SwitchAfter = False;
        }
        for ($I = 0 ; $I < (3 - (($NumPicsOnPage -1) % 4)) ; $I ++)
            $S .= "<td width=\"".ThumbnailLength."\">&nbsp;</td>\n";
        $S .= StrRowEnd();
        $S .= StrTableEnd();
    }
    return $S;
}

function ToggleArchived($AlbumSn, $Sort, $ArchivedAlbum)
{
    $AlbumName = pg_fetch_result(ResAlbum($AlbumSn), 0, "name");
    $T = $ArchivedAlbum ? "into" : "out of";
    if ($ArchivedAlbum)
    {
        $Q = "INSERT INTO archivedalbum (albumsn)";
        $Q .= " SELECT $1 WHERE NOT EXISTS";
        $Q .= " (SELECT * FROM archivedalbum WHERE albumsn = $1)";
    } else
    {
        $Q = "DELETE FROM archivedalbum WHERE albumsn = $1";
    }
    Query($Q, $AlbumSn);
    $T = "Moved album &quot;$AlbumName&quot; $T the archive.";
    $S .= ScreenAlbum($AlbumSn, $Sort, PagesToDisplay($AlbumSn),
                      False, True, "", $T);
    return $S;
}

function UpdatedLargeImage($AlbumPhotoSn, $Caption, $Page, $Ordinal)
{
    $Caption = trim($Caption);
    MoveAlbumPhoto($AlbumPhotoSn, $Page, $Ordinal);
    $Q = "UPDATE albumphoto SET caption = $1";
    $Q .= " WHERE albumphotosn = $2 AND COALESCE(caption, '') != $1";
    Query($Q, $Caption, $AlbumPhotoSn);
    $AlbumPageSn = pg_fetch_result(ResAlbumPhotoBySn($AlbumPhotoSn),
                                   0, "albumpagesn");
    return LargeImage($AlbumPageSn, $Ordinal);
}

function UpdatedMeta($AlbumSn, $Author, $PubDate, $BgColor, $Url,
                     $Notes, $TextColor, $VidBgColor, $Title)
{
    ForcePrefix($BgColor, $TextColor, $VidBgColor);
    list ($Z, $Title0, $BgColor0, $TextColor0, $VidBgColor0, $Author0,
          $Url0, $Notes0, $PubDate0)
        = pg_fetch_array(ResAlbum($AlbumSn), 0);
    $Title = trim($Title);
    $W = "";
    $Q = "";
    $Count = 1;
    $Params = array($AlbumSn);
    if ($Title != "")
        UpdateIfChanged($Q, $Params, $Count, $Title, $Title0,
                        "title");
    else
    	$W .= " Title may not be blank.";
    if (strtotime($PubDate) !== InvalidDateTime)
        UpdateIfChanged($Q, $Params, $Count,
                        StrDate($PubDate,  DateFormatPgDefault),
                        $PubDate0, "pubdate");
    else
        $W .= " Invalid publication date &quot;$PubDate&quot;.";
    UpdateIfChanged($Q, $Params, $Count, $BgColor, $BgColor0,
                    "bgcolor", True);
    UpdateIfChanged($Q, $Params, $Count, $TextColor, $TextColor0,
                    "textcolor", True);
    UpdateIfChanged($Q, $Params, $Count, $VidBgColor, $VidBgColor0,
                    "vidbgcolor", True);
    UpdateIfChanged($Q, $Params, $Count, $Author, $Author0,
                    "author", True);
    UpdateIfChanged($Q, $Params, $Count, $Url, $Url0, "url", True);
    UpdateIfChanged($Q, $Params, $Count, $Notes, $Notes0,
                    "notes", True);
    if ($Q != "")
    {
        $Q = "UPDATE album SET".substr($Q, 1);
        $Q .= " WHERE albumsn = $1";
        QueryArray($Q, $Params);
    }
    if ($W != "") $W = "Unchanged:$W";
    return ScreenEditTextFormat($AlbumSn, $W);
}

function UpdatedPageMeta($AlbumSn, $Heading, $NewPage, $Page,
                         $PageText, $TitlePicOrd, $TitlePicCaption)
{
    // TitlePicOrd could be unset on an empty page
    if ( ! isset($TitlePicOrd)) $TitlePicOrd = 0;
    $TitlePicOrd = BoundedValue($TitlePicOrd,
                                PicsOnPage($AlbumSn, $Page), 0);
    $Res = ResAlbumPage(False, $AlbumSn, $Page);
    if (pg_num_rows($Res) > 0)
        list ($Z1, $Z2, $Z3, $Heading0, $TitlePicOrd0,
              $TitlePicCaption0, $PageText0)
            = pg_fetch_array($Res, 0);
    $Q = "";
    $Count = 2;
    $Params = array($AlbumSn, $Page);
    if ($TitlePicOrd !== $TitlePicOrd0)
        $Q .= ", titlepicord = $TitlePicOrd";
    if ($TitlePicOrd > 0)
    {
        if ($TitlePicOrd !== $TitlePicOrd0)
            $TitlePicCaption = AlbumPhotoCaption($AlbumSn, $Page,
                                                 $TitlePicOrd);
        UpdateIfChanged($Q, $Params, $Count, $TitlePicCaption,
                        $TitlePicCaption0, "titlepiccaption", True);
    }
    UpdateIfChanged($Q, $Params, $Count, $Heading, $Heading0,
                    "heading", True);
    UpdateIfChanged($Q, $Params, $Count, $PageText, $PageText0,
                    "pagetext", True);
    if ( $Q != "" )
    {
        $Q = "UPDATE albumpage SET".substr($Q, 1);
        $Q .= " WHERE albumsn = $1 AND page = $2";
        QueryArray($Q, $Params);
    }
    $NewPage = abs($NewPage);
    if ($NewPage != $Page) MoveAlbumPage($AlbumSn, $Page, $NewPage);
    return ScreenEditTextFormat($AlbumSn);
}

function UpdateIfChanged(&$Q, &$Params, &$Count, $New, $Old, $Name,
                         $BlanksOk=False)
{
    $New = trim($New);
    if (($New != "" or $BlanksOk) and $New != $Old)
    {
        $Count++;
        $Params[] = $New;
        $Q .= ", $Name = $$Count";
    }
    return;
}

function WebPicUrl($AlbumPageSn, $Ordinal)
{
    $T = ThisScript()."?AlbumPageSn=$AlbumPageSn&Ordinal=$Ordinal";
    return $T;
}

//***************************************************************
// Fetch input variables
$FormVarList = array(
    "AlbumName",
    "AlbumSn",
    "AlbumPhotoSn",
    "ArchivedAlbum",
    "Author",
    "BgColor",
    "Caption",
    "ChangeSeq",
    "EditTextFormat",
    "Heading",
    "NewPage",
    "Notes",
    "NumPics",
    "Ordinal",
    "Page",
    "PageText",
    "PubDate",
    "ShowArchived",
    "Sort",
    "TargetPage",
    "TextColor",
    "Title",
    "TitlePicCaption",
    "TitlePicOrd",
    "Url",
    "VidBgColor"
);
$GetVarList = array(
    "AlbumPageSn",
    "AlbumSn",
    "EditTextFormat",
    "FetchVideo",
    "Ordinal",
    "Page"
);
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (! isset($ShowArchived)) $ShowArchived = False;
if (! isset($Sort)) $Sort = "Time";
if (isset($FetchVideo)) DownloadFile($FetchVideo);
elseif (isset($_GET["ViewLarge"]))
    $S = LargeImage($AlbumPageSn, $Ordinal);
elseif (isset($_GET["Forward"]))
    $S = LargeImage($AlbumPageSn, $Ordinal + 1);
elseif (isset($_GET["Backward"]))
    $S = LargeImage($AlbumPageSn, $Ordinal - 1);
elseif (isset($FormVars["Commit"]))
    $S = UpdatedLargeImage($AlbumPhotoSn, $Caption, $Page, $Ordinal);
elseif (isset($FormVars["DropOne"]))
    $S = DroppedWebPicFromAlbum($AlbumPhotoSn);
elseif (isset($FormVars["ResetCaption"]))
    $S = RevertedWebPicCaption($AlbumPhotoSn);
elseif (isset($FormVars["Move"]))
    $S = MovedToPage($AlbumSn, $Sort, $NumPics, $TargetPage);
elseif (isset($FormVars["ApplyCap"]))
    $S = AppliedCaption($AlbumSn, $Sort, $NumPics, $Caption);
elseif (isset($FormVars["Drop"]))
    $S = DroppedFromAlbum($AlbumSn, $Sort, $NumPics);
elseif (isset($FormVars["RevertCaption"]))
    $S = RevertedCaption($AlbumSn, $Sort, $NumPics);
elseif (isset($FormVars["Append"]))
    $S = AppendedToCaption($AlbumSn, $Sort, $NumPics);
elseif (isset($EditTextFormat))
    $S = ScreenEditTextFormat($AlbumSn);
elseif (isset($FormVars["UpdateMeta"]))
    $S = UpdatedMeta($AlbumSn, $Author, $PubDate, $BgColor, $Url,
                     $Notes, $TextColor, $VidBgColor, $Title);
elseif (isset($FormVars["UpdatePageMeta"]))
    $S = UpdatedPageMeta($AlbumSn, $Heading, $NewPage, $Page,
                         $PageText, $TitlePicOrd, $TitlePicCaption);
elseif (isset($FormVars["DeletePage"]))
    $S = DeletedPage($AlbumSn, $Page);
elseif (isset($FormVars["Compact"]))
    $S = CompactedPages($AlbumSn);
elseif (isset($FormVars["Create"]))
    $S = CreatedAlbum($AlbumName, $Sort);
elseif (isset($FormVars["Rename"]))
    $S = RenamedAlbum($AlbumSn, $Sort, $AlbumName);
elseif (!isset($AlbumSn))
{
    $S = StartOfPage("Album Worksheet");
    $S .= HdgSelectCreateDelete($ShowArchived, $Sort);
} elseif ($AlbumSn == "none" or $AlbumSn == "" or
          $AlbumSn == PromptNewAlbum)
{
    $S = StartOfPage("Worksheet for a Photo Album");
    /* $W = $ShowArchived */
    $W = isset($FormVars["ShowArchivedSwitch"])
       ? "" : "You must select an album or create a new one";
    $S .= HdgSelectCreateDelete($ShowArchived, $Sort, "", $W);
} elseif (isset($FormVars["DeleteAlbum"]))
        $S = PageConfirmDeleteAlbum($AlbumSn, $Sort);
elseif (isset($FormVars["ConfirmDeleteAlbum"]))
    $S = DeletedAlbum($AlbumSn, $Sort);
elseif (isset($FormVars["ToggleArchived"]))
    $S = ToggleArchived($AlbumSn, $Sort, $ArchivedAlbum);
else
{
    $Pages = PagesToDisplay($AlbumSn);
    if (isset($ChangeSeq) && is_numeric($ChangeSeq))
        ReorderPhotoOnPage($ChangeSeq, $FormVars);
    if (isset($FormVars["SelectRange"]))
        $S = SelectRange($AlbumSn, $Sort, $Pages, $NumPics,
                         $FormVars);
    else $S = ScreenAlbum($AlbumSn, $Sort, $Pages,
                          isset($FormVars["SelectAll"]),
                          $ShowArchived);
}
$S .= "</div></body></html>\n";
echo $S;

?>
