<?php
/*********************************************************************
editlocation.php
Location Management Module for PhotoTrove
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require ("lib_inputval.php");
require_once ("lib_layout.php");
require ("lib_photoset.php");
require_once ("local/loc_camera.php");

function BoxLocationSelect($LocationSn, $ShowArchived)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SearchByLocation", True);
    $S .= StrTableBegin("0", "", "", BoxStyle);
    $S .= RowSelectLocation($LocationSn, $ShowArchived);
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function DiscardLocation(&$LocationSn)
{
    $LocationName = ColFromSerialNumber("location", "name",
					$LocationSn);
    $QName = "location &quot;$LocationName&quot;";
    $NumHits = pg_num_rows(ResPhotosAt($LocationSn));
    if ($NumHits == 0)
    {
	Query("BEGIN");
	Query("DELETE FROM archivedlocation WHERE locationsn = $1",
	      $LocationSn);
	Query("DELETE FROM location WHERE locationsn = $1",
	      $LocationSn);
	Query("COMMIT");
	$LocationSn = False;
	return StrWarning("Discarded $QName", NoticeColor);
    } else
    {
	$E = "$NumHits photo".(($NumHits > 1) ? "s" : "");
	$E .= " still located there";
	$E = BulletItem($E);
	$E = Strong("Can't discard $QName:")."\n<ul>$E\n</ul>";
	return  StrWarningBox($E, "50%");
    }
}

function FormArchivedLocCheckbox($LocationSn, $ArchivedLoc)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("LocationSn", $LocationSn);
    $S .= HtmlHidden("ToggleArchived", 1);
    $S .= "<label>";
    $S .= HtmlCheckbox("ArchivedLoc", $ArchivedLoc,
		       "onchange=\"this.form.submit()\"");
    $S .= "Location is archived</label>";
    $S .= StrFormEnd();
    return $S;
}

function FormRenameLocation($LocationSn, $LocationName)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("LocationSn", $LocationSn);
    $S .= HtmlTextInput("LocationName", "22", $LocationName, "",
			"text-align: right");
    $S .= "&nbsp;";
    $S .= HtmlSubmitButton("Rename");
    $S .= StrFormEnd();
    return $S;
}

function FormShowArchivedLocs($LocationSn, $ShowArchived)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("LocationSn", $LocationSn);
    $S .= "<label>";
    $S .= HtmlCheckbox("ShowArchived", $ShowArchived,
		       "onchange=\"this.form.submit()\"");
    $S .= "Show <span style=\"background: ".ArchivedLocationColor;
    $S .= "\">archived</span> locations in menu</label>";
    $S .= StrFormEnd();
    return $S;
}

function HeadingLocationPage($NumHits, $LocationName, $Notes,
			     $LocationSn)
{
    $QName = "&quot;$LocationName&quot;";
    $N = "located at<br />\n$QName";
    if ($NumHits > 0)
	$N = "$NumHits photo".(($NumHits > 1) ? "s" : "")." {$N}.";
    else
	$N = "No photos {$N} yet.";
    $N = Strong(HtmlColor(NoticeColor, $N));
    if ($NumHits == 0)
	$N .= "<br />\n".HtmlSubmitButton("DiscardLocation",
					  "Discard location $QName");
    $Cols = 40;
    $Rows = min(ceil((strlen($Notes) + 1) / $Cols), 4);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("LocationSn", $LocationSn);
    $S .= StrTableBegin("0", "", "80%");
    $S .= StrRowBegin();
    $S .= StrCell($N, "", "2");
    $S .= StrCell("Notes:");
    $S .= StrCell(HtmlSubmitButton("UpdateNotes", "Update Notes"),
		  "", "", "text-align: right");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell(HtmlTextArea("Notes", "$Cols", $Rows, $Notes,
			       "width: 100%"), "2");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function LineArchiveRename($ShowArchived, $LocationSn,
			   $IsLocationArchived)
{
    $S = StrTableBegin("0");
    $S .= StrRowBegin();
    $S .= StrCell(FormShowArchivedLocs($LocationSn, $ShowArchived or
				       $IsLocationArchived));
    if (is_numeric($LocationSn))
    {
	$S .= StrCell(FormArchivedLocCheckbox($LocationSn,
					      $IsLocationArchived),
		      "", "", "padding-left: 2em");
	$LocationName
	    = ColFromSerialNumber("location", "name", $LocationSn);
	$S .= StrCell(FormRenameLocation($LocationSn, $LocationName),
		      "", "", "padding-left: 4em");
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function PageLocations($LocationSn, $Quiet=False)
{
    /*****************************************************************
    Displays locations page with all widgets and first and last
    photos.
    *****************************************************************/
    if (! is_numeric($LocationSn))
	if ($Quiet) return;
	else return StrWarning("Select a location");
    $Res = ResPhotosAt($LocationSn);
    $NumRows = pg_num_rows($Res);
    $Q = "SELECT name, notes FROM location WHERE locationsn = $1";
    list($LocationName, $Notes)
	= pg_fetch_array(Query($Q, $LocationSn), 0);
    $S = HeadingLocationPage($NumRows, $LocationName, $Notes,
			     $LocationSn);
    if ($NumRows > 0)
    {
	$S .= PhotoStats($Res, 0, True);
	if ($NumRows > 1)
	    $S .= PhotoStats($Res, $NumRows - 1, False);
    }
    return $S;
}

function RenameLocation($LocationSn, $LocationName)
{
    $OldName = ColFromSerialNumber("location", "name", $LocationSn);
    if ($OldName != $LocationName)
    {
	$Q = "UPDATE location SET name = $2 WHERE locationsn = $1";
	Query($Q, $LocationSn, $LocationName);
	$N = "Renamed location &quot;$OldName&quot; to";
	$N .= " &quot;$LocationName&quot;";
    } else $N = "";
    return $N;
}

function RenamedLocation($LocationSn, $LocationName)
{
    $LocationName = stripslashes($LocationName);
    if (IsInDatabase("location", "name", $LocationName, True,
		     "locationsn", $LocationSn))
    {
	$W = "Couldn't rename location to &quot;$LocationName&quot;,";
	$W .= " as another location has that name.";
	$S = StrWarningBox($W, "50%");
    } else
	$S = StrWarning(RenameLocation($LocationSn, $LocationName),
			NoticeColor);
    return $S;
}

function ResPhotosAt($LocationSn)
{
    $Q = " SELECT photosn, foldername, filename, p.locationsn,";
    $Q .= " description, 1, 1, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay." AS timestamp";
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " v.vidfilename";
    $Q .= " FROM photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s, location l";
    $Q .= " WHERE p.locationsn = $1 AND s.photosetsn = p.photosetsn";
    $Q .= " AND l.locationsn = $1";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $LocationSn);
}

function RowSelectLocation($LocationSn, $ShowArchived)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("Select location:"));
    $S .= "<td align=\"right\">";
    $S .= MenuLocation($LocationSn, False, $ShowArchived,
		       "Select location:", True, False);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function ToggleArchived($LocationSn, $ArchivedLoc)
{
    $LocationName = ColFromSerialNumber("location", "name",
					$LocationSn);
    $T = $ArchivedLoc ? "into" : "out of";
    if ($ArchivedLoc)
    {
	$Q = "INSERT INTO archivedlocation(locationsn)";
	$Q .= " SELECT $1 WHERE NOT EXISTS";
	$Q .= " (SELECT * FROM archivedlocation";
	$Q .= "  WHERE locationsn = $1)";
    } else
	$Q = "DELETE FROM archivedlocation WHERE locationsn = $1";
    Query($Q, $LocationSn);
    $N = "Moved location &quot;$LocationName&quot; $T the archive.";
    return StrWarning($N, NoticeColor);
}

function UpdateNotes($LocationSn, $Notes)
{
    $Owner = LoggedInUser();
    $Notes = trim($Notes);
    $Q = "UPDATE location SET notes = $2";
    $Q .= " WHERE locationsn = $1 AND COALESCE(notes, '') != $2";
    Query ($Q, $LocationSn, $Notes);
    return;
}

//********************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "ArchivedLoc",
    "Notes",
    "ShowArchived",
    "LocationName",
    "LocationSn"
    );
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (! isset($LocationSn)) $LocationSn = False;
if (! isset($ShowArchived)) $ShowArchived = False;
$S = StartOfPage("Manage Locations");
if (isset($FormVars["DiscardLocation"]))
{
    $S .= DiscardLocation($LocationSn);
} elseif (isset($FormVars["ToggleArchived"]))
    $S .= ToggleArchived($LocationSn, $ArchivedLoc);
elseif (isset($FormVars["Rename"]))
    $S .= RenamedLocation($LocationSn, $LocationName);
$IsLocationArchived = IsLocationArchived($LocationSn);
$S .= BoxLocationSelect($LocationSn,
			($ShowArchived or $IsLocationArchived));
$S .= LineArchiveRename($ShowArchived, $LocationSn,
			$IsLocationArchived);
if (isset($FormVars["SearchByLocation"]) or isset($SearchByLocation))
    $S .= PageLocations($LocationSn);
elseif (isset($FormVars["UpdateNotes"]))
{
    UpdateNotes($LocationSn, $Notes);
    $S .= PageLocations($LocationSn);
} else
    $S .= PageLocations($LocationSn, True);
$S .= "</div></body></html>\n";
echo $S;
?>
