<?php
/*********************************************************************
editset.php
Edit Photo Set Module for PhotoTrove
______________________________________________________________________
Copyright 2024 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
session_cache_limiter("");
require_once ("lib_common.php");
require ("lib_album.php");
require_once ("lib_layout.php");
require ("lib_filesys.php");
require ("lib_graphicsmagick.php");
require ("lib_photoset.php");
require ("lib_tag.php");
require ("local/loc_camera.php");

$T = "Nothing to replace; no photo in the collection has that ID.";
$T .= " Perhaps you or someone else deleted that photo from a";
$T .= " different window or tab?";
define("ErrNoPhotoToReplace", $T);
$T = "Nothing to replace; no video in the collection has that ID.";
$T .= " Perhaps you or someone else deleted that video from a";
$T .= " different window or tab?";
define("ErrNoVideoToReplace", $T);
define("SortDefault", "Time");
$T = "You said you were uploading a video but the file you actually";
$T .= " uploaded is a still image. If you really meant to replace";
$T .= " the still image, continue, otherwise press Cancel and start";
$T .= " over.";
define("WarnStillForVideo", $T);
$T = "You said you were uploading a still image but the file you";
$T .= " actually uploaded is a video. If you really meant to replace";
$T .= " the video, continue, otherwise press Cancel and start over.";
define("WarnVideoForStill", $T);

function BtnDeleteEmptySet()
{
    $S = "border: medium solid ".WarningColor;
    return HtmlSubmitButton("DeletePhotoSet",
			    "Delete Empty Photo Set", "", $S);
}

function CellPickImage($Res, $Row, $NumRows)
{
    // Cell with thumbnail image and active radio button
    $S = "";
    if ($Row % 4 == 0)
    {
	if ($Row > 0) $S .= StrRowEnd();
	$S .= StrRowBegin();
    }
    list ($PhotoSn, $FolderName, $FileName, $Description, $ThumbW,
	  $ThumbH, $Time, $LocName) = pg_fetch_array($Res, $Row);
    list($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder ($FolderName);
    $T = "<label>".HtmlRadioButton("NewStill", $PhotoSn, False,
				   "onchange=\"this.form.submit()\"");
    $T .= "$FileName:<br />\n";
    $Path = "$ThumbFolder/$FileName";
    $T .= HtmlImage(LatestFile($Path),
		    "$FileName $Time ($LocName) $Description",
		    $ThumbH, $ThumbW, 1);
    $T .= "</label>";
    $S .= StrCell($T, "", "", $Style);
    return $S;
}

function ConfirmReplaceStill($PhotoSn, $FolderName, $FileName,
			     $Target, $NewFile, $State, $Row)
{
    $S = StartOfPage("Confirm Still Image Replacement");
    if ($Target == "video")
	$S .= StrWarningBox(WarnStillForVideo, "50%");
    $S .= FormConfReplImg($PhotoSn, $FolderName, $FileName, $NewFile,
			  $State, $Row);
    return $S;
}

function ConfirmReplaceVideo($Res, $PhotoSn, $FolderName, $FileName,
			     $Target, $NewName, $VidFileName,
			     $NewFile, $State, $Row)
{
    // Check that video exists in DB
    $VideoSn = pg_fetch_result($Res, 0, "videosn");
    if ($VideoSn != "")
    {
	$S = StartOfPage("Confirm Video Replacement");
	if ($Target == "still")
	    $S .= StrWarningBox(WarnVideoForStill, "50%");
    	$S .= FormConfReplVid($PhotoSn, $FolderName, $FileName,
			      $NewName, $VidFileName, $NewFile,
			      $State, $Row);
    } else
    {
	$W = StrWarningBox(ErrNoVideoToReplace, "50%");
	$W .= StrWarning("Video replacement failed.");
	$S = EditLargeImage($State, $Row, $W);
/*********************************************************************
Offer jump to add1pic with
foldername selected, to add a new video.
*********************************************************************/
    }
    return $S;
}

function ConfReplVidOrStill($NewName, $Res, $PhotoSn, $FolderName,
			    $FileName, $VidFileName, $VideoSn,
			    $Target, $NewFile, $State, $Row)
{
    $Finfo = finfo_open(FILEINFO_MIME_TYPE);
    $MediaType = MediaType($NewFile, $Finfo);
    finfo_close($Finfo);
    switch ($MediaType)
    {
    case "Video":
	$S = ConfirmReplaceVideo($Res, $PhotoSn, $FolderName,
				 $FileName, $Target, $NewName,
				 $VidFileName, $NewFile, $State, $Row);
	break;
    case "Image":
	$S = ConfirmReplaceStill($PhotoSn, $FolderName, $FileName,
				 $Target, $NewFile, $State, $Row);
	break;
    default:
	$FailTitle = "Replace Photo &quot;$FileName&quot;";
	$S = StartOfPage($FailTitle);
	$T = "Uploaded file &quot;$NewName&quot; seems to be neither";
	$T .= " a video nor an image. Please try again.";
	$S .= StrWarning($T);
	$S .= StrWarning("Replacement failed.");
	$S .= FormVideoSelect($PhotoSn, $FileName, $State, $Row,
			      $VidFileName);
    }
    return $S;
}

function DeleteEmptySet($PhotoSetSn)
{
    $FolderName
        = ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    if ($FolderName !== False)
    {
        if (CountPhotosInSet($PhotoSetSn) == 0)
        {
            list ($FullFolder, $WebFolder, $ThumbFolder)
                = ImageFolder($FolderName);
            Query("BEGIN");
            $Q = "DELETE FROM photoset WHERE photosetsn = $1";
            Query($Q, $PhotoSetSn);
            $Q = "INSERT INTO deletedset (foldername)";
            $Q .= " SELECT $1 WHERE NOT EXISTS";
            $Q .= " (SELECT 1 FROM deletedset WHERE foldername = $1)";
            Query($Q, $FolderName);
            Query("COMMIT");
            exec("rm -r $FullFolder");
            exec("rm -r $WebFolder");
            $S = HtmlPara("Deleted empty photo set $FolderName");
            $PhotoSetSn = NextPhotoSetSn($FolderName);
            $S .= PageThumbGallery($PhotoSetSn);
        } else
        {
            $S = "Photo set $FolderName isn't empty, can't delete";
            $S = StrWarning($S);
            $S .= PageThumbGallery($PhotoSetSn);
        }
    } else
	{
	    $S = "No photo set has this ID. Already deleted?";
	    $S = StrWarning($S);
	    $S .= BoxPhotoSetSelect("", "Time");
	}
    return $S;
}

function EditLargeBySn($PhotoSn, $Msg=False)
{
    // Handle requests from other tools to edit the webpic with a
    // fresh state, using default order by time, filename
    $PhotoSetSn = pg_fetch_result(ResPhoto($PhotoSn), 0,
				  "photosetsn");
    $State = PackedState(array($PhotoSetSn, SortDefault, False,
			       False, False));
    $Row = TupleNumInPhotoSetBySn($PhotoSn);
    return EditLargeImage($State, $Row, $Msg);
}

function EditLargeImage($State, $Row, $DepMsg=False)
{
    list ($PhotoSetSn, $Sort, $HideLocDone, $HideDescDone,
	  $HideTimeDone, $HideTagged) = UnPackedState($State);
    $Res = ResPhotosInSet($PhotoSetSn, $Sort, $HideLocDone,
			  $HideDescDone, $HideTimeDone, $HideTagged);
    $NumRows = pg_num_rows($Res);
    if ($NumRows > 0)
	$S = EditLargeImageForm($State, $Row, $DepMsg, $Res,
				$NumRows);
    else
    {
	$S = "There are no photos left to display in this series,";
	$S .= " perhaps because there really are none left in the";
	$S .= " photo set";
	$T = "";
	if ($HideLocDone) $T .= " and location";
	if ($HideDescDone) $T .= " and description";
	if ($HideTimeDone) $T .= " and date/time";
	if ($HideTagged) $T .= " and any tag";
	if ($T > "")
	{
	    $T = substr($T, 5);
	    $S .= ", but more likely because you've set the";
	    $S .= " worksheet to &quot;Hide photos with $T already";
	    $S .= " set&quot;";
	}
	$S .= ".<div style=\"text-align: center\">";
	$S .= StrFormBegin("javascript:self.close()");
	$S .= HtmlSubmitButton("Close", "Close", "Close",
			       StyleCloseButton);
	$S .= StrFormEnd();
	$S .= "</div>";
	$S = StrWarningBox("$S.", "50%");
	$S = StartOfPage("Photo List is Empty", "", "").$S;
    }
    return $S;
}

function EditLargeImageForm($State, $Row, $DepMsg, $Res, $NumRows)
{
    $Row = BoundedValue($Row, $NumRows - 1, 0);
    list ($PrevRow, $NextRow) = AdjacentRows($Row, $NumRows);
    $State = htmlentities($State);
    /* $State = htmlentities(stripslashes($State)); */
    // Identify the photo currently at $Row:
    $PhotoSn = pg_fetch_result($Res, $Row, "photosn");
    list ($FolderName, $FileName, $PhotoSetSn, $LocationSn,
	  $Description, $Width, $Height, $ThumbW, $ThumbH, $Time,
	  $LocName, $VidFileName, $VideoSn, $VWidth, $VHeight)
	= pg_fetch_array(ResPhoto($PhotoSn), 0);
    if ($Time == NullTimeFlag) $Time = ""; // Easier to fill in
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $IsVideo = ($VidFileName > "");
    $HrefFwd = ImageHref("State", $State, $NextRow);
    $HrefBack = ImageHref("State", $State, $PrevRow);
    if ($IsVideo)
    {
	$S = StartOfPage("$VidFileName (video)", "" , "", True);
	if ($DepMsg) $S .= $DepMsg;
    } else
    {
	$S = StartOfPage("$FileName (large)", "", "", True);
	if ($DepMsg) $S .= $DepMsg;
	$S .= "<map name=\"PicMap\">\n";
	$S .= "<area shape=\"rect\" coords=\"0,0,".PageEdge;
	$S .= ",$Height\" $HrefBack.>\n";
	$S .= "<area shape=\"rect\" coords=\"".($Width - PageEdge);
	$S .= ",0,$Width,$Height\" $HrefFwd>\n";
	$S .= "</map>\n";
    }
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("State", $State);
    $S .= HtmlHidden("Row", $Row);
    $S .= HtmlHidden("PhotoSn", $PhotoSn);
    $S .= EditLargeImageTable($FolderName, $FileName, $WebFolder,
			      $Width, $Height, $Description, $Time,
			      $LocationSn, $VidFileName, $HrefFwd,
			      $HrefBack, $PhotoSn);
    $S .= StrFormEnd();
    return $S;
}

function EditLargeImageTable($FolderName, $FileName, $WebFolder,
			     $Width, $Height, $Description, $Time,
			     $LocationSn, $VidFileName, $HrefFwd,
			     $HrefBack, $PhotoSn)
{
    $IsVideo = ($VidFileName > "");
    $S = "<table border=\"0\" cellspacing=\"0\"";
    if ($IsVideo) $S .= " style=\"background: ".HighlightBg."\"";
    $S .= " cellpadding=\"2\">";
    $S .= "\n<tbody>\n";
    if ($IsVideo)
    {
	$S .= RowWebVid($WebFolder, $VidFileName, 3);
	$S .= RowNavButtons($HrefBack, $HrefFwd, "3");
    } else $S .= RowWebPic($WebFolder, $FileName, $Width, $Height,
			   $Description);
    // Description, buttons
    $S .= RowPicData1($FolderName, $FileName, $Description,
		      $VidFileName);
    // Time
    $S .= RowPicData2($Time);
    // Location
    $S .= RowPicData3($LocationSn, $PhotoSn);
    // OtherLoc field
    $S .= RowPicData4();
    $S .= RowPicData5($PhotoSn);
    $S .= StrTableEnd();
    return $S;
}

function EditNotesPage($PhotoSetSn, $GenerateNotes=False, $Message="")
{
    list($PhotoSetSn, $FolderName, $Nickname, $Notes)
	= pg_fetch_array(ResPhotoSet($PhotoSetSn), 0);
    $S = StartOfPage("Photo Set Worksheet", "", "", False, True);
    if ($Message > "") $S .= StrWarning($Message, NoticeColor);
    $T = "Edit Metadata for Photo Set &quot;$FolderName&quot;";
    $S .= StrSubTitle($T);
    if ($GenerateNotes) $Notes = SummaryNotes($PhotoSetSn);
    $S .= StrFormBeginRecurse ();
    $S .= StrBoxBegin("1", "left", TableColor2);
    $T = HtmlTextInput("Nickname", "50", $Nickname);
    $S .= HtmlPara(Strong("Photo set nickname: ").$T);
    $T = Strong("Photo set notes:")."<br />\n";
    $T .= FittedTextArea("Notes", 65, 65, $Notes);
    $S .= HtmlPara($T);
    $S .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $S .= StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("UpdateNickAndNotes",
				   "Save Changes"));
    $S .= "<td align=\"right\">";
    $S .= HtmlSubmitButton("GenerateNotes", "Summarize Descriptions");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= StrBoxEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function FormConfReplImg($PhotoSn, $FolderName, $FileName, $NewFile,
			 $State, $Row)
{
    $NewName = $_FILES["UserFile"]["name"];
    $VideoSn = LinkedVideo($PhotoSn);
    if ($VideoSn !== False and $NewName != $FileName
        and PhotoNameInUse($FolderName, $NewName))
    {
        $S = WarnStillNameInUse($FileName, $NewName, $FolderName,
                                $NewFile);
        UnlinkIfExists($NewFile);
        $S .= FormVideoSelect($PhotoSn, $FileName, $State, $Row,
                              $VidFileName);
    } else
    {
        chmod($NewFile, 0644);
        ClearExifRotation($NewFile);
        list($S, $Width0, $Height0)
            = ViewPicCurrent($FolderName, $FileName);
        list($S, $Width, $Height, $LengthW)
            = ViewPicReplacement($S, $NewFile);
        $S .= StrFormBeginRecurse();
        $S .= WebSizeSettings($PhotoSn, $LengthW);
        $W = WarnSizeChange($Width, $Height, $Width0, $Height0);
        $W .= ($VideoSn)
           ? WarnNameChange($NewName, $FileName)
           : WarnNameDiff($NewName, $FileName);
        if ($W > "")
        {
            $W = StrSubTitle("Possible Problems:")."\n<ul>\n".$W;
            $W .= "</ul>\n";
            $W .= WarnNoUndo();
            $S .= StrWarningBox($W, "50%");
        }
        $S .= HtmlHidden("PhotoSn", $PhotoSn);
        $S .= HtmlHidden("NewName", $NewName);
        $S .= HtmlHidden("State", htmlentities($State));
     /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
        $S .= HtmlHidden("Row", $Row);
        $T = HtmlSubmitButton("CommitRepl", "Confirm Replacement");
        $T .= "&nbsp; &nbsp; &nbsp;";
        $T .= HtmlSubmitButton("CancelRepl", "Cancel");
        $S .= HtmlPara($T, "center");
        $S .= StrFormEnd();
    }
    return $S;
}

function FormConfReplVid($PhotoSn, $FolderName, $FileName, $NewName,
			 $VidFileName, $NewFile, $State, $Row)
{
    if ($NewName != $VidFileName
	and VideoNameInUse($FolderName, $NewName))
    {
	$S =  WarnVideoNameInUse($FolderName, $VidFileName, $NewName,
				 $NewFile);
	UnlinkIfExists($NewFile);
	$S .= FormVideoSelect($PhotoSn, $FileName, $State, $Row,
			      $VidFileName);
    } else
    {
	chmod($NewFile, 0644);
	$S = ViewVidCurrent($FolderName, $VidFileName);
	$S .= ViewVidReplacement($NewFile);
	$T = $NewName;
	if ($VidFileName != $NewName)
	    $T .= ", replacing $VidFileName";
	$S .= HtmlPara($T);
	$W = WarnNameChange($NewName, $VidFileName);
	if ($W > "")
	{
	    $W = StrSubTitle("Possible Problems:")."\n<ul>\n".$W;
	    $W .= "</ul>\n";
	    $W .= WarnNoUndo(True);
	    $S .= StrWarningBox($W);
	}
	$S .= StrFormBeginRecurse();
	$S .= HtmlHidden("PhotoSn", $PhotoSn);
	$S .= HtmlHidden("NewName", $NewName);
	$S .= HtmlHidden("State", htmlentities($State));
	/* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
	$S .= HtmlHidden("Row", $Row);
	$T = HtmlSubmitButton("CommitReplVid", "Confirm Replacement");
	$T .= "&nbsp; &nbsp; &nbsp;";
	$T .= HtmlSubmitButton("CancelRepl", "Cancel");
	$S .= HtmlPara($T, "center");
	$S .= StrFormEnd();
    }
    return $S;
}

function FormPhotoUpload($PhotoSn, $FolderName, $FileName, $State,
			 $Row)
{
    // Form for user to specify file to upload.
    $T = ParsedHumanNum(UploadMaxFileSize, True);
    $UploadSizeLimit = ceil($T[1]);
    $S .= StrFormBeginPostUpload(ThisScript(), UploadMaxFileBytes);
    $S .= HtmlHidden("PhotoSn", $PhotoSn);
    $S .= HtmlHidden("State", htmlentities($State));
    /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
    $S .= HtmlHidden("Row", $Row);
    $S .= StrTableBegin(0, "", "", BoxStyle);
    $S .= StrRowBegin();
    $T = "Select the file (max ".UploadMaxFileSize." bytes) you";
    $T .= " wish to upload to<br>replace photo &quot;$FileName&quot;";
    $T .= " in photo set &quot;$FolderName&quot;:<br>\n";
    $T .= HtmlInputPostUpload("UserFile", 60);
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $T = HtmlSubmitButton("Upload", "Upload File");
    $T .= "&nbsp; &nbsp; &nbsp;".HtmlSubmitButton("CancelUpload",
						  "Cancel");
    $S .= StrCell($T, "", "", "text-align: center");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function FormPickNewStill($PhotoSetSn, $FolderName, $FileName,
			  $VidFileName, $VideoSn, $State, $RowThumbs)
{
    $S = StrTableBegin(0, "", "", BoxStyle);
    $S .= "<tr><td>\n";
    $S .= " &quot;$FileName&quot; is a still image linked to the";
    $S .= " video &quot;$VidFileName&quot;";
    $S .= "<ol start=\"2\">\n";
    $S .= "<li>Select a new still image from the available images in";
    $S .= " &quot;$FolderName&quot;:\n";
    $Res = ResAvailableStills($PhotoSetSn);
    $NumRows = pg_num_rows($Res);
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("VideoSn", $VideoSn);
    $S .= HtmlHidden("State", htmlentities($State));
    /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
    $S .= HtmlHidden("Row", $RowThumbs);
    $S .= StrTableBegin("0");
    for ($Row=0 ; $Row < $NumRows ; $Row++ )
	$S .= CellPickImage($Res, $Row, $NumRows);
    for ($I=0 ; $I < (3 - (($NumRows -1) % 4)) ; $I++)
	$S .= StrCell("&nbsp;");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= HtmlPara("or ".HtmlSubmitButton("CancelNewStill",
					  "Cancel"));
    $S .= StrFormEnd();
    $S .= "</li>\n";
    $S .= "</ol>\n";
    $S .= StrBoxEnd();
    return $S;
}

function FormVideoMode($PhotoSn, $Target, $State, $Row)
{
	list ($FolderName, $FileName, $PhotoSetSn, $Z1, $Z2, $Z3, $Z4,
	      $Z5, $Z6, $Z7, $Z8, $VidFileName, $VideoSn)
	= pg_fetch_array(ResPhoto($PhotoSn), 0);
    $S = StartOfPage("Replace Photo &quot;$FileName&quot;");
    $S .= ($Target == "usepic")
	? FormPickNewStill($PhotoSetSn, $FolderName, $FileName,
			   $VidFileName, $VideoSn, $State, $Row)
	: FormVideoUpload($PhotoSn, $FolderName, $FileName, $Target,
			  $VidFileName, $State, $Row);
    return $S;
}

function FormVideoSelect($PhotoSn, $FileName, $State, $Row,
			 $VidFileName)
{
    // Form for user to choose whether to replace the still image
    // ("thumbnail") or the video file itself and to specify file to
    // upload.
    $S = StrTableBegin(0, "", "", BoxStyle);
    $S .= "<tr><td>\n";
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("PhotoSn", $PhotoSn);
    $S .= HtmlHidden("State", htmlentities($State));
    /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
    $S .= HtmlHidden("Row", $Row);
    $S .= HtmlHidden("SelectMode", "1");
    $S .= "Photo &quot;$FileName&quot; is a still image linked to the";
    $S .= " video &quot;$VidFileName&quot;";
    $S .= "<ol>\n";
    $S .= "<li>Choose:<br>\n";
    $A = "onchange=\"this.form.submit()\"";
    $T = "<label>".HtmlRadioButton("Target", "still", False, $A);
    $T .= "Upload new still image to replace &quot;$FileName&quot;";
    $T .= "</label><br />\n";
    $T .= "<label>".HtmlRadioButton("Target", "video", False, $A);
    $T .= "Upload new video to replace &quot;$VidFileName&quot;";
    $T .= "</label><br />\n";
    $T .= "<label>".HtmlRadioButton("Target", "usepic", False, $A);
    $T .= "Select another image from the folder as the still for";
    $T .= " &quot;$VidFileName&quot;";
    $T .= "</label>";
    $S .= HtmlPara($T);
    $T = "<em>In every case, the new file name will apply.</em>";
    $S .= HtmlPara($T);
    $S .= "</li>\n";
    $S .= "</ol>\n";
    $S .= StrFormEnd();
    $S .= StrBoxEnd();
    return $S;
}

function FormVideoUpload($PhotoSn, $FolderName, $FileName, $Target,
			 $VidFileName, $State, $Row)
{
    $S = StrTableBegin(0, "", "", BoxStyle);
    $S .= "<tr><td>\n";
    $T = ParsedHumanNum(UploadMaxFileSize, True);
    $UploadSizeLimit = ceil($T[1]);
    $S .= StrFormBeginPostUpload(ThisScript(), UploadMaxFileBytes);
    $S .= HtmlHidden("PhotoSn", $PhotoSn);
    $S .= HtmlHidden("Target", $Target);
    $S .= HtmlHidden("State", htmlentities($State));
    /* $S .= HtmlHidden("State", htmlentities(stripslashes($State))); */
    $S .= HtmlHidden("Row", $Row);
    $S .= "Photo &quot;$FileName&quot; is a still";
    $S .= " &quot;thumbnail&quot; linked to the video";
    $S .= " &quot;$VidFileName&quot;";
    $S .= "<ol start=\"2\">\n";
    $T = ($Target == "still") ? "still image" : "video" ;
    $S .= "<li>Select the $T (max ".UploadMaxFileSize." bytes) you";
    $S .= " wish to upload to photo set &quot;$FolderName&quot;:\n";
    $S .= HtmlPara(HtmlInputPostUpload("UserFile", 60));
    $S .= "</li>\n";

    $S .= "<li>When you have selected a file, click the button to";
    $S .= " upload it:";
    $T = HtmlSubmitButton("UploadVid", "Upload File");
    $T .= "&nbsp; &nbsp; &nbsp;".HtmlSubmitButton("CancelUpload",
						  "Cancel");
    $S .= HtmlPara($T, "center");
    $S .= "</li>\n";
    $S .= "</ol>\n";
    $S .= StrFormEnd();
    $S .= StrBoxEnd();
    return $S;
}

function LineNotesDoneHide($PhotoSetSn, $NumRows, $Sort, $IsSetEmpty,
			   $HideLocDone=False, $HideDescDone=False,
			   $HideTimeDone=False, $HideTagged=False)
{
    // Line below photoset selection box with Edit Notes button,
    // number of photos, and buttons to hide photos with location or
    // description or any tag already set 
    $S = StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $T = StrFormBeginRecurse("", "_blank");
    $T .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
    $T .= HtmlSubmitButton("EditNotes", "Edit Notes");
    $T .= StrFormEnd();
    $S .= "<td style=\"text-align: left\">$T</td>\n";
    $S .= StrCell(Strong("$NumRows photos"));
    $T = StrFormBeginRecurse();
    $T .= HtmlHidden("PhotoSetSn", $PhotoSetSn);

    if ($IsSetEmpty) $T .= BtnDeleteEmptySet();
    else
    {
	$T .= HtmlHidden("ToggleHideDone", True);
	$T .= HtmlHidden("Sort", $Sort);
	$T .= HtmlGoButton("Go");
	$T .= " Hide photos with {<label>";
	$T .= HtmlCheckbox("HideLocDone", $HideLocDone,
			   "onchange=\"this.form.submit()\"");
	$T .= "location</label> <label>";
	$T .= HtmlCheckbox("HideDescDone", $HideDescDone,
			   "onchange=\"this.form.submit()\"");
	$T .= "description</label> <label>";
	$T .= HtmlCheckbox("HideTimeDone", $HideTimeDone,
			   "onchange=\"this.form.submit()\"");
	$T .= "date/time</label> <label>";
	$T .= HtmlCheckbox("HideTagged", $HideTagged,
			   "onchange=\"this.form.submit()\"");
	$T .= "any tag</label>} already set";
    }
    $T .= StrFormEnd();
    $S .= "<td style=\"text-align: right\">$T</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function LinkedVideo($PhotoSn)
{
    // Given an image, return videosn of linked video if there is
    // one, else False
    $Q = "SELECT videosn FROM video WHERE stillsn = $1";
    $Res = Query($Q, $PhotoSn);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_result($Res, 0, "videosn") : False;
}

function OptionRotatePhotos()
{
    $S = StrRowBegin();
    $S .= StrCell("Turn selected photos:");
    $C = HtmlSubmitButton("TurnRight", "90&#176; Right");
    $C .= "&nbsp;&nbsp;&nbsp;";
    $C .= HtmlSubmitButton("TurnLeft", "90&#176; Left");
    $C .= "&nbsp;&nbsp;&nbsp;";
    $C .= HtmlSubmitButton("Turn180", "180&#176;");
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function PageConfirmReplacePhoto($PhotoSn, $State, $Row)
{
    $Res = ResPhoto($PhotoSn);
    // Check that photo exists in DB, fetch $FolderName, $FileName
    if ($Res && pg_num_rows($Res) > 0)
    {
        list($FolderName, $FileName) = pg_fetch_array($Res, 0);
        $FailTitle = "Replace Photo &quot;$FileName&quot;";
        if ($_FILES["UserFile"]["tmp_name"] == "none")
        {
            $S = StartOfPage($FailTitle);
            $T = "Specify a file no larger than ".UploadMaxFileSize;
            $T .= " bytes to upload";
            $S .= StrWarning($T);
            $S .= StrWarning("Upload failed.");
            $S .= FormPhotoUpload($PhotoSn, $FolderName, $FileName,
                                  $State, $Row);
        } else
        {
            list($FullUploads, $WebUploads, $T)
                = ImageFolder(".uploads");
            if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
            $NewFile = "$FullUploads/".LoggedInUser();
            if (move_uploaded_file($_FILES["UserFile"]["tmp_name"],
                                   $NewFile)
                and filesize($NewFile) > 0)
            {
                $S = StartOfPage("Confirm Photo Replacement");
                $S .= FormConfReplImg($PhotoSn, $FolderName,
                                      $FileName, $NewFile, $State,
                                      $Row);
            } else
            {
                UnlinkIfExists($NewFile);
                $S = StartOfPage($FailTitle);
                $S .= StrWarning("Upload failed.");
                $S .= FormPhotoUpload($PhotoSn, $FolderName,
                                      $FileName, $State, $Row);
            }
        }
    } else
    {
        $W = StrWarningBox(ErrNoPhotoToReplace, "50%");
        $W .= StrWarning("Photo replacement failed.");
        $S = EditLargeImage($State, $Row, $W);
        /*************************************************************
          Offer (if fullsize) jump to add1pic with
          foldername selected, to add a new image.
        *************************************************************/
    }
    return $S;
}

function PageConfirmReplaceVideo($PhotoSn, $Target, $State, $Row)
{
    $Res = ResPhoto($PhotoSn);
    // Check that photo exists in DB, fetch $FolderName, $FileName
    if ($Res && pg_num_rows($Res) > 0)
    {
	list ($FolderName, $FileName, $Z1, $Z2, $Z3, $Z4, $Z5, $Z6,
	      $Z7, $Z8, $Z9, $VidFileName, $VideoSn)
	    = pg_fetch_array($Res, 0);
	$FailTitle = "Replace Photo &quot;$FileName&quot;";
	if ($_FILES["UserFile"]["tmp_name"] == "none")
	{
	    $S = StartOfPage($FailTitle);
	    $T = "Specify a file no larger than ".UploadMaxFileSize;
	    $T .= " bytes to upload";
	    $S .= StrWarning($T);
	    $S .= StrWarning("Upload failed.");
	    $S .= FormVideoSelect($PhotoSn, $FileName, $State, $Row,
				  $VidFileName);
	} else
	{
	    list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
	    if (! file_exists($FullUploads))
		mkdir($FullUploads, 0755);
	    $NewFile = "$FullUploads/".LoggedInUser();
	    if (move_uploaded_file($_FILES["UserFile"]["tmp_name"],
				   $NewFile)
		and filesize($NewFile) > 0)
	    {
		$NewName = $_FILES["UserFile"]["name"];
		$S = ConfReplVidOrStill($NewName, $Res, $PhotoSn,
					$FolderName, $FileName,
					$VidFileName, $VideoSn,
					$Target, $NewFile, $State,
					$Row);
	    } else
	    {
		UnlinkIfExists($NewFile);
		$S = StartOfPage($FailTitle);
		$S .= StrWarning("Upload failed.");
		$S .= FormVideoSelect($PhotoSn, $FileName, $State,
				      $Row, $VidFileName);
	    }
	}
    } else
    {
	$W = StrWarningBox(ErrNoPhotoToReplace, "50%");
	$W .= StrWarning("Photo replacement failed.");
	$S = EditLargeImage($State, $Row, $W);
/*********************************************************************
Offer (if fullsize) jump to add1pic with
foldername selected, to add a new image.
*********************************************************************/
    }
    return $S;
}

function PageReplacePhoto($PhotoSn, $State, $Row)
{
    // If $PhotoSn is a thumbnail for a video, offer video upload
    // form, else offer photo upload form
    $Res = ResPhoto($PhotoSn);
    // Check that photo exists in DB, fetch vidfilename if any
    if ($Res && pg_num_rows($Res) > 0)
    {
	list ($FolderName, $FileName, $PhotoSetSn, $LocationSn,
	      $Description, $Width, $Height, $ThumbW, $ThumbH, $Time,
	      $LocName, $VidFileName, $VideoSn, $VWidth, $VHeight)
	    = pg_fetch_array($Res, 0);
	$S = StartOfPage("Replace Photo &quot;$FileName&quot;");
	$S .= ($VidFileName > "")
	    ? FormVideoSelect($PhotoSn, $FileName, $State, $Row,
			      $VidFileName)
	    : FormPhotoUpload($PhotoSn, $FolderName, $FileName,
			      $State, $Row);
    } else
    {
	$W = StrWarningBox(ErrNoPhotoToReplace, "50%");
	$W .= StrWarning("Photo replacement failed.");
	$S = EditLargeImage($State, $Row, $W);
    }
    return $S;
}

function PageThumbGallery($PhotoSetSn, $Sort=SortDefault,
			  $SelectAll=False, $HideLocDone=False,
			  $HideDescDone=False, $HideTimeDone=False,
			  $HideTagged=False, &$RangeMarks=False)
{
    /*****************************************************************
    Displays photo set page with all widgets and photo thumbnails.
    Calls ImageCells to display thumbnails with checkboxes set
    according to $SelectAll and $RangeMarks.
    *****************************************************************/
    $S = "";
    if (! isset($Sort)) $Sort = SortDefault;
    if ($PhotoSetSn == "" or $PhotoSetSn == "none")
        $S = StrWarning("Select a photo set");
    $State = PackedState(array($PhotoSetSn, $Sort, $HideLocDone,
                               $HideDescDone, $HideTimeDone,
			       $HideTagged));
    $Res = ResPhotosInSet($PhotoSetSn, $Sort, $HideLocDone,
                          $HideDescDone, $HideTimeDone, $HideTagged);
    $NumRows = pg_num_rows($Res);
    $S .= BoxPhotoSetSelect($PhotoSetSn, $Sort);
    $IsSetEmpty = (CountPhotosInSet($PhotoSetSn) == 0);
    $S .= LineNotesDoneHide($PhotoSetSn, $NumRows, $Sort, $IsSetEmpty,
                            $HideLocDone, $HideDescDone,
			    $HideTimeDone, $HideTagged);
    if (! $IsSetEmpty)
    {
        $S .= StrFormBeginRecurse();
        $S .= HtmlHidden("State", htmlentities($State));
        $S .= HtmlHidden("NumRows", $NumRows);
        $S .= StrSelectionOptions();
        $S .= "<br>\n";
        $S .= PhotoActionOptions(True, OptionRotatePhotos());
        $S .= "<br>\n";
        if ($SelectAll) $RangeMarks = True;
        $S .= ImageCells($Res, $NumRows, $RangeMarks, "State", $State,
                         True, True);
        $S .= StrFormEnd();
    }
    return $S;
}

function PhotoNameInUse($FolderName, $FileName)
{
  // True if photo $FileName exists in photo set $FolderName
  $Q = "SELECT * FROM photo p, photoset s";
  $Q .= " WHERE foldername = $1";
  $Q .= " AND p.photosetsn = s.photosetsn";
  $Q .= " AND filename = $2";
  return (pg_num_rows(Query($Q, $FolderName, $FileName)) > 0);
}

function RenamePhoto($PhotoSn, $FileName, $NewFileName)
{
    /* Note: currently used only when photo is a replacement still for
     * a video (see function ReplacePhoto) */
    // Update filename in database entry, then rename all image files
    // (fullsize, webpics, thumbnails)
    // But first, check if we need to do anything
    if ($NewFileName != $FileName)
    {
	list ($OldFull, $OldWebPic, $OldThumbnail)
	    = ImagePath($PhotoSn);
	$Q = "UPDATE photo SET filename = $2 WHERE photosn = $1";
	Query($Q, $PhotoSn, $NewFileName);
	list ($NewFull, $NewWebPic, $NewThumbnail)
	    = ImagePath($PhotoSn);
	rename($OldFull, $NewFull);
	rename($OldWebPic, $NewWebPic);
	rename($OldThumbnail, $NewThumbnail);
    }
    return;
}

function ReplacePhoto($PhotoSn, $WebPicLength, $ThumbnailLength,
		      $NewName)
{
    list ($FullUploads, $WebUploads, $T) = ImageFolder(".uploads");
    list($FolderName, $FileName)
	= pg_fetch_array(ResPhoto($PhotoSn), 0);
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    // If this is a still image for a video, use the new file name
    if (LinkedVideo($PhotoSn))
    {
	RenamePhoto($PhotoSn, $FileName, $NewName);
	$FileName = $NewName;
    }
    // Replace fullsize, generate new webpic and thumbnail, update
    // geometry
    list($WebSize, $ThumbSize)
	= AddImages($FolderName, $FileName, $WebPicLength,
		    $ThumbnailLength);
    // Update existing photo
    $Q = "UPDATE photo SET";
    $Q .= " width = ".$WebSize["width"];
    $Q .= ", height = ".$WebSize["height"];
    $Q .= ", thumbw = ".$ThumbSize["width"];
    $Q .= ", thumbh = ".$ThumbSize["height"];
    $Q .= " WHERE photosn = $1";
    Query($Q, $PhotoSn);
    $GeoData = GPSFromFile("$FullFolder/$FileName");
    if ($GeoData !== False) UpdatePhotoGeoData($GeoData, $PhotoSn);
    return;
}

function ReplaceVideo($PhotoSn, $NewName)
{
    $Res = ResPhoto($PhotoSn);
    $FolderName = pg_fetch_result($Res, 0, "foldername");
    $VidFileName = pg_fetch_result($Res, 0, "vidfilename");
    $VideoSn = pg_fetch_result($Res, 0, "videosn");
    list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $NewFile = "$FullUploads/".LoggedInUser();
    $NewOggFile = "$FullUploads/".OggFile(LoggedInUser().".zzz");
    $OggFile = OggFile($NewName);
    // save/replace the file and its Ogg (Theora/Vorbis) version
    $Ok = rename($NewFile, "$FullFolder/$NewName");
    if ($Ok) $Ok = rename($NewOggFile, "$WebFolder/$OggFile");
    if ($Ok and ($NewName != $VidFileName))
    {
	// update video.vidfilename
	$Q = "UPDATE video SET vidfilename = $1";
	$Q .= " WHERE videosn = $VideoSn";
	Query($Q, $NewName);
	// there are old files to delete
	UnlinkIfExists("$FullFolder/$VidFileName");
	if ($NewName != OggFile($VidFileName))
	    UnlinkIfExists("$WebFolder/".OggFile($VidFileName));
    }
    if ($Ok)
    {
	// Update geometry
	list($Width, $Height)
	    = array_values(VideoSpecs("$FullFolder/$NewName"));
	$Q = "UPDATE video SET width = $Width, height = $Height";
	$Q .= " WHERE videosn = $VideoSn";
	Query($Q);
	$GeoData = GPSFromFile("$FullFolder/$NewName");
	if ($GeoData !== False)
	    UpdatePhotoGeoData($GeoData, $PhotoSn);
    }
    return $Ok;
}

function ResAvailableStills($PhotoSetSn)
{
    // Images in $PhotoSetSn not currently assigned as stills for
    // videos
    $Q = "SELECT photosn, foldername, filename, description, thumbw,";
    $Q .= " thumbh, time, l.name AS locname";
    $Q .= " FROM photoset s, photo p LEFT OUTER JOIN location l";
    $Q .= " USING (locationsn)";
    $Q .= " WHERE s.photosetsn = $1 AND p.photosetsn = s.photosetsn";
    $Q .= " AND NOT EXISTS";
    $Q .= " (SELECT * FROM video WHERE stillsn = p.photosn)";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $PhotoSetSn);
}

function RowPicData1($FolderName, $FileName, $Description,
		     $VidFileName)
{
    // Description, buttons
    $S = StrRowBegin();
    $S .= StrCell(Strong("What:"));
    $T = HtmlTextArea ("Description", "40", "3", $Description);
    $S .= StrCell($T);
    $T = HtmlSubmitButton("Commit", "Save Changes")."<br>\n";
    $T .= HtmlSubmitButton("Rotate90", "90&#176; Right");
    $T .= HtmlSubmitButton("Rotate90Left", "90&#176; Left");
    $T .= HtmlSubmitButton("Rotate180", "180&#176;");
    $C = HtmlPara($T, "right");
    $C .= LinkButtonPara($FolderName, $FileName, $VidFileName);
    $C .= StrDangerousButtons(
	array
	(array("Name" => "Replace", "Lbl" => ""),
	 array("Name" => "Delete1", "Lbl" => "Delete")
	));
    $S .= "<td rowspan=\"4\" align=\"right\">$C</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowPicData2($Time)
{
    // Time
    $S = StrRowBegin();
    $S .= StrCell(Strong("When:"));
    $T = HtmlTextInput("Time", "25", $Time);
    $S .= StrCell($T);
    $S .= StrRowEnd();
    return $S;
}

function RowPicData3($LocationSn, $PhotoSn)
{
    // Location
    $S = StrRowBegin();
    $S .= StrCell(Strong("Where: "));
    $T = MenuLocation($LocationSn);
    $G = GeoCoordinates($PhotoSn);
    if ($G !== False) $T .= "<br />$G";
    $S .= StrCell($T);
    $S .= StrRowEnd();
    return $S;
}

function RowPicData4()
{
    // OtherLoc field
    $S = StrRowBegin();
    $T = "If &quot;other,&quot; specify: ";
    $T .= HtmlTextInput("OtherLoc", "30");
    $S .= StrCell($T, "2");
    $S .= StrRowEnd();
    return $S;
}

function RowPicData5($PhotoSn)
{
    // album occurrences or tags, if any
    $T = TagOccurrences($PhotoSn);
    $T .= AlbumOccurrences($PhotoSn);
    if ($T > "")
    {
	$S = StrRowBegin();
	$S .= StrCell($T, "2");
	$S .= StrRowEnd();
    } else $S = "";
    return $S;
}

function RowWebPic($WebFolder, $FileName, $Width, $Height,
		   $Description)
{
    // Large image
    $S = StrRowBegin();
    $S .= "<td colspan=\"3\" align=\"center\">\n";
    $T = UrlDownLoad("FetchWebPic", "$WebFolder/$FileName");
    $S .= "<a href=\"$T\">";
    $Path = "$WebFolder/$FileName";
    $S .= HtmlImage(LatestFile($Path), $Description, $Height, $Width,
		    "1", $Description, "#PicMap");
//     $S .= "</a>";
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function SelectAllOrNone($State, $SelectAll=True)
{
    /*****************************************************************
    Sets or clears selection checkboxes on all images according to
    $SelectAll, then redisplays photo set page.
    *****************************************************************/
    /* $State = stripslashes($State); */
    list ($PhotoSetSn, $Sort, $HideLocDone, $HideDescDone,
	  $HideTimeDone, $HideTagged)
	= UnPackedState($State);
    return ($PhotoSetSn > "" and $PhotoSetSn != "none")
	? PageThumbGallery($PhotoSetSn, $Sort, $SelectAll,
			   $HideLocDone, $HideDescDone,
			   $HideTimeDone, $HideTagged)
	: "";
}

function SelectRange($State, $NumThumbs, $FormVars)
{
    /*****************************************************************
    If $NumThumbs > 0, builds an ordered array of selected thumbnails
    which specify starting and stopping points of ranges, then calls
    PageThumbGallery to redisplay photo set page with checkboxes set
    for all images in range(s).
    *****************************************************************/
    $RangeMarks = array();
    for ($Row=0 ; $Row < $NumThumbs ; $Row++ )
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
            $RangeMarks[] = $FormVars["PS$Row"];
    if (count($RangeMarks) > 0)
    {
	/* $State = stripslashes($State); */
	list ($PhotoSetSn, $Sort, $HideLocDone, $HideDescDone,
	      $HideTimeDone, $HideTagged) = UnPackedState($State);
	$T = PageThumbGallery($PhotoSetSn, $Sort, False, $HideLocDone,
			      $HideDescDone, $HideTimeDone,
			      $HideTagged, $RangeMarks);
	$S = HtmlPara($RangeMarks).$T;

    } else
    {
	$S = HtmlPara("No selection range specified.");
	$S .= SelectAllOrNone($State, False);
    }
    return $S;
}

function SummaryNotes($PhotoSetSn)
{
    $Q = "SELECT l.name as locname, description,";
    $Q .= " to_char(time, 'DD FMMonth') AS date,";
    $Q .= " to_char(time, 'YYYY') AS year";
    $Q .= " FROM photo LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " WHERE photosetsn = $1";
    $Q .= " AND description IS NOT NULL AND description != ''";
    $Q .= " ORDER BY time, lower(filename)";
    $Res = Query($Q, $PhotoSetSn);
    if (! $Res || pg_num_rows($Res) == 0)
	$S = "No image descriptions found";
    else
    {
	$NumRows = pg_num_rows($Res);
	list ($Loc, $Description, $Date, $Year)
	    = pg_fetch_array($Res, 0);
	$S = "$Date $Year $Loc: $Description";
	for ($Row = 1 ; $Row < $NumRows ; $Row++)
	{
	    $PrevLoc = $Loc;
	    $PrevDesc = $Description;
	    $PrevDate = $Date;
	    $PrevYear = $Year;
	    list ($Loc, $Description, $Date, $Year)
		= pg_fetch_array($Res, $Row);
	    if ("$Date $Year" !== "$PrevDate $PrevYear")
	    {
		$T = ($Year !== $PrevYear) ? " $Year "  : " ";
		$S .= ".\n\n${Date}${T}$Loc: $Description";
	    } elseif ($Loc !== $PrevLoc)
		$S .= ".\n\n$Loc: $Description";
	    elseif ($Description !== $PrevDesc)
		$S .= "; $Description";
	}
	$S .= ".";
    }
    return $S;
}

function SwapStill($VideoSn, $NewStill)
{
    // Install existing photo $NewStill as still for video $VideoSn
    $Q = "SELECT photosn, p.photosetsn, filename, description,";
    $Q .= " vidfilename FROM video v, photo p";
    $Q .= " WHERE videosn = $1 AND photosn = stillsn";
    list($OldStillSn, $PhotoSetSn, $OldStillName, $OldDescription,
	 $VidFileName) = pg_fetch_array(Query($Q, $VideoSn), 0);
    $Q = "UPDATE video SET stillsn = $2";
    $Q .= " WHERE videosn = $1";
    Query($Q, $VideoSn, $NewStill);
    list($FolderName, $NewStillName)
	= pg_fetch_array(ResPhoto($NewStill), 0);
    $T = "$NewStillName has replaced $OldStillName as the still";
    $T .= " image for video $VidFileName.";
    $M = StrWarning($T, NoticeColor);
    return EditLargeBySn($NewStill, $M);
}

function UpdateMetaData($PhotoSn, $Time, $LocationSn, $Description,
			$OtherLoc)
{
    // Updates time, locationsn, or description if changed. Handles
    // each separately since any may be null or invalid
    $Q1 = "";
    $Q2 = "";
    $Count = 1;
    $Params = array($PhotoSn);
    $Description = trim($Description);
    // Forbid setting description blank (looks like NULL but isn't):
    if ($Description != "")
    {
	$Count++;
	$Params[] = $Description;
	$Q1 .= ", description = $$Count";
	$Q2 .= " AND COALESCE(description, '') = $$Count";
    }
    $Time = trim($Time);
    if (strtotime($Time) !== InvalidDateTime)
    {
	$Count++;
	$Params[] = $Time;
	$Q1 .= ", time = $$Count";
	$Q2 .= " AND time IS NOT NULL AND time = $$Count";
    }
    $LocationSn = FinalLocationSn($LocationSn, $OtherLoc);
    if ($LocationSn)
    {
	$Count++;
	$Params[] = $LocationSn;
	$Q1 .= ", locationsn = $$Count";
	$Q2 .= " AND COALESCE(locationsn, 0) = $$Count";
    }
    if ($Q1 != "")
    {
	$Q1 = substr($Q1, 2);
	$Q2 = substr($Q2, 5);
	$Q = "UPDATE photo SET $Q1 WHERE photosn = $1 AND NOT ($Q2)";
	QueryArray($Q, $Params);
    }
    return;
}

function UpdateNickAndNotes($PhotoSetSn, $Notes, $Nickname)
{
    $Notes = trim($Notes);
    $Nickname = trim($Nickname);
    $Q = "UPDATE photoset SET notes = $2, nickname = $3";
    $Q .= " WHERE photosetsn = $1 AND NOT EXISTS";
    $Q .= " (SELECT 1 FROM photo WHERE notes = $2 AND nickname = $3)";
    Query ($Q, $PhotoSetSn, $Notes, $Nickname);
    return;
}

function VideoNameInUse($FolderName, $VidFileName)
{
    // True if video $VidFileName exists in photo set $FolderName
    $Q = "SELECT * FROM photoset s, photo p, video v";
    $Q .= " WHERE s.foldername = $1 AND v.vidfilename = $2";
    $Q .= " AND p.photosn = v.stillsn";
    $Q .= " AND p.photosetsn = s.photosetsn";
    return (pg_num_rows(Query($Q, $FolderName, $VidFileName)) > 0);
}

function ViewPicCurrent($FolderName, $FileName)
{
    list ($FullFolder, $WebFolder, $T) = ImageFolder($FolderName);
    $CurrentFile = "$FullFolder/$FileName";
    $CurrentWebPic = "$WebFolder/$FileName";
    $Title = "$FolderName/$FileName";
    // Display current pic.
    list($Width0, $Height0) = array_values(Geometry($CurrentFile));
    list($WidthW, $HeightW) = array_values(Geometry($CurrentWebPic));
    $S = StrSubTitle("Current Image");
    $S .= HtmlImage(LatestFile($CurrentWebPic), $Title, $HeightW,
                    $WidthW, "1", $Title);
    return array($S, $Width0, $Height0);
}

function ViewPicReplacement($S, $NewFile)
{
    // Make webpic (but no larger than fullsize)
    list($Width, $Height) = array_values(Geometry($NewFile));
    $LengthW = min(ScaledWebPicLength($Width, $Height),
		   max($Width, $Height));
    list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
    $NewWebPic = "$FullUploads/".LoggedInUser()."-web";
    list($WidthW, $HeightW) =
	array_values(ResizePath($LengthW, $NewFile, $NewWebPic));
    $S .= StrSubTitle("Uploaded Image");
    $S .= HtmlImage(LatestFile($NewWebPic), "uploaded file", $HeightW,
		    $WidthW, "1", "uploaded file");
    return array($S, $Width, $Height, $LengthW);
}

function ViewVidCurrent($FolderName, $VidFileName)
{
    $S = StrSubTitle("Current Video");
    list ($FullFolder, $WebFolder, $ThumbFolder)
	= ImageFolder($FolderName);
    $OggFile = OggFile($VidFileName);
    $S .= VideoBlock("$WebFolder/$OggFile");
    return $S;
}

function ViewVidReplacement($NewFile)
{
    $S = StrSubTitle("Uploaded Video");
    list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
    $OggFile = "$FullUploads/".OggFile(LoggedInUser().".zzz");
    // Don't abort just because it takes a while
    set_time_limit (0);
    ConvertToOgg($NewFile, $OggFile);
    $S .= VideoBlock($OggFile);
    return $S;
}

function WarnNameChange($NewFileName, $FileName)
{
    $W = "";
    if ($NewFileName != $FileName)
    {
	$W = "<li>".Strong("Name will change:")."<br>\n";
	$W .= "The uploaded file &quot;$NewFileName&quot;";
	$W .= " will completely<br>\nreplace the current file";
	$W .= " &quot;$FileName&quot;,<br>\n<em>including the";
	$W .= " name</em>, in the database.</li>\n";
    }
    return $W;
}

function WarnNameDiff($NewFileName, $FileName)
{
    $W = "";
    if ($NewFileName != $FileName)
    {
	$W = "<li>".Strong("Names differ:")."<br>\n";
	$W .= "Although the uploaded file is named";
	$W .= " &quot;$NewFileName&quot;<br>\nit will be saved";
	$W .= " under the current file name<br>\n";
	$W .= "&quot;$FileName&quot; in the database.</li>\n";
    }
    return $W;
}

function WarnNoUndo($IsVideo=False)
{
    $T = $IsVideo ? "a video" : "an image";
    $S = "Replacing $T is complete and irreversible,<br>";
    $S .= "erasing all trace of the previous version. Be sure!";
    return HtmlPara(Strong($S));
}

function WarnSizeChange($Width, $Height, $Width0, $Height0)
{
    // Returns a warning (bullet) if dimensions have changed
    $W = "";
    if ($Height != $Height0 or $Width != $Width0)
    {
	$W = "<li>".Strong("Dimensions differ:")."<br>\n";
	$W .= StrTableBegin("0");
	$W.= StrRowBegin();
	$W.= StrHeaderCell("&nbsp;");
	$W.= StrHeaderCell("Width");
	$W.= StrHeaderCell("Height");
	$W.= StrRowEnd();
	$W.= StrRowBegin();
	$W.= StrCell(Strong("Current:"));
	$W.= StrCell($Width0);
	$W.= StrCell($Height0);
	$W.= StrRowEnd();
	$W.= StrRowBegin();
	$W.= StrCell(Strong("Uploaded:"));
	$W.= StrCell($Width);
	$W.= StrCell($Height);
	$W.= StrRowEnd();
	$W .= StrTableEnd()."</li>\n";
    }
    return $W;
}

function WarnStillNameInUse($FileName, $NewName, $FolderName,
			    $NewFile)
{
    $T = "The still image you have uploaded to replace";
    $T .= " &quot;$FileName&quot; has a name, &quot;$NewName&quot;,";
    $T .= " that is already in use in photo set";
    $T .= " &quot;$FolderName&quot;.";
    return StrWarningBox($T, "50%");
}

function WarnVideoNameInUse($FolderName, $VidFileName, $NewName,
			    $NewFile)
{
    $T = "The video you have uploaded to replace";
    $T .= " &quot;$VidFileName&quot; has a name,";
    $T .= " &quot;$NewName&quot;, that is already in use in photo";
    $T .= " set &quot;$FolderName&quot;.";
    return StrWarningBox($T, "50%");
}

function WebSizeSettings($PhotoSn, $WebPicLength)
{
    list ($Z1, $Z2, $Z3, $Z4, $Z5, $Width, $Height, $ThumbW, $ThumbH)
	= pg_fetch_array(ResPhoto($PhotoSn), 0);
    $WebLength = max($Width, $Height);
    $ThumbLength = max($ThumbW, $ThumbH);
    $S = StrTableBegin(0, "", "", BoxStyle);
    $S .= "<tr><td>\n";
    $T = "Replacing the full-size image will generate<br>\n";
    $T .= "a new web-size image and a new<br>\n";
    $T .= "thumbnail, with these dimensions<br>\n";
    $T .= "(adjust if needed&mdash;in no case will<br>\n";
    $T .= "they exceed the full size):";
    $S .= HtmlPara($T);
    $S .= StrTableBegin(0);
    $S .= StrRowBegin();
    $S .= StrCell("Web-size image length:");
    $S .= StrCell(HtmlTextInput("WebPicLength", "5", $WebPicLength));
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell("Thumbnail image length:");
    $S .= StrCell(HtmlTextInput("ThumbnailLength", "5",
				ThumbnailLength));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    if ($WebLength != $WebPicLength
	or $ThumbLength != ThumbnailLength)
    {
	$T = Strong("Note:")."<br>\n";
	$T .= "The current webpic is $WebLength pixels long<br>\n";
	$T .= "and the current thumbnail is $ThumbLength pixels<br>";
	$T .= "\nlong.";
	$S .= HtmlPara($T);
    }
    $S .= StrBoxEnd();
    return $S;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "AlbumSn",
    "Description",
    "FileName",
    "PhotoSetSn",
    "HideDescDone",
    "HideLocDone",
    "HideTagged",
    "HideTimeDone",
    "LocationSn",
    "NewAlbum",
    "NewName",
    "NewStill",
    "NewTag",
    "Nickname",
    "Notes",
    "NumRows",
    "OtherLoc",
    "PhotoSetSn",
    "PhotoSn",
    "Row",
    "SelectAll",
    "Sort",
    "State",
    "TagSn",
    "Target",
    "ThumbnailLength",
    "Time",
    "VideoSn",
    "VidFileName",
    "ViewLarge",
    "WebPicLength"
    );
$GetVarList = array(
    "FetchFull",
    "FetchVideo",
    "FetchWebPic",
    "FileName",
    "PhotoSetSn",
    "Row",
    "State",
    "ViewLarge"
   );
$PostVarList = array(
    "FileName",
    "PhotoSetSn",
    "PhotoSn",
    "MAX_FILE_SIZE",
    "Row",
    "State",
    "Target",
    "UserFile",
    "VidFileName"
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (isset($FetchWebPic)) DownloadFile($FetchWebPic);
elseif (isset($FetchFull)) DownloadFile($FetchFull);
elseif (isset($FetchVideo)) DownloadFile($FetchVideo);
else
{
    if (! isset($Sort)) $Sort = "Time";
    $S = "";
    if (! isset($PhotoSetSn) and isset($State))
	list ($PhotoSetSn, $Sort, $HideLocDone, $HideDescDone,
	      $HideTimeDone, $HideTagged) = UnPackedState($State);

    if (isset($FormVars["EditLargeBySn"]))
	$S .= EditLargeBySn($PhotoSn);
    elseif (! isset($PhotoSetSn))
    {
	$S .= StartOfPage("Photo Set Worksheet");
	$S .= BoxPhotoSetSelect("", "Time");
    } elseif ($PhotoSetSn == "none")
    {
	$S .= StartOfPage("Photo Set Worksheet");
	$S .= StrWarning("You must select a photo set");
	$S .= BoxPhotoSetSelect("", "Time");
    } elseif (isset($ViewLarge))
 	$S .= EditLargeImage($State, $Row);
    elseif (isset($FormVars["Rotate90"]))
    {
	RotatePhoto($PhotoSn, 90);
	UpdateMetaData($PhotoSn, $Time, $LocationSn, $Description,
		       $OtherLoc);
	$S .= EditLargeImage($State, $Row);
    } elseif (isset($FormVars["Rotate90Left"]))
    {
	RotatePhoto($PhotoSn, -90);
	UpdateMetaData($PhotoSn, $Time, $LocationSn, $Description,
		       $OtherLoc);
	$S .= EditLargeImage($State, $Row);
    } elseif (isset($FormVars["Rotate180"]))
    {
	RotatePhoto($PhotoSn, 180);
	UpdateMetaData($PhotoSn, $Time, $LocationSn, $Description,
		       $OtherLoc);
	$S .= EditLargeImage($State, $Row);
    } elseif (isset($FormVars["Delete1"]))
    {
	$DepMsg = PhotoDependences($PhotoSn);
	if (!$DepMsg) DeletePhoto($PhotoSn);
// Warn if DeletePhoto returned False
	$S .= EditLargeImage($State, $Row, $DepMsg);
    } elseif (isset($FormVars["Commit"]))
    {
	UpdateMetaData($PhotoSn, $Time, $LocationSn, $Description,
		       $OtherLoc);
	$S .= EditLargeBySn($PhotoSn); // $Row may have changed
    } elseif (isset($FormVars["Replace"]))
	$S .= PageReplacePhoto($PhotoSn, $State, $Row);
    elseif (isset($_POST["Upload"]))
	$S .= PageConfirmReplacePhoto($PhotoSn, $State, $Row);
    elseif (isset($_POST["UploadVid"]))
	$S .= PageConfirmReplaceVideo($PhotoSn, $Target, $State,
				      $Row);
    elseif (isset($FormVars["SelectMode"]))
	$S .= FormVideoMode($PhotoSn, $Target, $State, $Row);
    elseif (isset($_POST["CancelUpload"])
	    or isset($FormVars["CancelNewStill"]))
    {
	$W = StrWarning("Photo replacement canceled.", NoticeColor);
	$S .= EditLargeImage($State, $Row, $W);
    } elseif (isset($NewStill))
	  $S .= SwapStill($VideoSn, $NewStill);
    elseif (isset($FormVars["CommitRepl"]))
    {
	ReplacePhoto($PhotoSn, $WebPicLength, $ThumbnailLength,
		     $NewName);
	$W = StrWarning("Photo replaced.", NoticeColor);
	$S .= EditLargeImage($State, $Row, $W);
    } elseif (isset($FormVars["CommitReplVid"]))
    {
	if (ReplaceVideo($PhotoSn, $NewName))
	    $W = StrWarning("Video replaced.", NoticeColor);
	else
	    $W = StrWarning("Video replacement failed.");
	$S .= EditLargeImage($State, $Row, $W);
    } elseif (isset($FormVars["CancelRepl"]))
    {
	$W = StrWarning("Photo replacement canceled.", NoticeColor);
	$S .= EditLargeImage($State, $Row, $W);
    } elseif (isset($FormVars["EditNotes"]))
	$S .= EditNotesPage($PhotoSetSn);
    elseif (isset($FormVars["GenerateNotes"]))
	$S .= EditNotesPage($PhotoSetSn, True);
    elseif (isset($FormVars["UpdateNickAndNotes"]))
    {
	UpdateNickAndNotes($PhotoSetSn, $Notes, $Nickname);
	$S .= EditNotesPage($PhotoSetSn, False, "Saved changes");
    } else
    {
	$S = StartOfPage("Photo Set Worksheet");
	if (isset($FormVars["ToggleHideDone"])) // The hide checkboxes
	    $S .= PageThumbGallery($PhotoSetSn, $Sort, False,
				   $HideLocDone, $HideDescDone,
				   $HideTimeDone, $HideTagged);
	elseif (isset($SelectAll) or isset($FormVars["ClearAll"]))
	    $S .= SelectAllOrNone($State, isset($SelectAll));
	elseif (isset($FormVars["SelectRange"]))
	    $S .= SelectRange($State, $NumRows, $FormVars);
	elseif (isset($FormVars["ApplyLoc"]))
	{
	    $S .= AppliedLocation($NumRows, $LocationSn, $OtherLoc);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["ApplyDesc"]))
	{
	    $S .= AppliedDescription($NumRows, $Description);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["AddToAlbum"]))
	{
	    $S .= AddedToAlbum($NumRows, $AlbumSn, $NewAlbum);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["ApplyTag"]))
	{
	    $S .= AppliedTag($NumRows, $TagSn, $NewTag);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["TurnRight"]))
	{
	    $S .= AppliedRotation($NumRows, 90);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["TurnLeft"]))
	{
	    $S .= AppliedRotation($NumRows, -90);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["Turn180"]))
	{
	    $S .= AppliedRotation($NumRows, 180);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["Delete"]))
	{
	    $S .= AppliedDeletion($NumRows);
	    $S .= SelectAllOrNone($State, False);
	} elseif (isset($FormVars["DeletePhotoSet"]))
	    $S .= DeleteEmptySet($PhotoSetSn);
	else
	    $S .= PageThumbGallery($PhotoSetSn, $Sort);
    }
    $S .= "</div></body></html>\n";
    echo $S;
}

?>
