<?php
/*********************************************************************
edittagged.php
Tagged-Image Operations Module for PhotoTrove
______________________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require_once ("lib_layout.php");
require ("lib_tag.php");
require ("lib_photoset.php");
require_once ("local/loc_camera.php");

function BoxTagSelect($TagSn, $ShowArchived)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SearchByTag", True);
    $S .= StrTableBegin("0", "", "", BoxStyle);
    $S .= RowSelectTag($TagSn, $ShowArchived);
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function DiscardTag($TagSn)
{
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    Query("BEGIN");
    Query("DELETE FROM archivedtag WHERE tagsn = $1", $TagSn);
    Query("DELETE FROM tag_photo WHERE tagsn = $1", $TagSn);
    Query("DELETE FROM tag WHERE tagsn = $1", $TagSn);
    Query("COMMIT");
    return StrWarning("Discarded tag &quot;$TagName&quot;",
		      NoticeColor);
}

function FormArchivedTagCheckbox($TagSn, $ArchivedTag)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= HtmlHidden("ToggleArchived", 1);
    $S .= "<label>";
    $S .= HtmlCheckbox("ArchivedTag", $ArchivedTag,
		       "onchange=\"this.form.submit()\"");
    $S .= "Tag is archived</label>";
    $S .= StrFormEnd();
    return $S;
}

function FormRenameTag($TagSn, $TagName)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= HtmlTextInput("TagName", "22", $TagName, "",
			"text-align: right");
    $S .= "&nbsp;";
    $S .= HtmlSubmitButton("Rename");
    $S .= StrFormEnd();
    return $S;
}

function FormShowArchivedTags($TagSn, $ShowArchived)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= "<label>";
    $S .= HtmlCheckbox("ShowArchived", $ShowArchived,
		       "onchange=\"this.form.submit()\"");
    $S .= "Show <span style=\"background: ".ArchivedTagColor;
    $S .= "\">archived</span> tags in menu</label>";
    $S .= StrFormEnd();
    return $S;
}

function HeadingTagPage($NumHits, $TagName, $Notes, $TagSn)
{
    if ($NumHits > 0)
    {
	$Plural = ($NumHits > 1) ? "s" : "";
	$N = "$NumHits photo$Plural tagged &quot;$TagName&quot;.";
    } else
	$N = "No photos tagged &quot;$TagName&quot; yet.";

    $Cols = 30;
    $Rows = min(ceil((strlen($Notes) + 1) / $Cols), 4);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("TagSn", $TagSn);
    $S .= StrTableBegin("0", "", "80%");
    $S .= StrRowBegin();
    $S .= StrCell(Strong(HtmlColor(NoticeColor, $N)), "", "2");
    $S .= StrCell("Notes:");
    $S .= StrCell(HtmlSubmitButton("UpdateNotes", "Update Notes"));
    $S .= "<td title=\"Remove this tag from any photos and discard";
    $S .= " it from the tag list\">";
    $S .= HtmlSubmitButton("DiscardTag",
			   "Discard tag &quot;$TagName&quot;");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell(HtmlTextArea("Notes", "$Cols", $Rows, $Notes), "2");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function LineArchiveRename($ShowArchived, $TagSn, $IsTagArchived)
{
    $S = StrTableBegin("0");
    $S .= StrRowBegin();
    $S .= StrCell(FormShowArchivedTags($TagSn, $ShowArchived
				       or $IsTagArchived));
    if (is_numeric($TagSn))
    {
	$S .= StrCell(FormArchivedTagCheckbox($TagSn, $IsTagArchived),
		      "", "", "padding-left: 2em");
	$TagName = ColFromSerialNumber("tag", "name", $TagSn);
	$S .= StrCell(FormRenameTag($TagSn, $TagName), "", "",
		      "padding-left: 4em");
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function PageTaggedPhotos($TagSn, $Quiet=False)
{
    /*****************************************************************
    Displays tagged photo page with all widgets and first and last
    photos.
    *****************************************************************/
    if (! is_numeric($TagSn))
	if ($Quiet) return;
	else return StrWarning("Select a tag");
    $Res = ResTaggedPhotos($TagSn);
    $NumRows = pg_num_rows($Res);
    $Q = "SELECT name, notes FROM tag WHERE tagsn = $1";
    list($TagName, $Notes) = pg_fetch_array(Query($Q, $TagSn), 0);
    $S = HeadingTagPage($NumRows, $TagName, $Notes, $TagSn);
    if ($NumRows > 0)
    {
	$S .= PhotoStats($Res, 0, True);
	if ($NumRows > 1)
	    $S .= PhotoStats($Res, $NumRows - 1, False);
    }
    return $S;
}

function RenameTag($TagSn, $TagName)
{
    $OldName = ColFromSerialNumber("tag", "name", $TagSn);
    if ($OldName != $TagName)
    {
	$Q = "UPDATE tag SET name = $2 WHERE tagsn = $1";
	Query($Q, $TagSn, $TagName);
	$N = "Renamed tag &quot;$OldName&quot; to";
	$N .= " &quot;$TagName&quot;";
    } else $N = "";
    return $N;
}

function RenamedTag($TagSn, $TagName)
{
    $TagName = trim($TagName);
    if (IsInDatabase("tag", "name", $TagName, True, "tagsn", $TagSn))
    {
	$W = "Couldn't rename tag to &quot;$TagName&quot;,";
	$W .= " as another tag has that name.";
	$S = StrWarningBox($W, "50%");
    } else
	$S = StrWarning(RenameTag($TagSn, $TagName), NoticeColor);
    return $S;
}

function ResTaggedPhotos($TagSn)
{
    $Q = " SELECT tp.photosn, foldername, filename, p.locationsn,";
    $Q .= " description, 1, 1, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay." AS timestamp";
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " v.vidfilename";
    $Q .= " FROM tag_photo tp, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= ", photoset s";
    $Q .= " WHERE tagsn = $1 AND p.photosn = tp.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY time, lower(filename)";
    return Query($Q, $TagSn);
}

function RowSelectTag($TagSn, $ShowArchived)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("Select tag:"));
    $S .= "<td align=\"right\">";
    $S .= MenuTag($TagSn, "TagSn", True, False, False, $ShowArchived);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function ToggleArchived($TagSn, $ArchivedTag)
{
    $TagName = ColFromSerialNumber("tag", "name", $TagSn);
    $T = $ArchivedTag ? "into" : "out of";
    if ($ArchivedTag)
    {
	$Q = "INSERT INTO archivedtag (tagsn)";
	$Q .= " SELECT $1 WHERE NOT EXISTS";
	$Q .= " (SELECT * FROM archivedtag WHERE tagsn = $1)";
    } else
    {
	$Q = "DELETE FROM archivedtag WHERE tagsn = $1";
    }
    Query($Q, $TagSn);
    $N = "Moved tag &quot;$TagName&quot; $T the archive.";
    return StrWarning($N, NoticeColor);
}

function UpdateNotes($TagSn, $Notes)
{
    $Owner = LoggedInUser();
    $Notes = trim($Notes);
    $Q = "UPDATE tag SET notes = $2";
    $Q .= " WHERE tagsn = $1 AND COALESCE(notes, '') != $2";
    Query ($Q, $TagSn, $Notes);
    return;
}

//********************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "ArchivedTag",
    "Notes",
    "ShowArchived",
    "TagName",
    "TagSn"
    );
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (! isset($TagSn)) $TagSn = False;
if (! isset($ShowArchived)) $ShowArchived = False;
$S = StartOfPage("Manage Tags");
if (isset($FormVars["DiscardTag"]))
{
    $S .= DiscardTag($TagSn);
    $TagSn = False;
} elseif (isset($FormVars["ToggleArchived"]))
    $S .= ToggleArchived($TagSn, $ArchivedTag);
elseif (isset($FormVars["Rename"]))
    $S .= RenamedTag($TagSn, $TagName);
$IsTagArchived = IsTagArchived($TagSn);
$S .= BoxTagSelect($TagSn, ($ShowArchived or $IsTagArchived));
$S .= LineArchiveRename($ShowArchived, $TagSn, $IsTagArchived);
if (isset($FormVars["SearchByTag"]) or isset($SearchByTag))
    $S .= PageTaggedPhotos($TagSn);
elseif (isset($FormVars["UpdateNotes"]))
{
    UpdateNotes($TagSn, $Notes);
    $S .= PageTaggedPhotos($TagSn);
} else
    $S .= PageTaggedPhotos($TagSn, True);
$S .= "</div></body></html>\n";
echo $S;
?>
