<?php
/*********************************************************************
gk_common.php

Common Definitions for PHP Scripts

Do not edit; put application-specific definitions in parent folder's
lib_common.php
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require ("local/loc_env.php");

if (file_exists("local/loc_pwkmenu.php"))
{
    require("local/loc_pwkmenu.php");
    if (EnablePwkMenu) require("pwkmenu/pwkmenu.php");
}
if (! function_exists("IsPwkMenuOkHere"))
{
    // Normally defined in lib_pwkmenu.php but not loaded if
    // EnablePwkMenu is False
    function IsPwkMenuOkHere()
    {
	return False;
    }
}

if (file_exists("gk/gk_phpretro.php"))
    require("gk/gk_phpretro.php");

// Regular expression to match email address, from
// http://www.phpbuilder.com/annotate/message.php3?id=1012622 which is
// a comment to original article at
// http://www.phpbuilder.com/columns/dario19990616.php3?page=3
// Updated to use preg_match instead of deprecated eregi
define ("RegExIEmail",
"/^[_[:alnum:]-]+(\.[_[:alnum:]-]+)*@[[:alnum:]-]+(\.[[:alnum:]-]+)*(\.([[:alpha:]]){2,4})$/i");

// Regular expression to match a non-US telephone number using
// preg_match
define ("RegExINonUsPhone", "/^011-[[:digit:]]+(-[[:digit:]]+)+(x[[:digit:]]+)?$/i");
// define ("RegExINonUsPhone", "^011-[0-9]+(-[0-9]+)+(x[0-9]+)?$");

// Regular expression to match a US telephone number using preg_match
define ("RegExIUsPhone",
"/^[2-9][[:digit:]]{2}-[2-9][[:digit:]]{2}-[[:digit:]]{4}(x[[:digit:]]+)?$/i");

// define ("RegExIUsPhone",
// "^[2-9][0-9]{2}-[2-9][0-9]{2}-[0-9]{4}(x[0-9]+)?$");

define ("InvalidDateTime", (PHP_VERSION >= "5.1") ? False : -1);

function ArrayPack($Array)
{
    return ArrayWrap(serialize($Array));
}

function ArrayUnpack($WrappedSerializedArray)
{
    return unserialize(ArrayUnwrap($WrappedSerializedArray));
}

function ArrayUnwrap($WrappedSerializedArray)
{
    return stripslashes($WrappedSerializedArray);
}

function ArrayWrap($SerializedArray)
{
    return htmlentities(stripslashes($SerializedArray));
}

function BounceStrangers($UserId, $Tool)
{
    global $NoActiveLogin;
    // Deny access to users with insufficient privileges
    if (IsToolPublic($Tool)) return;
    if (IsReportPublic($Tool)) return;
    if (! $UserId)
    {
	$NoActiveLogin = True;
	include(LoginScript);
	exit;   
    } elseif (! UserMayUseTool($UserId, $Tool))
    {
	$T = "This service is accessible only to users who log in";
	$T .= " and have the requisite privilege.";
	$T .= ($UserId > "")
	    ? " User &quot;$UserId&quot; (that's you)"
	    : " An unidentified user (for example, you)";
	$T .= " <strong>may NOT</strong> run $Tool; how did you";
	$T .= " get here? If you require access, please contact";
	$T .= " the system administrator.";
	DisplayErrorPage($T);
	exit;   
    }
}

function ClearSession()
{
    session_start();
    session_unset();
    session_destroy();
    session_start();
    return;
}

function ColFromId($Table, $Col, $Id)
{
    // Return value of column $Col of tuple with serial ID $Id,
    // where ID column is named "{$Table}id"; the inverse
    // of function IdFromCol
    $Res = Query("SELECT $Col FROM $Table WHERE {$Table}id = $1", $Id);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_result($Res, 0, $Col) : False;
}

function ColFromSerialNumber($Table, $Col, $Sn)
{
    // Return value of column $Col of tuple with serial number $Sn,
    // where serial number column is named "{$Table}sn"; the inverse
    // of function SerialNumber
    $Res = Query("SELECT $Col FROM $Table WHERE {$Table}sn = $1", $Sn);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_result($Res, 0, $Col) : False;
}

// Even scripts that don't need gk/gk_layout may need this:
function DisplayErrorPage ($Message, $ButtonText="Exit",
			   $ButtonUrl="")
{
    global $SystemTitle;
    include_once("gk/gk_layout.php");
    if ($ButtonUrl == "")
    {
	$ButtonUrl = LoginScript;
    }
    $Button = array
    (
	"Text" => $ButtonText,
	"Url"  => $ButtonUrl
    );
    $B = "font-family: helvetica, sans-serif; color: Navy;";
    $B .= " background-color: #ffb062;";
    $B .= " background-image: url(gk/images/flames.jpg)";
    $L = "A:link { color: Blue }\nA:visited { color: Blue }\n";
    $S = HeadAndTitle("", "", $B, $L);
    $S .= "<div align=\"center\">\n";
    $S .=  PageTitleWithButton("Zounds!", $Button);
    $S .=  StrWarningBox($Message, "50%");
    $S .=  "</div></body></html>\n";
    echo $S;
}

function HitsInDatabase($Table, $Col, $Value)
{
    // Returns number of records in $Table where $Col = $Value
    $Q = "SELECT * FROM $Table WHERE $Col";
    $Res = (is_string($Value) and $Value == "")
	? Query("$Q IS NULL") : Query("$Q = $1", $Value);
    return pg_num_rows($Res);
}

function IdFromCol($Table, $Key, $Value, $KeyIsText=True)
{
    // Return serial ID of tuple whose $Key column = $Value, where
    // ID column is named "{$Table}id"; the inverse of
    // function ColFromId
    $Q = "SELECT {$Table}id FROM $Table WHERE $Key = $1";
    $Res = Query($Q, $Value);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_result($Res, 0, "{$Table}id") : False;
}

function IsInDatabase($Table, $Col, $Value, $IgnoreCase=False,
		      $XCol=False, $XVal='')
{
    /* True if $Value is nonblank and $Table contains $Value in $Col */
    /* Optionally, ignores records containing $XVal in $XCol */
    /* Escapes $Value and $XVal via prepared statement */
    /* $Table, $Col, $XCol must not be user input, since not escaped */
    if ($Value > '' or is_numeric($Value))
    {
	$Q = "SELECT * FROM $Table WHERE $Col ";
	$Q .= ($IgnoreCase and is_string($Value)) ? "ILIKE" : "=";
	$Q .= " $1";
	if ($XCol)
	{
	    $Q .= " AND $XCol != $2";
	    $Res = Query($Q, $Value, $XVal);
	} else
	    $Res = Query($Q, $Value);
	return (pg_num_rows($Res) > 0);
    } else return False;
}

function IsNonToolScript($ToolName)
{
    // Certain scripts aren't "tools" so they're not in the database:
    return (in_array($ToolName,
		     array(LoginScript,
			   "selectreport.php"))
	);
}

function IsReportPublic($ReportName)
{
    $Q = "SELECT * FROM report r, report_priv rp, privilege p";
    $Q .= " WHERE r.name= $1 AND rp.reportsn = r.reportsn";
    $Q .= " AND rp.privilegesn = p.privilegesn";
    $Q .= " AND p.privname = 'public'";
    $Ok = (pg_num_rows(Query($Q, $ReportName)) > 0);
    return $Ok;
}

function IsToolPublic($ToolName)
{
    // Certain scripts aren't "tools" so they're not in the database:
    $Ok = IsNonToolScript($ToolName);
    if (! $Ok)
    {
	$Q = "SELECT * FROM tool t, tool_priv tp, privilege p";
	$Q .= " WHERE t.name= $1 AND tp.toolsn = t.toolsn";
	$Q .= " AND tp.privilegesn = p.privilegesn";
	$Q .= " AND p.privname = 'public'";
	$Ok = (pg_num_rows(Query($Q, $ToolName)) > 0);
    }
    if (! $Ok and function_exists("PublicToolAccess"))
	$Ok = PublicToolAccess($ToolName);
    return $Ok;
}

function IsTrimmedUserKeyValid(&$LastName, &$FirstName, &$User)
{
    // Trims components of user search key, rejects null key
    $LastName = trim($LastName);
    $FirstName = trim($FirstName);
    $User = trim($User);
    return ($LastName.$FirstName.$User > "");
}

function IsValidEmailAddress($Email)
{
    return preg_match(RegExIEmail, $Email);
}

function IsValidPhoneNumber($Phone)
{
    return (preg_match(RegExIUsPhone, $Phone) or
	    preg_match(RegExINonUsPhone, $Phone));
}

function LoggedInUser()
{
    return (isset($_SESSION["UserRec"])
	    && array_key_exists("userid", $_SESSION["UserRec"])
	    && $_SESSION["UserRec"]["userid"] > "")
	? $_SESSION["UserRec"]["userid"] : False ;
}

function ParsedHumanNum($ValueString, $Binary=False)
{
    // Parses a human-friendly value to get sign, numeric value and
    // the implied precision (number of significant figures). If
    // parsing fails, $Value returns False.  $ValueString is of the
    // form $xx,xxx.ddK, where $ is the dollar sign and K is a K
    // (thousand) or M (million) suffix. $, K, commas, the decimal
    // point and digits d are all optional.  If $Binary is true, then
    // uses 1,024 and 1,048,576 for K and M
    $Kilo = $Binary ? 1024.0 : 1000.0;
    $Mega = $Binary ? 1048576.0 : 1000000.0;
    $ValueString = trim($ValueString);
    $FirstChar = substr($ValueString, 0, 1);
    if ($FirstChar == "+" or $FirstChar == "-")
	$ValueString = substr($ValueString, 1);
    $Suffix = strtoupper(substr($ValueString, -1));
    // Strip all but digits and decimal point:
    $S = preg_replace("/[^.[:digit:]]/", "", $ValueString);
    if (! is_numeric($S)) $Value = False;
    else
    {
	$SigFigs = (substr_count($S, ".") > 0)
	    ? ((abs($S) < 1)
	       ? strlen(ltrim($S, "0."))
	       : strlen(ltrim($S, "0")) - 1)
	    : strlen(rtrim($S, "0"));
	if ($SigFigs < 1) $SigFigs = 1;
	// Adjust for any K or M suffix
	switch ($Suffix)
	{
	case "K":
	    $Value = $S * $Kilo;
	    break;
	case "M":
	    $Value = $S * $Mega;
	    break;
	default:
	    $Value = $S;
	}
	$Sign = ($FirstChar == "-") ? "-" : "" ;
	$Value = ($FirstChar == "-") ? -$Value : $Value;
    }
    return array($Sign, $Value, $SigFigs);
}

function ParsedHumanNumValue($ValueString, $Binary=False)
{
    // return numeric value of a human-friendly value string formatted
    // as described in function ParsedHumanNum. False if can't parse.
    list($Sign, $Value, $SigFigs)
	= ParsedHumanNum($ValueString, $Binary);
    return $Value;
}

function Query($Sql)
{
    /* With a single argument, executes simple query $Sql; with
     * multiple arguments, executes prepared statement $Sql with the
     * remainder of the arguments as its parameters. Always use the
     * latter method for handling any user input, to prevent SQL
     * injection attacks. */
    global $Conn;
    if (func_num_args() == 1)

    {
	return pg_query($Conn, $Sql);
	/************************************************************
	To allow error reporting using pg_result_error, use this pair
	instead of plain pg_query:

	pg_send_query($Conn, $Sql);
	return pg_get_result($Conn);
	************************************************************/
    } else
    {
	$Args = func_get_args();
	$Params = array_slice($Args, 1);
	return pg_query_params($Conn, $Sql, $Params);
	/************************************************************
	To allow error reporting using pg_result_error, use this pair
	instead of plain pg_query_params:

	pg_send_query_params($Conn, $Sql, $Params);
	return pg_get_result($Conn);
	************************************************************/
    }
}

function QueryArray($Sql, $Params)
{
    /* Executes prepared statement $Sql with $Params an array of its
     * parameters, to prevent SQL injection attacks. */
    global $Conn;
    return pg_query_params($Conn, $Sql, $Params);
}

function ResInsertIfNew($Table, $Data)
{
    // Finds a record or inserts one, returns entire record as a
    // result pointer.
    // $Data is array (array ($Col, $Value) [, ...] )
    $QSelect = "SELECT * FROM $Table WHERE True";
    $C = "";
    $V = "";
    $Count = 0;
    $Params = array();
    foreach ($Data as $Set)
    {
	list($Col, $Value) = $Set;
	$Count++;
	$Params[] = $Value;
	// Construct selection query:
	$QSelect .= " AND $Col = $$Count";
	// Construct insertion query:
	$C .= ", $Col";	
	$V .= ", $$Count";
    }
    // Complete insertion query with selection query as filter:
    $Q = "INSERT INTO $Table (".substr($C, 2);
    $Q .= ") SELECT ".substr($V, 2)." WHERE NOT EXISTS ($QSelect)";
    // Apply insertion query to insert if needed:
    QueryArray($Q, $Params);
    // Apply selection query to get full record:
    $Res = QueryArray($QSelect, $Params);
    return ((pg_num_rows($Res) > 0) ? $Res : False);
}

function ResInsertOrUpdate($Table, $Keys, $Data)
{
    // Update existing (match keys) record, else insert. Return entire
    // record as result pointer.
    // $Keys is array (array ($Col, $Value) [, ...] )
    // $Data is array (array ($Col, $Value) [, ...] )
    // $Keys and $Data must suffice (with table defaults) to INSERT
    // Check if record exists (and prepare INSERT clause in case not)
    $ClauseKey = "True";
    $C = "";
    $V = "";
    $Count = 0;
    $Params = array();
    foreach ($Keys as $KeySet)
    {
	list($Col, $Value) = $KeySet;
	$Count++;
	$Params[] = $Value;
	$ClauseKey .= " AND $Col = $$Count";
	$C .= ", $Col";	
	$V .= ", $$Count";
    }
    $Q = "SELECT * FROM $Table WHERE $ClauseKey";
    $Sel = QueryArray($Q, $Params);
    if (!$Sel) return False; // Error flag
    $KeyParams = $Params;
    if (pg_num_rows($Sel) > 0)
    {
	// Record exists; update with new values
	$Q = "";
	foreach ($Data as $Set)
	{
	    list($Col, $Value) = $Set;
	    $Count++;
	    $Params[] = $Value;
	    $Q .= ", $Col = $$Count";
	}
	$Q = "UPDATE $Table SET".substr($Q, 1)." WHERE $ClauseKey";
    } else
    {
	// None exists; create new record
	foreach ($Data as $Set)
	{
	    list($Col, $Value) = $Set;
	    $Count++;
	    $Params[] = $Value;
	    $C .= ", $Col";	
	    $V .= ", $$Count";
	}
	$Q = "INSERT INTO $Table (".substr($C, 2);
	$Q .= ") VALUES (".substr($V, 2).")";
    }
    $Res = QueryArray($Q, $Params);
    return ($Res)
	? QueryArray("SELECT * FROM $Table WHERE $ClauseKey",
		     $KeyParams)
	: False;
}

function ResUserByNameLogin($LastName, $FirstName, $UserId)
{
    // Retrieve user records matching arguments
    // Note: * allows changing person schema
    $Q = "SELECT r.userid, r.passwd_hash, r.passwd_changed, p.*";
    $Q .= " FROM reguser r, person p";
    $Q .= " WHERE p.personsn = r.personsn";
    $Count = 0;
    $Params = array();
    if ($LastName > "")
    {
	$Count++;
	$Params[] = $LastName;
	$Q .= " AND p.lastname ILIKE $$Count || '%'";
    }
    if ($FirstName > "")
    {
	$Count++;
	$Params[] = $FirstName;
	$Q .= " AND p.firstname ILIKE $$Count || '%'";
    }
    if ($UserId > "")
    {
	$Count++;
	$Params[] = $UserId;
	$Q .= " AND r.userid = $$Count";
    }
    $Q .= " ORDER BY lastname, firstname";
    return QueryArray($Q, $Params);
}

function RoundToSigFigs($Value, $SigFigs)
{
    // Rounds $Value to $SigFigs significant figures
    if ($Value == 0) return 0;
    elseif ($Value < 0) return -RoundToSigFigs(-$Value, $SigFigs);
    else
    {
	$Scale =  pow(10, floor(log10($Value)) + 1 - $SigFigs);
	return round($Value / $Scale) * $Scale;
    }
}

function SerialNumber($Table, $Key, $Value)
{
    // Return serial number of tuple whose $Key column = $Value, where
    // serial number column is named "{$Table}sn"; the inverse of
    // function ColFromSerialNumber
    $Q = "SELECT {$Table}sn FROM $Table WHERE $Key = $1";
    $Res = Query($Q, $Value);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_result($Res, 0, "{$Table}sn") : False;
}

function SerialNumberCompound($Table, $Keys)
{
    // Return serial number of tuple whose $Keys match,
    // where serial number column is named "{$Table}sn"
    // and $Keys is array (array ($Col, $Value) [, ...] )
    $ClauseKey = "True";
    $Count = 0;
    $Params = array();
    foreach ($Keys as $KeySet)
    {
	list($Col, $Value) = $KeySet;
	$Count++;
	$Params[] = $Value;
	$ClauseKey .= " AND $Col = $$Count";	
    }
    $Q = "SELECT {$Table}sn FROM $Table WHERE $ClauseKey";
    $Res = QueryArray($Q, $Params);
    return (pg_num_rows($Res) > 0)
	? pg_fetch_result($Res, 0, "{$Table}sn") : False;
}

function StateStrip($StateArray)
{
    // Removes unneeded (possibly harmful) elements
    $S = array();
    foreach ($StateArray as $Key => $Value)
	switch ($Key)
	{
	case "PHPSESSID":
	    break;
	default:
	    $S[$Key] = $Value;
	}
    return $S;
}

function StrHumanNum($Value, $SigFigs, $Binary=False)
{
    // Converts $Value to "human-readable" string ending in "K" or "M"
    // with comma-separated thousands and rounded to $SigFigs
    // significant figures
    // If $Binary is true, then uses 1,024 and 1,048,576 for K and M
    $Kilo = $Binary ? 1024.0 : 1000.0;
    $Mega = $Binary ? 1048576.0 : 1000000.0;
    $Sign = ($Value < 0) ? "-" : "" ;
    $Value = RoundToSigFigs(abs($Value), $SigFigs);
    if ($Value >= $Mega)
    {
	$Value = $Value / $Mega;
	$Suffix = "M";
    } elseif ($Value >= $Kilo)
    {
	$Value = $Value / $Kilo;
	$Suffix = "K";
    } else
    {
	$Rem = $Value % 1.0;
	$Suffix = "";
    }
    if ($Value == 0) $Dec = $SigFigs - 1;
    else
    {
	$Log = log10($Value);
	$Ceil = ceil($Log);
	$Dec = (abs($Ceil - $Log) < 0.0001)
	    ? $SigFigs - $Ceil - 1 : $SigFigs - $Ceil;
    }
    if ($Dec > 0) return $Sign.number_format($Value, $Dec).$Suffix;
    else return $Sign.number_format($Value).$Suffix;
}

function ThisScript ()
{
    return basename($_SERVER["SCRIPT_NAME"]);
}

function TimeString($TimeStampString, $Format=DateTimeFormatFull,
		    $FailureValue=NullTimeFlag)
{
    // Converts $TimeStampString to conform to $Format; if not a
    // recognizable timestamp, returns $FailureValue
    $TimeStamp = strtotime($TimeStampString);
    return ($TimeStamp !== InvalidDateTime)
	? date($Format, $TimeStamp) : $FailureValue;
}

function UnlinkIfExists($FilePath)
{
    // Avoid scary error messages if reloading a page repeats a delete
    // action
    if (file_exists($FilePath)) unlink($FilePath);
    return;
}

function UserHasMetaPrivilege($UserId, $Privilege)
{
    $Q = "SELECT * FROM user_metapriv up, reguser r, privilege p";
    $Q .= " WHERE userid = $1 AND privname = $2";
    $Q .= " AND r.regusersn = up.regusersn";
    $Q .= " AND p.privilegesn = up.privilegesn";
    return (pg_num_rows(Query($Q, $UserId, $Privilege)) > 0);
}

function UserHasPrivilege($UserId, $Privilege)
{
    $Q = "SELECT * FROM user_priv up, reguser r, privilege p";
    $Q .= " WHERE userid = $1 AND privname = $2";
    $Q .= " AND r.regusersn = up.regusersn";
    $Q .= " AND p.privilegesn = up.privilegesn";
    return (pg_num_rows(Query($Q, $UserId, $Privilege)) > 0);
}

function UserMayUseTool($UserId, $Tool)
{
    // Assumes $Tool is either private or for any logged-in user.
    // $Tool may be name of a report rather than a tool
    if ($UserId == "") return False;
    $ReguserSn = SerialNumber("reguser", "userid", $UserId);
    $Q = "SELECT * FROM tool t, tool_priv tp, privilege p,";
    $Q .= " user_priv up";
    $Q .= " WHERE t.name = $1 AND tp.toolsn = t.toolsn";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    $Q .= " AND (";
    // If tool privilege is anylogin, allow
    $Q .= "  p.privname = 'anylogin'";
    // If user has tool privilege, allow
    $Q .= "  OR (p.privilegesn = up.privilegesn";
    $Q .= "   AND up.regusersn = $ReguserSn)";
    // If tool privilege is grant, allow if user has any metaprivilege
    $Q .= "  OR (p.privname = 'grant' AND EXISTS";
    $Q .= "   (SELECT * FROM user_metapriv";
    $Q .= "    WHERE regusersn = $ReguserSn))";
    $Q .= ")";
    $Ok = (pg_num_rows(Query($Q, $Tool)) > 0);
    // Check for local exceptions allowing access:
    if (! $Ok and function_exists("SpecialToolAccess"))
	$Ok = SpecialToolAccess($UserId, $Tool);
    // Check whether tool is a private report and user may run it
    if (! $Ok)
    {
	$Q = "SELECT * FROM report r, report_priv rp, privilege p,";
	$Q .= " user_priv up";
	$Q .= " WHERE r.name = $1 AND rp.reportsn = r.reportsn";
	$Q .= " AND rp.privilegesn = p.privilegesn";
	$Q .= " AND (";
	// If report privilege is anylogin, allow
	$Q .= "  p.privname = 'anylogin'";
	// If user has report privilege, allow
	$Q .= "  OR (p.privilegesn = up.privilegesn";
	$Q .= "   AND up.regusersn = $ReguserSn))";
	$Ok = (pg_num_rows(Query($Q, $Tool)) > 0);
    }
    return $Ok;
}

//********************************************************************
// Useful Constants

define ("SecsPerDay", 86400);
date_default_timezone_set(trim(shell_exec("cat /etc/timezone")));

// Identify working environment from path

$RelPath = "/".substr($_SERVER["SCRIPT_NAME"], strlen($ServiceRoot));
$Pos = strrpos($RelPath,"/gk/");
// Might be included from within gk/, so check:
if ($Pos === False) $Pos = strrpos($RelPath,"/");
$Path = substr($RelPath, 0, $Pos + 1);
SetDefaultFormMethodAndTimeout($Path);
$SystemTitle = $SysEnv[$Path]["SysTitle"];
$DbUser = $SysEnv[$Path]["User"];
$Database = $SysEnv[$Path]["Db"];
$SessionTimeoutSecs = SessionTimeout * 60;

// Connect to appropriate Postgresql database
$Conn = pg_connect("user=$DbUser dbname=$Database");

// Alternative: persistent connection:
//$Conn = pg_pconnect("user=$DbUser  dbname=$Database");
 
//see if our connection was successful
if (!$Conn)
{
    // Connection failed - exit the page with an error
    $T = "Couldn't connect to database $Database as user $DbUser";
    DisplayErrorPage($T);
    exit;
}

// Set up/restore session state
session_start();

// Any form in a public tool should set $LoginFlag if a user is logged
// in, so we know to prompt for a login after a timeout; for a
// non-public tool, we always prompt after a timeout.
$TimedOut = ((isset($_REQUEST["LoginFlag"])
	      or
	      ! (IsToolPublic(ThisScript())
		 or
		 IsReportPublic(ThisScript())))
	     and
	     (! isset($_SESSION["Watchdog"])
	      or
	      ((time() - $_SESSION["Watchdog"])
	       > $SessionTimeoutSecs)));

if ($TimedOut and ! IsNonToolScript(ThisScript()))
{
    $StateAtTimeOut = array(
	"_POST" => StateStrip($_POST),
	"_GET" => StateStrip($_GET),
	"_REQUEST" => StateStrip($_REQUEST))
	+ array ("CalledScript" => ThisScript());
    $SavedStateAtTimeOut = htmlentities(serialize($StateAtTimeOut));
    ClearSession();
    $_SESSION["Watchdog"] = time();
    include (LoginScript);
    exit;
}

if ($TimedOut or isset($_REQUEST["LoggedOut"])
    or isset($_REQUEST["CancelLogin"]))
{
    // Note: ClearSession leaves variables set here
    ClearSession();
    if (ThisScript() != LoginScript)
    {
	include (LoginScript);
	exit;
    }
}

BounceStrangers(LoggedInUser(), ThisScript());

// We're OK to proceed
$_SESSION["Watchdog"] = time();	// Watchdog timer for session timout
// User record, associative array:
if (! isset($_SESSION["UserRec"])) $_SESSION["UserRec"] = array();

?>
