<?php
/*********************************************************************
gk_filesys.php

Filesystem Access Definitions for PHP Scripts

Do not edit; put application-specific definitions in parent folder's
lib_filesys.php
______________________________________________________________________
Copyright 2021 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
// Specification that must match function GoodFileName
$T = "an alphanumeric followed by alphanumerics, underscores or";
$T .= " hyphens";
define ("FileNameSpec", $T);

function FileNameBase($File)
{
    /* Return the base name of a file (strip period and extension) */
    $DotPos = strrpos($File, ".");
    return ($DotPos !== False) ? substr($File, 0, $DotPos) : $File;
}

function FileNameExtension($File)
{
    /* Return the extension of a file (from period to end) */
    $Ext = strrchr($File, ".");
    if ($Ext === False or $Ext == ".") $Ext = "[ no extension ]";
    else $Ext = substr($Ext, 1);
    return $Ext;
}

function GoodFileName($Name)
{
    // True if $Name is a valid file name (matches FileNameSpec)
    return preg_match("/^[[:alnum:]][[:alnum:]_-]*$/", $Name);
}

function DownloadFile($FullPath, $MimeType=False)
{
    // streams file to client
    $FileName = basename($FullPath);
    if (! $MimeType)
    {
        $Finfo = finfo_open(FILEINFO_MIME_TYPE);
        $MimeType = finfo_file($Finfo, $FullPath);
        finfo_close($Finfo);
    }
    header("Content-type: $MimeType");
    header("Content-Disposition: attachment; filename=$FileName");
    readfile($FullPath);
}

function Invisible($FileName)
{
    // True if named file is to be ignored (in directory search)
    return ($FileName[0] == "." or $FileName == "CVS") ;
}

function Subdirectories($Directory)
{
    // All subdirectories in specified directory, sorted
    // Omits hidden files, "index.html", "index.php"
    $Subdirectories = array();
    $Dir = dir($Directory);
    while (False !== ($File = $Dir->read()))
    {
        if ( ! Invisible($File) and is_dir($Directory.'/'.$File) )
        {
            $Subdirectories[] = $File;
        }
    }
    $Dir->close();
    sort($Subdirectories);
    return $Subdirectories;
}

?>
