<?php
/*********************************************************************
gk_grant.php

Grant and revoke privileges to users

Designed to be accessed via an include/require statement in the
otherwise empty ../grant.php script
______________________________________________________________________
Copyright 2013 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");

function StrSearchResults($LastName, $FirstName, $UserId,
			  $ActiveUserId)
{
    if ( ! IsTrimmedUserKeyValid($LastName, $FirstName, $UserId))
    {
	$S = StrWarning("You must enter a search key (partial ok).");
	$S .= StrUserSearchForm();
    } else
    {
	$Res = ResUserByNameLogin($LastName, $FirstName, $UserId);
	$Hits = pg_num_rows($Res);
	if (! $Res or $Hits < 1)
	    $S = StrUserNotFound($LastName, $FirstName, $UserId);
	elseif ($Hits > 1)
	    $S = StrUsersMultiple($LastName, $FirstName, $UserId,
				  $Res, $Hits);
	else $S = StrUserPrivilegeScreen($Res, $ActiveUserId);
    }
    return $S;
}

function StrUpdateResults($UserId, $NumRows, $ActiveUserId, $Meta)
{
    if ( ! $Meta or UserHasPrivilege($ActiveUserId, "metameta"))
	UpdateUserPrivileges($UserId, $NumRows, $ActiveUserId, $Meta);
    $T = "Updated ".($Meta ? "meta" : "")."privileges for $UserId";
    $T .= " where authorized.";
    $S = StrWarning($T, NoticeColor);
    $Res = ResUserByNameLogin("", "", $UserId);
    $S .= StrUserPrivilegeScreen($Res, $ActiveUserId);
    return $S;
}

function StrUserPrivilegeForm($Grantee, $UserId, $Prv, $Meta=False)
{
    $NumRows = pg_num_rows($Prv);
    $T = $Meta ? "Metaprivileges" : "Privileges";
    $T .= " granted to $Grantee ($UserId)";
    $S = StrSubTitle($T);
    $S .= StrFormBeginRecurse();
    $S .= StrTableBegin("1", TableColor);
    $S .= StrRowBegin();
    $S .= StrHeaderCell("Grant");
    $S .= StrHeaderCell("Name");
    $S .= StrHeaderCell("Description");
    $S .= StrRowEnd();
    $S .= StrUserPrivilegeFormRows($UserId, $NumRows, $Prv, $Meta);
    $S .= StrTableEnd();
    $S .= "<table border=\"0\" cellspacing=\"10\" cellpadding=\"4\">";
    $S .= "\n".StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("UpdatePriv", "Update"));
    $S .= StrCell(HtmlHidden("Meta", $Meta ? 1 : 0));
    $S .= StrCell(HtmlResetButton());
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= HtmlHidden("NumRows", $NumRows);
    $S .= HtmlHidden("UserId", $UserId);
    $S .= StrFormEnd();
    return $S;
}

function StrUserPrivilegeFormRows($UserId, $NumRows, $Prv,
				  $Meta=False)
{
    for ($Row = 0 ; $Row < $NumRows ; $Row ++)
    {
	// For each priv, check box if target user has it
	list($Name, $Description) = pg_fetch_array($Prv, $Row);
	if ($Meta) $Checked = UserHasMetaPrivilege($UserId, $Name);
	else $Checked = UserHasPrivilege($UserId, $Name);
	$S .= StrRowBegin();
	$S .= StrCell(HtmlCheckbox("Grant$Row", $Checked));
	$S .= StrCell($Name);
	$T = "<label for=\"Grant$Row\">$Description</label>";
	$S .= StrCell(HtmlHidden("Priv$Row", $Name).$T);
	$S .= StrRowEnd();
    }
    return $S;
}

function StrUserPrivilegeForms($Res, $ActiveUserId)
{
    $UserId = pg_fetch_result($Res, $Row, "userid");
    $LastName = pg_fetch_result($Res, $Row, "lastname");
    $FirstName = pg_fetch_result($Res, $Row, "firstname");
    $ActiveUserSn = SerialNumber("reguser", "userid", $ActiveUserId);
    // Get list of all privileges that active user can grant
    $Q = "SELECT p.privname, p.description";
    $Q .= " FROM user_metapriv m, privilege p";
    $Q .= " WHERE m.regusersn = $1";
    $Q .= " AND p.privilegesn = m.privilegesn";
    $Q .= " AND NOT EXISTS";
    $Q .= " (SELECT * FROM nogrant WHERE privilegesn = p.privilegesn)";
    $Q .= " ORDER BY p.privname";
    $Prv = Query($Q, $ActiveUserSn);
    $S = StrUserPrivilegeForm("$FirstName $LastName", $UserId, $Prv,
			      False);
    if (UserHasPrivilege($ActiveUserId, "metameta"))
    {
	// Get list of all privileges
	$Q = "SELECT privname, description FROM privilege p";
	$Q .= " WHERE NOT EXISTS";
	$Q .= " (SELECT * FROM nogrant";
	$Q .= " WHERE privilegesn = p.privilegesn)";
	$Q .= " ORDER BY privname";
	$Prv = Query($Q);
	$S .= StrUserPrivilegeForm("$FirstName $LastName", $UserId,
				   $Prv, True);
    }
    return $S;
}

function StrUserPrivilegeScreen($Res, $ActiveUserId)
{
    $S = StrUserPrivilegeForms($Res, $ActiveUserId);
    $S .= "<h3 align=\"center\">Select another user:</h3>\n";
    $S .= StrUserSearchForm();
    return $S;
}

function UpdateUserPrivileges($UserId, $NumRows, $ActiveUserId,
			      $Meta=False)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    for ($Row = 0 ; $Row < $NumRows ; $Row ++)
	UpdateUserPrivilege($UserId, $FormVars["Priv$Row"],
			    $FormVars["Grant$Row"], $ActiveUserId,
			    $Meta);
}

function UpdateUserPrivilege($UserId, $Priv, $Grant, $ActiveUserId,
			     $Meta=False)
{
    $ActiveUserSn = SerialNumber("reguser", "userid", $ActiveUserId);
    $UserSn = SerialNumber("reguser", "userid", $UserId);
    $PrivSn = SerialNumber("privilege", "privname", $Priv);
    if ($Meta)
    {
	$TargetTable = "user_metapriv";
	$Auth = "";
    } else
    {
	$TargetTable = "user_priv";
	$Auth = " AND EXISTS";
	$Auth .= " (SELECT * FROM user_metapriv";
	$Auth .= " WHERE regusersn = $ActiveUserSn";
	$Auth .= " AND privilegesn = $PrivSn)";
    }
    if ($Grant)
    {
	$Q = "INSERT INTO $TargetTable(regusersn, privilegesn)";
	$Q .= " SELECT $UserSn, $PrivSn";
	$Q .= " WHERE NOT EXISTS";
	$Q .= " (SELECT * FROM $TargetTable";
	$Q .= " WHERE regusersn = $UserSn AND privilegesn = $PrivSn)";
	$Q .= $Auth;
    } else
    {
	$Q = "DELETE FROM $TargetTable";
	$Q .= " WHERE regusersn = $UserSn AND privilegesn = $PrivSn";
	$Q .= $Auth;
    }
    Query($Q);
    return;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "FirstName",
    "LastName",
    "Meta",
    "NumRows",
    "SearchForUser",
    "UserId"
    );
$GetVarList = array(
    "SearchForUser",
    "UserId"
    );
$PostVarList = array(
    
    );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$T = "Manage Users' Privileges";
$S = HeadAndTitle("$SystemTitle $T");
$S .= PageTitleWithButton($T, BtnMainMenu());
$S .= "<div align=\"center\">\n";

if (isset($SearchForUser))
    $S .= StrSearchResults($LastName, $FirstName, $UserId,
			   LoggedInUser());
elseif (isset($FormVars["UpdatePriv"]))
    $S .= StrUpdateResults($UserId, $NumRows, LoggedInUser(), $Meta);
else
{
    $S .= "<h3 align=\"center\">Select a User</h3>\n";
    $S .= StrUserSearchForm();
}

$S .= "</div></body></html>\n";
echo $S;

?>
