<?php
/*********************************************************************
gk_layout.php

Layout Definitions for PHP Scripts

Do not edit; put application-specific definitions in parent folder's
lib_layout.php
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

Notes: Certain functions have alternate forms; the preferred form uses
HTML 4 tags and other modern standards that that may not work with
older browsers. The Boolean constant WebStandards (in
local/loc_env.php) determines which form is used.

*********************************************************************/
$T = "color: ".WarningColor."; font-weight: bold;";
$T .= " background-color: ".TableColor;
define ("StyleButtonDanger", $T);

function Border($Edges="", $Color="", $Width="", $Style="")
{
    // Returns a style string with borders at top, right, bottom
    // and/or left, respectively. More compact than repeated calls to
    // CssBorder.
    // $Edges = { "T" | "R" | "B" | "L" } or empty => all borders
    $Attrib = array("t" => "top",    "r" => "right",
                    "b" => "bottom", "l" => "left");
    $S = "";
    if ($Edges > "")
    {
        foreach ($Attrib as $Code => $Value)
            if (stristr($Edges, $Code))
                $S .= "; ".CssBorder($Value, $Color, $Width, $Style);
        $S = substr($S, 2);
    } else $S = CssBorder("", $Color, $Width, $Style);
    return $S;
}

function Btn($Text, $Url, $Color=ButtonColor)
{
    return array
        (
            "Text" 	=> $Text,
            "Url"  	=> $Url,
            "Color"	=> $Color
        );
}

function BtnClose($Color=CloseButtonColor)
{
    return Btn("Close", "javascript:self.close()", $Color);
}

function BtnDone($Args="")
{
    return Btn("Done", ThisScript().$Args, ButtonColor2);
}

function BtnHome($Color=ButtonColor)
{
    return Btn("Home", LoginScript, $Color);
}

function BtnLogin($Color=ButtonColor)
{
    return Btn("Log In", LoginScript."?LogIn=True", $Color);
}

// note to self: this should be changed to a constant in local/loc_env

function BtnLogout($Color=ButtonColor)
{
    return Btn("Log Out", LoginScript."?LoggedOut=True", $Color);
}

function BtnMainMenu($Color=ButtonColor)
{
    return Btn("Main Menu", "index.php", $Color);
}

function BtnMyAccount($Color=ButtonColor)
{
    return Btn("My Account", LoginScript."?MngAcct=True", $Color);
}

function BtnStartOver($Color=ButtonColor)
{
    return Btn("Start Over", ThisScript(), $Color);
}

function CssBorder($Where="", $Color="", $Width="", $Style="")
{
    // Return a CSS style string to put a border on a cell
    // $Where = {top | right | bottom | left } or empty => all borders
    // $Color = any legal color specification, e.g., "blue", "#ffffd0"
    // $Width = {thin | medium | thick | <length>}
    // $Style = {none | dotted | dashed | solid | double | groove |
    // ridge | inset | outset}
    $S = "border".(($Where > "") ? "-$Where: " : ": ");
    if ($Width > "") $S .= "$Width ";
    $S .= ($Style > "") ? "$Style " : "solid ";
    $S .= ($Color > "") ? $Color : TextColor;
    return $S;
}

function FittedTextArea($Name, $Cols, $MaxRows, $Value="", $Style="")
{
    // Shrinks to shorter initial text value
    $Size = strlen($Value);
    $EstRows = ceil($Size/$Cols);
    // Adjust if embedded newlines
    $NewLines = substr_count($Value, "\n");
    if ($NewLines > 0) $EstRows += ceil($NewLines / 2.0) + 1;
    $Rows = ($EstRows < $MaxRows) ? $EstRows : $MaxRows;
    return HtmlTextArea($Name, $Cols, $Rows, $Value, $Style);
}

function FormEditButton($ButtonName, $Size=16, $Title="Edit",
                        $Extra="", $Action="")
{
    /*
      Return html for an edit$Size.png image button surrounded by a
      form; use $Extra for additional form content like hidden
      variables.
      To handle, check whether isset "{$ButtonName}_x"
    */
    // Three sizes available: 16px, 24px, 32px. Choose closest:
    $Size = min(max(round($Size/8, 0), 2), 4) * 8;
    $Src = "gk/images/edit".$Size.".png";

    return FormImageButton($ButtonName, $Src, $Title, $Extra, $Action);
}

function FormImageButton($ButtonName, $Src, $Title="", $Extra="",
                         $Action="", $Value="", $Style="")
{
    // Return html for an image button surrounded by a form; use
    // $Extra for additional form content like hidden variables.
    // To handle, check whether isset "{$ButtonName}_x"
    if ($Action == "") $Action = ThisScript();
    $S = StrFormBegin($Action);
    $S .= HtmlImageButton($ButtonName, $Src, $Title, $Value, $Style);
    $S .= $Extra.StrFormEnd();
    return $S;
}

function FormLine($Tip, $Label, $Content, $Align="right")
{
    // Return a formatted table row used in forms
    $S = StrRowBegin();
    $S .= "<td align=\"$Align\" title=\"$Tip\">";
    if (strtolower($Align) == "right") $Label .= " : ";
    $S .= Strong($Label)."</td>\n";
    $S .= StrCell($Content);
    $S .= StrRowEnd();
    return $S;
}

function HeadAndTitle($TitleText="", $ExtraTags="", $BodyStyle="",
                      $LinkStyle="")
{
    /*****************************************************************
      Return complete <head> tag and <body> start tag.
      Optional $ExtraTags is for stuff like styles, javascript
      definitions (e.g., in gk_jscalendar.php), <meta> tags, etc.
      Optional $LinkStyle is of the form:
      A:link { color: $LinkColor }
      A:visited { color: $VLinkColor }
      A:active { color: $ALinkColor }
    *****************************************************************/
    global $SystemTitle;
    if ($TitleText == "") $TitleText = $SystemTitle;
    $S = "<html><head>\n";
    $S .= "<META HTTP-EQUIV=\"expires\"";
    $S .= " CONTENT=\"Mon, 01 Jan 2000 01:00:00 GMT\">\n";
    if ($BodyStyle == "" and defined("DefaultBodyStyle"))
        $BodyStyle = DefaultBodyStyle;
    if ($BodyStyle != "")
    {
        $S .= "<style type=\"text/css\">body { $BodyStyle }";
        $S .= "</style>\n";
    }
    if ($LinkStyle != "")
        $S .= "<style type=\"text/css\">\n$LinkStyle</style>\n";
    if ($ExtraTags != "") $S .= "$ExtraTags\n";
    $S .= "<title>$TitleText</title>\n";
    $S .= "</head><body>\n";
    return $S;
}

function HintSpan($Hint, $Content)
{
    return "<span title=\"$Hint\">$Content</span>\n";
}

function HtmlCheckbox($Name, $Checked=False, $Attribs=False)
{
    $S = "<input type=\"checkbox\" name=\"$Name\" id=\"$Name\"";
    if ($Checked) $S .= " checked";
    if ($Attribs > "") $S .= " $Attribs";
    $S .= ">";
    return $S;
}

function HtmlCleanText($Text)
{
    /* Ensure cleanly-renderable and database-safe text by converting
     * encoded characters to HTML entities. This is a wrapper for PHP
     * function htmlentities, whose default behavior will encode
     * existing html entities ($double_encode = True). */
    return htmlentities ($Text, ENT_COMPAT | ENT_HTML401, "UTF-8",
                         False);
}

function HtmlCleanTitle($Text)
{
    /* Ensure a cleanly-renderable title attribute (mouseover) by
     * stripping any HTML tags and then encoding any characters with
     * special significance in HTML. This is a wrapper to supply
     * currently-needed arguments for PHP function htmlspecialchars,
     * whose default behavior can change between PHP releases */
    return htmlspecialchars(strip_tags($Text),
                            ENT_COMPAT | ENT_HTML401, "UTF-8");
}

function HtmlColor($Color, $Text)
{
    return SpanStyle($Text, "color: $Color");
}

function HtmlGoButton($ButtonName, $Value="", $Style="")
{
    return HtmlImageButton($ButtonName, "gk/images/check.png",
                           "click to confirm selection", $Value,
                           $Style);
}

function HtmlHidden($Name, $Value)
{
    $S = "<input type=\"hidden\" name=\"$Name\" id=\"$Name\"";
    $S .= " value=\"$Value\">\n";
    return $S;
}

function HtmlImage($Src, $Alt="", $Height="", $Width="", $Border="0",
                   $Title="", $UseMap="")
{
    $Title = HtmlCleanTitle($Title);
    $Alt = HtmlCleanTitle($Alt);
    $S = "<img src=\"$Src\" border=\"$Border\"";
    if ($Alt > "") $S .= " alt=\"$Alt\"";
    if ($Height > "") $S .= " height=\"$Height\"";
    if ($Width > "") $S .= " width=\"$Width\"";
    if ($Title > "") $S .= " title=\"$Title\"";
    elseif ($Alt > "") $S .= " title=\"$Alt\"";
    if ($UseMap > "") $S .= " usemap=\"$UseMap\"";
    $S .= " />";
    return $S;
}

function HtmlImageButton($ButtonName, $Src, $Title="", $Value="",
                         $Style="")
{
    if ($Title == "") $Title = $ButtonName;
    $Title = HtmlCleanTitle($Title);
    $S = "<input type=\"image\" src=\"$Src\"";
    $S .= " title=\"$Title\" alt=\"$Title\"";
    $S .= " name=\"$ButtonName\"";
    if ($Style > "") $S .= " style=\"$Style\"";
    if ($Value == "") $Value = $ButtonName;
    $S .= " value=\"".strip_tags($Value)."\">";
    return $S;
}

function HtmlInputPostUpload($Name, $Size, $Value="")
{
    // Display a file selection widget for uploading files
    // (see function StrFormBeginPostUpload)
    $S = "<input type=\"file\" name=\"$Name\" size=\"$Size\"";
    // Consider sanitizing $Value, e.g., with function HtmlCleanTitle
    if ($Value != "") $S .= " value=\"$Value\"";
    $S .= ">";
    return $S;
}

function HtmlPara($Text, $Align="")
{
    $S = "<p";
    if ($Align > "") $S .= " align=\"$Align\"";
    $S .= ">$Text</p>\n";
    return $S;
}

function HtmlRadioButton($Name, $Value, $Checked=False,
                         $Attribs=False)
{
    $S = "<input type=\"radio\" name=\"$Name\" value=\"$Value\"";
    $S .= " id=\"$Name-$Value\"";
    if ($Checked) $S .= " checked";
    if ($Attribs > "") $S .= " $Attribs";
    $S .= ">";
    return $S;
}

function HtmlResetButton($Label="")
{
    if (WebStandards)
    {
        $S = "<button type=\"reset\">";
        if ($Label == "") $Label = "Reset";
        $S .= "$Label</button>";
    } else
    {
        $S = "<input type=\"reset\"";
        if ($Label != "") $S .= " value=\"".strip_tags($Label)."\"";
        $S .= " />";
    }
    return $S;
}

function HtmlSelectBegin($Name, $Attribs="")
{
    $S = "<select name=\"$Name\"";
    if ($Attribs > "") $S .= " $Attribs";
    return "$S>\n";
}

function HtmlSelectEnd()
{
    return "</select>\n";
}

function HtmlSelectOption($Value, $Label, $Selected=False, $Hint="",
                          $Style="")
{
    $Hint = HtmlCleanTitle($Hint);
    if ($Hint > "") $Hint = " title=\"$Hint\"";
    if ($Style > "") $Style = " style=\"$Style\"";
    $S = "<option value=\"$Value\"";
    if ($Selected) $S .= " selected=\"selected\"";
    $S .= "$Hint $Style>$Label</option>\n";
    return $S;
}

function HtmlSubmitButton($ButtonName, $Label="", $Value="",
                          $Style="")
{
    if ($Label == "") $Label = $ButtonName;
    if (WebStandards)
    {
        $S = "<button type=\"submit\" name=\"$ButtonName\"";
        if ($Value == "") $Value = $ButtonName;
        $S .= " value=\"$Value\"";
        if ($Style > "") $S .= " style=\"$Style\"";
        $S .= ">$Label</button>";
    } else
    {
        $S = "<input type=\"submit\" name=\"$ButtonName\"";
        if ($Style > "") $S .= " style=\"$Style\"";
        $S .= " value=\"".strip_tags($Label)."\">";
    }
    return $S;
}

function HtmlTextArea($Name, $Cols, $Rows, $Value="", $Style="")
{
    $S = "<textarea name=\"$Name\" cols=\"$Cols\"";
    $S .= " rows=\"$Rows\"";
    if ($Style > "") $S .= " style=\"$Style\"";
    $S .= ">";
    if ($Value != "") $S .= $Value;
    $S .= "</textarea>";
    return $S;
}

function HtmlTextInput($Name, $Size, $Value="", $MaxLength="",
                       $Style="")
{
    $Value = HtmlCleanTitle($Value);
    $S = "<input type=\"text\" name=\"$Name\" size=\"$Size\"";
    if ($MaxLength != "") $S .= " maxlength=\"$MaxLength\"";
    if ($Value != "") $S .= " value=\"$Value\"";
    if ($Style > "") $S .= " style=\"$Style\"";
    $S .= ">";
    return $S;
}

function PageTitle($Title, $Size=2)
{
    $S = PageTitleBegin();
    $S .= "&nbsp;";
    $S .= PageTitleEnd($Title, $Size);
    return $S;
}

function PageTitleBegin()
{
    $S = StrTableBegin("0", "", "", "width: 100%");
    $S .= StrRowBegin();
    $S .= "<td style=\"width: 20%\">";
    return $S;
}

function PageTitleEnd($Title, $Size=2)
{
    $S = "</td>\n";
    $S .= "<td style=\"text-align: center\">";
    $S .= "<h$Size>$Title</h$Size></td>\n";
    $S .= "<td style=\"width: 20%; text-align: right;";
    $S .= " font-size: smaller\">";
    $UserRec = $_SESSION["UserRec"];
    if (isset($_SESSION["UserRec"])
	    && array_key_exists("userid", $_SESSION["UserRec"])
	    && $_SESSION["UserRec"]["userid"] > "")
    {
        $S .= "Logged in as <a href=\"".LoginScript."?MngAcct=True\">";
        $S .= $UserRec["userid"]."</a>. <a href=\"";
        $S .= LoginScript."?LoggedOut=True\">Log out</a>";
    } else $S .= "<a href=\"".LoginScript."?LogIn=True\">Log in</a>";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function PageTitleWithButton($Title, $Button, $Size=2)
{
    $S = PageTitleBegin();
    $S .= StrButton($Button);
    $S .= PageTitleEnd($Title, $Size);
    return $S;
}

function ProgressBar($Fraction, $FrameWidth=500)
{
    /* Uses HTML5 <progress> tag with fallback to old
       ProgressBarFallBack for IE and Safari */
    $PerCent = round($Fraction * 100);
    $S = "<progress value=\"$PerCent\" max=\"100\"";
    $S .= " title=\"$PerCent%\" style=\"width: {$FrameWidth}px\">";
    $S .= ProgressBarFallBack($Fraction, $FrameWidth);
    $S .= "</progress>\n";
    return $S;
}

function ProgressBarFallBack($Fraction, $FrameWidth=500,
                             $FrameHeight=40)
{
    /* Construct a progress bar for browsers that don't support HTML5
     * progress tag */
    $Width = round($FrameWidth * $Fraction);
    $PerCent = round($Fraction * 100);
    $S = "<table border=\"2\" bgcolor=\"#ffffff\" cellspacing=\"0\"";
    $S .= " cellpadding=\"0\" width=\"".($FrameWidth + 5)."\">\n";
    $S .= "<tbody>\n";
    $S .= StrRowBegin();
    $T = "<img src=\"".ProgressBarImage."\" height=\"40\"";
    $T .= " width=\"$Width\" alt=\"$PerCent%\" title=\"$PerCent%\"/>";
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function Say($Text)
{
    // Use this function only for diagnostic purposes, not in
    // production code
    echo "<p>$Text</p>\n";
    return;
}

function SayArray($Array, $Label=False)
{
    // Use this function only for diagnostic purposes, not in
    // production code
    if ($Label) $Label = "<strong>$Label:</strong>";
    echo "<pre>$Label\n";
    print_r($Array);
    echo "</pre>\n";
    return;
}

function SimpleText($Text, $Style="")
{
    // Replaces double end-of-lines with paragraphs, single
    // end-of-lines with breaks. Useful for rendering preformatted
    // text in html.
    $P = ($Style == "") ? "p" : "p style=\"$Style\"";
    if ($Text > "")
    {
        $Trans = array("\n\n" => "</p>\n<$P>", "\n" => "<br>\n");
        $Text = strtr($Text, $Trans);
    }
    return "<$P>$Text</p>\n";
}

function SpanStyle($Text, $Style)
{
    return "<span style=\"$Style\">$Text</span>";
}

function StrBoxBegin($Border="1", $Align="left", $Color="", $Width="")
{
    $S = StrTableBegin($Border, $Color, $Width);
    $S .= "<tr><td align=\"$Align\">\n";
    return $S;
}

function StrBoxEnd()
{
    return "</td></tr></tbody></table>\n";
}

function StrBullet($Button)
{
    // $Button is array (Text, Url, Color)
    $S = "<li>";
    $S .= "<a href=\"".$Button["Url"]."\">".$Button["Text"]."</a>";
    $S .= "</li>\n";
    return $S;
}

function StrButton($Button, $Width="", $Title="")
{
    // $Button is array (Text, Url, Color)
    // Only text is live link
    $Title = HtmlCleanTitle($Title);
    $S = StrTableBegin("2", $Button["Color"], $Width);
    $S .= "<tr align=\"center\">\n";
    $T = "<a href=\"".$Button["Url"]."\"";
    if ($Title > "") $T .= " title=\"$Title\"";
    $T .= ">".$Button["Text"]."</a>";
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function StrButtonHot($Button, $Width="", $Title="")
{
    // $Button is array (Text, Url, Color)
    // Entire button (one-cell table) is live link
    $Title = HtmlCleanTitle($Title);
    $S = "<a href=\"".$Button["Url"];
    if ($Title > "") $S .= " title=\"$Title\">";
    $S .= StrTableBegin("2", $Button["Color"], $Width);
    $S .= "<tr align=\"center\">\n";
    $S .= StrCell($Button["Text"]);
    $S .= StrRowEnd();
    $S .= StrTableEnd()."</a>\n";
    return $S;
}

function StrButtonsHorz($Buttons, $Width="")
{
    // $Buttons is array (Button1, Button2, ...)
    $S = StrTableBegin("0", "", $Width);
    $S .= "<tr align=\"center\">\n";
    foreach ($Buttons as $Button)
    {
        $S .= StrCell(StrButton($Button));
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}


function StrButtonsVert($Buttons, $Color=ButtonColor)
{
    // $Buttons is array (index => (text, url))
    $S = StrTableBegin("2", $Color);
    foreach ($Buttons as $Button)
    {
        $S .= "<tr align=\"center\">\n";
        $S .= "<td bgcolor=\"".$Button["Color"]."\">";
        $S .= "<a href=\"".$Button["Url"]."\">".$Button["Text"]."</a>";
        $S .= "</td></tr>\n";
    }
    $S .= StrTableEnd();
    return $S;
}

function StrCell($CellContent, $Colspan="", $Rowspan="", $Style="")
{
    return StrTableCell("td", $CellContent, $Colspan, $Rowspan,
                        $Style);
}

function StrDangerousButtons($Buttons)
{
    // HTML submit buttons, framed in warning color. $Buttons is array
    // (array("Name", "Lbl"))
    $T = "";
    foreach ($Buttons as $Button)
    {
        $Label = $Button["Lbl"];
        if ($Label == "") $Label = $Button["Name"];
        $L = SpanStyle(" $Label ", StyleButtonDanger);
        $T .= "&nbsp;".HtmlSubmitButton($Button["Name"], $L);
    }
    $S = StrBoxBegin("1", "center", WarningColor);
    $S .= substr($T, 6);
    $S .= StrBoxEnd();
    return $S;
}

function StrDate($DateString, $Format=DateFormat_MdY)
{
    // Returns date string converted to preferred representation form,
    // if a legal date value, else returns it unchanged
    $TimeStamp = strtotime($DateString);
    return ($TimeStamp !== InvalidDateTime) ?
                        date($Format, $TimeStamp) :
                        $DateString;
}

function StrDeleteButtonCell($ButtonName, $Label="")
{
    // Delete button framed in color, as table cell
    if ($Label == "") $Label = "Delete";
    $S = "<td bgcolor=\"".WarningColor."\">";
    $S .= HtmlSubmitButton($ButtonName, " $Label ", "",
                           StyleButtonDanger);
    $S .= "</td>\n";
    return $S;
}

function StrDeleteButtonWidget($ButtonName, $Label="")
{
    // Delete button framed in color
    if ($Label == "") $Label = "Delete";
    return StrDangerousButtons(array(array("Name" => $ButtonName,
                                           "Lbl" => $Label)));
}

function StrEditButtons($What, $OldKey="", $Action="Add",
                        $DeleteOk=True)
{
    /*****************************************************************
      Makes a row of buttons for form actions (Add, Update/Delete):
      $What is the name of the entity to be added, updated, or deleted,
      for example, "User" or "File"
      $OldKey captures the key to allow, for example, backing out of a
      key change
      $Action should be "Add" or "Update"
    *****************************************************************/
    $S = "<table border=\"0\" cellspacing=\"10\" cellpadding=\"4\">";
    $S .= "\n<tbody>\n";
    $S .= StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("$Action$What", "$Action"));
    if ($OldKey > "") $S .= StrCell(HtmlHidden("OldKey", $OldKey));
    $S .= StrCell(HtmlResetButton());
    if ($Action != "Add" and $DeleteOk)
    {
        $S .= StrCell("&nbsp;");
        $S .= StrDeleteButtonCell("Delete$What", "Delete");
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function StrFormBegin($Action, $Name="", $Target="", $Style="")
{
    $S = "<form action=\"$Action\" method=\"".FormMethod."\"";
    if ($Name > "") $S .= " name=\"$Name\" id=\"$Name\"";
    if ($Target > "") $S .= " target=\"$Target\"";
    if ($Style > "") $S .= " style=\"$Style\"";
    $S .= ">\n";
    return $S;
}

function StrFormBeginPostUpload($Action, $MaxFileSize=51200)
{
    // Form tag for file uploads (see function HtmlInputPostUpload)
    // $MaxFileSize is in bytes, default 50K
    $S = "<form enctype=\"multipart/form-data\" method=\"post\"";
    $S .= " action=\"$Action\">\n";
    $S .= HtmlHidden("MAX_FILE_SIZE", $MaxFileSize);
    return $S;
}

function StrFormBeginRecurse($Name="", $Target="", $Style="")
{
    return StrFormBegin(ThisScript(), $Name, $Target, $Style);
}

function StrFormEnd()
{
    // Any form should set $LoginFlag if a user is logged in, so we
    // know to prompt for a login after a timeout
    $S = LoggedInUser()
       ? "<input type=\"hidden\" name=\"LoginFlag\" value=\"1\">\n"
       : "";
    $S .= "</form>\n";
    return $S;
}

function StrHeaderCell($CellContent, $Colspan="", $Rowspan="",
                       $Style="")
{
    return StrTableCell("th", $CellContent, $Colspan, $Rowspan,
                        $Style);
}

function StrHorizontalTable($Cells)
{
    // $Cells is array of cell contents
    $S = "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
    $S .= "<tbody>\n";
    $S .= "<tr valign=\"middle\">\n";
    foreach ($Cells as $Cell)
    {
        $S .= StrCell($Cell);
    }
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function StrLoginBox($Color=LoginBoxColor)
{
    $T = "background: $Color; border: 1ex outset $Color";
    $S = StrTableBegin("0", "", "", $T);
    $S .= "<tr><td>\n";
    $S .= StrTableBegin("0", "", "", "margin: 1cm");
    $S .= StrRowBegin();
    $S .= StrCell(Strong("User:"), "", "", "text-align: right");
    $S .= StrCell(HtmlTextInput("UserId", "16"));
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Password:"), "", "", "text-align: right");
    $T = "<input type=\"password\" name=\"Password\" size=\"16\">";
    $S .= StrCell($T);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrBoxEnd();
    $Buttons = array
             (
                 HtmlSubmitButton("Login", "Log In"),
                 "&nbsp;&nbsp;&nbsp;",
                 HtmlSubmitButton("CancelLogin", "Cancel")
             );
    $S .= HtmlPara(StrHorizontalTable($Buttons), "center");
    return $S;
}

function StrMenuBar()
{
    // Arguments (any number) are buttons (arrays)
    $S = "<hr width=\"30%\" align=\"left\">\n";
    $S .= StrButtonsHorz(func_get_args(), "100%");
    return $S;
}

function StrNote($Text)
{
    $S = "<dl compact=\"compact\">\n";
    $S .= "<dt><img src=\"gk/images/pinnote.png\" width=\"32\"";
    $S .= " height=\"32\" align=\"left\" alt=\"Note!\"></dt>\n";
    $S .= "<dd style=\"text-align: left\">$Text</dd>\n";
    $S .= "</dl>\n";
    return $S;
}

function Strong($Text)
{
    return "<strong>$Text</strong>";
}

function StrPaginationButtons($Page, $Pages)
{
    // Page number and count plus icons for First Page, Previous Page,
    // Next Page, Last Page
    // Used by function StrPaginationForm
    $S = StrTableBegin("0", "", "50%");
    $S .= StrRowBegin();
    $S .= "<td align=\"left\">";
    if ($Page > 1)
    {
        $S .= HtmlImageButton("FirstPage", "gk/images/2leftarrow.png",
                              "First Page");
        $S .= "&nbsp;";
        $S .= HtmlImageButton("PrevPage", "gk/images/1leftarrow.png",
                              "Previous Page");
        $S .= "</td>\n";
    } else $S .= "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;";
    $S .= "</td>\n";
    $S .= "<td align=\"center\">Page $Page of $Pages</td>\n";
    $S .= "<td align=\"right\">";
    if ($Page < $Pages)
    {
        $S .= HtmlImageButton("NextPage", "gk/images/1rightarrow.png",
                              "Next Page");
        $S .= "&nbsp;";
        $S .= HtmlImageButton("LastPage", "gk/images/2rightarrow.png",
                              "Last Page");
    } else $S .= "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;";
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function StrPaginationForm($Hits, $FormData, $StartRow)
{
    // $Hits = number of records.
    // $FormData is a single variable or a serialized array of many
    // variables, to preserve form data from one page to the next.
    // $StartRow is database record row that begins current page
    /************************************************************
      To assemble multiple variables as $FormData:
      $FormData = htmlentities(serialize(array(
      $First, $Second, ...)));
      To restore them:
      list ($First, $Second, ...) 
      = unserialize(stripslashes(html_entity_decode($FormData)))
    *************************************************************/
    $StartRow = min($Hits, $StartRow + HitsPerPage);
    $Pages = ceil($Hits/HitsPerPage);
    $Page = ceil($StartRow / HitsPerPage);
    $PrevStartRow = ($Page - 2) * HitsPerPage;
    $LastStartRow = ($Pages - 1) * HitsPerPage;
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("FormData", $FormData);
    $S .= HtmlHidden("StartRow", $StartRow);
    $S .= HtmlHidden("PrevStartRow", $PrevStartRow);
    $S .= HtmlHidden("LastStartRow", $LastStartRow);
    $S .=  StrPaginationButtons($Page, $Pages);
    $S .= StrFormEnd();
    return $S;
}

function StrRowBegin($Color="")
{
    $Style = "vertical-align: top";
    if ($Color !== "") $Style .= "; background-color: $Color";
    return "<tr style=\"$Style\">\n";
}

function StrRowEnd()
{
    return "</tr>\n";
}

function StrStdMenuBar($ShowMain=True)
{
    $Buttons = array();
    if ($ShowMain) $Buttons[] = BtnMainMenu();
    if (LoggedInUser() or UsersMayCreateOwnAccounts)
        $Buttons[] = BtnMyAccount();
    if (LoggedInUser())
        $Buttons[] = BtnLogout();
    else
        $Buttons[] = BtnLogin();
    $S = "<div align=\"center\">\n";
    $S .= StrButtonsHorz($Buttons);
    $S .= "</div>\n";
    return $S;
}

function StrSubTitle($Subtitle, $Size="3", $Align="center")
{
    return "<h$Size align=\"$Align\">$Subtitle</h$Size>\n";
}

function StrTableBegin($Border, $Color="", $Width="", $Style="")
{
    $S = "<table border=\"$Border\"";
    if ($Color !== "") $S .= " bgcolor=\"$Color\"";
    if ($Width !== "") $S .= " width=\"$Width\"";
    if ($Style !== "") $S .= " style=\"$Style\"";
    $S .= ">\n<tbody>\n";
    return $S;
}

function StrTableCell($Tag, $CellContent, $Colspan="", $Rowspan="",
                      $Style="")
{
    $StartTag = "<$Tag";
    if ($Colspan != "") $StartTag .= " colspan=\"$Colspan\"";
    if ($Rowspan != "") $StartTag .= " rowspan=\"$Rowspan\"";
    if ($Style != "") $StartTag .= " style=\"$Style\"";
    $StartTag .= ">";
    return "$StartTag$CellContent</$Tag>\n";
}

function StrTableEnd()
{
    return "</tbody></table>\n";
}

function StrUserKeyDescription($LastName, $FirstName, $UserId)
{
    // Return description of key(s) supplied
    $S = ($LastName > "") ? " last name $LastName" : "";
    if ($FirstName > "")
    {
        if ($S > "") $S .= ",";
        $S .= " first name $FirstName";
    }
    if ($UserId > "")
    {
        if ($S > "") $S .= ",";
        $S .= " user ID $UserId";
    }
    return $S;
}

function StrUserLinks($Res, $Hits, $SepWin=False)
{
    // Given multiple users, display a selection link for each
    $S = StrBoxBegin("2", "left", TableColor);
    $S .= "<ul>\n";
    for ($Row = 0 ; $Row < $Hits ; $Row++)
    {
        $UserId = pg_fetch_result($Res, $Row, "userid");
        $LastName = pg_fetch_result($Res, $Row, "lastname");
        $FirstName = pg_fetch_result($Res, $Row, "firstname");
        $S .= "<li>";
        $S .= "<a href=\"".ThisScript()."?UserId=$UserId";
        $S .= "&SearchForUser=Search&SepWin=$SepWin\">";
        $S .= "$LastName, $FirstName ($UserId)</a>";
        $S .= "</li>\n";
    }
    $S .= "</ul>\n";
    $S .= StrBoxEnd();
    return $S;
}

function StrUserNotFound($LastName, $FirstName, $UserId,
                         $SepWin=False)
{
    $W = "Found no user matching";
    $W .= StrUserKeyDescription($LastName, $FirstName, $UserId);
    $S = StrWarning($W);
    $S .= StrUserSearchForm("", $LastName, $FirstName, $UserId,
                            $SepWin);
    return $S;
}

function StrUserSearchBox($BgColor="", $LastName="", $FirstName="",
                          $UserId="")
{
    if ($BgColor == "") $BgColor = LoginBoxColor;
    $S = StrBoxBegin("1", "left", $BgColor);
    $S .= StrTableBegin("0");

    $S .= StrRowBegin();
    $S .= StrCell("Last Name:");
    $S .= StrCell(HtmlTextInput("LastName", "10", $LastName));
    $S .= StrCell("First Name:");
    $S .= StrCell(HtmlTextInput("FirstName", "10", $FirstName));
    $S .= StrRowEnd();

    $S .= StrRowBegin();
    $S .= StrCell("User ID:");
    $S .= StrCell(HtmlTextInput("UserId", "10", $UserId));
    //    $S .= StrCell("&nbsp", "2");
    $S .= "<td colspan=\"2\" align=\"center\">";
    $S .= HtmlSubmitButton("SearchForUser", "Search");
    $S .= "</td>\n";
    $S .= StrRowEnd();

    $S .= StrTableEnd();
    $S .= StrBoxEnd();
    return $S;
}

function StrUserSearchForm($BgColor="", $LastName="", $FirstName="",
                           $UserId="", $SepWin=False)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SepWin", $SepWin);
    $S .= StrUserSearchBox($BgColor, $LastName, $FirstName, $UserId);
    $S .= StrFormEnd();
    return $S;
}

function StrUsersMultiple($LastName, $FirstName, $UserId, $Res, $Hits,
                          $SepWin=False)
{
    $W = "Found multiple users matching";
    $W .= StrUserKeyDescription($LastName, $FirstName, $UserId);
    $S = StrWarning($W, NoticeColor);
    $S .= StrUserLinks($Res, $Hits, $SepWin);
    return $S;
}

function StrWarning($Warning, $Color=WarningColor)
{
    return HtmlPara(Strong(HtmlColor($Color, $Warning)), "center");
}

function StrWarningBox($Warning, $Width="", $Color=WarningColor)
{
    $S = "<div style=\"margin: auto; border: thick double $Color";
    $S .= "; background: ".NoticeBg."; color: $Color";
    if ($Width > "") $S .= "; width: $Width";
    $S .= "\">";
    $S .= StrNote($Warning);
    $S .= "</div>\n";
    return $S;
}

function StrWarningInRow($Warning, $Color=WarningColor, $Colspan="")
{
    $S = StrRowBegin();
    $S .= StrTableCell("th", Strong(HtmlColor($Color, $Warning)),
                       $Colspan);
    $S .= StrRowEnd();
    return $S;
}

function StrWithGoButton($Content, $ButtonName, $Value="", $Style="")
{
    // Places a Go button to the right of a widget
    $S = "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
    $S .= "<tbody>\n";
    $S .= "<tr valign=\"middle\">\n";
    $S .= StrCell($Content);
    $S .= StrCell("&nbsp;");
    $S .= StrCell(HtmlGoButton($ButtonName, $Value, $Style));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function TableMultiCol($Res, $NumCols, $CellFunction, $Args=False,
                       $Style="")
{
    // Arranges the ordered set of tuples indicated by $Res into
    // $NumCols columns, column-by-column. Invokes supplied function
    // $CellFunction on each tuple to populate the corresponding table
    // cell. Any cells with no corresponding tuples (in the last
    // column) get an HTML space.

    // The function named by $CellFunction must take, as the first
    // argument, the array fetched for the corresponding tuple. $Args
    // (optional) is a value (or an array of values) to be passed
    // through to $CellFunction. Obviously, $CellFunction must return
    // HTML text.

    // $Style (optional) is for the table.
    $NumTuples = pg_num_rows($Res);
    $RowsPerCol = ceil($NumTuples / $NumCols);
    $S = StrTableBegin("0", "", "", $Style);
    for ($Row = 0 ; $Row < $RowsPerCol ; $Row ++)
    {
        $S .= StrRowBegin();
        for ($Col = 0 ; $Col < $NumCols ; $Col ++)
        {
            $Tuple = ($Col * $RowsPerCol) + $Row;
            $T = ($Tuple < $NumTuples)
               ? $CellFunction(pg_fetch_array($Res, $Tuple), $Args)
               : "&nbsp;";
            $S .= StrCell($T);
        }
        $S .= StrRowEnd();
    }
    $S .= StrTableEnd();
    return $S;
}

?>
