<?php
/*********************************************************************
gk_main.php

Main Page for PHP service

Designed to be accessed via an include/require statement in the
otherwise empty ../index.php script

______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");

define("BadLoginMessage", "Login incorrect");
define("LoggedOutMessage", "Logged out");
define("NoLoginMessage",
       "Nobody's logged in&mdash;session may have timed out");
define("PasswordChangedMessage", "Password changed");
$T = "Session timed out; please reenter your credentials to";
$T .= " continue:";
define("TimeoutPrompt", $T);

function PageLogin($SavedStateAtTimeOut)
{
    $S = "<div align=\"center\">\n";
    if (! isset($SavedStateAtTimeOut)) $S .= HtmlPara(LoginPrompt);
    $S .= "<form action=\"index.php\" method=\"post\">\n";
    $S .= StrLoginBox();
    if (isset($SavedStateAtTimeOut))
	$S .= HtmlHidden("SavedStateAtTimeOut", $SavedStateAtTimeOut);
    $S .= StrFormEnd();
    if (! isset($SavedStateAtTimeOut)
	and (AlternateAuthOK or UsersMayCreateOwnAccounts))
    {
	$S .= "<br>\n"; 
	$S .= StrTableBegin("0", "", "70%");
	if (AlternateAuthOK)
	{
	    $S .= StrRowBegin();
	    $S .= StrCell(AltAuthPrompt);
	    $T = StrFormBeginRecurse();
	    $T .= HtmlSubmitButton("TryAltAuth", "Use Alternate");
	    $T .= StrFormEnd();
	    $S .= StrCell($T);
	    $S .= StrRowEnd();
	}
	if (UsersMayCreateOwnAccounts)
	{
	    $S .= StrRowBegin();
	    $S .= StrCell(AccountInvitation);
	    $T = StrFormBegin("myaccount.php");
	    $T .= HtmlSubmitButton("CreateAcct", "Create Account");
	    $T .= StrFormEnd();
	    $S .= StrCell($T);
	    $S .= StrRowEnd();
	}
	$S .= StrTableEnd();
    }
    return $S;
}

function PageMainMenu($Path)
{
    $S = StrStdMenuBar(False);
    $T = "<em>".ServiceTitle."</em><br>\n";
    $T .= ServiceDescription;
    $S = PageTitle($T);
    $S .= "<div align=\"center\">\n";
    $S .= StrWarnIfNotProduction($Path);
    $S .= StrMessages();
    $S .= StrToolMenu();
    return $S;
}

function ResPrivateTools($UserId)
{
    // Return record of private tools $UserId may use, including any
    // with the pseudo-privilege "anylogin" and any with
    // application-specific special-access rules
    $SATools = "";
    if (function_exists("SpecialToolAccess"))
    {
	$SATools = "";
	$Tnm = Query("SELECT name FROM tool");
	for ($Row = 0 ; $Row < pg_num_rows($Tnm) ; $Row ++)
	{
	    $Tool = pg_fetch_result($Tnm, $Row, "name");
	    if (SpecialToolAccess($UserId, $Tool))
		$SATools .= ", '$Tool'";
	}
	$SATools = ($SATools > "") ? substr($SATools, 2) : False;
    }
    $ReguserSn = SerialNumber("reguser", "userid", $UserId);
    $Q = "SELECT DISTINCT ON (t.dispord, t.name) t.name,";
    $Q .= " t.description";
    $Q .= " FROM tool t, tool_priv tp, privilege p, user_priv up";
    $Q .= " WHERE t.toolsn = tp.toolsn";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    $Q .= " AND NOT EXISTS";
    $Q .= " (SELECT * FROM hiddentool WHERE toolsn = t.toolsn)";
    $Q .= " AND (";
    // If tool privilege is anylogin, allow
    $Q .= "  p.privname = 'anylogin'";
    // If user has tool privilege, allow
    $Q .= "  OR (p.privilegesn = up.privilegesn";
    $Q .= "   AND up.regusersn = $ReguserSn)";
    // If tool privilege is grant, allow if user has any metaprivilege
    $Q .= "  OR (p.privname = 'grant' AND EXISTS";
    $Q .= "   (SELECT * FROM user_metapriv";
    $Q .= "    WHERE regusersn = $ReguserSn))";
    if ($SATools) $Q .= " OR t.name IN ($SATools)";
    $Q .= ")";
    $Q .= " ORDER BY t.dispord, t.name";
    return Query($Q);
}

function ResPublicTools()
{
    // Identify Public Services
    $Q = "SELECT DISTINCT ON (t.dispord, t.name) t.name,";
    $Q .= " t.description";
    $Q .= " FROM tool t, tool_priv tp, privilege p";
    $Q .= " WHERE t.toolsn = tp.toolsn";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    $Q .= " AND p.privname = 'public'";
    $Q .= " ORDER BY t.dispord, t.name";
    return Query($Q);
}

function StrMessages()
{
    $S = "";
    $Today = date("Ymd");
    // Display Message of the Day, if any:
    if (file_exists($Today))
    {
	$S .= StrBoxBegin("4", "left", WarningColor, "60%");
	$S .= StrBoxBegin("0", "left", MotdColor, "100%");
	$S .= file_get_contents($Today);
	$S .= StrBoxEnd();
	$S .= StrBoxEnd();    
    }
    // Display Message for Every Day, if any:
    if (file_exists("mfed"))
    {
	$S .= StrBoxBegin("4", "left", NoticeColor, "60%");
	$S .= StrBoxBegin("0", "left", NoticeBg, "100%");
	$S .= file_get_contents("mfed");
	$S .= StrBoxEnd();
	$S .= StrBoxEnd();    
    }
    // Display app-specific messages, if any:
    if (function_exists("AppMessages")) $S .= AppMessages();
    return $S;
}

function StrToolBullets($Res, $NumRows)
{
    $S = "";
    for ($Row=0 ; $Row < $NumRows ; $Row++)
    {
	list($Url, $Text) = pg_fetch_array($Res, $Row);
	$Button = array("Text" => $Text, "Url" => $Url);
	$S .= StrBullet($Button);
    }
    return $S;
}

function StrToolMenu()
{
    $PubTools = ResPublicTools();
    $NumPub = $PubTools != False ? pg_num_rows($PubTools) : 0;
    $UserId = LoggedInUser();
    if ($UserId != False)
    {
	$PrivTools = ResPrivateTools($UserId);
	$NumPriv = $PrivTools != False ? pg_num_rows($PrivTools) : 0;
    } else $NumPriv = 0;
    if ($NumPub + $NumPriv > 0)
    {
	$S = StrBoxBegin("2", "left", TableColor);
	$S .= "<ul>\n";
	$S .= StrToolBullets($PubTools, $NumPub);
	if ($UserId != False)
	    $S .= StrToolBullets($PrivTools, $NumPriv);
	$S .= "</ul>\n";
	$S .= StrBoxEnd();
    } elseif ($UserId != False) $S = HtmlPara(NoPrivsMessage); 
    else $S = HtmlPara(LoginPrompt); 
    return $S;
}

function StrWarnIfNotProduction($Path)
{
    // Display warning that this isn't production service (can
    // suppress by including path in array $NoProductionLink in
    // local/loc_env.php)
    global $NoProductionLink;
    $S = "";
    if (! in_array($Path, $NoProductionLink))
    {
	$W = "This is ".Strong("not");
	$W .= " the production version of the ";
	$W .= ServiceTitle.".<br>\n";
	$W .= "<ul>\n<li>";
	$W .= "Services available here may change without warning.";
	$W .= "</li>\n<li>";
	$W .= "Data entered here will eventually be lost.";
	$W .= "</li>\n</ul>\n";
	$S .= StrWarningBox($W, "50%");
    }
    return $S;
}

//********************************************************************
$S = HeadAndTitle();

if (isset($_POST["Login"]))
{
    include("gk/gk_validate.php");
    if (isset($BadLogin))
    {
	$S .= StrWarning(BadLoginMessage, NoticeColor);
	if (isset($SavedStateAtTimeOut))
	    $SavedStateAtTimeOut = ArrayWrap($SavedStateAtTimeOut);
	$S .= PageLogin($SavedStateAtTimeOut);
    } else
	$S .= PageMainMenu($Path);
} elseif (isset($_REQUEST["LoggedOut"]))
{
    $S .= StrWarning(LoggedOutMessage, NoticeColor);
    $S .= PageMainMenu($Path);
} elseif(isset($_REQUEST["LogIn"])) // Distinct from Login (I not i)
    $S .= PageLogin($SavedStateAtTimeOut);
elseif ($TimedOut)
{
    $S .= StrWarning(TimeoutPrompt, NoticeColor);
    $S .= PageLogin($SavedStateAtTimeOut);
} elseif ($NoActiveLogin)
{
    $S .= StrWarning(NoLoginMessage, NoticeColor);
    $S .= PageLogin($SavedStateAtTimeOut);
} elseif (isset($_POST["ChangePassword"]))
{
    include("gk/gk_validate.php");
    if (isset($PasswordChanged))
    {
	$S .= StrWarning(PasswordChangedMessage, NoticeColor);
	$S .= PageMainMenu($Path);
    }
} elseif (isset($_REQUEST["TryAltAuth"]))
{
    include("gk/gk_altauth.php");
    if (isset($PasswordChanged))
    {
	$S .= StrWarning(PasswordChangedMessage, NoticeColor);
	$S .= PageMainMenu($Path);
    }
} elseif (isset($_REQUEST["MngAcct"])
)
{
    include("gk/gk_myaccount.php");
    if (isset($WarnDeletedUser))
	$S = HeadAndTitle().$WarnDeletedUser.PageMainMenu($Path);
} else $S .= PageMainMenu($Path);

$S .= "</div></body></html>\n";

echo $S;
	
?>
