<?php
/*********************************************************************
gk_managepriv.php

Add and delete user privileges

Designed to be accessed via an include/require statement in the
otherwise empty ../manageuser.php script

______________________________________________________________________
Copyright 2013 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");

// Return array($PrivName, $Description, $Grantable)
$Q = "SELECT privilegesn, privname, description, NOT EXISTS";
$Q .= " (SELECT * FROM nogrant WHERE privilegesn = p.privilegesn)";
$Q .= " FROM privilege p";
define("QueryPrivilege", $Q);

function AddPrivilege($PrivName, $Description, $IsGrantable)
{
    $N = "";
    $W = "";
    if ($PrivName > "" and $Description > "")
    {
	if (IsInDatabase("privilege", "privname", "$PrivName"))
	    $W = "Privilege &quot;$PrivName&quot; already exists";
	else
	{
	    Query("BEGIN");
	    $Q = "INSERT INTO privilege(privname, description)";
	    $Q .= " VALUES ($1, $2)";
	    Query($Q, $PrivName, $Description);
	    $PrivilegeSn = SerialNumber("privilege", "privname",
					"$PrivName");
	    if (! $IsGrantable)
	    {
		$Q = "INSERT INTO nogrant (privilegesn)";
		$Q .= " VALUES ($PrivilegeSn) ";
		Query($Q);
	    }
	    Query("COMMIT");
	    $N = "Added privilege &quot;$PrivName&quot;";
	}
    } else $W = "You must enter both a name and a description.";
    return array($N, $W);
}

function DeletePrivilege($PrivilegeSn)
{
    $N = "";
    $W = "";
    list($Z, $PrivName, $Description, $IsGrantable)
	= PrivilegeDetails($PrivilegeSn);
    $Tables = array(
	"nogrant",
	"tool_priv",
	"user_metapriv",
	"user_priv",
	"report_priv",
	"privilege"); 
    Query("BEGIN");
    foreach ($Tables as $Table)
    {
	Query("DELETE FROM $Table WHERE privilegesn = $PrivilegeSn");
    }
    Query("COMMIT");
    $N = "Deleted privilege &quot;$PrivName&quot;";
    return array($N, $W);
}
function DeleteTool($ToolSn)
{
    list($ToolName, $Description) = ToolNameAndDesc($ToolSn);
    Query("BEGIN");
    foreach (array("hiddentool", "tool_priv", "tool") as $Table)
    {
	Query("DELETE FROM $Table WHERE toolsn = $ToolSn");
    }
    Query("COMMIT");
    return "Deleted $ToolName<br />\n";
}


function FormPrivilege($PrivilegeSn=False, $Update=False, $PrivName="",
		       $Description="", $IsGrantable=True)
{
    $S = StrFormBeginRecurse();
    if ($PrivilegeSn) $S .= HtmlHidden("PrivilegeSn", $PrivilegeSn);
    $S .= StrTableBegin("0", TableColor, "", "border: thin solid");
    $S .= StrRowBegin();
    $S .= StrHeaderCell("Name");
    $S .= StrHeaderCell("Description");
    $S .= StrHeaderCell("Grantable?");
    $T = $PrivilegeSn ? "Update" : "Add";
    $S .= StrCell(HtmlSubmitButton($T), "", "2",
		  "vertical-align: bottom");
    if ($PrivilegeSn)
	$S .= StrCell(StrDeleteButtonWidget("Delete"), "", "2",
		      "vertical-align: bottom; padding-left: 3em");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrCell(HtmlTextInput("PrivName", "10", $PrivName));
    $S .= StrCell(HtmlTextInput("Description", "30", $Description));
    $S .= StrCell(HtmlCheckbox("IsGrantable", $IsGrantable), "", "",
		  "text-align: center");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
    if (! $Update) list($Z, $PrivName, $Description, $IsGrantable)
		       = PrivilegeDetails($PrivilegeSn);
}

function FormPrivileges()
{
    $S = StrFormBeginRecurse();
    $S .= StrTableBegin("0", TableColor, "", "border: thin solid");
    $S .= StrRowBegin();
    // $S .= StrHeaderCell("");
    $S .= StrHeaderCell("Name", "2");
    $S .= StrHeaderCell("Description");
    $S .= StrHeaderCell("Grantable?");
    $S .= StrRowEnd();
    $Res = Query(QueryPrivilege." ORDER BY privname");
    for ($Row = 0 ; $Row < pg_num_rows($Res) ; $Row ++)
    {
	list($PrivilegeSn, $PrivName, $Description, $Grantable)
	    = pg_fetch_array($Res, $Row);
	$S .= StrRowBegin();
	$T = FormEditButton("Edit", 16, "Edit privilege \"$PrivName\"",
			    HtmlHidden("PrivilegeSn", $PrivilegeSn));
	$S .= StrCell($T);
	$S .= StrCell($PrivName);
	$S .= StrCell($Description);
	$S .= StrCell(($Grantable == "t")
		      ? HtmlImage("gk/images/check.png", "Yes") : "",
		  "", "", "text-align: center");
	$S .= StrRowEnd();
    }
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function PageEditPrivilege($PrivilegeSn, $Update=False, $PrivName="",
			   $Description="", $IsGrantable=True, $N="",
			   $W="")
{
    global $SystemTitle;
    if ($PrivilegeSn and ! $Update)
	list($Z, $PrivName, $Description, $IsGrantable)
	    = PrivilegeDetails($PrivilegeSn);
    $S = HeadAndTitle("$SystemTitle Privilege Management");
    $T = "Edit Privilege";
    if ($PrivilegeSn) $T .= "&quot;$PrivName&quot;";
    $S .= PageTitleWithButton($T, BtnDone());
    $S .= "<div align=\"center\">\n";
    if ($W > "") $S .= StrWarningBox($W);
    if ($N > "") $S .= StrWarning($N, NoticeColor);
    $S .= FormPrivilege($PrivilegeSn, $Update, $PrivName, $Description,
			$IsGrantable);
    return $S;
}

function PagePrivileges($N, $W)
{
    global $SystemTitle;
    $S = HeadAndTitle("$SystemTitle Privilege Management");
    $S .= PageTitleWithButton("Privilege Management", BtnMainMenu());
    $S .= "<div align=\"center\">\n";
    if ($W > "") $S .= StrWarningBox($W);
    if ($N > "") $S .= StrWarning($N, NoticeColor);
    $S .= FormPrivilege();
    $S .= FormPrivileges();
    return $S;
}

function PrivilegeDetails($PrivilegeSn)
{
    // Return array($PrivilegeSn, $PrivName, $Description,
    // $IsGrantable)
    $Q = QueryPrivilege." WHERE privilegesn = $PrivilegeSn";
    list($Z, $PrivName, $Description, $Grantable)
	= pg_fetch_array(Query($Q));
    $IsGrantable = ($Grantable == "t");
    return array($PrivilegeSn, $PrivName, $Description, $IsGrantable);
}

function UpdatePrivilege($PrivilegeSn, $PrivName, $Description,
			 $IsGrantable)
{
    $N = "";
    $W = "";
    /* $Description = stripslashes($Description); */
    if ($PrivName > "" and $Description > "")
    {
	if (IsInDatabase("privilege", "privname", "$PrivName", True,
			 "privilegesn", $PrivilegeSn))
	{
	    $W = "Privilege name &quot;$PrivName&quot; is taken;";
	    $W .= " reverting to original name";
	    list($Z, $PrivName, $Z1, $Z2)
		= PrivilegeDetails($PrivilegeSn);
	} else
	{
	    Query("BEGIN");
	    $Q = "UPDATE privilege SET privname = $1,";
	    $Q .= " description = $2";
	    $Q .= " WHERE privilegesn = $PrivilegeSn AND";
	    $Q .= " (privname != $1 OR description != $2)";
	    Query($Q, $PrivName, $Description);
	    if ($IsGrantable)
	    {
		$Q = "DELETE FROM nogrant";
		$Q .= " WHERE privilegesn = $PrivilegeSn";
	    } else
	    {
		$Q = "INSERT INTO nogrant (privilegesn)";
		$Q .= " SELECT $PrivilegeSn WHERE NOT EXISTS";
		$Q .= " (SELECT * FROM nogrant";
		$Q .= " WHERE privilegesn = $PrivilegeSn)";
	    }
	    Query($Q);
	    Query("COMMIT");
	    $N = "Updated privilege &quot;$PrivName&quot;";
	}
    } else $W = "You must enter both a name and a description.";
    return PageEditPrivilege($PrivilegeSn, True, $PrivName,
			     $Description, $IsGrantable, $N, $W);
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "Description",
    "IsGrantable",
    "PrivilegeSn",
    "PrivName"
    );
$GetVarList = array(
    
   );
$PostVarList = array(
    
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$N = "";
$W = "";
if (isset($FormVars["Add"]))
{
    list($N, $W) = AddPrivilege($PrivName, $Description, $IsGrantable);
    $S = PagePrivileges($N, $W);
} elseif (isset($FormVars["Edit_x"]))
    $S .= PageEditPrivilege($PrivilegeSn);
elseif (isset($FormVars["Update"]))
    $S .= UpdatePrivilege($PrivilegeSn, $PrivName, $Description,
			  $IsGrantable);
elseif (isset($FormVars["Delete"]))
{
    list($N, $W) = DeletePrivilege($PrivilegeSn);
    $S = PagePrivileges($N, $W);
}else
    $S = PagePrivileges($N, $W);
$S .= "</div></body></html>\n";
echo $S;

?>
