<?php
/*********************************************************************
gk_managereport.php

Manage report privileges

Designed to be accessed via an include/require statement in the
otherwise empty ../managereport.php script
______________________________________________________________________
Copyright 2013 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");

function AllowForAnyLogin($ReportSn)
{
    // First reset all privileges for report, then set "anylogin"
    Query("DELETE FROM report_priv WHERE reportsn = $ReportSn");
    $AnyLoginSn = SerialNumber("privilege", "privname", "anylogin");
    UpdateReportPrivilege($ReportSn, $AnyLoginSn, True);
    return;
}

function DeleteReport($ReportSn)
{
    list($ReportName, $Description) = ReportNameAndDesc($ReportSn);
    Query("BEGIN");
    Query("DELETE FROM report_priv WHERE reportsn = $ReportSn");
    Query("DELETE FROM report WHERE reportsn = $ReportSn");
    Query("COMMIT");
    return "Deleted $ReportName<br />\n";
}

function MakeReportPublic($ReportSn)
{
    // First reset all privileges for report, then set "public"
    Query("DELETE FROM report_priv WHERE reportsn = $ReportSn");
    $PublicSn = SerialNumber("privilege", "privname", "public");
    UpdateReportPrivilege($ReportSn, $PublicSn, True);
    return;
}

function ReorderReport($ReportName, $NewOrdinal, $NumReports)
{
    $Q = "SELECT dispord FROM report WHERE name = $1";
    $Ord = Query($Q, $ReportName);
    $OldOrdinal = pg_fetch_result($Ord, 0, "dispord");
    if ($NewOrdinal != $OldOrdinal)
    {
	$NewOrdinal = BoundedValue($NewOrdinal, $NumReports);
	$PublicSn = SerialNumber("privilege", "privname", "public");
	$IsPublic = (IsReportPublic($ReportName)
		     ? "" : "!")."= $PublicSn";
	if ($NewOrdinal < $OldOrdinal)
	{
	    $Q = "UPDATE report t SET dispord = (dispord+1)";
	    $Q .= " WHERE dispord >= $1";
	    $Q .= " AND dispord < $OldOrdinal";
	    $Q .= " AND EXISTS (SELECT * FROM report_priv";
	    $Q .= " WHERE reportsn = t.reportsn";
	    $Q .= " AND privilegesn $IsPublic)";
	    Query($Q, $NewOrdinal);
	} elseif ($NewOrdinal > $OldOrdinal)
	{
	    $Q = "UPDATE report t SET dispord = (dispord-1)";
	    $Q .= " WHERE dispord > $OldOrdinal";
	    $Q .= " AND dispord <= $1";
	    $Q .= " AND EXISTS (SELECT * FROM report_priv";
	    $Q .= " WHERE reportsn = t.reportsn";
	    $Q .= " AND privilegesn $IsPublic)";
	    Query($Q, $NewOrdinal);
	}
	$Q = "UPDATE report SET dispord = $1";
	$Q .= " WHERE name = $2";
	Query($Q, $NewOrdinal, $ReportName);
    }
    return;
}

function StrAddReport($Name, $Seq, $Description)
{
    $Name = trim($Name);
    $Seq = trim($Seq);
    $Description = trim($Description);
    /* $Description = stripslashes(trim($Description)); */
    $N = "";
    if ( ! ($Description > "") )
	$N .= "You must provide a description<br />\n";
    if ( ! ($Name > "") ) $N .= "You must name the report<br />\n";
    if (IsInDatabase("report", "name", $Name))
	$N .= "That name is already taken<br />\n";
    if ($N > "") return $N;
    // Open a slot for new report
    $Q = "UPDATE report SET dispord = dispord + 1";
    $Q .= " WHERE dispord >= $1 AND NOT EXISTS";
    $Q .= " (SELECT * FROM report WHERE name = $2)";
    Query($Q, $Seq, $Name);
    // Add the report
    $Q = "INSERT INTO report(name, dispord, description)";
    $Q .= " SELECT $1, $2, $3 WHERE NOT EXISTS";
    $Q .= " (SELECT * FROM report WHERE name = $1)";
    if (Query($Q, $Name, $Seq, $Description))
	return "$Name added<br />\n";
    else return "$Name not added<br />\n";
}

function StrAddReportTable()
{
    $S = StrTableBegin("1", TableColor);
    $S .= StrRowBegin();
    $S .= StrHeaderCell("Name");
    $S .= StrHeaderCell("Seq");
    $S .= StrHeaderCell("Description");
    $S .= StrHeaderCell("Maintenance");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrFormBeginRecurse();
    $S .= StrCell(HtmlTextInput("Report","25"));
    $S .= StrCell(HtmlTextInput("Seq","5"));
    $S .= StrCell(HtmlTextInput("Description","45"));
    $C = "<td align=\"center\">";
    $C .= HtmlSubmitButton("AddPub","Add Public", "Public");
    $C .= HtmlSubmitButton("AddPriv","Add Private", "Private");
    $C .= HtmlSubmitButton("AddAnyLogin","Add for Any Login",
			   "AnyLogin");
    $C .= "</td>";
    $S .= $C;
    $S .= StrFormEnd();
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function StrBeginReportTable()
{
    $S = StrTableBegin("1", TableColor);
    $S .= StrRowBegin();
    $S .= StrHeaderCell("Name");
    $S .= StrHeaderCell("Position");
    $S .= StrHeaderCell("Description");
    $S .= StrHeaderCell("Required Privileges");
    $S .= StrHeaderCell("Maintenance");
    $S .= StrRowEnd();
    return $S;
}

function StrSelectOrdinal($ReportName, $Ordinal, $NumReports)
{
    // Draws the drop-down box
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("ReportName", $ReportName);
    $S .= HtmlHidden("NumReports", $NumReports);
    $S .= HtmlSelectBegin("Ordinal", 'onchange="this.form.submit()"');
    for ($I = 1; $I <= $NumReports ; $I++)
	$S .= HtmlSelectOption($I, $I, ($I == $Ordinal));
    $S .= HtmlSelectEnd();
    $S .= StrFormEnd();
    return $S;
}

function StrReportDisplayTable($Reports)
{
    $NumReports = pg_num_rows($Reports);
    for ($Row = 0 ; $Row < $NumReports ; $Row ++)
    {
	list($Name, $Dispord, $Description, $ReportSn)
	    = pg_fetch_array($Reports, $Row);
	$Q = "SELECT privname FROM report_priv tp, privilege p";
	$Q .= " WHERE reportsn = $ReportSn";
	$Q .= " AND p.privilegesn = tp.privilegesn";
	$Q .= " ORDER BY privname";
	$Prv = Query($Q);
	$P = "";
	for ($PRow = 0 ; $PRow < pg_num_rows($Prv) ; $PRow ++)
	{
	    $P .= ", ".pg_fetch_result($Prv, $PRow, "privname");
	}
	if ($Dispord != $Row+1) 
	{
	    $Q = "UPDATE report SET dispord = $Row+1";
	    $Q .= " WHERE name = '$Name'";
	    Query($Q);
	}
	$S .= StrRowBegin();
	$S .= StrCell($Name);
	$S .= StrCell(StrSelectOrdinal($Name, $Row+1, $NumReports));
	if ("$Description" == "") $Description = "&nbsp;";
	$S .= StrCell($Description);
	$S .= ($P > "") ? StrCell(SubStr($P, 2)) : StrCell("&nbsp;");
	$C = "<td align=\"center\">";
	$C .= StrFormBeginRecurse();
	$C .= HtmlHidden("Report", $Name);
	$C .= HtmlSubmitButton("Manage");
	$C .= StrFormEnd();
	$C .= "</td>";
	$S .= $C;
	$S .= StrRowEnd();
    }
    $S .= StrRowBegin();
    $S .= StrRowEnd();
    return $S;
}

function StrReportMaintenanceTable($Name)
{
    $Q = "SELECT name, dispord, description, reportsn FROM report";
    $Q .= " WHERE name = $1";
    list($Name, $Dispord, $Description, $ReportSn)
	= pg_fetch_array(Query($Q, $Name), 0);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("ReportSn", $ReportSn);
    $S .= StrBeginReportTable();
    $S .= StrRowBegin();
    $S .= StrCell(HtmlTextInput("NewReportName", "25", $Name));
    $S .= StrCell($Dispord, "", "2");
    $S .= StrCell(HtmlTextInput("NewDescription", "45",
				$Description));
    $Q = "SELECT privname, privilegesn FROM privilege";
    $Q .= " ORDER BY privname";
    $Res = Query($Q);
    $C ="";
    for ($PRow = 0 ; $PRow < pg_num_rows($Res) ; $PRow ++)
    {
	list($CurrPriv, $PrivilegeSn)
	    = pg_fetch_array($Res, $PRow);
	$Q = "SELECT * FROM report_priv WHERE reportsn = $ReportSn";
	$Q .= " AND privilegesn = $PrivilegeSn";
	$C .= "<label>";
	$C .= HtmlCheckbox("Grant".$PRow, pg_num_rows(Query($Q)));
	$C .= "$CurrPriv</label>";
	$C .= HtmlHidden("PrvSn".$PRow, $PrivilegeSn);
	$C .= "<br />\n";
    }
    $S .= StrCell($C, "", "", "white-space: nowrap");
    $C = HtmlSubmitButton("UpdateReport", "Submit Changes", $Name);  
    $Q = "SELECT * FROM report_priv tp, privilege p";
    $Q .= " WHERE reportsn = $ReportSn AND privname = 'public'";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    if (pg_num_rows(Query($Q)) == 0)
	$C .= HtmlSubmitButton("Public", "Make Public", $Name);
    $Q = "SELECT * FROM report_priv tp, privilege p";
    $Q .= " WHERE reportsn = $ReportSn AND privname = 'anylogin'";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    if (pg_num_rows(Query($Q)) == 0)
	$C .= HtmlSubmitButton("AnyLogin", "Allow for Any Login ",
			       $Name);
    $C .= HtmlSubmitButton("Cancel");
    $S .= "<td style=\"text-align: center; height: 100%\">";
    $S .= StrTableBegin("0", "", "", "height: 100%").StrRowBegin();
    $S .= "<td align=\"center\">".$C."</td>";
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $C = HtmlSubmitButton("Delete", "", "",
			  "border: medium solid ".WarningColor);
    $S .= "<td style=\"text-align: right; vertical-align: bottom\">";
    $S .= $C."</td>";
    $S .= StrRowEnd().StrTableEnd()."</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function TablesOfReports()
{
    // We're in report display mode; show public reports
    $S = StrAddReportTable();
    $S .= StrSubTitle("Public Reports");
    $S .= StrBeginReportTable();
    $Q = "SELECT name, dispord, description, reportsn";
    $Q .= " FROM report t0 WHERE EXISTS";
    $Q .= " (SELECT * FROM report_priv tp, report t, privilege p";
    $Q .= " WHERE t.name = t0.name AND privname='public'";
    $Q .= " AND tp.reportsn = t.reportsn";
    $Q .= " AND tp.privilegesn = p.privilegesn)";
    $Q .= " ORDER BY dispord, name";
    $S .= StrReportDisplayTable(Query($Q));
    $S .= StrTableEnd();
    // show private reports
    $S .= StrSubTitle("Private Reports");
    $S .= StrBeginReportTable();
    $Q = "SELECT name, dispord, description, reportsn";
    $Q .= " FROM report t0 WHERE NOT EXISTS";
    $Q .= " (SELECT * FROM report_priv tp, report t, privilege p";
    $Q .= " WHERE t.name = t0.name AND privname='public'";
    $Q .= " AND tp.reportsn = t.reportsn";
    $Q .= " AND tp.privilegesn = p.privilegesn)";
    $Q .= " ORDER BY dispord, name";
    $S .= StrReportDisplayTable(Query($Q));
    $S .= StrTableEnd();
    return $S;
}

function ReportNameAndDesc($ReportSn)
{
    $Q = "SELECT name, description FROM report WHERE reportsn = $1";
    return pg_fetch_array(Query($Q, $ReportSn), 0);
}

function UpdateReportNameAndDesc($ReportSn, $NewReportName,
				 $NewDescription)
{
    list($OldName, $OldDescription) = ReportNameAndDesc($ReportSn);
    $S = "";
    /* $NewDescription = stripslashes($NewDescription); */
    /* $OldDescription = stripslashes($OldDescription); */
    $Name = $OldName;
    if ($NewReportName != "" AND $NewReportName != $OldName)
    {
	$Q = "UPDATE report SET name = $1";
	$Q .= " WHERE reportsn = $ReportSn AND name != $1";
	Query($Q, $NewReportName);
	$S .= "Renamed $OldName to $NewReportName<br />\n";
	$Name = $NewReportName;
    }
    if ($NewDescription != "" AND $NewDescription != $OldDescription)
    {
	$Q = "UPDATE report SET description = $1";
	$Q .= " WHERE reportsn = $ReportSn";
	$Q .= " AND description != $1";
	Query($Q, $NewDescription);
	$S .= "Updated description for $Name<br />\n";
    }
    return $S;
}

function UpdateReportPrivilege($ReportSn, $PrivilegeSn, $Grant=False)
{
    if ($Grant)
    {
	$Q = "INSERT INTO report_priv(reportsn, privilegesn) SELECT";
	$Q .= " $ReportSn, $PrivilegeSn WHERE NOT EXISTS";
	$Q .= " (SELECT * FROM report_priv WHERE";
	$Q .= " reportsn = $ReportSn AND privilegesn = $PrivilegeSn)";
    } else
    {
	$Q = "DELETE FROM report_priv WHERE";
	$Q .= " reportsn = $ReportSn AND privilegesn = $PrivilegeSn";
    }
    Query($Q);
}

function UpdateReportPrivileges($ReportSn, $NumRows)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    Query("BEGIN");
    for ($Row = 0 ; $Row < $NumRows ; $Row ++)
	UpdateReportPrivilege($ReportSn, $FormVars["PrvSn$Row"],
			    $FormVars["Grant$Row"]);
    $PublicSn = SerialNumber("privilege", "privname", "public");
    $AnyLoginSn = SerialNumber("privilege", "privname", "anylogin");
    $Q = "DELETE FROM report_priv WHERE reportsn = $ReportSn";
    $Q .= " AND privilegesn IN ($PublicSn, $AnyLoginSn) AND EXISTS";
    $Q .= " (SELECT * FROM report_priv WHERE reportsn = $ReportSn";
    $Q .= " AND privilegesn NOT IN ($PublicSn, $AnyLoginSn))";
    Query($Q);
    Query("COMMIT");
    list($ReportName, $Description) = ReportNameAndDesc($ReportSn);
    return "Privileges updated for $ReportName<br \>\n";
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "Description",
    "NewDescription",
    "NewReportName",
    "NumReports",
    "Ordinal",
    "Report",
    "ReportName",
    "ReportSn",
    "Seq"
    );
$GetVarList = array(
    
   );
$PostVarList = array(
    
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S = HeadAndTitle("$SystemTitle Report Management");

$S .= PageTitleWithButton("Report Management", BtnMainMenu());
$S .= "<div align=\"center\">\n";

$N = "";

if (isset($Ordinal)) ReorderReport($ReportName, $Ordinal, $NumReports);
elseif (isset($FormVars["UpdateReport"]))
{
    $Q = "SELECT privname, privilegesn FROM privilege";
    $Prv = Query($Q);
    $NumRows = pg_num_rows($Prv);
    $N .= UpdateReportPrivileges($ReportSn, $NumRows);
    $N .= UpdateReportNameAndDesc($ReportSn, $NewReportName,
				  $NewDescription);
} elseif (isset($FormVars["Public"])) MakeReportPublic($ReportSn);
elseif (isset($FormVars["AnyLogin"])) AllowForAnyLogin($ReportSn);

elseif (isset($FormVars["AddPriv"]))
    $N .= StrAddReport($Report, $Seq, $Description);
elseif (isset($FormVars["AddPub"]))
{
    $N .= StrAddReport($Report, $Seq, $Description);
    $ReportSn = SerialNumber("report", "name", $Report);
    MakeReportPublic($ReportSn);
} elseif (isset($FormVars["AddAnyLogin"]))
{
    $N .= StrAddReport($Report, $Seq, $Description);
    $ReportSn = SerialNumber("report", "name", $Report);
    AllowForAnyLogin($ReportSn);
} elseif (isset($FormVars["Delete"])) $N .= DeleteReport($ReportSn);

if ($W > "") $S .= StrWarningBox($W);

if ($N > "") $S .= StrWarning($N, NoticeColor);

if (isset($FormVars["Manage"])) // We're in report management mode
    $S .= StrReportMaintenanceTable($Report);
else $S .= TablesOfReports();

$S .= "</div></body></html>\n";
echo $S;

?>
