<?php
/*********************************************************************
gk_managetool.php

Manage tool privileges

Designed to be accessed via an include/require statement in the
otherwise empty ../managetool.php script
______________________________________________________________________
Copyright 2013 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require_once ("lib_common.php");
require_once ("lib_layout.php");

function AllowForAnyLogin($ToolSn)
{
    // First reset all privileges for tool, then set "anylogin"
    Query("DELETE FROM tool_priv WHERE toolsn = $ToolSn");
    $AnyLoginSn = SerialNumber("privilege", "privname", "anylogin");
    UpdateToolPrivilege($ToolSn, $AnyLoginSn, True);
    return;
}

function DeleteTool($ToolSn)
{
    list($ToolName, $Description) = ToolNameAndDesc($ToolSn);
    Query("BEGIN");
    foreach (array("hiddentool", "tool_priv", "tool") as $Table)
    {
	Query("DELETE FROM $Table WHERE toolsn = $ToolSn");
    }
    Query("COMMIT");
    return "Deleted $ToolName<br />\n";
}

function MakeToolPublic($ToolSn)
{
    // First reset all privileges for tool, then set "public"
    Query("DELETE FROM tool_priv WHERE toolsn = $ToolSn");
    $PublicSn = SerialNumber("privilege", "privname", "public");
    UpdateToolPrivilege($ToolSn, $PublicSn, True);
    return;
}

function ReorderTool($ToolName, $NewOrdinal, $NumTools)
{
    $Q = "SELECT dispord FROM tool WHERE name = $1";
    $Ord = Query($Q, $ToolName);
    $OldOrdinal = pg_fetch_result($Ord, 0, "dispord");
    if ($NewOrdinal != $OldOrdinal)
    {
	$NewOrdinal = BoundedValue($NewOrdinal, $NumTools);
	$PublicSn = SerialNumber("privilege", "privname", "public");
	$IsPublic = (IsToolPublic($ToolName)
		     ? "" : "!")."= $PublicSn";
	if ($NewOrdinal < $OldOrdinal)
	{
	    $Q = "UPDATE tool t SET dispord = (dispord+1)";
	    $Q .= " WHERE dispord >= $1";
	    $Q .= " AND dispord < $OldOrdinal";
	    $Q .= " AND EXISTS (SELECT * FROM tool_priv";
	    $Q .= " WHERE toolsn = t.toolsn";
	    $Q .= " AND privilegesn $IsPublic)";
	    Query($Q, $NewOrdinal);
	} elseif ($NewOrdinal > $OldOrdinal)
	{
	    $Q = "UPDATE tool t SET dispord = (dispord-1)";
	    $Q .= " WHERE dispord > $OldOrdinal";
	    $Q .= " AND dispord <= $1";
	    $Q .= " AND EXISTS (SELECT * FROM tool_priv";
	    $Q .= " WHERE toolsn = t.toolsn";
	    $Q .= " AND privilegesn $IsPublic)";
	    Query($Q, $NewOrdinal);
	}
	$Q = "UPDATE tool SET dispord = $1 WHERE name = $2";
	Query($Q, $NewOrdinal, $ToolName);
    }
    return;
}

function StrAddTool($Name, $Seq, $Description)
{
    $Name = trim($Name);
    $Seq = trim($Seq);
    $Description = trim($Description);
    /* $Description = stripslashes(trim($Description)); */
    $N = "";
    if ( ! ($Description > "") )
	$N .= "You must provide a description<br />\n";
    if ( ! ($Name > "") ) $N .= "You must name the tool<br />\n";
    if (IsInDatabase("tool", "name", $Name))
	$N .= "That name is already taken<br />\n";
    if ($N > "") return $N;
    // Open a slot for new tool
    $Q = "UPDATE tool SET dispord = dispord + 1";
    $Q .= " WHERE dispord >= $1 AND NOT EXISTS";
    $Q .= " (SELECT * FROM tool WHERE name = $2)";
    Query($Q, $Seq, $Name);
    // Add the tool
    $Q = "INSERT INTO tool(name, dispord, description)";
    $Q .= " SELECT $1, $2, $3 WHERE NOT EXISTS";
    $Q .= " (SELECT * FROM tool WHERE name = $1)";
    if (Query($Q, $Name, $Seq, $Description))
	return "$Name added<br />\n";
    else return "$Name not added<br />\n";
}

function StrAddToolTable()
{
    $S = StrTableBegin("1", TableColor);
    $S .= StrRowBegin();
    $S .= StrHeaderCell("Name");
    $S .= StrHeaderCell("Seq");
    $S .= StrHeaderCell("Description");
    $S .= StrHeaderCell("Maintenance");
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $S .= StrFormBeginRecurse();
    $S .= StrCell(HtmlTextInput("Tool","25"));
    $S .= StrCell(HtmlTextInput("Seq","5"));
    $S .= StrCell(HtmlTextInput("Description","45"));
    $C = "<td align=\"center\">";
    $C .= HtmlSubmitButton("AddPub","Add Public", "Public");
    $C .= HtmlSubmitButton("AddPriv","Add Private", "Private");
    $C .= HtmlSubmitButton("AddAnyLogin","Add for Any Login",
			   "AnyLogin");
    $C .= "</td>";
    $S .= $C;
    $S .= StrFormEnd();
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function StrBeginToolTable()
{
    $S = StrTableBegin("1", TableColor);
    $S .= StrRowBegin();
    $S .= StrHeaderCell("Name");
    $S .= StrHeaderCell("Position");
    $S .= StrHeaderCell("Description");
    $S .= StrHeaderCell("Required Privileges");
    $S .= StrHeaderCell("Maintenance");
    $S .= StrRowEnd();
    return $S;
}

function StrSelectOrdinal($ToolName, $Ordinal, $NumTools)
{
    // Draws the drop-down box
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("ToolName", $ToolName);
    $S .= HtmlHidden("NumTools", $NumTools);
    $S .= HtmlSelectBegin("Ordinal", 'onchange="this.form.submit()"');
    for ($I = 1; $I <= $NumTools ; $I++)
	$S .= HtmlSelectOption($I, $I, ($I == $Ordinal));
    $S .= HtmlSelectEnd();
    $S .= StrFormEnd();
    return $S;
}

function StrToolDisplayTable($Tools)
{
    $NumTools = pg_num_rows($Tools);
    for ($Row = 0 ; $Row < $NumTools ; $Row ++)
    {
	list($Name, $Dispord, $Description, $ToolSn)
	    = pg_fetch_array($Tools, $Row);
	$Q = "SELECT privname FROM tool_priv tp, privilege p";
	$Q .= " WHERE toolsn = $ToolSn";
	$Q .= " AND p.privilegesn = tp.privilegesn";
	$Q .= " ORDER BY privname";
	$Prv = Query($Q);
	$P = "";
	for ($PRow = 0 ; $PRow < pg_num_rows($Prv) ; $PRow ++)
	{
	    $P .= ", ".pg_fetch_result($Prv, $PRow, "privname");
	}
	if ($Dispord != $Row+1) 
	{
	    $Q = "UPDATE tool SET dispord = $Row+1";
	    $Q .= " WHERE name = '$Name'";
	    Query($Q);
	}
	$S .= StrRowBegin();
	$S .= StrCell($Name);
	$S .= StrCell(StrSelectOrdinal($Name, $Row+1, $NumTools));
	if ("$Description" == "") $Description = "&nbsp;";
	$S .= StrCell($Description);
	$S .= ($P > "") ? StrCell(SubStr($P, 2)) : StrCell("&nbsp;");
	$C = "<td align=\"center\">";
	$C .= StrFormBeginRecurse();
	$C .= HtmlHidden("Tool", $Name);
	$C .= HtmlSubmitButton("Manage");
	$C .= StrFormEnd();
	$C .= "</td>";
	$S .= $C;
	$S .= StrRowEnd();
    }
    $S .= StrRowBegin();
    $S .= StrRowEnd();
    return $S;
}

function StrToolMaintenanceTable($Name)
{
    $Q = "SELECT name, dispord, description, toolsn FROM tool";
    $Q .= " WHERE name = $1";
    list($Name, $Dispord, $Description, $ToolSn)
	= pg_fetch_array(Query($Q, $Name), 0);
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("ToolSn", $ToolSn);
    $S .= StrBeginToolTable();
    $S .= StrRowBegin();
    $S .= StrCell(HtmlTextInput("NewToolName", "25", $Name));
    $S .= StrCell($Dispord, "", "2");
    $S .= StrCell(HtmlTextInput("NewDescription", "45",
				$Description));
    $Q = "SELECT privname, privilegesn FROM privilege";
    $Q .= " ORDER BY privname";
    $Res = Query($Q);
    $C ="";
    for ($PRow = 0 ; $PRow < pg_num_rows($Res) ; $PRow ++)
    {
	list($CurrPriv, $PrivilegeSn)
	    = pg_fetch_array($Res, $PRow);
	$Q = "SELECT * FROM tool_priv WHERE toolsn = $ToolSn";
	$Q .= " AND privilegesn = $PrivilegeSn";
	$C .= "<label>";
	$C .= HtmlCheckbox("Grant".$PRow, pg_num_rows(Query($Q)));
	$C .= "$CurrPriv</label>";
	$C .= HtmlHidden("PrvSn".$PRow, $PrivilegeSn);
	$C .= "<br />\n";
    }
    $S .= StrCell($C, "", "", "white-space: nowrap");
    $C = HtmlSubmitButton("UpdateTool", "Submit Changes", $Name);  
    $Q = "SELECT * FROM tool_priv tp, privilege p";
    $Q .= " WHERE toolsn = $ToolSn AND privname = 'public'";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    if (pg_num_rows(Query($Q)) == 0)
	$C .= HtmlSubmitButton("Public", "Make Public", $Name);
    $Q = "SELECT * FROM tool_priv tp, privilege p";
    $Q .= " WHERE toolsn = $ToolSn AND privname = 'anylogin'";
    $Q .= " AND tp.privilegesn = p.privilegesn";
    if (pg_num_rows(Query($Q)) == 0)
	$C .= HtmlSubmitButton("AnyLogin", "Allow for Any Login ",
			       $Name);
    $C .= HtmlSubmitButton("Cancel");
    $S .= "<td style=\"text-align: center; height: 100%\">";
    $S .= StrTableBegin("0", "", "", "height: 100%").StrRowBegin();
    $S .= "<td align=\"center\">".$C."</td>";
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $C = HtmlSubmitButton("Delete", "", "",
			  "border: medium solid ".WarningColor);
    $S .= "<td style=\"text-align: right; vertical-align: bottom\">";
    $S .= $C."</td>";
    $S .= StrRowEnd().StrTableEnd()."</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function TablesOfTools()
{
    // We're in tool display mode; show public tools
    $S = StrAddToolTable();
    $S .= StrSubTitle("Public Tools");
    $S .= StrBeginToolTable();
    $Q = "SELECT name, dispord, description, toolsn";
    $Q .= " FROM tool t0 WHERE EXISTS";
    $Q .= " (SELECT * FROM tool_priv tp, tool t, privilege p";
    $Q .= " WHERE t.name = t0.name AND privname='public'";
    $Q .= " AND tp.toolsn = t.toolsn";
    $Q .= " AND tp.privilegesn = p.privilegesn)";
    $Q .= " ORDER BY dispord, name";
    $S .= StrToolDisplayTable(Query($Q));
    $S .= StrTableEnd();
    // show private tools
    $S .= StrSubTitle("Private Tools");
    $S .= StrBeginToolTable();
    $Q = "SELECT name, dispord, description, toolsn";
    $Q .= " FROM tool t0 WHERE NOT EXISTS";
    $Q .= " (SELECT * FROM tool_priv tp, tool t, privilege p";
    $Q .= " WHERE t.name = t0.name AND privname='public'";
    $Q .= " AND tp.toolsn = t.toolsn";
    $Q .= " AND tp.privilegesn = p.privilegesn)";
    $Q .= " ORDER BY dispord, name";
    $S .= StrToolDisplayTable(Query($Q));
    $S .= StrTableEnd();
    return $S;
}

function ToolNameAndDesc($ToolSn)
{
    $Q = "SELECT name, description FROM tool WHERE toolsn = $ToolSn";
    return pg_fetch_array(Query($Q), 0);
}

function UpdateToolNameAndDesc($ToolSn, $NewToolName, $NewDescription)
{
    list($OldName, $OldDescription) = ToolNameAndDesc($ToolSn);
    $S = "";
    /* $NewDescription = stripslashes($NewDescription); */
    /* $OldDescription = stripslashes($OldDescription); */
    $Name = $OldName;
    if ($NewToolName != "" AND $NewToolName != $OldName)
    {
	$Q = "UPDATE tool SET name = $1";
	$Q .= " WHERE toolsn = $ToolSn AND name != $1";
	Query($Q, $NewToolName);
	$S .= "Renamed $OldName to $NewToolName<br />\n";
	$Name = $NewToolName;
    }
    if ($NewDescription != "" AND $NewDescription != $OldDescription)
    {
	$Q = "UPDATE tool SET description = $1";
	$Q .= " WHERE toolsn = $2";
	$Q .= " AND description != $1";
	Query($Q, $NewDescription, $ToolSn);
	$S .= "Updated description for $Name<br />\n";
    }
    return $S;
}

function UpdateToolPrivilege($ToolSn, $PrivilegeSn, $Grant=False)
{
    if ($Grant)
    {
	$Q = "INSERT INTO tool_priv(toolsn, privilegesn) SELECT";
	$Q .= " $ToolSn, $PrivilegeSn WHERE NOT EXISTS";
	$Q .= " (SELECT * FROM tool_priv WHERE";
	$Q .= " toolsn = $ToolSn AND privilegesn = $PrivilegeSn)";
    } else
    {
	$Q = "DELETE FROM tool_priv WHERE";
	$Q .= " toolsn = $ToolSn AND privilegesn = $PrivilegeSn";
    }
    Query($Q);
}

function UpdateToolPrivileges($ToolSn, $NumRows)
{
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    Query("BEGIN");
    for ($Row = 0 ; $Row < $NumRows ; $Row ++)
	UpdateToolPrivilege($ToolSn, $FormVars["PrvSn$Row"],
			    $FormVars["Grant$Row"]);
    $PublicSn = SerialNumber("privilege", "privname", "public");
    $AnyLoginSn = SerialNumber("privilege", "privname", "anylogin");
    $Q = "DELETE FROM tool_priv WHERE toolsn = $ToolSn";
    $Q .= " AND privilegesn IN ($PublicSn, $AnyLoginSn) AND EXISTS";
    $Q .= " (SELECT * FROM tool_priv WHERE toolsn = $ToolSn";
    $Q .= " AND privilegesn NOT IN ($PublicSn, $AnyLoginSn))";
    Query($Q);
    Query("COMMIT");
    list($ToolName, $Description) = ToolNameAndDesc($ToolSn);
    return "Privileges updated for $ToolName<br \>\n";
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "Description",
    "NewDescription",
    "NewToolName",
    "NumTools",
    "Ordinal",
    "Seq",
    "Tool",
    "ToolSn",
    "ToolName"
    );
$GetVarList = array(
    
   );
$PostVarList = array(
    
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S = HeadAndTitle("$SystemTitle Tool Management");

$S .= PageTitleWithButton("Tool Management", BtnMainMenu());
$S .= "<div align=\"center\">\n";

$N = "";

if (isset($Ordinal)) ReorderTool($ToolName, $Ordinal, $NumTools);
elseif (isset($FormVars["UpdateTool"]))
{
    $Q = "SELECT privname, privilegesn FROM privilege";
    $Prv = Query($Q);
    $NumRows = pg_num_rows($Prv);
    $N .= UpdateToolPrivileges($ToolSn, $NumRows);
    $N .= UpdateToolNameAndDesc($ToolSn, $NewToolName,
				$NewDescription);
} elseif (isset($FormVars["Public"])) MakeToolPublic($ToolSn);
elseif (isset($FormVars["AnyLogin"])) AllowForAnyLogin($ToolSn);

elseif (isset($FormVars["AddPriv"]))
    $N .= StrAddTool($Tool, $Seq, $Description);
elseif (isset($FormVars["AddPub"]))
{
    $N .= StrAddTool($Tool, $Seq, $Description);
    $ToolSn = SerialNumber("tool", "name", $Tool);
    MakeToolPublic($ToolSn);
} elseif (isset($FormVars["AddAnyLogin"]))
{
    $N .= StrAddTool($Tool, $Seq, $Description);
    $ToolSn = SerialNumber("tool", "name", $Tool);
    AllowForAnyLogin($ToolSn);
} elseif (isset($FormVars["Delete"])) $N .= DeleteTool($ToolSn);

if ($W > "") $S .= StrWarningBox($W);

if ($N > "") $S .= StrWarning($N, NoticeColor);

if (isset($FormVars["Manage"])) // We're in tool management mode
    $S .= StrToolMaintenanceTable($Tool);
else $S .= TablesOfTools();

$S .= "</div></body></html>\n";
echo $S;

?>