<?php
/*********************************************************************
gk_manageuser.php

Manage user accounts

Designed to be accessed via an include/require statement in the
otherwise empty ../manageuser.php script

______________________________________________________________________
Copyright 2010 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_crypto.php");
require_once ("lib_layout.php");
require ("lib_usermgt.php");

function StrSearchResults($LastName, $FirstName, $UserId,
			  $SepWin=False)
{
    if ( ! IsTrimmedUserKeyValid($LastName, $FirstName, $UserId))
    {
	$S = StrWarning("You must enter a search key (partial ok).");
	$S .= StrUserSearchForm("", "", "", "", $SepWin);
	$S .= HtmlPara(Strong("or"));
	$S .= StrUserAccountForm(False, False, $SepWin);
    } else
    {
	$Res = ResUserByNameLogin($LastName, $FirstName, $UserId);
	$Hits = pg_num_rows($Res);
	if (! $Res or $Hits < 1)
	{
	    $S = StrUserNotFound($LastName, $FirstName, $UserId,
				 $SepWin);
	    $S .= HtmlPara(Strong("or"));
	    $S .= StrUserAccountForm(False, False, $SepWin);
	} elseif ($Hits > 1)
	    $S = StrUsersMultiple($LastName, $FirstName, $UserId,
				  $Res, $Hits, $SepWin);
	else $S = StrUserAccountScreen($Res, False, $SepWin);
    }
    return $S;
}

function StrUserAccountScreen($Res=False, $InProgress=False,
			      $SepWin=False)
{
    $S = StrUserAccountForm($Res, $InProgress, $SepWin);
    $S .= "<h3 align=\"center\">Select another user:</h3>\n";
    $S .= StrUserSearchForm("", "", "", "", $SepWin);
    $S .= HtmlPara(Strong("or"));
    $S .= StrUserAccountForm(False, False, $SepWin);
    return $S;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "FirstName",
    "LastName",
    "OldUserId",
    "Password",
    "Password2",
    "SearchForUser",
    "SepWin",
    "UserId"
    );
$GetVarList = array(
    "SearchForUser",
    "SepWin",
    "UserId"
   );
$PostVarList = array(
    
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S = HeadAndTitle("$SystemTitle Manage User Accounts");
$S .= "<div align=\"center\">\n";
$S .= PageTitleWithButton("Manage User Accounts",
			  $SepWin ? BtnClose() : BtnMainMenu());

if (isset($SearchForUser))
    $S .= StrSearchResults($LastName, $FirstName, $UserId, $SepWin);
elseif (isset($FormVars["AddUser"]))
{
    $Err = UserAddStatus($UserId, $LastName, $FirstName, $Password,
			 $Password2);
    if ($Err > "") $S .= StrWarning($Err);
    else
    {
	$T = "Added new user $FirstName $LastName ($UserId)";
	$S .= StrWarning($T, NoticeColor);
    }
    $S .= "<h3 align=\"center\">Select a User</h3>\n";
    $S .= StrUserSearchForm("", "", "", "", $SepWin);
    $S .= HtmlPara(Strong("or"));
    $S .= StrUserAccountForm(False, ($Err > ""), $SepWin);
} elseif (isset($FormVars["DeleteUser"]))
{
    $Err = UserDeleteErrors($OldUserId);
    if ($Err > "")
    {
	$S .= StrWarningBox($Err);
	$UserId = $OldUserId;
	$Res = ResUserByNameLogin("", "", $OldUserId);
	$S .= StrUserAccountScreen($Res, True, $SepWin);
    } else
    {
	DeleteUser($OldUserId);
	$T = "Deleted user $OldUserId";
	$S .= StrWarning($T, NoticeColor);
	$S .= "<h3 align=\"center\">Select a User</h3>\n";
	$S .= StrUserSearchForm("", "", "", "", $SepWin);
	$S .= HtmlPara(Strong("or"));
	$S .= StrUserAccountForm(False, False, $SepWin);
    }
} elseif (isset($FormVars["UpdateUser"]))
{
    $Err = UserUpdateStatus($OldUserId, $LastName, $FirstName,
			    $Password, $Password2);
    if ($Err > "")
    {
	$Res = ResUserByNameLogin("", "", $OldUserId);
	$S .= StrWarning($Err);
    } else
    {
	/* $FirstName = stripslashes($FirstName); */
	/* $LastName = stripslashes($LastName); */
	$Res = ResUserByNameLogin("", "", $OldUserId);
	$T = "Updated user $FirstName $LastName ($OldUserId)";
	$S .= StrWarning($T, NoticeColor);
    }
    $S .= StrUserAccountScreen($Res, ($Err > ""), $SepWin);
} else
{
    $S .= "<h3 align=\"center\">Select a User</h3>\n";
    $S .= StrUserSearchForm("", "", "", "", $SepWin);
    $S .= HtmlPara(Strong("or"));
    $S .= StrUserAccountForm(False, False, $SepWin);
}

$S .= "</div></body></html>\n";
echo $S;

?>
