<?php
/*********************************************************************
gk_myaccount.php

Manage logged-in user's own account
______________________________________________________________________
Copyright 2013 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
********************************************************************/
require ("lib_crypto.php");
require ("lib_usermgt.php");

function AltAuthStatus($UserId, $Input)
{
    $Challenge1 = $Input["Challenge1"];
    $Response1 = $Input["Response1"];
    $Response1b = $Input["Response1b"];
    $Challenge2 = $Input["Challenge2"];
    $Response2 = $Input["Response2"];
    $Response2b = $Input["Response2b"];
    $MyUserRec = ResUserByNameLogin("", "", $UserId);
    $E = ErrBlank("Challenge 1", $Challenge1);
    $E .= ErrMisMatch("Response 1", $Response1, $Response1b);
    $E .= ErrBlank("Challenge 2", $Challenge2);
    $E .= ErrMisMatch("Response 2", $Response2, $Response2b);
    $Response1 = SansSpecialChars($Response1);
    $Response2 = SansSpecialChars($Response2);
    $E .= ErrBlank("Response 1", $Response1);
    $E .= ErrBlank("Response 2", $Response2);
    if ($E > "")
    {
	$S = StrWarningBox("Update failed:<br />\n<ul>\n.$E.</ul>\n");
	$S .= StrUserAccountForm($MyUserRec, False, False,
				 // Tag so main can route properly
				 HtmlHidden("MngAcct", "True"));
	$S .= FormAltAuthSetup($MyUserRec, $Challenge1, $Challenge2);
    } else
    {
	UpdateAltAuth($UserId, $Challenge1, $Response1, $Challenge2,
		      $Response2);
	$S = StrWarning("Updated alternate authentication");
	$S .= PageMyAccount($MyUserRec);
    }
    return $S;
}

function ErrBlank($Label, $Value)
{
    return (trim($Value) > "") ? ""
	: ErrBullet("$Label may not be blank");
}

function ErrBullet($Text)
{
    // return list item enclosing text ended in period
    return "<li>$Text.</li>\n";
}

function ErrMisMatch($Label, $Value1, $Value2)
{
    return ($Value1 == $Value2) ? ""
	: ErrBullet("The two entries for $Label must agree");
}

function FormAltAuthSetup($MyUserRec, $Challenge1="", $Challenge2="")
{
    $UserId = pg_fetch_result($MyUserRec, 0, "userid");
    if ($Challenge1.$Challenge2 == "")
    {
	$Res = ResAltAuth($UserId);
	if ($Res)
	    list ($Challenge1, $Challenge2) = pg_fetch_array($Res, 0);
    }
    $S = StrSubTitle("Alternate Authentication");
    $S .= "<p style=\"width: 70%; text-align: left\">";
    $S .= AltAuthSetupInvitation."</p>\n";
    $S .= StrWarningBox(AltAuthWarning, "50%");
    $S .= StrFormBeginRecurse();
    $S .= StrTableBegin("0");
    $S .= RowAltAuthSetup(1, $Challenge1);
    $S .= "<tr><td colspan=\"2\">&nbsp;</td></tr>\n";
    $S .= RowAltAuthSetup(2, $Challenge2);
    $S .= StrTableEnd();
    $S .= "<table border=\"0\" cellspacing=\"10\"";
    $S .= " cellpadding=\"4\">\n".StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("UpdateAltAuth", "Update"));
    $S .= StrCell("");
    $S .= StrCell(HtmlResetButton());
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    // Tag so main can route properly
    $S .= HtmlHidden("MngAcct", "True");
    $S .= StrFormEnd();
    return $S;
}

function PageMyAccount($MyUserRec)
{
    $S = StrUserAccountForm($MyUserRec, False, False,
			    // Tag so main can route properly
			    HtmlHidden("MngAcct", "True"));
    if (AlternateAuthOK) $S .= FormAltAuthSetup($MyUserRec);
    return $S;
}

function ResAltAuth($UserId)
{
    $Q = "SELECT challenge1, challenge2 FROM altauth a, reguser r";
    $Q .= " WHERE r.userid = $1 and a.regusersn = r.regusersn";
    $Res = Query($Q, $UserId);
    return (pg_num_rows($Res) > 0) ? $Res : False;
}

function RowAltAuthSetup($Ordinal, $Challenge="")
{
    $T = HtmlTextArea("Challenge$Ordinal", "60", "2", $Challenge);
    $S = "Question or hint for which only you know the response";
    $S = FormLine($S, "Challenge $Ordinal", $T);
    $S .= StrUserAccountFormLine("Answer to challenge $Ordinal",
				 "Response $Ordinal",
				 "Response$Ordinal", "60", "", True);
    $S .= StrUserAccountFormLine("Retype to guard against mistyping",
				 "Repeat Response $Ordinal",
				 "Response${Ordinal}b",
				 "60", "", True);
    return $S;
}

function UpdateAltAuth($UserId, $Challenge1, $Response1, $Challenge2,
		       $Response2)
{
    $Cypher1 = Encrypted(strtolower($Response1));
    $Cypher2 = Encrypted(strtolower($Response2));
    $ReguserSn = SerialNumber("reguser", "userid", $UserId);
    $Keys = array(array("regusersn", $ReguserSn));
    $Data = array(array("challenge1", $Challenge1),
		  array("response1", $Cypher1),
		  array("challenge2", $Challenge2),
		  array("response2", $Cypher2));
    ResInsertOrUpdate("altauth", $Keys, $Data);
    return;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "FirstName",
    "LastName",
    "OldUserId",
    "Password",
    "Password2"
    );
$GetVarList = array(
    
   );
$PostVarList = array(
    
   );
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

$S0 = HeadAndTitle("$SystemTitle Manage My User Account");
// Page header might change if we change user's name:
$ST = PageTitleWithButton("Manage My User Account", BtnMainMenu());
$S = "<div align=\"center\">\n";

$UserId = LoggedInUser();
if ($UserId)
{
    if (isset($FormVars["UpdateUser"]))
    {
	$Err = UserUpdateStatus($OldUserId, $LastName, $FirstName,
				$Password, $Password2, True);
	$Res = ResUserByNameLogin("", "", $OldUserId);
	if ($Err > "") $S = $S0.$ST.$S.StrWarning($Err);
	else
	{
	    $T = "Updated user $FirstName $LastName ($OldUserId)";
	    // Refresh user record
	    $Q = "SELECT r.userid, r.passwd_hash, r.passwd_changed,";
	    $Q .= " r.personsn, p.*";
	    $Q .= " FROM reguser r, person p";
	    $Q .= " WHERE r.userid = $1";
	    $Q .= " AND p.personsn = r.personsn";
	    $_SESSION["UserRec"]
		= pg_fetch_array(Query($Q, $UserId), 0);
	    // Update page header in case user's name changed:
	    $ST = PageTitleWithButton("Manage My User Account",
				      BtnMainMenu());
	    $S = $S0.$ST.$S.StrWarning($T, NoticeColor);
	}
	$S .= PageMyAccount($Res);
    } elseif (AlternateAuthOK and isset($FormVars["UpdateAltAuth"]))
    {
	$S = $S0.$ST.$S;
	$S .= AltAuthStatus($UserId, $FormVars);
    } elseif (isset($FormVars["DeleteUser"]))
    {
	$S = $S0.$ST.$S;
	$Err = UserDeleteErrors($OldUserId);
	if ($Err > "")
	{
	    $S .= StrWarningBox($Err);
	    $Login = $OldUserId;
	    $Res = ResUserByNameLogin("", "", $OldUserId);
	    $S .= PageMyAccount($Res, True);
	} else
	{
	    DeleteUser($OldUserId);
	    $T = "Deleted user $OldUserId";
	    ClearSession();
	    $WarnDeletedUser = StrWarning($T, NoticeColor);
	}
    } else
    {
	$S = $S0.$ST.$S;
	// Offer update user form
	$Res = ResUserByNameLogin("", "", $UserId);
	if (! $Res or pg_num_rows($Res) != 1)
	{
	    $W = "Found no user matching";
	    $W .= StrUserKeyDescription("", "", $UserId);
	    $S .= StrWarningBox($W);
	} else $S .= PageMyAccount($Res);
    }
} else
{
    $S = $S0.$ST.$S;
    if (isset($FormVars["AddUser"]) and UsersMayCreateOwnAccounts)
    {
	$Err = UserAddStatus($Login, $LastName, $FirstName, $Password,
			     $Password2);
	if ($Err > "")
	{
	    $S .= StrWarning($Err);
	    $S .= StrUserAccountForm(False, True, False,
				     // Tag so main can route properly
				     HtmlHidden("MngAcct", "True"));
	} else
	{
	    $T = "Added new user $FirstName $LastName ($Login).";
	    $S .= StrWarning($T, NoticeColor);
	    $T = "$FirstName $LastName is now logged in.<br>\n";
	    $T .= "Be sure to log out when done.<br>\n";
	    $S .= StrWarningBox($T);
	    // retrieve the user record corresponding to userid
	    $Q = "SELECT r.userid, r.passwd_hash, r.passwd_changed,";
	    $Q .= " r.personsn, p.*";
	    $Q .= " FROM reguser r, person p";
	    $Q .= " WHERE r.userid = $1";
	    $Q .= " AND p.personsn = r.personsn";
	    $U = Query($Q, $Login);
	    // Populating $_SESSION["UserRec"] creates a logged-in
	    // session
	    $_SESSION["UserRec"] = pg_fetch_array($U, 0);
	}
    } else
    {
	$W = "To manage your account, you must first log in.";
	$S .= StrWarning($W, NoticeColor);
	$S .= StrWarning(AccountInvitation, NoticeColor);
	if (UsersMayCreateOwnAccounts)
	    $S .= StrUserAccountForm(False, False, False,
				     // Tag so main can route properly
				     HtmlHidden("MngAcct", "True"));
    }
}

?>
