<?php
/*********************************************************************
gk_reports.php

Definitions related to report generation and access
______________________________________________________________________
Copyright 2014 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
define ("ClauseReportsAnyLogin",
"FROM privilege p, report_priv rp, report r
WHERE privname = 'anylogin' AND rp.privilegesn = p.privilegesn
AND r.reportsn = rp.reportsn AND EXISTS
(SELECT 1 FROM reguser WHERE userid = $1)");

define ("ClauseReportsPrivate",
"FROM reguser u, user_priv up, report_priv rp, report r
WHERE userid = $1 AND up.regusersn = u.regusersn
AND rp.privilegesn = up.privilegesn and r.reportsn = rp.reportsn");

define ("ClauseReportsPublic",
"FROM privilege p, report_priv rp, report r
WHERE privname = 'public' AND rp.privilegesn = p.privilegesn
AND r.reportsn = rp.reportsn");

function FieldLabel($Field)
{
    global $FieldLabels;
    return $FieldLabels[$Field];
}

function HiddenArgs($Keys, $OtherHidden="")
{
    $S = "";
    foreach ($Keys as $Index => $Key)
	$S .= HtmlHidden("K$Index", $Key);
    $S .= $OtherHidden;
    $S .= HtmlHidden("KeysSelected", "True");
    return $S;
}

function KeyDescription($KeySQL)
{
    global $FieldLabels, $FieldKeys;
    return FieldLabel(array_search($KeySQL, $FieldKeys));    
}

function KeyDescriptions($Keys)
{
    $S = "";
    foreach ($Keys as $KeySQL)
	$S .= ", ".KeyDescription($KeySQL);
    return "(".substr($S, 2).")";
}

function KeySelections($Input)
{
    global $FieldKeys;
    $OldKey = array($Input["K0"]);
    if (isset($Input["K1"]))
    {
	$OldKey[] = $Input["K1"];
	if (isset($Input["K2"])) $OldKey[] = $Input["K2"];
    }
    $NewKey = False;
    foreach (array_keys($FieldKeys) as $Field)
    {
	$Key = "K$Field";
	if (isset($Input[$Key])) $NewKey = $FieldKeys[$Field];
    }
    $OldKey = array_diff($OldKey, array($NewKey));
    return $NewKey
	? array_slice(array_merge(array($NewKey), $OldKey), 0, 3)
	: $OldKey;
}

function MenuAllowedReports($Selected, $AutoSubmit=True)
{
    $UserId = LoggedInUser();
    $Res = ResAllowedReports($UserId);
    $NumRows = pg_num_rows($Res);
    if ($NumRows > 0)
    {
	$Attribs = ($AutoSubmit)
	    ? "onchange=\"this.form.submit()\"" : "";
	$S = HtmlSelectBegin("ReportName", $Attribs);
	if ($Selected == "" or $Selected == "none")
	    $S .= HtmlSelectOption("none",
				   "&mdash;Select a report&mdash;");
	for ($Row = 0 ; $Row < $NumRows ; $Row++)
	{
	    list ($Name, $Description) = pg_fetch_array($Res, $Row);
	    $S .= HtmlSelectOption($Name, $Description,
				   ($Selected === $Name));
	}
	$S .= HtmlSelectEnd();
    } else
    {
	$T = "There are currently no reports available for ";
	$T .= $User ? "user &quot;$User&quot;." : "public use." ;
	$S = StrWarningBox($T);
    }
    return $S;
}

function OrderSpec($Keys)
{
    $S = "";
    foreach ($Keys as $Key) $S .= ", $Key";
    return substr($S, 2);
}

function ResAllowedReports($UserId)
{
    /* Return list of reports $UserId may use, including public
     * reports and any with the pseudo-privilege "anylogin" */
    $T = "SELECT r.name, r.description, dispord ";
    $Params=array();
    $Q = $T.ClauseReportsPublic;
    if ($UserId !== False)
    {
	$Params[] = $UserId;
	$Q = "SELECT name, description FROM ($Q";
	$Q .= " UNION $T".ClauseReportsPrivate;
	$Q .= " UNION $T".ClauseReportsAnyLogin.") as z";
    }
    $Q .= " ORDER BY dispord";
    return QueryArray($Q, $Params);
}

function ThisReportDescription()
{
    $Q = "SELECT description FROM report WHERE name = $1";
    return pg_fetch_result(Query($Q, ThisScript()), 0, "description");
}

?>