<?php
/*********************************************************************
gk_showfunctions.php

Displays a table of all functions defined in all shared PHP programs
in current directory and subdirectories. Shared programs are those
intended to be included by others. The application identifies shared
programs by the convention that they have names of the forms:

    gk_*.php
    lib_*.php
    loc_*php

Designed to be accessed via an include/require statement in the
otherwise empty ../showfunctions.php script
______________________________________________________________________
Copyright 2010 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require_once ("lib_common.php");
require ("lib_filesys.php");
require_once ("lib_layout.php");

function FunctionsByName ($Files)
{
    $Cmd ="for File in $Files\n";
    $Cmd .= "do\n";
    $Cmd .= "    cat \$File";
    $Cmd .= " | sed -e 's/^function/zqzfunction/'";
    $Cmd .= ' | tr -s " 	" "\n\n"';
    $Cmd .= " | sed -ne '/^zqzfunction\$/,/)/p'";
    $Cmd .= ' | tr "\n" " "';
    $Cmd .= " | sed -e 's/) /)\\\n/g'";
    $Cmd .= " | sed -e 's/^zqzfunction /<tr valign=\"top\"><td><strong>/'";
    $Cmd .= " -e 's/ *(/<\/strong> (/'";
    $Cmd .= " -e 's#$#</td><td>'\${File}'</td></tr>#'\n";
    $Cmd .= "done | sort \n";
    return $Cmd;
}

function FunctionsByScript($Script)
{
    $Cmd = "cat $Script";
    $Cmd .= " | sed -e 's/^function/zqzfunction/'";
    $Cmd .= ' | tr -s " 	" "\n\n"';
    $Cmd .= " | sed -ne '/^zqzfunction/,/)/p'";
    $Cmd .= ' | tr "\n" " "';
    $Cmd .= " | sed -e 's/) /)\\\n/g'";
    $Cmd .= " | sed -e 's/^zqzfunction /<strong>/'";
    $Cmd .= " -e 's/ *(/<\/strong> (/'";
    $Cmd .= " -e 's/$/<br \/>/'\n";
    return $Cmd;
} 

function IsScript ($File)
{
    $Extension = strrchr($File, ".");
    if ($Extension == ".php")
	return (strpos($File, "gk_") === 0 
		or strpos($File, "lib_") === 0 
		or strpos($File, "loc_") === 0);
    else return False;
}

function ScriptsIn ($Directory)
{
    // All PHP scripts in specified directory, sorted
    // Omits hidden files
    $ScriptsIn = array();
    $Dir = dir ($Directory);
    while ($File = $Dir->read())
	if ( ! Invisible ($File) and IsScript($File) )
	    $ScriptsIn[] = $File;
    $Dir->close();
    sort ($ScriptsIn);
    return $ScriptsIn;
}

//********************************************************************
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
$SortByScript = (isset($FormVars["ByScript"]));
$S = HeadAndTitle("$SystemTitle Shared Functions");

$Cwd = getcwd();
$Top = basename($Cwd);
$Title = "Shared Functions in $Top";
$AllScripts = ScriptsIn($Cwd);

$SubDirs = Subdirectories ($Cwd);
foreach ($SubDirs as $SubDir)
{
    $Scripts = ScriptsIn("$Cwd/$SubDir");
    if (sizeof($Scripts) > 0)
    {
	foreach ($Scripts as $Script)
	    $AllScripts[] = "$SubDir/$Script";
	$Title .= ", ".$Top."/".$SubDir;
    }
}

$S .= PageTitleWithButton($Title, BtnMainMenu());

$S .= StrTableBegin("1", TableColor);

$S .= StrRowBegin();
$S .= "<td colspan=\"2\">";
$S .= StrTableBegin("0", "", "100%");
$S .= StrRowBegin();
$S .= "<td>";
if ($SortByScript)
{
    $S .= StrSubTitle("By Defining Script");
    $S .= "</td><td align=\"right\">";
    $S .= StrFormBeginRecurse();
    $S .= HtmlSubmitButton("ByName", "Order by Function Name");
} else
{
    $S .= StrSubTitle("By Function Name");
    $S .= "</td><td align=\"right\">";
    $S .= StrFormBeginRecurse();
    $S .= HtmlSubmitButton("ByScript", "Order by Defining Script");
}
$S .= StrFormEnd();
$S .= "</td>";
$S .= StrRowEnd();
$S .= StrTableEnd();
$S .= "</td>";
$S .= StrRowEnd();

$S .= StrRowBegin();
if ($SortByScript)
    $S .= "<th>Defining Script</th><th>Function Name</th>";
else $S .= "<th>Function Name</th><th>Defining Script</th>";
$S .= StrRowEnd();

if ($SortByScript)
{
    foreach ($AllScripts as $Script)
    {
	$S .= StrRowBegin();
	$S .= "<td>".$Script."</td><td>";
	$S .= shell_exec (FunctionsByScript($Script));
	$S .= "</td>";
	$S .= StrRowEnd();
    }
} else
{
    $Files = "";
    foreach ($AllScripts as $Script) $Files .= " ".$Script;
    $S .= shell_exec (FunctionsByName($Files));
}

$S .= StrTableEnd();
$S .= "</body></html>\n";
echo $S;

?>
