<?php
/*********************************************************************
gk_usermgt.php

Common definitions for managing user accounts
______________________________________________________________________
Copyright 2013 Theodore B. Ruegsegger
This file is part of the GAPP Kit.

The GAPP Kit is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

The GAPP Kit is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with the GAPP Kit. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
require("gk/gk_filesys.php");

function DeleteUser($UserId)
{
    $Res = ResUserByNameLogin("", "", $UserId);
    $PersonSn = pg_fetch_result($Res, 0, "personsn");
    $ReguserSn = SerialNumber("reguser", "userid", $UserId);
    Query("BEGIN");
    foreach (UserPointers() as $Table => $Col)
    {
	Query("DELETE FROM $Table WHERE $Col = $ReguserSn");
    }
    if (IsOrphanPerson($PersonSn))
    {
	Query("DELETE FROM person WHERE personsn = $PersonSn");
    }
    Query("COMMIT");
    return;
}

function ErrBlankEntry($Col, $Label=False)
{
    $PersonAttributes = PersonAttributes();
    if (! $Label) $Label = $PersonAttributes[$Col]["Lbl"];
    return "$Label must be present<br>\n";
}

function ErrNonBlankCheck($Col, $Value, $Label=False)
{
    // Return error string if value is blank or null
    if ($Value > "")
    {
	return "";
    } else
    {
	return ErrBlankEntry($Col, $Label);
    }

}

function IsOrphanPerson($PersonSn)
{
    // Returns False if any other table points to this personsn
    if (PersonPointers())
    {
	foreach (PersonPointers() as $Table => $Col)
	{
	    if (IsInDatabase($Table, $Col, $PersonSn))
		return False;
	}
    }
    return True;
}

function SansSpecialChars($Text)
{
    // strips spaces and punctuation to facilitate matching
    return str_replace(array(" ", "'", '"', "-", "."), "", $Text);
}

function StrUserAccountForm($Res=False, $InProgress=False,
			    $SepWin=False, $ExtraLines=False)
{
    $New = (! $Res);
    if ($InProgress)
    {
	// Already have $OldUserId, $UserId, $LastName and $FirstName
	$OldUserId = $_REQUEST["OldUserId"];
	$UserId = $_REQUEST["UserId"];
	$LastName = $_REQUEST["LastName"];
	$FirstName = $_REQUEST["FirstName"];
	if ($New)
	{
	    $T = "Add a New User:";
	} else
	{
	    $T = "Account Details for $OldUserId";
	}
    } elseif ($New)
    {
	$T = "Add a New User:";
    } else
    {
	$UserId = pg_fetch_result($Res, 0, "userid");
	$LastName = pg_fetch_result($Res, 0, "lastname");
	$FirstName = pg_fetch_result($Res, 0, "firstname");
	$T = "Account Details for $UserId";
    }
    $S = StrSubTitle("$T");
    $S .= StrFormBeginRecurse();
    $S .= HtmlHidden("SepWin", $SepWin);
    $S .= StrTableBegin("0");
    $S .= StrUserAccountFormLine("Arbitrary string used for login",
				 "User ID", "UserId", 10, $UserId,
				 False, !$New);
    $S .= StrUserAccountFormLine("Last name", "Last name", "LastName",
				 40, $LastName);
    $S .= StrUserAccountFormLine("First name", "First name",
				 "FirstName", 40, $FirstName);
// if ($New or (UserHasPrivilege(LoggedInUser(), "mnguser")
// 		 and $UserId != "admin"))
// {
    $Tip = $New
	? "Initial password" : "Leave blank unless changing";
    $Label = $New? "Password" : "New Password";
    $S .= StrUserAccountFormLine($Tip, $Label, "Password", 16, "",
				 True);
    $Tip = "$Label, again";
    $Label = "Repeat $Label";
    $S .= StrUserAccountFormLine($Tip, $Label, "Password2", 16, "",
				 True);
// }
    // Allow user records with additional fields
    $PersonAttributes = PersonAttributes();
    if ($PersonAttributes)
    {
	foreach ($PersonAttributes as $Col => $Specs)
	{
	    $VarName = ucfirst($Col);
	    if ($InProgress)
	    {
		$Value = trim($_REQUEST[$VarName]);
	    } else
	    {
	    $Value = ($New) ? "" : pg_fetch_result($Res, 0, $Col);
	    }
	    $S .= StrUserAccountFormLine($Specs["Hint"],
					 $Specs["Lbl"], $VarName,
					 $Specs["Siz"], $Value);
	}
    }
    $S .= StrTableEnd();
    if ($UserId != "admin" or LoggedInUser() == "admin")
    {
	$S .= "<table border=\"0\" cellspacing=\"10\"";
	$S .= " cellpadding=\"4\">\n".StrRowBegin();
	$Action = ($New) ? "Add" : "Update";
	$S .= StrCell(HtmlSubmitButton("${Action}User", "$Action"));
	if (!$InProgress) $OldUserId = $UserId;
	$S .= StrCell(HtmlHidden("OldUserId", $OldUserId));
	$S .= StrCell(HtmlResetButton());
	if (!$New and ! UserDeleteErrors($OldUserId))
	{
	    $S .= StrCell("&nbsp;");
	    $S .= StrDeleteButtonCell("DeleteUser", "Delete");
	}
	$S .= StrRowEnd();
	$S .= StrTableEnd();
    }
    // Allow inserting extra lines into form, e.g, hidden vars as
    // flags
    if ($ExtraLines) $S.= $ExtraLines;
    $S .= StrFormEnd();
    return $S;
}

function StrUserAccountFormLine($Tip, $Label, $VarName, $Size,
				$Value="", $IsPassword=False,
				$DisplayOnly=False)
{
    // create a line in the form
    $S = StrRowBegin();
    $S .= "<td align=\"right\" title=\"$Tip\">";
    $S .= Strong("$Label : ")."</td>\n";
    if ($DisplayOnly)
    {
	$T = Strong($Value);
    } elseif ($IsPassword)
    {
	$T = "<input type=\"password\" name=\"$VarName\"";
	$T .= " size=\"$Size\">";
    } else
    {
	$T = HtmlTextInput($VarName, $Size, $Value);
    }
    $S .= StrCell($T);
    $S .= StrRowEnd();
    return $S;
}

function UserAddStatus($UserId, $LastName, $FirstName, $Password,
		       $Password2)
{
    // UserId, LastName, FirstName, Password, Password2, otherfields
    $UserId = trim($UserId);
    $Err = ErrNonBlankCheck("userid", $UserId, "User ID");
    if ($Err == "" and ! GoodFileName($UserId))
	$Err .= "User ID must be ".FileNameSpec."<br>\n";
    if ($UserId > "" and
	pg_num_rows(ResUserByNameLogin("", "", $UserId)) > 0)
	$Err .= "User &quot;$UserId&quot; already exists<br>\n";

    if ($Password != $Password2)
    {
	$Err .= "Repeated password doesn't match<br>\n";
    } elseif ( ! PasswordLegal($Password))
    {
	$Err .= LegalPasswordMsg."<br>\n";
    }

    $LastName = trim($LastName);
    $Err .= ErrNonBlankCheck("lastname", $LastName, "Last Name");

    $FirstName = trim($FirstName);
    $Err .= ErrNonBlankCheck("firstname", $FirstName, "First Name");

    $Attr = array();
    $Attr[] = array("lastname", $LastName);
    $Attr[] = array("firstname", $FirstName);
    $PersonAttributes = PersonAttributes();
    if ($PersonAttributes)
    {
	foreach ($PersonAttributes as $Col => $Specs)
	{
	    $VarName = ucfirst($Col);
	    $Value = trim($_REQUEST[$VarName]);
	    if ($Specs["Rqd"])
	    {
		$Err .= ErrNonBlankCheck($Col, $Value,
					 $Specs["Lbl"]);
	    }
	    $Attr[] = array($Col, $Value);
	}
	// App-specific validations (in app_usermgt.php)
	$Err .= PersonAttributeErrors();
    }

    if ($Err > "")
    {
	return $Err;
    } else
    {
	Query("BEGIN");
	$Res = ResInsertIfNew("person", $Attr);
	$PersonSn = pg_fetch_result($Res, 0, "personsn");
	$CypherText = Encrypted($Password);
	$Q = "INSERT INTO reguser (userid, passwd_hash,";
	$Q .= " passwd_changed, personsn)";
	$Q .= " VALUES ($1, '$CypherText'";
	// If user is adding self, no need to timeout password
	$Q .= LoggedInUser() ? ", NULL" : ", CURRENT_DATE";
	$Q .= ", $PersonSn)";
	Query($Q, $UserId);
	Query("COMMIT");
	return "";
    }
}

function UserDeleteErrors($UserId)
{
    $Err = "";
    $Disp = "&quot;$UserId&quot;";
    if (pg_num_rows(ResUserByNameLogin("", "", $UserId)) == 0)
    {
	$Err .= "User $Disp not found.<br>\n";
    }
    if ($Err == "")
    {
	$Err = WhyCannotDeleteUser($UserId);
	if ($UserId == "admin"
	    or UserHasPrivilege($UserId, "adminsvc"))
	    $Err .= "User $Disp has administrator privileges.<br>\n";
    }
    $S = ($Err > "")
	? "${Err}Cannot delete user $Disp.<br>\n" : False;
    return $S;
}

function UserUpdateStatus($UserId, $LastName, $FirstName, $Password,
			  $Password2, $SetDate=False)
{
    $Err = "";
    $UserId = trim($UserId);
    if (! IsInDatabase("reguser", "userid", $UserId, True))
	$Err .= "No user has ID &quot;$UserId&quot;<br>";
    if ($Password > "")
    {
	if ($Password != $Password2)
	{
	    $Err .= "Repeated password doesn't match<br>\n";
	} elseif ( ! PasswordLegal($Password))
	{
	    $Err .= LegalPasswordMsg."<br>\n";
	}
    }
    $Res = ResUserByNameLogin("", "", $UserId);
    $PersonSn = pg_fetch_result($Res, 0, "personsn");
    $LastName = trim($LastName);
    $Err .= ErrNonBlankCheck("lastname", $LastName, "LastName");

    $FirstName = trim($FirstName);
    $Err .= ErrNonBlankCheck("firstname", $FirstName, "FirstName");

    $Q = "UPDATE person SET lastname = $1, firstname = $2";
    $Count = 2;
    $Params = array($LastName, $FirstName);
    $PersonAttributes = PersonAttributes();
    if ($PersonAttributes)
    {
	foreach ($PersonAttributes as $Col => $Specs)
	{
	    $VarName = ucfirst($Col);
	    $Value = trim($_REQUEST[$VarName]);
	    if ($Specs["Rqd"])
	    {
		$Err .= ErrNonBlankCheck($Col, $Value,
					 $Specs["Lbl"]);
	    }
	    $Count++;
	    $Q .= ", $Col = $$Count";
	    $Params[] = $Value;
	}
	// App-specific validations (in app_usermgt.php)
	$Err .= PersonAttributeErrors();
    }
    if ($Err > "")
    {
	return $Err;
    } else
    {
	$Q .= " WHERE personsn = $PersonSn";
	QueryArray($Q, $Params);
	if ($Password > "")
	{
	    $CypherText = Encrypted($Password);
	    $Q = "UPDATE reguser set passwd_hash = '$CypherText'";
	    $Date = ($SetDate ? "CURRENT_DATE" : "NULL");
	    $Q .= ", passwd_changed = $Date";
	    $Q .= " WHERE userid = $1";
	    Query($Q, $UserId);
	}
	return "";
    }
}

//********************************************************************

?>
