<?php
/*********************************************************************
lib_album.php
Album Management Definitions for PhotoTrove
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
function AddedToAlbum($NumRows, $AlbumSn, $NewAlbum)
{
    // Adds photos to last position of page zero, creating album and
    // page zero if it doesn't exist
    $S = "";
    $Count = 0;
    $AlbumSn = FinalAlbumSn($AlbumSn, $NewAlbum);
    if ($AlbumSn === False)
	$S = StrWarningBox(WarnBadAlbumName($NewAlbum), "50%");
    else
    {
	$AlbumName = pg_fetch_result(ResAlbum($AlbumSn), 0, "name");
	$FormVars = (FormMethod == "get") ? $_GET : $_POST;
	for ($Row=0 ; $Row < $NumRows ; $Row++ )
	{
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
	    {
            $PhotoSn = $FormVars["PS$Row"];
            AppendToAlbum($AlbumSn, $PhotoSn);
            $S .= ", ".PhotoPointer($PhotoSn);
            $Count ++;
	    }
	}
	if ($S > "")
	{
	    $S = substr($S, 1);
	    $T = "Appended $Count photos to page 0 of album";
	    $T .= " &quot;$AlbumName&quot;:";
	} else
	{
	    $T = "No photos selected; none appended to album";
	    $T .= " &quot;$AlbumName&quot;.";
	}
	$S = HtmlPara($T.$S);
    }
    return $S;
}

function AlbumSn($AlbumName)
{
    // Returns albumsn of $AlbumName in album table
    // Inserts album if it doesn't already exist
    // Note: assumes good file name; use FinalAlbumSn to check
    $AlbumName = trim($AlbumName);
    $AlbumSn = SerialNumber("album", "name", "$AlbumName");
    if ($AlbumSn === False)
    {
	$Q = "INSERT INTO album (name, title, pubdate)";
	$Q .= " VALUES ($1, $1, CURRENT_DATE)";
	Query($Q, $AlbumName);
	$AlbumSn = AlbumSn($AlbumName);
	// Every album has a page 0
	CreateAlbumPage($AlbumSn, 0);
    }
    return $AlbumSn;
}

function AppendToAlbum($AlbumSn, $PhotoSn)
{
    // Adds photo to last position of page zero
    $Q = "INSERT INTO albumphoto";
    $Q .= " (albumpagesn, photosn, ordinal, caption)";
    $Q .= " SELECT pg.albumpagesn, $2,";
    $Q .= " (SELECT COALESCE(max(ordinal), 0) + 1 FROM albumphoto";
    $Q .= " WHERE albumpagesn = pg.albumpagesn), description";
    $Q .= " FROM albumpage pg, photo p";
    $Q .= " WHERE albumsn = $1 AND page = 0 AND p.photosn = $2";
    Query($Q, $AlbumSn, $PhotoSn);
    return;
}

function CreateAlbumPage($AlbumSn, $Page)
{
    // Creates page $Page in album $AlbumSn if it doesn't already
    // exist, returns $AlbumPageSn
    $Res = ResInsertIfNew("albumpage",
			  array(array("albumsn", $AlbumSn),
				array("page", $Page)));
    return pg_fetch_result($Res, 0, "albumpagesn");
}

function FinalAlbumSn($AlbumSn, $NewAlbum)
{
    if ($AlbumSn == "newalbum" or $AlbumSn == "none")
    {
	$NewAlbum = trim($NewAlbum);
	if (GoodFileName($NewAlbum))
	    $AlbumSn = AlbumSn($NewAlbum);
	else
	    $AlbumSn = False;
    }
    return $AlbumSn;
}

function IsContiguousAlbum($AlbumSn)
{
    return (PageCount($AlbumSn) == LastPage($AlbumSn) + 1);
}

function LastPage($AlbumSn)
{
    // Highest page number in album (compare PageCount)
    $Q = "SELECT max(page) FROM albumpage WHERE albumsn = $1";
    return pg_fetch_result(Query($Q, $AlbumSn), 0, 0);
}

function PageCount($AlbumSn)
{
    // Number of pages in album (compare LastPage)
    $Q = "SELECT count(*) FROM albumpage WHERE albumsn = $1";
    return pg_fetch_result(Query($Q, $AlbumSn), 0, 0);
}

function PicsOnPage($AlbumSn, $Page)
{
    $Q = "SELECT count(*) FROM albumphoto WHERE albumpagesn =";
    $Q .= " (SELECT albumpagesn FROM albumpage";
    $Q .= " WHERE albumsn = $1 AND page = $2)";
    return pg_fetch_result(Query($Q, $AlbumSn, $Page), 0, 0);
}

function ResAlbum($AlbumSn)
{
    // Return all attributes of a single album
    $Q = "SELECT name, title, bgcolor, textcolor, vidbgcolor,";
    $Q .= " author, url, notes, pubdate";
    $Q .= " FROM album WHERE albumsn = $1";
    return Query($Q, $AlbumSn);
}

function ResAlbumPage($AlbumPageSn, $AlbumSn=False, $Page=False)
{
    // Return array of $AlbumPageSn, $AlbumSn, $Page, $Heading,
    // $TitlePicOrd, $TitlePicCaption, $PageText for album page
    // specified either by $AlbumPageSn or by $AlbumSn and $Page (and
    // False for $AlbumPageSn)
    $Q = "SELECT albumpagesn, albumsn, page, heading, titlepicord,";
    $Q .= " titlepiccaption, pagetext";
    $Q .= " FROM albumpage WHERE ";
    $Res = ($AlbumPageSn !== False)
	? Query($Q."albumpagesn = $1", $AlbumPageSn)
	: Query($Q."albumsn = $1 AND page = $2", $AlbumSn, $Page);
    return ($Res && pg_num_rows($Res) > 0) ? $Res : False;
}

function TitlePicOrd($AlbumPageSn)
{
    return pg_fetch_result(ResAlbumPage($AlbumPageSn), 0,
			   "titlepicord");
}

function WarnBadAlbumName($AlbumName)
{
    if ($AlbumName == "")
	$W = "An album name can't be blank.";
    else
    {
	$W = "&quot;$AlbumName&quot; contains spaces or weird";
	$W .= " characters and can't be an album name. An album";
	$W .= " name must be ".FileNameSpec.".";
    }
    return $W;
}

?>
