<?php
/*********************************************************************
lib_albumgen.php
Resources useful for assembling albums
______________________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
define ("TAB", "\t");		// TAB character as delimiter
define ("TagNamePano", "panorama");

function AdjacentPics($AlbumSn, $Page, $Ordinal, $LastPage,
                      $Php=True, $PageDigits=2, $OrdinalDigits=2)
{
    $NumPics = PicsOnPage($AlbumSn, $Page);
    if ($Ordinal == 1)
    {
        list($PageBefore, $Z) = PageBefore($AlbumSn, $Page);
        $PrevUrl = PageLinkUrl($AlbumSn, $PageBefore, $Php);
    } else
        $PrevUrl = LargeImageUrl($AlbumSn, $Page, $Ordinal - 1, $Php,
                                 $PageDigits, $OrdinalDigits);
    if ($Ordinal == $NumPics)
    {
        if ($Page < $LastPage)
        {
            list($PageAfter, $Z) = PageAfter($AlbumSn, $Page);
            $NextUrl = PageLinkUrl($AlbumSn, $PageAfter, $Php);
        } else
            $NextUrl = PageLinkUrl($AlbumSn, 0, $Php);
    } else
        $NextUrl = LargeImageUrl($AlbumSn, $Page, $Ordinal + 1, $Php,
                                 $PageDigits, $OrdinalDigits);
    return array($PrevUrl, $NextUrl);
}

function AlbumBodyStyle($BgColor, $TextColor, $VidBgColor)
{
    $S = "font-family: helvetica, sans-serif;";
    if ($TextColor > "") $S .= " color: $TextColor;";
    if ($BgColor > "") $S .= " background-color: $BgColor";
    if ($TextColor > "") $S .= "}\na {color: $TextColor;";
    $S .= "}\n.vid {border: medium solid $VidBgColor";
    // $S .= "}\n.xout {text-decoration: line-through";
    return $S;
}

function AlbumLinkStyle($TextColor)
{
    $L = "A:link { color: $TextColor }\n";
    $L .= "A:visited { color: $TextColor }\n";
    $L .= "A:active { color: $TextColor }\n";
    return $L;
}

function CountPanoramas($AlbumSn)
{
    // Return number of occurences of a panorama on any page of the
    // album
    $Q = "SELECT COUNT(*)";
    $Q .= " FROM albumphoto p, albumpage pg, tag t, tag_photo tp";
    $Q .= " WHERE pg.albumsn = $1 AND p.albumpagesn = pg.albumpagesn";
    $Q .= " AND t.name = '".TagNamePano."'";
    $Q .= " AND p.photosn = tp.photosn AND tp.tagsn = t.tagsn";
    return pg_fetch_result(Query($Q, $AlbumSn), 0);
}

function LargeImageUrl($AlbumSn, $Page, $Ordinal, $Php=True,
                       $PageDigits=2, $OrdinalDigits=2)
{
    if ($Php)
    {
        $Url = ThisScript()."?AlbumSn=$AlbumSn&Page=$Page";
        $Url .= "&Ordinal=$Ordinal&ViewLarge=1";
    } else
    {
        $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
        // Pages and images in published albums have padded numbers:
        $Page = str_pad($Page, $PageDigits, "0", STR_PAD_LEFT);
        $Ordinal = str_pad($Ordinal, $OrdinalDigits, "0",
                           STR_PAD_LEFT);
        $Url = "$AlbumName-$Page-$Ordinal.html";
    }
    return $Url;
}

function LinkBar($AlbumSn, $Page, $LastPage, $Php=True)
{
    $S = StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    if ($Page == 0)
    {
        $S .= "<td align=\"right\">";
        $S .= $Php
           ? StrHomePageLink() : "<a href=\"../index.html\">Home";
        $S .= "</td>\n";
    } else
    {
        $S .= LinkBarCell($AlbumSn, $Page, "left", $Php);
        $S .= LinkBarCell($AlbumSn, $Page, "center", $Php);
    }
    if ($Page < $LastPage)
        $S .= LinkBarCell($AlbumSn, $Page, "right", $Php);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function LinkBarCell($AlbumSn, $Page, $Position, $Php=True)
{
    $S = "<td align=\"$Position\"";
    if ($Position == "left")
    {
        list($LinkPage, $Heading) = PageBefore($AlbumSn, $Page);
        $Heading = HtmlCleanTitle($Heading);
        $T = "Previous Page";
    } elseif ($Position == "right")
      {
          list($LinkPage, $Heading) = PageAfter($AlbumSn, $Page);
          $Heading = HtmlCleanTitle($Heading);
          $T = "Next Page";
      } else
        {
            list($LinkPage, $Heading) = PageBefore($AlbumSn, 1);
            $Heading = HtmlCleanTitle($Heading);
            $T = "Main";
        }
    $S .= " title=\"p.$LinkPage";
    if ($Heading > "") $S .= ", $Heading";
    $S .= "\"><a href=\"";
    $S .= PageLinkUrl($AlbumSn, $LinkPage, $Php);
    $S .= "\">$T</a></td>\n";
    return $S;
}

function PageAfter($AlbumSn, $ThisPage)
{
    // Return array($Page, Heading) for first existing page after
    // $ThisPage; if $ThisPage is last, returns its values
    $Q = "SELECT page, heading FROM albumpage";
    $Q .= " WHERE albumsn = $1 AND page >= $2";
    $Q .= " ORDER BY page";
    $Res = Query($Q, $AlbumSn, $ThisPage);
    $Row = (pg_num_rows($Res) > 1) ? 1 : 0;
    return pg_fetch_array($Res, $Row);
}

function PageBefore($AlbumSn, $ThisPage)
{
    // Return array($Page, Heading) for last existing page before
    // $ThisPage; if $ThisPage is first, returns its values
    $Q = "SELECT page, heading FROM albumpage";
    $Q .= " WHERE albumsn = $1 AND page <= $2";
    $Q .= " ORDER BY page DESC";
    $Res = Query($Q, $AlbumSn, $ThisPage);
    $Row = (pg_num_rows($Res) > 1) ? 1 : 0;
    return pg_fetch_array($Res, $Row);
}

function PageLinkUrl($AlbumSn, $Page, $Php=True)
{
    return $Php ? ThisScript()."?AlbumSn=$AlbumSn&Page=$Page"
                              : (($Page > 0) ? "page$Page.html" : "index.html") ;
}

function PageZero($AlbumSn, $LastPage, $Author, $Url, $Php=True,
                  $WithFullSize=False)
{
    $T = "All the pictures are on the following page";
    if ($LastPage != 1) $T .= "s";
    $T .= ". Click the small images to see them larger. Click the";
    $T .= " left and right arrows under the large images to step";
    $T .= " through a whole page as a slideshow.\n";
    if (CountPanoramas($AlbumSn) > 0)
    {
        $T .= "<em><strong>Some images are panoramas, wider than";
        $T .= " most displays. Scroll sideways using your browser's";
        $T .= " scroll bar, or simply use the arrow keys on your";
        $T .= " keyboard.</strong></em>\n";
    }
    if ($Php or $WithFullSize)
    {
        $T .= "For printing or editing, fetch the high-resolution";
        $T .= " full-size image by clicking the\n";
        $T .= Strong("Full-Size");
        $T .= " link at the bottom right of each large photo.\n";
    }
    $S = HtmlPara($T);
    $T = IsValidEmailAddress($Url) ? "mailto:$Url" : $Url;
    $T = "$Author<br />\n<a href=\"$T\">$Url</a>";
    if ($LastPage > 0)
    {
        $S .= "<dl><dt>Individual pages:</dt>\n<dd>\n";
        for ($Pg = 1 ; $Pg <= $LastPage ; $Pg++)
        {
            $Q = "SELECT heading FROM albumpage";
            $Q .= " WHERE albumsn = $1 AND page = $2";
            $Res = Query($Q, $AlbumSn, $Pg);
            if (pg_num_rows($Res) > 0)
            {
                $Url = PageLinkUrl($AlbumSn, $Pg, $Php);
                $S .= "<a href=\"$Url\">$Pg.";
                $S .= " ".pg_fetch_result($Res, 0, 0);
                $S .= "</a><br />\n";
            }
        }
        $S .= "</dl>\n";
        $S .= HtmlPara($T);
        $S .= LinkBar($AlbumSn, 0, $LastPage, $Php);
    } else
    {
        $S .= HtmlPara("No other pages found.");
        $S .= HtmlPara($T);
        $S .= $Php
           ? HtmlPara(StrHomePageLink())
           : HtmlPara("<a href=\"../index.html\">Home</a>");
    }
    return $S;
}

function StrHomePageLink()
{
    $HomeButton = BtnHome();
    $S = "<a href=\"".$HomeButton["Url"]."\">".$HomeButton["Text"];
    $S .= "</a>";
    return $S;
}

function StrImageWithMap($Width, $Height, $Url, $PrevUrl, $NextUrl,
                         $Caption, $Php=True, $WithFullSize=True)
{
    $S = "<map name=\"PicMap\">\n";
    $S .= "<area shape=\"rect\" coords=\"0,0,".PageEdge.",$Height\"";
    $S .= " href=\"$PrevUrl\">\n";
    $S .= "<area shape=\"rect\" coords=\"".($Width - PageEdge).",0,";
    $S .= "$Width,$Height\"";
    $S .= " href=\"$NextUrl\">\n";
    $S .= "</map>\n";
    $Label = HtmlCleanTitle($Caption);
    $S .= "<div align=\"center\">\n";
    if ($Php)
    {
        $S .= "<a href=\"".ThisScript()."?FetchPic=$Url\">";
        $V = "&nbsp";
        if (! $WithFullSize) $O = "&nbsp;";
        else
        {
            $O = "<a href=\"".ThisScript()."?FetchPic=fullsize";
            $O .= substr($Url, strpos($Url, "/"));
            $O .= "\">Full-Size</a>";
        }
    } else
    {
        $S .= "<a target=\"_blank\" href=\"$Url\">";
        $V = "&nbsp";
        if (! $WithFullSize) $O = "&nbsp;";
        else
        {
            $O = "<a target=\"_blank\" href=\"fullsize/$Url\">";
            $O .= "Full-Size</a>";
        }
    }
    $S .= HtmlImage($Url, $Label, $Height, $Width, "0", $Label,
                    "#PicMap")."</a>\n";
    
    $S .= StrTableBegin("0", "", "100%");
    $S .= RowNavButtons("href=\"$PrevUrl\"", "href=\"$NextUrl\"", 1,
                        "#808080", ($Php ? "gk/images" : "images"));
        $S .= StrTableEnd();

    $S .= StrTableBegin("0", "", "", "margin-top: 1em");
    $S .= StrRowBegin();
    $S .= "<td width=\"15%\">&nbsp;</td>\n";
    $S .= "<td align = \"center\">$Caption</td>\n";
    $S .= "<td align = \"center\">$V</td>\n";
    $S .= "<td width=\"15%\" align = \"right\">$O</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= "</div>\n";
    return $S;
}

function VideoInPlayer($Url, $PrevUrl, $NextUrl, $Caption, $VidName,
                       $OggName, $OggWidth, $OggHeight, $VidBgColor,
                       $Php=True, $WithFullSize=True)
{
    $S = "<div align=\"center\">\n";
    if ($Php)
    {
        if (! $WithFullSize) $V = "&nbsp;";
        else
        {
            $V = "<a href=\"".ThisScript();
            $V .= "?FetchVid=fullsize";
            $T = substr($Url, strpos($Url, "/"));
            $V .= substr($T, 0, strrpos($T, "/"))."/$VidName\">";
            $V .= "Download original-format video</a>";
        }
    } else
    {
        if (! $WithFullSize) $V = "&nbsp;";
        else
        {
            $V = "<a target=\"_blank\"";
            $V .= " href=\"fullsize/$VidName\">";
            $V .= "Original-format video</a>";
        }
    }
    $S .= VideoBlock($OggName, $OggWidth, $OggHeight, $Php);
    $S .= StrTableBegin("0", "", $OggWidth, "margin-top: 1em");
    $S .= RowNavButtons("href=\"$PrevUrl\"", "href=\"$NextUrl\"", 3,
                        $VidBgColor, ($Php ? "gk/images" : "images"));
    $S .= StrRowBegin();
    $S .= "<td width=\"15%\">&nbsp;</td>\n";
    $S .= "<td align = \"center\">$Caption</td>\n";
    $S .= "<td width=\"15%\" align = \"right\">$V</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= "</div>\n";
    return $S;
}

function WarnPageGone($Page, $AlbumName, $Title)
{
    $T = "Album &quot;$AlbumName: $Title&quot; has no page $Page.";
    $S .= StrWarningBox($T, "50%");
    return $S;
}

//***************************************************************

?>
