<?php
/*********************************************************************
lib_common.php
Common Definitions for Application
______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
// Put require statements after function SpecialToolAccess because
// gk/gk_common.php looks for it

/*
function SpecialToolAccess($UserId, $Tool)
{
    // Additional criteria for tool access by logged-in users
    //***************************************************************
    // For tool access, check whether users have privileges for any
    // possible case (e.g., within a particular project). Tool itself
    // will handle the finer details of which cases apply. Some tools
    // will have an associated privilege (e.g., adminsvc) and others
    // won't (or will have a never-assigned placeholder privilege);
    // such tools will be accessible only via the SpecialToolAccess
    // feature. Typically, one or more Boolean functions will grant
    // access according to table lookups or database queries.
    //***************************************************************
    switch ($Tool)
    {
    case "launchthemissile.php":
	$Ok = UserMayLaunchMissile($UserId);
	break;
    case "editproject.php":
    case "assigntasks.php":
    case "updateschedule.php":
	$Ok = (UserIsProjectLeader($UserId)
	       or UserIsTheBigBoss($UserId)
	       or UserIsProjectAdmin($UserId));
	break;
    default:
	$Ok = False;
    }
    return $Ok;
}
*/

// Put require statements here because gk/gk_common.php looks for
// function SpecialToolAccess
require ("gk/gk_common.php");

define("NullLocationFlag", "[location unknown]");
define("NullTimeFlag", "[date/time unknown]");

$T = "Password for account &quot;admin&quot; is still set to the";
$T .= " PHP Web Kit default value. Change this soon!";
define ("WarningAdminPasswordStillDefault", $T);

$T = "Hyphen-delimited phone number with area code (prefix non-US";
$T .= " numbers with 011). Only digits and hyphens plus x for";
$T .= " optional extension.";
define ("PhoneNumberFormatHint", $T);

define ("PhoneNumberFormatTable",
"<table border=\"1\"><tr><th>Country</th>
<th>Format</th><th>Example</th></tr>
<tr><td>US numbers</td>
<td>Area Code - Exchange - Number</td>
<td>703-555-1234</td></tr>
<tr><td>Non-US numbers</td>
<td>Begin with 011 - Country Code -</td>
<td>011-49-123...</td></tr>
<tr><td>Any number</td>
<td>Optional extension, any length</td>
<td>703-555-1234x987</td></tr></table>");

// Fetch from PHP configuration (a string, perhaps with K, M, or G):
define ("UploadMaxFileSize", ini_get("upload_max_filesize"));
// Convert to bytes as an actual number:
define ("UploadMaxFileBytes", ReturnBytes(UploadMaxFileSize));

function AllAfter ($Flag, $String)
{
    $Posn = strpos($String, $Flag);
    return $Posn ? substr($String, $Posn + strlen($Flag)) : "";
}

function AllBefore ($Flag, $String)
{
    $Posn = strpos($String, $Flag);
    return $Posn ? substr($String, 0, strpos($String, $Flag)) : "";
}

function AppMessages()
{
    // Returns HTML for any application-specific, ad-hoc messages, to
    // be displayed on main menu page (index.php)
    $S = "";

    // Use: test for some condition, e.g., whether logged-in user has
    // a particular privilege, then generate the appropriate message
    // inside StrWarningBox().

    // A good start is to warn if the admin user's password is still
    // set to the GAPP Kit default:
    $Q = "SELECT passwd_hash FROM reguser WHERE userid = 'admin'";
    $CypherText = pg_fetch_result(Query($Q), 0, "passwd_hash");
    /* if (crypt(stripcslashes("I'm the BOFH"), $CypherText) */
    if (crypt("I'm the BOFH", $CypherText) == $CypherText)
	$S .= StrWarningBox(WarningAdminPasswordStillDefault, "50%");
    return $S;
}

function BoundedValue($Value, $Max, $Min=1)
{
    // Limits value within boundary values
    if ($Value < $Min) return $Min;
    elseif ($Value > $Max) return $Max;
    else return $Value;
}

function Geometry($Image)
{
    // Returns width and height of an image
    list($Width, $Height) = getimagesize($Image);
    return array ("width" => $Width, "height" => $Height);
}

function HmsTime($Seconds)
{
    // Returns time as hrs, min, sec
    if ( $Seconds < 60 )
    {
	$Format = "s \s\\e\c";
    } elseif ( $Seconds < 3600 )
    {
	$Format = "i \m\i\\n, s \s\\e\c";
    } else
    {
	$Format = "H \h\\r, i \m\i\\n, s \s\\e\c";
    }
    return gmdate ($Format, $Seconds);
}

function InitCaps($Text)
{
    // Convert the first letter of every distinct alpha sequence
    $LcText = strtolower($Text);
    preg_match_all('/(\w*[[:^lower:]]+)+/U', $LcText, $Words); 
    preg_match("/(\w+)$/", $LcText, $LastWord);
    $Words[0][] = $LastWord[0]; 
    foreach($Words[0] as $Word) $CapWords[] = ucfirst($Word); 
    return implode("", $CapWords); 
}

function PhotoSerialNumber($FileName, $PhotoSetSn, $FolderName=False)
{
    if (! is_numeric($PhotoSetSn))
	$PhotoSetSn = SerialNumber("photoset", "foldername",
				   $FolderName);
    $Keys = array(array("photosetsn", $PhotoSetSn),
		  array("filename", $FileName));
    return SerialNumberCompound("photo", $Keys);
}

function ReturnBytes($SizeStr)
{
    /* Taken from PHP manual, function.ini-get, user comment by Stas
     * Trefilov
     */
    switch (substr($SizeStr, -1))
    {
        case 'M': case 'm': return (int)$SizeStr * 1048576;
        case 'K': case 'k': return (int)$SizeStr * 1024;
        case 'G': case 'g': return (int)$SizeStr * 1073741824;
        default: return $SizeStr;
    }
}

function ScaleSize(&$Width, &$Height, $MaxLength)
{
    if ($Width > $Height)
    {
	$Height = round($MaxLength * ($Height / $Width));
	$Width = $MaxLength;
    } else
    {
	$Width = round($MaxLength * ($Width / $Height));
	$Height = $MaxLength;
    }
    return;
}

function UrlDownLoad($Name, $FilePath)
{
    return ThisScript()."?$Name=$FilePath";
}

//********************************************************************

?>
