<?php
/*********************************************************************
lib_layout.php
Layout Definitions for Application
______________________________________________________________________
Copyright 2019 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
$T = "padding: 1mm; border: thin solid ".NoticeColor;
$T .= "; background: ".TableColor;
define ("BoxStyle", $T);
define("SubmitIfChange", "onchange=\"this.form.submit()\"");

function BtnGetVideo($FolderName, $VidFileName)
{
    if ($VidFileName == "") return "";
    else
    {
	list ($FullFolder, $WebFolder, $ThumbFolder)
	    = ImageFolder($FolderName);
	return StrButton(array
	     (
		 'Text'	 => "Original-Format Video",
		 'Url'	 => UrlDownLoad("FetchVideo",
					"$FullFolder/$VidFileName"),
		 'Color' => ButtonColor
	     ));
    }
}

function FormMenuAlbum($Sort="PubDate", $Selected="",
		       $AutoSubmit=True, $OfferCreate=False,
		       $Prompt="&mdash; Select album &mdash;",
		       $ShowArchived=False)
{
    $S = "<div style=\"".BoxStyle."; display: table\">\n";
    $S .= StrFormBeginRecurse("", "", "margin-bottom: 0");
    $S .= HtmlHidden("Sort", $Sort);
    $S .= MenuAlbum($Selected, $AutoSubmit, $OfferCreate, $Prompt,
		    $ShowArchived, $Sort);
    $S .= StrFormEnd();
    if ($AutoSubmit)
    {
	$S .= StrFormBeginRecurse("", "", "margin-bottom: 0");
	$ByName = ($Sort == "Name");
	$T = "onchange=\"this.form.submit()\"";
	$S .= Strong("Sort: ")."<label>";
	$S .= HtmlRadioButton("Sort", "PubDate", ! $ByName, $T);
	$S .= "most recent first</label><label>";
	$S .= HtmlRadioButton("Sort", "Name", $ByName, $T);
	$S .= " by name</label>";
	$S .= StrFormEnd();
    }
    $S .= "</div>\n";
    return $S;
}

function MenuAlbum($Selection="", $AutoSubmit=True,
		   $OfferCreate=False,
		   $Prompt="&mdash; Select album &mdash;",
		   $ShowArchived=False, $Sort="PubDate")
{
    // Drop-down menu of names in album table in database
    $Q = "SELECT a.albumsn, name, title,";
    $Q .= " to_char(pubdate, 'YYYY-Mon-DD'), ";
    $Q .= $ShowArchived ? "aa.albumsn IS NOT NULL" : "False";
    $Q .= " AS isarchived FROM album a";
    $Q .= " LEFT OUTER JOIN archivedalbum aa USING (albumsn)";
    if (! $ShowArchived) $Q .= " WHERE aa.albumsn IS NULL";
    $Q .= " ORDER BY ";
    $Q .= ($Sort == "Name") ? "lower(name)" : "pubdate DESC";
    $Res = Query($Q);
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin("AlbumSn", $Attribs);
    if ($Selection == "")
	$S .= HtmlSelectOption("none", $Prompt, True);
    if ($OfferCreate)
	$S .= HtmlSelectOption(
	    "newalbum", "&mdash; Create new album &mdash;", False,
	    "Be sure to enter a name for the new album");
    for ($Row=0 ; $Row < pg_num_rows($Res) ; $Row++)
    {
	list($AlbumSn, $Name, $Title, $PubDate, $IsArchived)
	    = pg_fetch_array($Res, $Row);
	$Style = ($IsArchived == "t")
	    ? "background: ".ArchivedAlbumColor : "";
	$Label = ($Sort == "Name")
	    ? "$Name ($PubDate)" : "$PubDate: $Name";
	$S .= HtmlSelectOption($AlbumSn, $Label,
			       ($AlbumSn == $Selection), $Title,
			       $Style);
    }
    $S .= HtmlSelectEnd();
    return $S;
}

function RowNavButtons($HrefBack, $HrefFwd, $Colspan="2",
		       $BgColor=HighlightBg, $SrcDir="gk/images")
{
    // Since video player doesn't use image map, make a slim row with
    // buttons to go to next/previous image
    $S = StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $T = "<a $HrefBack><img border=\"0\"";
    $T .= " src=\"$SrcDir/1leftarrow.png\" /></a>";
    $S .= StrCell($T, "", "", "text-align: left");
    $T = "<a $HrefFwd><img border=\"0\"";
    $T .= " src=\"$SrcDir/1rightarrow.png\" /></a>";
    $S .= StrCell($T, "", "", "text-align: right");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S = StrRowBegin($BgColor).StrCell($S, $Colspan);
    $S .= StrRowEnd();
    return $S;
}

function RowWebVid($WebFolder, $VidFileName, $Colspan="2",
		   $BgColor=HighlightBg)
{
    $OggFile = OggFile($VidFileName);
    $S = StrRowBegin($BgColor);
    $S .= "<td align=\"center\" colspan=\"$Colspan\">\n";
    $S .= VideoBlock("$WebFolder/$OggFile");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function StartOfPage($PageTitle, $ExtraTags="", $BodyStyle="",
		     $SuppressText=False, $SepWin=False)
{
    $S = HeadAndTitle($PageTitle, $ExtraTags, $BodyStyle);
    if (!$SuppressText)
    {
	$Button = $SepWin ? BtnClose() : BtnMainMenu();
	$S .= PageTitleWithButton($PageTitle, $Button);
    }
    $S .= "<div align=\"center\">\n";
    return $S;
}

function StrNoticeBox($Text)
{
    // For multi-line messages, too long for StrWarning
    $S = StrBoxBegin("0");
    $S .= Strong(HtmlColor(NoticeColor, $Text));
    $S .= StrBoxEnd();
    return $S;
}

function StrSeparatorRow($Colspan="", $Text="")
{
    // Empty row as separator
    if ($Text == "") $Text = "&nbsp;";
    return "<tr>".StrCell($Text, $Colspan)."</tr>\n";
}

function StrToolPageTitle($ExtraTags="", $ForceTitle=False)
{
    // A handy page-starting function for typical applications; for
    // non-typical applications, edit to suit.
    global $SystemTitle;
    $T = $ForceTitle ? $ForceTitle : ThisToolDescription();
    $S = HeadAndTitle("$T: $SystemTitle", $ExtraTags);
    $S .= PageTitleWithButton($T, BtnMainMenu());
    $S .= "<div align=\"center\">\n";
    return $S;
}

function ThisToolDescription()
{
    $Q = "SELECT description FROM tool WHERE name = $1";
    return pg_fetch_result(Query($Q, ThisScript()), 0, "description");
}

function VideoBlock($OggFile, $OggWidth="", $OggHeight="", $Php=True)
{
    // Creates a <video> block for the Ogg (Theora/Vorbis) file
    if ($OggWidth == "") list($OggWidth, $OggHeight)
			     = array_values(VideoSpecs($OggFile));
    $S = "<video width=\"$OggWidth\" height=\"$OggHeight\" preload";
    $S .= " controls>\n";
    $S .= "  <source src=\"".LatestFile($OggFile, $Php);
    $S .= "\" type=\"video/ogg\">\n";
    $S .= VideoBlockFallBack($OggWidth, $OggHeight);
    $S .= "</video>\n";
    return $S;
}

function VideoBlockFallBack($Width, $Height)
{
    // Display a warning that browser doesn't support video tag
    if ($Width == "") $Width = "80%";
    if ($Height == "")
    {
	$Height = "50%";
	$Margin = "25%";
    } else $Margin = $Height/2;
    $S = "<div style=\"width: $Width; height: $Height;";
    $S .= " background-image: url(gk/images/flames.jpg);";
    $S .= " border: thick outset\">\n";
    $S .= "<h3 style=\" margin-top: $Margin; font-style: oblique\">";
    $S .= "Your browser does not support the";
    $S .= " <code>&lt;video&gt;</code> tag.</h3>\n";
    $S .= "</div>\n";
    return $S;
}

//********************************************************************
require ("gk/gk_layout.php");

?>
