<?php
/*********************************************************************
lib_photoset.php

Definitions related to photoset management
______________________________________________________________________
Copyright 2024 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
$T = "COALESCE(to_char(time, '".DateFormatSQL_dMY."'), '";
$T .= NullTimeFlag."')";
define("ClauseDateForDisplay", $T);

$T = "COALESCE(to_char(time, '".DateTimeFormatSQLFull."'), '";
$T .= NullTimeFlag."')";
define("ClauseTimeForDisplay", $T);

$T = "To use a location not already in the list, select this option";
$T .= " and fill in the description below";
define("HintOtherLocation", $T);

function AddImages($FolderName, $FileName, $WebPicLength,
                   $ThumbnailLength)
{
    /* Given an uploaded photo and possibly a tentative webpic under
     * .uploads/, make physical images, return sizes */
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    // Install fullsize
    list($Uploads, $Z, $Z1) = ImageFolder(".uploads");
    rename("$Uploads/".LoggedInUser(), "$FullFolder/$FileName");
    chmod("$FullFolder/$FileName", 0644);
    // Make new webpic if size changed since tentative webpic
    $NewWebPic = "$Uploads/".LoggedInUser()."-web";
    $NewWebPicLength = (file_exists($NewWebPic))
                     ? max(Geometry($NewWebPic)) : 0;
    $FullLength = max(Geometry("$FullFolder/$FileName"));
    $WebPicLength = min($WebPicLength, $FullLength);
    $WebSize = ($NewWebPicLength == $WebPicLength)
             ? Geometry($NewWebPic)
             : ResizePath($WebPicLength, "$FullFolder/$FileName",
                          $NewWebPic);
    // Move new webpic to web folder
    rename($NewWebPic, "$WebFolder/$FileName");
    // Make thumbnail (but no larger than fullsize)
    $ThumbnailLength = min($ThumbnailLength, $FullLength);
    $ThumbSize = Resize($ThumbnailLength, $FileName, $FullFolder,
                        $ThumbFolder);
    return array($WebSize, $ThumbSize);
}

function AddPhoto($PhotoSetSn, $FileName, $LocationSn, $TimeStamp,
                  $Description, $WebSize, $ThumbSize, $TimeZone)
{
    // Insert photo into database, return photosn
    global $PhotoMetaData;
    $Description = trim($Description);
    $Count = 6;
    $Params = array(
        $PhotoSetSn, $FileName, $WebSize["width"], $WebSize["height"],
        $ThumbSize["width"], $ThumbSize["height"]);
    $C = "photosetsn, filename, width, height, thumbw, thumbh";
    $V = "$1, $2, $3, $4, $5, $6";
    if (is_numeric($LocationSn))
    {
        $Count++;
        $Params[] = $LocationSn;
        $C .= ", locationsn";
        $V .= ", $$Count";
    }
    if (strtotime($TimeStamp) !== InvalidDateTime)
    {
        $Count++;
        $Params[] =$TimeStamp ;
        $C .= ", time";
        $V .= ", $$Count";
    }
    if ($Description != "")
    {
        $Count++;
        $Params[] = $Description;
        $C .= ", description";
        $V .= ", $$Count";
    }
    $FolderName
        = ColFromSerialNumber("photoset", "foldername", $PhotoSetSn);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $Info = ImageInfo($FullFolder, $FileName, $TimeZone);
    foreach ($PhotoMetaData as $Key)
    {
        if (array_key_exists($Key, $Info) && $Info[$Key] != "")
        {
            $Count++;
            $Params[] = $Info[$Key];
            $C .= ", $Key";
            $V .= ", $$Count";
        }
    }
    $Q = "INSERT INTO photo ($C) VALUES ($V)";
    QueryArray($Q, $Params);
    $PhotoSn = PhotoSerialNumber($FileName, $PhotoSetSn);
    if ($Info["IsPanorama"])
        TagPhoto(TagSn("panorama"), $PhotoSn);
    return $PhotoSn;
}

function AdjacentRows($Row, $NumRows)
{
    $PrevRow = ($Row == 0) ? $Row : $Row - 1;
    $NextRow = ($Row == $NumRows - 1 ) ? $Row : $Row + 1;
    return array($PrevRow, $NextRow);
}

function AlbumOccurrences($PhotoSn)
{
    $S = "";
    $Q = "SELECT DISTINCT ON (name, a.albumsn) title, name";
    $Q .= " FROM albumphoto ap, albumpage pg, album a";
    $Q .= " WHERE photosn = $1 AND pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND a.albumsn = pg.albumsn ORDER BY name";
    $Res = Query($Q, $PhotoSn);
    $NumRows = pg_num_rows($Res);
    if ($NumRows > 0)
    {
        $S = "<dl>\n<dt><strong>In album(s):</strong></dt>\n";
        for ($Row = 0 ; $Row < $NumRows ; $Row ++)
        {
            list ($Title, $Name) = pg_fetch_array($Res, $Row);
            $S .= "<dd>$Name";
            if ($Title != $Name) $S .= ": &quot;$Title&quot;";
            $S .= "</dd>\n";
        }
        $S .= "</dl>\n";
    }
    return $S;
}

function AppliedDeletion($NumRows)
{
    $S = "";
    $E = "";
    $Count = 0;
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    for ($Row=0 ; $Row < $NumRows ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            $PhotoSn = $FormVars["PS$Row"];
            $DepMsg = PhotoDependences($PhotoSn);
            if ($DepMsg)
                $E .= "<br />\n$DepMsg";
            elseif (pg_num_rows(ResPhoto($PhotoSn)) > 0)
            {
                $S .= ", ".PhotoPointer($PhotoSn);
                DeletePhoto($PhotoSn);
                $Count ++;
            }
        }
    }
    if ($S > "")
        $S = $E.HtmlPara("Deleted $Count photos:".substr($S, 1));
    elseif ($E > "")
        $S = $E.HtmlPara("No photos deleted.");
    else
        $S = HtmlPara("No photos selected; none deleted.");
    return $S;
}

function AppliedDescription($NumRows, $Description)
{
    $S = "";
    $Count = 0;
    $Description = trim($Description);
    if ($Description != "")
    {
        $FormVars = (FormMethod == "get") ? $_GET : $_POST;
        for ($Row=0 ; $Row < $NumRows ; $Row++ )
        {
            if (array_key_exists("Sel$Row", $FormVars)
                && $FormVars["Sel$Row"] == "on")
            {
                $PhotoSn = $FormVars["PS$Row"];
                $Q = "UPDATE photo SET description = $2";
                $Q .= " WHERE photosn = $1";
                Query($Q, $PhotoSn, $Description);
                $S .= ", ".PhotoPointer($PhotoSn);
                $Count ++;
            }
        }
        if ($S > "")
        {
            $S = substr($S, 1);
            $T = "Applied description &quot;$Description&quot; to";
            $T .= " $Count photos:";
        } else
        {
            $T = "No photos selected; couldn't apply description";
            $T .= " &quot;$Description&quot;.";
        }
        $S = $T.$S;
    } else
    {
        $S = "No description specified; couldn't update description";
        $S .= " for selected photos.";
    }
    return HtmlPara($S);
}

function AppliedLocation($NumRows, $LocationSn, $OtherLoc)
{
    $S = "";
    $Count = 0;
    $LocationSn = FinalLocationSn($LocationSn, $OtherLoc);
    if ($LocationSn)
    {
        $FormVars = (FormMethod == "get") ? $_GET : $_POST;
        for ($Row=0 ; $Row < $NumRows ; $Row++ )
        {
            if (array_key_exists("Sel$Row", $FormVars)
                && $FormVars["Sel$Row"] == "on")
            {
                $PhotoSn = $FormVars["PS$Row"];
                $Q = "UPDATE photo SET locationsn = $2";
                $Q .= " WHERE photosn = $1";
                Query($Q, $PhotoSn, $LocationSn);
                $S .= ", ".PhotoPointer($PhotoSn);
                $Count ++;
            }
        }
        $LocName = Location($LocationSn);
        if ($S > "")
        {
            $S = substr($S, 1);
            $T = "Applied location &quot;$LocName&quot; to $Count";
            $T .= " photos:";
        } else
        {
            $T = "No photos selected; couldn't apply location";
            $T .= " &quot;$LocName&quot;.";
        }
        $S = $T.$S;
    } else
    {
        $S = "No location specified; couldn't update location for";
        $S .= " selected photos.";
    }
    return HtmlPara($S);
}

function AppliedRotation($NumRows, $Angle)
{
    $S = "";
    $Count = 0;
    $FormVars = (FormMethod == "get") ? $_GET : $_POST;
    for ($Row=0 ; $Row < $NumRows ; $Row++ )
    {
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
        {
            $PhotoSn = $FormVars["PS$Row"];
            RotatePhoto($PhotoSn, $Angle);
            $S .= ", ".PhotoPointer($PhotoSn);
            $Count ++;
        }
    }
    $T = ($Angle < 0) ? -$Angle."&#176; counter-clockwise"
       : $Angle."&#176; clockwise";
    $T = ($Angle < 0) ? -$Angle."&#176; counter-clockwise"
       : (($Angle == 180)
          ? $Angle."&#176;" : $Angle."&#176; clockwise");
    if ($S > "")
        $S = "Turned $Count photos $T:".substr($S, 1);
    else
        $S = "No photos selected; nothing to turn $T.";
    return HtmlPara($S);
}

function AppliedTag($NumRows, $TagSn, $NewTag)
{
    $S = "";
    $Count = 0;
    $TagSn = FinalTagSn($TagSn, $NewTag);
    if ($TagSn)
    {
        $TagName = ColFromSerialNumber("tag", "name", $TagSn);
        $FormVars = (FormMethod == "get") ? $_GET : $_POST;
        for ($Row=0 ; $Row < $NumRows ; $Row++ )
        {
            if (array_key_exists("Sel$Row", $FormVars)
                && $FormVars["Sel$Row"] == "on")
            {
                $PhotoSn = $FormVars["PS$Row"];
                TagPhoto($TagSn, $PhotoSn);
                $S .= ", ".PhotoPointer($PhotoSn);
                $Count ++;
            }
        }
        if ($S > "")
        {
            $S = substr($S, 1);
            $T = "Applied tag &quot;$TagName&quot; to $Count photos:";
        } else
        {
            $T = "No photos selected; couldn't apply tag";
            $T .= " &quot;$TagName&quot;.";
        }
        $S = $T.$S;
    } else
        $S = "No tag specified; nothing to apply to selected photos.";
    return HtmlPara($S);
}

function BoxPhotoSetSelect($PhotoSetSn, $Sort)
{
    $S = StrFormBeginRecurse();
    $S .= HtmlHidden("SelectPhotoSet", True);
    $S .= StrTableBegin(0, "", "", BoxStyle);
    $S .= RowSelectPhotoSet($PhotoSetSn);
    $S .= RowSearchSortOrder($Sort, True);
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function BtnGetFullSize($FolderName, $FileName, $IsVideo=False)
{
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    return StrButton(array
                     (
                         'Text'	=> ($IsVideo ? "Still Image": "Full Size"),
                         'Url'	=> UrlDownLoad("FetchFull",
                                               "$FullFolder/$FileName"),
                         'Color'	=> ButtonColor
                     ));
}

function CountPhotosInSet($PhotoSetSn)
{
    $Q = "SELECT COUNT(*) FROM photo WHERE photosetsn = $1";
    return pg_fetch_result(Query($Q, $PhotoSetSn), 0);
}

function DegToDegMin($Coordinate, $HemPos, $HemNeg, $Width)
{
    // For geo-coordinates, returns a string H [D]DD° MM.MMM", e.g.,
    // W 072° 07.730"
    if ($Coordinate < 0)
	{
	    $Hemisphere = $HemNeg;
	    $Coordinate = -$Coordinate;
	} else $Hemisphere = $HemPos;
	$Degrees = floor($Coordinate);
	$Minutes = round(($Coordinate - $Degrees) * 60, 3);
	$Degrees = sprintf("%0{$Width}u", $Degrees);
	$Minutes = sprintf("%06.3f", $Minutes);
	return "$Hemisphere $Degrees&deg; $Minutes'";
}

function DeletePhoto($PhotoSn)
{
    // Delete all image files (fullsize, webpics, thumbnails), any
    // related videos (including Ogg versions), database entry and
    // dependences including any parent links.  Caller must ensure,
    // before invoking, that there are no parent-child (derived_photo)
    // or albumphoto dependences that must be preserved.
    list ($FullImage, $WebPic, $Thumbnail, $VideoSn, $VidFile,
          $OggFile) = ImagePath($PhotoSn);
    Query("BEGIN");
    $Q = "DELETE FROM tag_photo WHERE photosn = $1";
    Query($Q, $PhotoSn);
    $Q = "DELETE FROM derived_photo WHERE childphotosn = $1";
    Query($Q, $PhotoSn);
    $Q = "DELETE FROM derived_photo WHERE parentphotosn = $1";
    Query($Q, $PhotoSn);
    $Q = "DELETE FROM video WHERE stillsn = $1";
    Query($Q, $PhotoSn);
    $Q = "DELETE FROM photo WHERE photosn = $1";
    Query($Q, $PhotoSn);
    if (Query("COMMIT"))
    {
        UnlinkIfExists($FullImage);
        UnlinkIfExists($WebPic);
        UnlinkIfExists($Thumbnail);
        if (is_numeric($VideoSn))
        {
            UnlinkIfExists($VidFile);
            UnlinkIfExists($OggFile);
        }
        $Success = True;
    } else $Success = False;
    return $Success;
}

function FinalLocationSn($LocationSn, $OtherLoc)
{
    if ($LocationSn == "other" or $LocationSn == "unknown")
    {
        $OtherLoc = trim($OtherLoc);
        if ($OtherLoc != "")
        {
            $LocationSn = LocationSn($OtherLoc);
            UpdateLocationTimeStamp($LocationSn);
        } else
            $LocationSn = False;
    } else UpdateLocationTimeStamp($LocationSn);
    return $LocationSn;
}

function GeoCoordinates($PhotoSn)
{
    /*****************************************************************
      Given a photo in the database, returns a GeoLocation() string
      with latitude, longitude, altidude, and an html link to
      geohack.toolforge.org or False if photo has no geo-data.
    *****************************************************************/
    $Res = ResGeoCoordinates($PhotoSn);
    return ($Res !== False)
                  ? GeoLocation(pg_fetch_array($Res, 0,  PGSQL_ASSOC))
                  : False;
}

function GeoLocation($CoordinateArray)
{
    /*****************************************************************
      Given an array(latitude, longitude, altitude), returns a string
      that is an html link to geohack.toolforge.org displaying the
      coordinates in degrees and decimal minutes, altitude in meters,
      e.g.,

      N 42° 14.500' W 072° 38.883' / 366.4m
      or, if no altitude available:
      N 42° 14.500' W 072° 38.883'
    *****************************************************************/
    list($Latitude, $Longitude, $Altitude)
        = array_values($CoordinateArray);
    $URL = "https://geohack.toolforge.org/geohack.php";
    $URL .= "?params=$Latitude;$Longitude";
    $LatDM = DegToDegMin($Latitude, "N", "S", 2);
    $LonDM = DegToDegMin($Longitude, "E", "W", 3);
    $Display = "$LatDM $LonDM";
    if (is_numeric($Altitude))
        $Display .= " / ".sprintf("%0.1f", round($Altitude, 1))."m";
    return "<a href=\"$URL\" target=\"_blank\">$Display</a>";
}

function ImageCell($Res, $Row, $NumRows, $StateVarName=False,
                   $StateVarValue=False, $Selected=False,
                   $EditOk=False, $ShowLDT=False)
{
    list ($PhotoSn, $FolderName, $FileName, $LocationSn, $Description,
          $Width, $Height, $ThumbW, $ThumbH, $Time, $LocName,
          $VidFileName, $Z0, $Z1, $Z2, $IsTagged)
        = pg_fetch_array($Res, $Row);
    list($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    if ($EditOk)
    {
        $S = "<label>".HtmlCheckbox("Sel$Row", $Selected);
        $S .= HtmlHidden("PS$Row", $PhotoSn);
        if ($ShowLDT)
        {
            $E = is_null($LocationSn) ? " L" : "";
            $E .= ($Description == "") ? " D" : "";
            $E .= ($IsTagged == "f") ? " T" : "";
            if ($E > "")
                $S .= Strong(HtmlColor(WarningColor, $E))." ";
        }
        $S .= "$FileName</label>";
    } else $S = "$FileName";
    $S .= "<br />\n<a target=\"_blank\" ";
    $S .= ($StateVarName)
       ? ImageHref($StateVarName, htmlentities($StateVarValue), $Row)
       : "href=\"".ImageUrl($PhotoSn)."\"";
    $S .= ">";
    $Path = "$ThumbFolder/$FileName";
    $V = ($VidFileName > "") ? "(Video) " : "";
    $Alt = "$FileName $V$Time ($LocName) $Description";
    $Alt .= ImageTags($PhotoSn);
    $S .= HtmlImage(LatestFile($Path), $Alt, $ThumbH, $ThumbW, 1);
    $S .= "</a>";
    $Style = ($VidFileName > "") ? "background: ".HighlightBg : "";
    return StrCell($S, "", "", $Style);
}

function ImageCells($Res, $NumRows, &$RangeMarks=False,
                    $StateVarName=False, $StateVarValue=False,
                    $EditOk=False, $ShowLDT=False)
{
    /*****************************************************************
    Display photo thumbnails in rows of four.
    If $RangeMarks is False, clear all thumbnail selection checkboxes.
    If it's True (and Boolean, rather than an array), set them all.

    Otherwise, $RangeMarks is an array of elements (Photosn), each of
    which starts or ends a range. If a range has been started but no
    marker ends it, it will extend to the last image on the page.
    *****************************************************************/
    $IsByRange = is_array($RangeMarks);
    $SwitchAfter = False;
    $NumSelected = 0;
    if ($IsByRange)
    {
        $NumRanges = ceil(count($RangeMarks)/2);
        $Selected = False;
        $NextMark = array_shift($RangeMarks);
    } else $Selected = $RangeMarks;
    $S = StrTableBegin("0");
    for ($Row=0 ; $Row < $NumRows ; $Row++ )
    {
        if ($IsByRange)
        {
            $PhotoSn = pg_fetch_result($Res, $Row, "photosn");
            if ($PhotoSn == $NextMark)
            {
                if ($Selected) $SwitchAfter = True;
                else $Selected = True;
                $NextMark = array_shift($RangeMarks);
            }
        }
        if ($Row % 4 == 0)
        {
            if ($Row > 0) $S .= StrRowEnd();
            $S .= StrRowBegin();
        }
        $S .= ImageCell($Res, $Row, $NumRows, $StateVarName,
                        $StateVarValue, $Selected, $EditOk,
                        $ShowLDT);
        if ($Selected) $NumSelected++;
        if ($SwitchAfter) $Selected = False;
        $SwitchAfter = False;
    }
    for ($I=0 ; $I < (3 - (($NumRows -1) % 4)) ; $I++)
        $S .= StrCell("&nbsp;");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    if ($IsByRange)
    {
        // Use the $RangeMarks argument to return statistics
        $Plural = ($NumRanges > 1) ? "s" : "";
        $T = "Selected $NumSelected photos in $NumRanges specified";
        $T .= " range$Plural.";
        $RangeMarks = $T;
    }
    return $S;
}

function ImageHref($StateVarName, $StateVarValue, $Row)
{
    $H = "href=\"".ThisScript();
    $H .= "?$StateVarName=$StateVarValue&Row=$Row&ViewLarge=1\"";
    return $H;
}

function ImagePath($PhotoSn)
{
    list($FolderName, $FileName, $VideoSn, $VidFileName)
        = pg_fetch_array(ResPhotoVideo($PhotoSn), 0);
    $FullFolder = "fullsize/$FolderName";
    $FullImage = "$FullFolder/$FileName";
    $WebFolder = "webpics/$FolderName";
    $WebPic = "$WebFolder/$FileName";
    $Thumbnail = "$WebFolder/thumbnails/$FileName";
    if (is_numeric($VideoSn))
    {
        $VidFile = "$FullFolder/$VidFileName";
        $OggFile = "$WebFolder/".OggFile($VidFileName);
    } else
    {
        $VidFile = False;
        $OggFile = False;
    }
    return array
        (
            $FullImage,
            $WebPic,
            $Thumbnail,
            $VideoSn,
            $VidFile,
            $OggFile
        );
}

function ImageTags($PhotoSn)
{
    // Enumerates any tags applied to photo $PhotoSn, as a string to
    // append to the thumbnail mouseover title
    $S = "";
    $Res = ResPhotoTags($PhotoSn);
    $NumRows = pg_num_rows($Res);
    if ($NumRows > 0)
    {
        $S = ", tagged \"".pg_fetch_result($Res, 0, 0)."\"";
        for ($Row = 1 ; $Row < $NumRows ; $Row ++)
        {
            $S .= ", \"".pg_fetch_result($Res, $Row, 0)."\"";
        }
    }
    return $S;
}

function ImageUrl($PhotoSn)
{
    return "viewset.php?PhotoSn=$PhotoSn&ViewLarge=1";
}

function IsLocationArchived($LocationSn)
{
    if (is_numeric($LocationSn))
    {
        $Q = "SELECT 1 FROM archivedlocation WHERE locationsn = $1";
        return (pg_num_rows(Query($Q, $LocationSn)) > 0);
    } else return False;
}

function LinkButtonPara($FolderName, $FileName, $VidFileName)
{
    $IsVideo = ($VidFileName > "");
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $FetchVideo = BtnGetVideo($FolderName, $VidFileName);
    $FetchFull = BtnGetFullSize($FolderName, $FileName, $IsVideo);
    $ThisScript = ThisScript();
    if ($ThisScript != "editset.php" and $ThisScript != "viewset.php")
    {
        $PhotoSetSn = SerialNumber("photoset", "foldername",
                                   "$FolderName");
        $ViewSet = StrButton(array
                             (
                                 "Text"	=> "View Set",
                                 "Url"	=> "viewset.php?PhotoSetSn=$PhotoSetSn",
                                 "Color"	=> ButtonColor
                             ));
    } else $ViewSet = "";
    $T = $FetchVideo.$FetchFull.$ViewSet.StrButton(BtnClose());
    return HtmlPara($T, "right");
}

function Location($LocSn)
{
    if (is_numeric($LocSn))
    {
        $Location = ColFromSerialNumber("location", "name", $LocSn);
        if ($Location === False) $Location = NullLocationFlag;
    } else $Location = NullLocationFlag;
    return $Location;
}

function LocationsByTime()
{
    // Return array of most-recently-selected locations
    // Each row is an array(locationsn, name, notes)
    $Locs = array();
    $Q = "SELECT locationsn, name, notes FROM location";
    $Q .= " ORDER BY last_used DESC";
    $Res = Query($Q);
    // for each most-recently-used location:
    $Rows = pg_num_rows($Res);
    if ($Rows > RecentLocs) $Rows = RecentLocs;
    for ($Row=0 ; $Row < $Rows ; $Row++ )
        $Locs[$Row] = pg_fetch_array ($Res, $Row);
    return $Locs;
}

function LocationSn($LocName)
{
    // Returns locationsn of $LocName in location table
    // Inserts it if it doesn't already exist
    $LocName = trim($LocName);
    $Sn = SerialNumber("location", "name", $LocName);
    if ($Sn === False)
    {
        $Q = "INSERT INTO location (name, last_used)";
        $Q .= " VALUES ($1, CURRENT_TIMESTAMP)";
        Query($Q, $LocName);
        $Sn = LocationSn($LocName);
    }
    return $Sn;
}

function MenuLocation($LocationSn=Null, $ShowRecent=True,
                      $ShowArchived=False, $NullLabel="(unknown)",
                      $AutoSubmit=False, $AskOther=True,
                      $AlwaysOfferNull=False, $OfferAny=False)

{
    $ShowArchived = ($ShowArchived or
                     IsLocationArchived($LocationSn));
    $LocsByName = ResLocnsByName($ShowArchived);
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin("LocationSn", $Attribs);
    if ($OfferAny)
        $S .= HtmlSelectOption("any", "(anywhere)",
                               ($LocationSn == "any"));
    if (! is_numeric($LocationSn) or $OfferAny)
    {
        $S .= HtmlSelectOption("unknown", $NullLabel,
                               ($LocationSn == "unknown"));
        // for each most-recently-used location:
        if ($ShowRecent)
        {
            $LocsByTime = LocationsByTime();
            foreach ($LocsByTime as $Loc)
                $S .= HtmlSelectOption(
                    $Loc["locationsn"], $Loc["name"],
                    False, $Loc["notes"], "background: ".BgColor);
            if ($AskOther)
                $S .= HtmlSelectOption("other", "other", False,
                                       HintOtherLocation);
        }
    } elseif ($AlwaysOfferNull)
          $S .= HtmlSelectOption("unknown", $NullLabel, True);
    // for each record in location table:
    for ($Row=0 ; $Row < pg_num_rows($LocsByName) ; $Row++ )
    {
        list($Sn, $Name, $Notes, $IsArchived)
            = pg_fetch_row($LocsByName, $Row);
        $Style = ($IsArchived == "t")
               ? "background: ".ArchivedLocationColor : "";
        $S .= HtmlSelectOption($Sn, $Name, ($Sn == $LocationSn),
                               $Notes, $Style);
    }
    if ($AskOther) $S .= HtmlSelectOption("other", "other", False,
                                          HintOtherLocation);
    $S .= HtmlSelectEnd ();
    if ($AutoSubmit) $S = StrWithGoButton($S, "Go");
    return $S;
}

function MenuPhotoSet($Selection="", $AutoSubmit=False,
                      $Prompt="&mdash; Select photo set &mdash;")
{
    // Drop-down menu of foldernames in photoset table
    $Res  = ResPhotoSet();
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin("PhotoSetSn", $Attribs);
    if ($Selection == "" or $Selection == "none")
        $S .= HtmlSelectOption("none", $Prompt, True);
    for ($Row = 0 ; $Row < pg_num_rows($Res) ; $Row ++)
    {
        list($PhotoSetSn, $FolderName, $Nickname)
            = pg_fetch_array($Res, $Row);
        $S .= HtmlSelectOption($PhotoSetSn, "$FolderName  $Nickname",
                               ($PhotoSetSn == $Selection));
    }
    $S .= HtmlSelectEnd();
    //     if ($AutoSubmit) $S = StrWithGoButton($S, "Go");
    return $S;
}

function NextPhotoSetSn($FolderName)
{
    /* Return sn of photo set whose foldername immediately follows
     * $FolderName in menu or, if none, of that whose foldername
     * immediately precedes $FolderName, */
    $Q = "SELECT photosetsn FROM photoset WHERE foldername > $1";
    $Q .= " ORDER BY foldername";
    $NextSn = pg_fetch_result(Query($Q, $FolderName), 0);
    if (is_numeric($NextSn)) return $NextSn;
    else
    {
        $Q = "SELECT photosetsn FROM photoset WHERE foldername < $1";
        $Q .= " ORDER BY foldername DESC";
        return pg_fetch_result(Query($Q, $FolderName), 0);
    }
}

function OptionAddToAlbum()
{
    $S = StrRowBegin();
    $S .= StrCell("Add selected photos<br />\nto this album:");
    $C = WidgetSelectAlbum();
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionAssignTag()
{
    $S = StrRowBegin();
    $S .= StrCell("Tag selected photos:");
    $C = WidgetSelectTag();
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionDeletePhotos()
{
    $S = StrRowBegin();
    $S .= StrCell("Delete selected photos:");

    $C = StrTableBegin("0", "", "100%");
    $C .= StrRowBegin();
    $C .= "<td align=\"right\">This is final, and can't be undone.";
    $C .= " Be sure!</td>\n";
    $C .= "<td align=\"right\">".StrDeleteButtonWidget("Delete");
    $C .= "</td>\n";
    $C .= StrRowEnd();
    $C .= StrTableEnd();
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionSetDescription()
{
    $S = StrRowBegin();
    $T = "Assign this description<br />\nto selected photos:";
    $S .= StrCell($T);
    $C = WidgetEnterDescription();
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function OptionSetLocation($LocationSn=Null)
{
    $S = StrRowBegin();
    $S .= StrCell("Assign this location<br />\nto selected photos:");
    $C = WidgetSelectLocation($LocationSn);
    $S .= StrCell($C, "", "", BoxStyle);
    $S .= StrRowEnd();
    return $S;
}

function PackedState($StateArray)
{
    // return htmlentities(serialize($StateArray));
    return serialize($StateArray);
}

function PhotoActionOptions($AllowRisky, $StrOptionalRows="")
{
    // Vertical array of actions that may be applied to selected
    // photos in thumbnail gallery
    $S = StrTableBegin("0");
    $S .= OptionSetLocation();
    if ($AllowRisky) $S .= OptionSetDescription();
    $S .= OptionAddToAlbum();
    $S .= OptionAssignTag();
    $S .= $StrOptionalRows;
    if ($AllowRisky) $S .= OptionDeletePhotos();
    $S .= StrTableEnd();
    return $S;
}

function PhotoDependences($PhotoSn)
{
    // If any album photos or derived photos depend on this one, or
    // the photo has any tags owned by other users, return a list of
    // them, else return False
    $S = PhotoDependencesAlbum($PhotoSn);
    $S .= PhotoDependencesDerived($PhotoSn);
    if ($S > "")
    {
        list($FolderName, $FileName)
            = pg_fetch_array(ResPhoto($PhotoSn), 0);
        $R = StrBoxBegin("5", "center", WarningColor);
        $R .= StrBoxBegin("0", "left", TableColor2);
        $T = "Cannot delete photo $FolderName/$FileName";
        $R .= StrSubTitle(HtmlColor(WarningColor, $T));
        $R .= $S;
        $R .= StrBoxEnd();
        $R .= StrBoxEnd();
        return $R;
    } else return False;
}

function PhotoDependencesAlbum($PhotoSn)
{
    // Return any albums that include this photo
    $Q = "SELECT a.name, pg.page FROM albumphoto ap, albumpage pg,";
    $Q .= " album a";
    $Q .= " WHERE photosn = $1 AND pg.albumpagesn = ap.albumpagesn";
    $Q .= " AND a.albumsn = pg.albumsn";
    $Res = Query($Q, $PhotoSn);
    $Rows = pg_num_rows($Res);
    if ($Rows > 0)
    {
        $T = "Photo occurs in album(s):";
        $S = HtmlPara(Strong($T));
        $S .= StrTableBegin("0", TableColor2);
        $S .= StrRowBegin();
        $S .= StrCell(Strong("Album"));
        $S .= StrHeaderCell("Page");
        $S .= StrRowEnd();
        for ($Row = 0 ; $Row < $Rows ; $Row++)
        {
            list ($AlbumName, $Page) = pg_fetch_array ($Res, $Row);
            $S .= StrRowBegin();
            $S .= StrCell($AlbumName);
            $S .= "<td align=\"center\">$Page</td>\n";
            $S .= StrRowEnd();
        }
        $S .= StrTableEnd();
    } else $S ="";
    return $S;
}

function PhotoDependencesDerived($PhotoSn)
{
    // Return any photos derived from this one
    $Res = ResDerivedPhotos($PhotoSn);
    if ($Res)
    {
        $S = HtmlPara(Strong("Photo has derived photo(s):"));
        $S .= StrTableBegin("0", TableColor2);
        for ($Row = 0 ; $Row < pg_num_rows($Res) ; $Row++)
        {
            list($FolderName, $FileName) = pg_fetch_array($Res, $Row);
            $S .= StrRowBegin();
            $S .= StrCell("$FolderName/$FileName");
            $S .= StrRowEnd();
        }
        $S .= StrTableEnd();
    } else $S ="";
    return $S;
}

function PhotoPointer($PhotoSn)
{
    // Return filename of photo for inclusion in a list. Where
    // foldername can vary within the list (when not directly viewing
    // or editing a single photoset), prepend foldername.
    list($FolderName, $FileName)
        = pg_fetch_array(ResPhoto($PhotoSn), 0);
    $ThisScript = ThisScript();
    return ($ThisScript == "editset.php"
            or $ThisScript == "viewset.php"
            or $ThisScript == "stitchpano.php")
                        ? $FileName : "$FolderName/$FileName";
}

function PhotoStats($Res, $Row, $IsFirst)
{
    // Display either the first or the last photo in $Res, with
    // statistics
    list ($PhotoSn, $FolderName, $FileName, $LocationSn, $Description,
          $Z0, $Z1, $ThumbW, $ThumbH, $Time, $LocName, $VidFileName)
        = pg_fetch_array($Res, $Row);
    list($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $Path = "$ThumbFolder/$FileName";
    $V = "";
    $CellStyle = "text-align: right";
    if ($VidFileName > "")
    {
        $V = "(Video) ";
        $CellStyle .= "; background: ".HighlightBg;
    }
    $S = StrTableBegin(0, "", "",
                       BoxStyle."; width: 60%; margin-bottom: 1ex");
    $S .= StrRowBegin();
    $T = "<span style=\"background: yellow\">";
    $T .= ($IsFirst ? "First" : "Last")." photo:</span><br />\n";
    $T .= "$Time<br />\n";
    $T .= "File ".PhotoPointer($PhotoSn)."<br />\n";
    $T .= "Location: ${LocName}";
    $T .= HtmlPara($Description);
    $S .= StrCell($T);
    $T = HtmlImage(LatestFile($Path),
                   "$FileName $V$Time ($LocName) $Description",
                   $ThumbH, $ThumbW, 1);
    $S .= StrCell($T, "", "", $CellStyle);
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}
function ResDerivedPhotos($ParentPhotoSn)
{
    // Return all photos derived from $ParentPhotoSn ("children")
    $Q = "SELECT foldername, filename";
    $Q .= " FROM derived_photo dp, photo p, photoset s";
    $Q .= " WHERE parentphotosn = $1 AND photosn = childphotosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY foldername, filename";
    $Res = Query($Q, $ParentPhotoSn);
    return (pg_num_rows($Res) > 0) ? $Res : False;
}

function ResGeoCoordinates($PhotoSn)
{
    // Returns a tuple with latitude, longitude[, altitude] or False
    $Q = "SELECT latitude, longitude, altitude FROM photo";
    $Q .= " WHERE photosn = $1 AND latitude IS NOT NULL";
    $Q .= " AND longitude IS NOT NULL";
    $Res = Query($Q, $PhotoSn);
    return (pg_num_rows($Res) > 0) ? $Res : False;
}

function ResLocnsByName($ShowArchived=False)
{
    /* Return a query result (locationsn, name, notes, isarchived) */
    $Q = "SELECT l.locationsn, name, notes, ";
    $Q .= $ShowArchived ? "al.locationsn IS NOT NULL" : "False";
    $Q .= " AS isarchived FROM location l";
    $Q .= " LEFT OUTER JOIN archivedlocation al USING (locationsn)";
    if (! $ShowArchived) $Q .= " WHERE al.locationsn IS NULL";
    $Q .= " ORDER BY lower(name)";
    return Query($Q);
}

function ResPhoto($PhotoSn)
{
    // Return all attributes of a single photo
    global $PhotoMetaData;
    $Q = "SELECT foldername, filename, p.photosetsn, p.locationsn,";
    $Q .= " description, p.width, p.height, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay;
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " vidfilename, videosn, v.width AS vwidth,";
    $Q .= " v.height AS vheight";
    foreach ($PhotoMetaData as $Key)
    {
        $Q .= ", $Key";
    }
    $Q .= " FROM photoset s, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= " WHERE photosn = $1 AND s.photosetsn = p.photosetsn";
    return Query($Q, $PhotoSn);
}

function ResPhotoSet($PhotoSetSn=False, $FolderName=False)
{
    /*****************************************************************
      Return all attributes of all photo sets, ordered by foldername, or
      of a single photo set specified by either $PhotoSetSn or
      $FolderName, so result is:
      ($PhotoSetSn, $FolderName, $Nickname, $Notes)
    *****************************************************************/
    $Q = "SELECT photosetsn, foldername, nickname, notes";
    $Q .= " FROM photoset";
    if ($PhotoSetSn !== False)
        $Res = Query($Q." WHERE photosetsn = $1", $PhotoSetSn);
    elseif ($FolderName !== False)
        $Res = Query($Q." WHERE foldername = $1", $FolderName);
    else
        $Res = Query($Q." ORDER BY foldername DESC");
    return $Res;
}

function ResPhotosInSet($PhotoSetSn, $Sort="Time", $HideLocDone=False,
                        $HideDescDone=False, $HideTimeDone=False,
			$HideTagged=False)
{
    /*****************************************************************
      Return all attributes of all photos in set $PhotoSetSn, ordered
      by $Sort and filtered by $HideLocDone, $HideDescDone,
      $HideTimeDone and $HideTagged:
      ($PhotoSn, $FolderName, $FileName, $LocationSn, $Description,
      $Width, $Height, $ThumbW, $ThumbH, $Time, $LocName,
      $VidFileName, $VWidth, $VHeight)
    *****************************************************************/
    $Q = "SELECT photosn, foldername, filename, p.locationsn,";
    $Q .= " description, p.width, p.height, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay;
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " vidfilename, v.width AS vwidth, v.height AS vheight,";
    $Q .= " time, EXISTS (SELECT 1 FROM tag_photo";
    $Q .= "  WHERE photosn = p.photosn) AS istagged";
    $Q .= " FROM photoset s, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= " WHERE s.photosetsn = p.photosetsn";
    $Q .= " AND p.photosetsn = $1";
    if ($HideLocDone or $HideDescDone or $HideTimeDone or $HideTagged)
    {
        $Q .= " AND (FALSE";
        if ($HideLocDone) $Q .= " OR p.locationsn IS NULL";
        if ($HideDescDone)
            $Q .= " OR description IS NULL OR description = ''";
        if ($HideTimeDone)
            $Q .= " OR time IS NULL";
        if ($HideTagged)
        {
            $Q .= " OR NOT EXISTS (SELECT 1 FROM tag_photo";
            $Q .= "  WHERE photosn = p.photosn)";
        }
        $Q .= ")";
    }
    $Q .= " ORDER BY ";
    $Q .= ($Sort == "Time") ? "time" : "locname, time";
    $Q .= ", lower(filename)";
    return Query($Q, $PhotoSetSn);
}

function ResPhotoTags($PhotoSn)
{
    $Q = "SELECT name FROM tag_photo tp, tag t";
    $Q .= " WHERE photosn = $1 AND tp.tagsn = t.tagsn ORDER BY name";
    return Query($Q, $PhotoSn);
}

function ResPhotoVideo($PhotoSn)
{
    // Return some attributes of a single photo
    $Q = "SELECT foldername, filename, videosn, vidfilename";
    $Q .= " FROM photoset s, photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
    $Q .= " WHERE photosn = $1 AND s.photosetsn = p.photosetsn";
    return Query($Q, $PhotoSn);
}

function RotatePhoto($PhotoSn, $Rotation)
{
    // Rotates all image files (fullsize, webpic, thumbnail)
    // and updates dimensions in database
    list ($FullImage, $WebPic, $Thumbnail, $VideoSn, $VidFile,
          $OggFile) = ImagePath($PhotoSn);
    $FullSize = RotatedSize($FullImage, $Rotation);
    $WebSize = RotatedSize($WebPic, $Rotation);
    $ThumbSize = RotatedSize($Thumbnail, $Rotation);
    $Q = "UPDATE photo SET";
    $Q .= " width = ".$WebSize["width"];
    $Q .= ", height = ".$WebSize["height"];
    $Q .= ", thumbw = ".$ThumbSize["width"];
    $Q .= ", thumbh = ".$ThumbSize["height"];
    $Q .= " WHERE photosn = $1";
    Query($Q, $PhotoSn);
    if (is_numeric($VideoSn))
    {
        list($VWidth, $VHeight, $Format, $Codec)
            = array_values(VideoSpecs($VidFile));
        RotateVideo($VidFile, $Codec, $Rotation);
        // Update dimensions after rotation
        list($VWidth, $VHeight) = array_values(VideoSpecs($VidFile));
        // Don't abort just because it takes a while
        set_time_limit (0);
        ConvertToOgg($VidFile, $OggFile);
        $Q = "UPDATE video SET width = $2, height = $3";
        $Q .= " WHERE videosn = $1";
        Query($Q, $VideoSn, $VWidth, $VHeight);
    }
    return;
}

function RotateVideo($VidFile, $Codec, $Rotation)
{
    // Rotates video file $VidFile $Rotation degrees (90, -90, 180)
    // using tool appropriate for codec
    /* Can ignore ID_VID_0_ROTATE arguments, since ffmpeg handles
     * correctly */
    switch ($Codec)
    {
    case "ffhevc":
    case "ffh265":
    case "ffh264":
        RotateVideoFfmpeg($VidFile, $Rotation);
        break;
    case "ffodivx":
    case "ffmjpeg":
    case "fftheora":
        RotateVideoMencoder($VidFile, $Rotation);
        break;
    default:
        // Consider some warning about untested codec $Codec; for now,
        // do nothing
    }
    return;
}

function RotateVideoFfmpeg($VidFile, $Rotation)
{
    // Rotates video file $VidFile $Rotation degrees (90, -90, 180)
    // using ffmpeg tool
    switch ($Rotation)
    {
    case 90:
        $Options = "-vf transpose=1";
        break;
    case 0:
        $Options = "";
        break;
    case -90:
        $Options = "-vf transpose=2";
        break;
    case 180:
        $Options = "-vf hflip,vflip";
        break;
    default:
        $Options = False;
        // Consider some warning about allowed rotations; for now, do
        // nothing
    }
    if ($Options !== False)
    {
        // Avoid known bug in ffmpeg:
        $Options .= " -max_muxing_queue_size 9999";
        list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
        if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
        $Tempfile = "$FullUploads/".basename($VidFile);
        $Rot = "ffmpeg -i $VidFile $Options";
        $Rot .= " -strict experimental $Tempfile";
        $Rotate = shell_exec($Rot);
        rename($Tempfile, $VidFile);
    }
    return;
}

function RotateVideoMencoder($VidFile, $Rotation)
{
    // Rotates video file $VidFile $Rotation degrees (90, -90, 180)
    // using mencoder tool
    switch ($Rotation)
    {
    case 90:
        $Options = "-vf rotate=1";
        break;
    case 0:
        $Options = "";
        break;
    case -90:
        $Options = "-vf rotate=2";
        break;
    case 180:
        $Options = "-vf mirror,flip";
        break;
    default:
        $Options = False;
        // Consider some warning about allowed rotations; for now, do
        // nothing
    }
    if ($Options !== False)
    {
        list($FullUploads, $Z, $Z1) = ImageFolder(".uploads");
        if (! file_exists($FullUploads)) mkdir($FullUploads, 0755);
        $Tempfile = "$FullUploads/".basename($VidFile);
        $Rot = "mencoder $VidFile -o $Tempfile $Options";
        $Rot .= " -oac pcm -ovc lavc";
        $Rotate = shell_exec($Rot);
        rename($Tempfile, $VidFile);
    }
    return;
}

function RowImgData($FullFolder, $FolderName, $FileName, $PhotoSn,
                    $Description, $Time, $LocationSn, $VidFileName)
{
    $Color = ($VidFileName > "") ? HighlightBg : TableColor;
    $S = StrRowBegin($Color);
    $T = HtmlPara($Description);
    $T .= "Taken $Time";
    $T .= " at ".Location($LocationSn);
    $G = GeoCoordinates($PhotoSn);
    if ($G !== False) $T .= "<br />$G";
    $T .= "<br />File ".PhotoPointer($PhotoSn);
    $T .= TagOccurrences($PhotoSn);
    $T .= AlbumOccurrences($PhotoSn);
    $S .= StrCell($T, "", "2");
    if (ThisScript() != "editset.php"
        and UserHasPrivilege(LoggedInUser(), "edit"))
    {
        $T = StrFormBegin("editset.php");
        $T .= HtmlHidden("PhotoSn", $PhotoSn);
        $T .= HtmlSubmitButton("EditLargeBySn", "Edit");
        $T .= StrFormEnd();
    } else $T = "&nbsp;";
    $S .= "<td align=\"right\">$T<td>\n";
    $S .= StrRowEnd();
    $S .= StrRowBegin($Color);
    $T = LinkButtonPara($FolderName, $FileName, $VidFileName);
    $S .= "<td align=\"right\" valign=\"bottom\">$T<td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowSearchSortOrder($Sort, $AutoSubmit=False)
{
    if (! isset($Sort)) $Sort = "Time";
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = StrRowBegin();
    $S .= StrCell(Strong("Sort photos:"));
    $C = HtmlRadioButton("Sort", "Time", ($Sort == "Time"), $Attribs);
    $C .= " by time";
    $C = "<label>$C</label><label>";
    $C .= HtmlRadioButton("Sort", "LocTime", ($Sort == "LocTime"),
                          $Attribs);
    $C .= " by place and time</label>";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSelectPhotoSet($PhotoSetSn)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("Select photo set:"));
    $S .= "<td align=\"right\">";
    $S .= MenuPhotoSet($PhotoSetSn, True);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowWebImg($WebFolder, $FileName, $Width, $Height,
                   $Description, $VidFileName)
{
    // Large image
    $Color = ($VidFileName > "") ? HighlightBg : "";
    $S = StrRowBegin($Color);
    $S .= "<td align=\"center\" colspan=\"2\">\n";
    $T = UrlDownLoad("FetchWebPic", "$WebFolder/$FileName");
    $S .= "<a href=\"$T\">";
    $S .= HtmlImage("$WebFolder/$FileName", $Description, $Height,
                    $Width, "1", $Description, "#PicMap");
    $S .= "</a>";
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function ScaledWebPicLength($Width, $Height)
{
    /* For extra-wide or extra-tall (aspect ratio > 3:2) images,
     * adjust longer webpic dimension so that shorter webpic dimension
     * is 3/4 WebPicLength */
    $Long = max($Width, $Height);
    $Short = min($Width, $Height);
    return ($Short >= floor(2 * $Long) / 3)
                    ? WebPicLength : round(0.75 * $Long * WebPicLength / $Short);
}

function StrLargeImageForm($State, $Res, $Row)
{
    $NumRows = pg_num_rows($Res);
    $Row = BoundedValue($Row, $NumRows - 1, 0);
    list ($PhotoSn, $FolderName, $FileName, $LocationSn, $Description,
          $Width, $Height, $ThumbW, $ThumbH, $Time, $LocName,
          $VidFileName, $VWidth, $VHeight)
        = pg_fetch_array($Res, $Row);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $State = htmlentities($State);
    /* $State = htmlentities(stripslashes($State)); */
    list ($PrevRow, $NextRow) = AdjacentRows($Row, $NumRows);
    $HrefFwd = ImageHref("State", $State, $NextRow);
    $HrefBack = ImageHref("State", $State, $PrevRow);
    $IsVideo = ($VidFileName > "");
    if ($IsVideo)
        $S = StartOfPage("$VidFileName (video)", "", "", True);
    else
    {
        $S = StartOfPage("$FolderName/$FileName (large)", "", "",
                         True);
        $S .= "<map name=\"PicMap\">\n";
        $S .= "<area shape=\"rect\" coords=\"0,0,".PageEdge;
        $S .= ",$Height\" $HrefBack>\n";
        $S .= "<area shape=\"rect\" coords=\"".($Width - PageEdge);
        $S .= ",0,$Width,$Height\" $HrefFwd>\n";
        $S .= "</map>\n";
    }
    $S .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">";
    $S .= "\n<tbody>\n";
    if ($IsVideo)
    {
        $S .= RowWebVid($WebFolder, $VidFileName);
        $S .= RowNavButtons($HrefBack, $HrefFwd);
    } else $S .= RowWebImg($WebFolder, $FileName, $Width, $Height,
                           $Description, $VidFileName);
    // need PhotoSn here
    $S .= RowImgData($FullFolder, $FolderName, $FileName, $PhotoSn,
                     $Description, $Time, $LocationSn, $VidFileName);
    $S .= StrTableEnd();
    return $S;
}

function StrSelectionOptions()
{
    $S = StrTableBegin("0", "");
    $S .= StrRowBegin();
    $S .= StrCell(HtmlSubmitButton("SelectAll", "Select All"));
    $S .= "<td style=\"width: 10%\">&nbsp;</td>\n";
    $S .= StrCell(HtmlSubmitButton("SelectRange", "Select Range"));
    $S .= "<td style=\"width: 40%\">&nbsp;</td>\n";
    // Can't easily handle toggle; when can, uncomment
    //     $S .= StrCell(HtmlSubmitButton("Toggle", "Toggle Selections"));
    //     $S .= "<td style=\"width: 15%\">&nbsp;</td>\n";
    $S .= StrCell(HtmlSubmitButton("ClearAll", "Clear Selections"));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function TagOccurrences($PhotoSn)
{
    // Enumerates any tags applied to photo $PhotoSn, as a definition
    // list
    $S = "";
    $Res = ResPhotoTags($PhotoSn);
    $NumRows = pg_num_rows($Res);
    if ($NumRows > 0)
    {
        $S = "<dl>\n<dt><strong>Tagged:</strong></dt>\n";
        for ($Row = 0 ; $Row < $NumRows ; $Row ++)
        {
            $Name = pg_fetch_result($Res, $Row, 0);
            $S .= "<dd>$Name";
            $S .= "</dd>\n";
        }
        $S .= "</dl>\n";
    }
    return $S;
}

function TupleNumInPhotoSetBySn($PhotoSn)
{
    // Return tuple number (ordinal) of photo $PhotoSn in its
    // containing photo set using default order by time, filename
    $Q = "SELECT photosetsn, time, filename";
    $Q .= " FROM photo WHERE photosn = $1";
    $Res = Query($Q, $PhotoSn);
    if (pg_num_rows($Res) > 0)
    {
        list($PhotoSetSn, $Time, $FileName) = pg_fetch_array($Res, 0);
        $Q = "SELECT 1 FROM photo WHERE photosetsn = $1";
        if ($Time > "")
        {
            $Q .= " AND (time < '$Time'";
            $Q .= " OR (time = '$Time' AND filename < $2))";
        } else $Q .= " AND filename < $2";
        $Row = pg_num_rows(Query($Q, $PhotoSetSn, $FileName));
    } else $Row = 0;
    return $Row;
}

function UnPackedState($State)
{
    return unserialize(html_entity_decode($State));
    /* return unserialize(stripslashes(html_entity_decode($State))); */
}

function UpdatePhotoGeoData($GeoData, $StillSn)
{
    list($Latitude, $Longitude, $Altitude) = array_values($GeoData);
    $NullAlt = ($Altitude == "NULL");
    $Q = "UPDATE photo SET";
    $Q .= " latitude = $2, longitude = $3,";
    $Q .= " altitude = ".($NullAlt ? "NULL" : "$4");
    $Q .= " WHERE photosn = $1";
    if ($NullAlt)
        Query($Q, $StillSn, $Latitude, $Longitude);
    else
        Query($Q, $StillSn, $Latitude, $Longitude, $Altitude);
    return;
}

function UpdateLocationTimeStamp($LocationSn)
{
    $Q = "UPDATE location SET last_used = CURRENT_TIMESTAMP";
    $Q .= " WHERE locationsn = $1";
    Query($Q, $LocationSn);
    return;
}

function WidgetEnterDescription($Description="")
{
    $S = StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $T = Strong("Description: ")."<br />\n";
    $T .= HtmlTextArea ("Description", "40", "3", $Description);
    $S .= StrCell($T);
    $S .= "<td align=\"right\">";
    $S .= HtmlSubmitButton("ApplyDesc", "Apply");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function WidgetSelectAlbum($Selection="")
{
    $S = "<table border=\"0\" width=\"100%\" cellspacing=\"0\"";
    $S .= " cellpadding=\"0\"><tbody>";
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Album: "));
    $S .= "<td align=\"right\">".MenuAlbum($Selection, False, True);
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= StrRowBegin();
    $C = StrTableBegin("0", "", "100%");
    $C .= StrRowBegin();
    $C .= StrCell("New Album name: ".HtmlTextInput("NewAlbum", "25"));
    $C .= "<td align=\"right\">";
    $C .= HtmlSubmitButton("AddToAlbum", "Append");
    $C .= "</td>\n";
    $C .= StrRowEnd();
    $C .= StrTableEnd();
    $S .= StrCell($C, "2");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function WidgetSelectLocation($LocationSn=Null)
{
    $S = Strong("Where taken: ")."\n";
    $S .= MenuLocation($LocationSn);
    $S .= "<br />If \"other\", specify:<br />\n";
    $S .= StrTableBegin("0", "", "100%");
    $S .= StrRowBegin();
    $S .= StrCell(HtmlTextInput("OtherLoc", "35"));
    $S .= "<td align=\"right\">";
    $S .= HtmlSubmitButton("ApplyLoc", "Apply");
    $S .= "</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

function WidgetSelectTag()
{
    $S = "<table border=\"0\" width=\"100%\" cellspacing=\"0\"";
    $S .= " cellpadding=\"0\"><tbody>";
    $S .= StrRowBegin();
    $S .= StrCell(Strong("Tag name: "));
    $S .= "<td align=\"right\">".MenuTag();
    $S .= "</td>\n";
    $S .= StrRowEnd();

    $S .= StrRowBegin();
    $C = StrTableBegin("0", "", "100%");
    $C .= StrRowBegin();
    $C .= StrCell("New tag name: ".HtmlTextInput("NewTag", "25"));
    $C .= "<td align=\"right\">";
    $C .= HtmlSubmitButton("ApplyTag", "Apply");
    $C .= "</td>\n";
    $C .= StrRowEnd();
    $C .= StrTableEnd();
    $S .= StrCell($C, "2");
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    return $S;
}

//********************************************************************

?>
