<?php
/*********************************************************************
lib_tag.php

Definitions related to tagged-photo management
______________________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
function FinalTagSn($TagSn, $NewTag)
{
    if ($TagSn == "newtag" or $TagSn == "none")
    {
        $NewTag = trim($NewTag);
        if ($NewTag != "")
        {
            $TagSn = TagSn($NewTag);
            UpdateTagTimeStamp($TagSn);
        } else
            $TagSn = False;
    } else UpdateTagTimeStamp($TagSn);
    return $TagSn;
}

function IsTagArchived($TagSn)
{
    $Q = "SELECT * FROM archivedtag WHERE tagsn = $1";
    return (is_numeric($TagSn)
	    ? pg_num_rows(Query($Q, $TagSn)) > 0
	    : False);
}

function MenuTag($Selection="", $VarName="TagSn", $AutoSubmit=False,
                 $OfferCreate=True, $ShowRecent=True,
                 $ShowArchived=False,
                 $Prompt="&mdash; Select tag &mdash;")
{
    // Drop-down menu of available tags
    $ShowArchived = ($ShowArchived or IsTagArchived($TagSn));
    $TagsByName = ResTagsByName($ShowArchived);
    $Attribs = ($AutoSubmit) ? "onchange=\"this.form.submit()\"" : "";
    $S = HtmlSelectBegin($VarName, $Attribs);
    if ($Selection == "" or $Selection == "none")
        $S .= HtmlSelectOption("none", $Prompt, True);
    // for each most-recently-used tag:
    if ($ShowRecent)
    {
        $TagsByTime = TagsByTime();
        foreach ($TagsByTime as $Tag)
            $S .= HtmlSelectOption(
                $Tag["tagsn"], $Tag["name"],
                False, $Tag["notes"], "background: ".BgColor);
    }
    if ($OfferCreate)
        $S .= HtmlSelectOption(
            "newtag", "&mdash; Create new tag &mdash;", False,
            "Be sure to enter a name for the new tag");
    for ($Row=0 ; $Row < pg_num_rows($TagsByName) ; $Row++)
    {
        list($TagSn, $Name, $Notes, $IsArchived)
            = pg_fetch_array($TagsByName, $Row);
        $Style = ($IsArchived == "t")
               ? "background: ".ArchivedTagColor : "";
        $S .= HtmlSelectOption(
            $TagSn, $Name, ($TagSn == $Selection), $Notes, $Style);
    }
    $S .= HtmlSelectEnd();
    if ($AutoSubmit) $S = StrWithGoButton($S, "Go");
    return $S;
}

function ResTagsByName($ShowArchived=False)
{
    /* Return a query result (tagsn, name, notes, isarchived) */
    $Q = "SELECT t.tagsn, name, notes, ";
    $Q .= $ShowArchived ? "at.tagsn IS NOT NULL" : "False";
    $Q .= " AS isarchived FROM tag t";
    $Q .= " LEFT OUTER JOIN archivedtag at USING (tagsn)";
    if (! $ShowArchived) $Q .= " WHERE at.tagsn IS NULL";
    $Q .= " ORDER BY lower(name)";
    return Query($Q);
}
function TagPhoto($TagSn, $PhotoSn)
{
    $Q = "INSERT INTO tag_photo (tagsn, photosn)";
    $Q .= " SELECT $1, $2 WHERE NOT EXISTS";
    $Q .= " (SELECT 1 FROM tag_photo WHERE tagsn = $1";
    $Q .= "  AND photosn = $2)";
    Query($Q, $TagSn, $PhotoSn);
    return;
}

function TagsByTime()
{
    // Return array of most-recently-selected tags
    // Each row is an array(tagsn, name, notes)
    $Tags = array();
    $Q = "SELECT tagsn, name, notes FROM tag";
    $Q .= " ORDER BY last_used DESC, name";
    $Res = Query($Q);
    // for each most-recently-used tag:
    $Rows = pg_num_rows($Res);
    if ($Rows > RecentTags) $Rows = RecentTags;
    for ($Row=0 ; $Row < $Rows ; $Row++ )
        $Tags[$Row] = pg_fetch_array ($Res, $Row);
    return $Tags;
}

function TagSn($TagName)
{
    // Returns tagsn of $TagName in tag table
    // Inserts tag if it doesn't already exist
    $TagName = trim($TagName);
    $Sn = SerialNumber("tag", "name", $TagName);
    if ($Sn === False)
    {
        $Q = "INSERT INTO tag (name, last_used)";
        $Q .= " VALUES ($1, CURRENT_TIMESTAMP)";
        Query($Q, $TagName);
        $Sn = TagSn($TagName);
    }
    return $Sn;
}

function UnTagPhoto($TagSn, $PhotoSn)
{
    $Q = "DELETE FROM tag_photo";
    $Q .= " WHERE tagsn = $1 AND photosn = $2";
    Query($Q, $TagSn, $PhotoSn);
    return;
}

function UpdateTagTimeStamp($TagSn)
{
    $Q = "UPDATE tag SET last_used = CURRENT_TIMESTAMP";
    $Q .= " WHERE tagsn = $1";
    Query($Q, $TagSn);
    return;
}

//********************************************************************

?>
