<?php
/*********************************************************************
loc_env.php
Local Environment Definitions for PHP Scripts

______________________________________________________________________
Copyright 2023 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________
*********************************************************************/
// DEBUG mode: set True to control diagnostics
//define ("DEBUG", False);
define ("DEBUG", True);

// Use current Web standards? Better layout, but breaks old browsers:
define ("WebStandards", False);

define ("ServiceTitle", "PhotoTrove");
define ("ServiceName", "trove");
//define ("ServiceDescription", "PhotoTrove");
define ("ServiceDescription", "");
$DbName = ServiceName;

define ("LoginScript", "index.php");

// Service Root is relative to document root:
//$ServiceRoot = "/";
$ServiceRoot = "/".ServiceName."/";
// Don't use a user's document root if there is a non-ssl webserver:
//$ServiceRoot = "/~someuser/".ServiceName."/";

// Note: Though it may coincidentally have the same value,
// DefaultDbUser has no relation to the webserver's GNU/Unix user or
// group accounts:
define ("DefaultDbUser", "www-data");	// the webserver account
define ("LoginPrompt",
	"To access your personal configuration, please log in:");
//define ("LoginPrompt", "To use ".ServiceTitle.", please log in.");
$T = "Strangely, you don't seem to have privileges for any available";
$T .= " tools. Contact the system administrator.";
define ("NoPrivsMessage", $T);
define ("UsersMayCreateOwnAccounts", False);
if (UsersMayCreateOwnAccounts)
{
    $T = "If you don't have a login account, you may create one,";
    $T .= " specifying a user ID of your own choosing.";
} else
{
    $T = "If you don't have a login account, contact the service";
    $T .= " administrator.";
}
define ("AccountInvitation", $T);

define ("AlternateAuthOK", True);
if (AlternateAuthOK)
{
    $T = "You may enter two challenge-response pairs for use in case";
    $T .= " you forget your password. You may change this";
    $T .= " information any time you are logged in. The responses";
    $T .= " are stored encrypted, so they cannot be displayed.";
    define ("AltAuthSetupInvitation", $T);
    $T = "Be extremely careful to select information that no one but";
    $T .= " you can discover.";
    define ("AltAuthWarning", $T);
    $T = "If you have forgotten your password, you may reset it";
    $T .= " yourself, provided you have set up alternate";
    $T .= " authentication.";
    define ("AltAuthPrompt", $T);
}

$T = "Please contact the service administrator to reset your";
$T .= " password.";
define ("PasswordResetInstructions", $T);

//define ("WallPaper", "images/wallpaper.jpg");	// Default wallpaper

/*********************************************************************
Special Patch so development page has different color
*********************************************************************/
$RelPath = "/".substr($_SERVER["SCRIPT_NAME"], strlen($ServiceRoot));
$Path = substr($RelPath, 0, strrpos($RelPath,"/") + 1);
// Default background color
if ($Path == "/dev/") define ("BgColor", "#f0e0e0"); // beige-pink
else define ("BgColor", "#d0e0ff");	// Default periwinkle
define ("BgColor2", "#b0f0a0");	// Alternate background palegreen
define ("BgColor3", "#f0b080");	// Alternate background color caramel
define ("BgColor4", "#d0ffa0");	// Alternate background  pale green
define ("TextColor", "");  // Text color (browser default)
define ("ArchivedAlbumColor", "#e8e96b"); // album menu highlighting
define ("ArchivedLocationColor", "#e8e96b"); // location menu hilite
define ("ArchivedTagColor", "#e8e96b"); // tag menu highlighting

// Default style for all webpages:
$T = "font-family: helvetica, sans-serif;";
$T .= ((TextColor > "") ? (" color: ".TextColor.";") : "");
$T .= " background-color: ".BgColor;
define("DefaultBodyStyle", $T);

$T = "font-family: helvetica, sans-serif;";
$T .= ((TextColor > "") ? (" color: ".TextColor.";") : "");
$T .= " background-color: ".BgColor2;
define("DefaultBodyStyle2", $T);

define ("TableColor", "#fffff0"); // Table background color beige
define ("TableColor2", "#ffffd0"); // Alt table background cheese
define ("ButtonColor", "lightblue");	// Button color (lightblue)
define ("ButtonColor2", "lightgreen");	// Button color (lightgreen)
define ("ButtonOffColor", "#e2e0e2");	// Button color (pale grey)
define ("LoginBoxColor", "lightblue");	// Login box color
define ("CloseButtonColor", "#ffb6c5"); // Close button color (pink)
define ("ProgressBarImage", "images/blue75x100.jpg");
define ("MotdColor", "#ffe900");      // Message of the day
// Warning Colors:
//define ("WarningColor", "red");     // Error/urgent (typical)
define ("WarningColor", "#e06000");   // Error/urgent (508 compliant)
define ("WarningBg", "#ffcece");      // Error/urgent background
//define ("NoticeColor", "green");    // Informational (typical)
define ("NoticeColor", "#000080");    // Informational (508 compliant)
define ("NoticeBg", "#fffff0");       // Informational bg beige
define ("HighlightBg", "#ffff90");    // Highlighted bg pale yellow
define ("OkBg", "#d0ffa0");	      // OK background pale green

$T = "display: inline; background: ".CloseButtonColor;
$T .= "; border: medium outset ".WarningColor;
define("StyleCloseButton", $T);

// max number of photos on overview pages:
define ("HitsPerPage", 20);
// max number of photos to find in searches:
define ("MaxSearchHits", 400);

define ("LocalTimeZone", trim(shell_exec("date +%Z")));
define ("LocalTimeZoneOffset", trim(shell_exec("date +%z")));

define ("DefaultCameraTimeZone", "UTC");
//define ("DefaultCameraTimeZone", LocalTimeZone);

// date/time representations for PHP functions:
// date representation PostgreSQL default, e.g., 2011-02-27:
define ("DateFormatPgDefault", "Y-m-d");
// date representation, e.g., Feb 4, 2004:
define ("DateFormat_MdY", "M d, Y");
// date representation, e.g., 1 Aug 2010:
define ("DateFormat_dMY", "j M Y");
// date representation for pop-up calendar, e.g., 02/04/2004:
define ("CalFormat", "m/d/Y");
// date representation for album publication date, e.g., 2011-Feb-27:
define ("CalPubDateFormat", "%Y-%b-%d"); // for jscalendar
define ("PubDateFormat", "Y-M-d"); // for PHP date()
// date-time representation, e.g., 01 August 2010 20:09:13 EDT:
define ("DateTimeFormatFull", "d M Y H:i:s T");
// day-date representation, e.g., Wed, Feb 4, 2004:
define ("DayDateFormat_DMdY", "D, M d, Y");
// time representation, e.g., 14:36:32:
define ("TimeFormat_HMS", "H:i:s");

// date/time representations in SQL-ese for function to_char:
// date representation, e.g., Feb 4, 2004:
define ("DateFormatSQL_MdY", "FMMonth FMDD, YYYY");
// date representation, e.g., 1 Aug 2010:
define ("DateFormatSQL_dMY", "FMDD FMMonth YYYY");
// date-time representation, e.g., 01 August 2010 20:09:13 EDT:
define ("DateTimeFormatSQLFull", "DD FMMonth YYYY HH24:MI:SS TZ");
// day-date representation, e.g., Wed, Feb 4, 2004:
define ("DayDateFormatSQL_DMdY", "Dy, FMMonth FMDD, YYYY");
// time representation, e.g., 14:36:32:
define ("TimeFormatSQL_HMS", "HH24:MI:SS");
// date representation for pop-up calendar, e.g., 02/04/2004:
define ("CalFormatSQL", "MM/DD/YYYY");

function SetDefaultFormMethodAndTimeout($Path)
{
    // Ensures that production always uses post and has minimum
    // timeout
    // Note that PHP by default (session.gc_maxlifetime in php.ini)
    // considers session data "expired" after 24 minutes of inactivity
    // and destroys it in the next garbage-collection pass, which
    // could happen at any time (the frequency of garbage collection
    // is load-dependent, by default once every 100 hits per
    // session.gc_probability). This means we can't guarantee that a
    // session will stay active longer than 24 minutes. However, all
    // this means is that, if the session times out, the user will
    // have to reenter the login credentials, with no loss of form
    // data.
    if ($Path == "/dev/" and DEBUG)
    {
	define ("FormMethod", "get");
	define ("SessionTimeout", 20); // minutes
    } else
    {
	// POST hides values, handles big data:
	define ("FormMethod", "post");
	define ("SessionTimeout", 20);	// minutes
    }
    return;
}

function StrWarnNoLogin()
{
    // Called when a page is accessed but no login is active. Handy
    // for displaying custom messages for pages likely to be reached
    // from a URL without a login.
    if (ThisScript() == "example.php")
    {
	$S = "<p>To reach the Example screen from a URL,<br>\n";
	$S .= "you must be logged into ".$SystemTitle." when you";
	$S .= " click on it.</p>\n";
    } else $S = "";
    return $S;	
}

// Paths for which we do NOT display a "not production" warning and a
// link to the production service on main menu page (index.php):
$NoProductionLink = array
(
    "/",
    "/proto/",
    "/tng/",
    "/tvl/"
);

$SysEnv = array
(
  "/"	=> array
    (			// Production
    "SysTitle"	=> ServiceTitle,
    "Db"	=> $DbName,
    "User"	=> DefaultDbUser
    ),
  "/tst/" => array
    (			// Test
    "SysTitle"	=> ServiceTitle." [Note: Test System]",
    "Db"	=> "${DbName}_tst",
    "User"	=> DefaultDbUser
    ),
  "/dev/" => array
    (
    "SysTitle"	=> ServiceTitle." [Note: Development System]",
    "Db"	=> "${DbName}_dev",
    "User"	=> DefaultDbUser
    ),
  "/tvl/" => array
    (
    "SysTitle"	=> ServiceTitle." [Note: Portable System for Travel]",
    "Db"	=> "${DbName}_tvl",
    "User"	=> DefaultDbUser
    )
);

//***************************************************************
// originalphotos is a symlink to the actual photo upload directory
define ("OriginalPhotos", "originalphotos");
//define ("OriginalPhotos", "/home/public/photos");
define ("MaxSetSize", 50);     // default max number of photos in a set
// Number of most-recently-used locations to display in location menu:
define ("RecentLocs", 8);
// Number of most-recently-used tags to display in tag menu:
define ("RecentTags", 3);
define ("ThumbnailLength", 192);// pixels
define ("WebPicLength", 768);	// pixels
// Width of clickable region on large image, pixels:
define ("PageEdge", 100);
// Largest dimension of image on album title page, pixels:
define ("TitlePicSize", 512);
// Processing speed per MB for video (convert to Ogg) vs for image
// (make webpic & thumbnail); Video conversion approx ten times slower:
define ("VidToPicTimeRatio", 10);

?>
