<?php
/*********************************************************************
search.php
Search Module for PhotoTrove
______________________________________________________________________
Copyright 2024 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
session_cache_limiter("");
require ("lib_album.php");
require_once ("lib_common.php");
require ("lib_filesys.php");
require_once ("lib_layout.php");
require ("lib_photoset.php");
require ("lib_tag.php");
require ("local/loc_camera.php");

$T = " FROM photo p";
$T .= " LEFT OUTER JOIN albumphoto USING (photosn)";
$T .= " LEFT OUTER JOIN location l USING (locationsn)";
$T .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn)";
$T .= ", photoset s";
$T .= " WHERE s.photosetsn = p.photosetsn";
define("ClauseFrom", $T);
$T = " EXISTS (SELECT TRUE FROM tag_photo WHERE photosn =";
$T .= " p.photosn AND tagsn";
define("ClauseTagsApply", $T);
$T = "Click selects one, Shift-click extends selection,";
$T .= " Control-click (de)selects individual tags";
define("HintTagSelect", $T);
define("PromptMultiTag", "&mdash; Select one or more tags &mdash;");

function ActionsForAll($State, $FormVars, $NewLocationSn, $OtherLoc,
                       $AlbumSn, $NewAlbum, $TagSn, $NewTag, $UnTagSn)
{
    /* Since we're displaying only the first and last photos in the
     * series, $FormVars has no "Sel$Row" or "PS$Row" elements to
     * which to apply the actions individually; instead, we're
     * treating the entire series as selected. Reapply the query
     * criteria to select the photos and execute a single UPDATE on
     * all tuples.  */
    if (isset($FormVars["ApplyLoc"]))
        $S .= AllAppliedLocation($State, $NewLocationSn, $OtherLoc);
    elseif (isset($FormVars["AddToAlbum"]))
        $S .= AllAddedToAlbum($State, $AlbumSn, $NewAlbum);
    elseif (isset($FormVars["ApplyTag"]))
        $S .= AllTagged($State, $TagSn, $NewTag);
    elseif (isset($FormVars["Untag"]))
        $S .= AllUntagged($State, $UnTagSn);
    $S .= SelectAll($State, False, True);
    return $S;
}

function AllAddedToAlbum($State, $AlbumSn, $NewAlbum)
{
    // Adds photos to last position of page zero, creating album and
    // page zero if it doesn't exist
    $AlbumSn = FinalAlbumSn($AlbumSn, $NewAlbum);
    if ($AlbumSn === False)
        $S = StrWarningBox(WarnBadAlbumName($NewAlbum), "50%");
    else
    {
        Query("BEGIN");
        $AlbumPageSn = CreateAlbumPage($AlbumSn, 0);
        $Q = "CREATE TEMPORARY SEQUENCE ordinal_seq";
        Query($Q);
        $Q = "SELECT SETVAL('ordinal_seq',";
        $Q .= " (SELECT COALESCE(max(ordinal), 0) + 1";
        $Q .= " FROM albumphoto WHERE albumpagesn = $1))";
        Query($Q, $AlbumPageSn);
        $KeyDescription = "";
        $Count = 1;
        $Params = array($AlbumPageSn);
        $Clauses
            = ClausesActionsForAll($State, $KeyDescription,
                                   $ClauseOrder, $Count, $Params);
        $Q = "INSERT INTO albumphoto (albumpagesn, photosn, ordinal,";
        $Q .= " caption)";
        $Q .= " SELECT $1, photosn, NEXTVAL('ordinal_seq'),";
        $Q .= " description";
        $Q .= " FROM";
        $Q .= " (SELECT p.photosn, description,";
        $Q .= " COALESCE(l.name, '".NullLocationFlag."') AS locname";
        $Q .= " $Clauses $ClauseOrder) AS matches";
        QueryArray($Q, $Params);
        $Q = "DROP SEQUENCE ordinal_seq";
        Query($Q);
        Query("COMMIT");
        $AlbumName = pg_fetch_result(ResAlbum($AlbumSn), 0, "name");
        $S = "Appended to page 0 of album &quot;$AlbumName&quot;";
        $S .= " all photos $KeyDescription.";
        $S = StrNoticeBox($S);
    }
    return $S;
}

function AllAppliedLocation($State, $LocationSn, $OtherLoc)
{
    $LocationSn = FinalLocationSn($LocationSn, $OtherLoc);
    $KeyDescription = "";
    $Count = 1;
    $Params = array($LocationSn);
    $Clauses = ClausesActionsForAll($State, $KeyDescription, $Z,
                                    $Count, $Params);
    if ($LocationSn)
    {
        $Q = "UPDATE photo SET locationsn = $1";
        $Q .= " WHERE photosn = ANY (SELECT p.photosn $Clauses)";
        QueryArray($Q, $Params);
        $LocName = Location($LocationSn);
        $S = "Applied location &quot;$LocName&quot; to all";
        $S .= " photos $KeyDescription.";
        $S = StrNoticeBox($S);
    } else
    {
        $S = "No location specified; couldn't update location for";
        $S .= " photos $KeyDescription.";
        $S = StrWarningBox($S, "50%");
    }
    return $S;
}

function AllTagged($State, $TagSn, $NewTag)
{
    $TagSn = FinalTagSn($TagSn, $NewTag);
    $KeyDescription = "";
    $Count = 1;
    $Params = array($TagSn);
    $Clauses = ClausesActionsForAll($State, $KeyDescription, $Z,
                                    $Count, $Params);
    if ($TagSn)
    {
        $TagName = ColFromSerialNumber("tag", "name", $TagSn);
        $Q = "INSERT INTO tag_photo (tagsn, photosn)";
        $Q .= " SELECT DISTINCT ON (photosn) $TagSn, p.photosn";
        $Q .= " $Clauses AND NOT EXISTS";
        $Q .= " (SELECT 1 FROM tag_photo";
        $Q .= " WHERE tagsn = $1 AND photosn = p.photosn)";
        QueryArray($Q, $Params);
        $S = "Applied tag &quot;$TagName&quot; to all";
        $S .= " photos $KeyDescription.";
        $S = StrNoticeBox($S);
    } else
    {
        $S = "No tag specified; nothing to apply to all photos";
        $S .= " $KeyDescription.";
        $S = StrWarningBox($S, "50%");
    }
    return $S;
}

function AllUntagged($State, $UnTagSn)
{
    if (is_numeric($UnTagSn))
    {
        $KeyDescription = "";
        $Count = 1;
        $Params = array($UnTagSn);
        $Clauses = ClausesActionsForAll($State, $KeyDescription, $Z,
                                        $Count, $Params);
        $TagName = ColFromSerialNumber("tag", "name", $UnTagSn);
        $Q = "DELETE FROM tag_photo";
        $Q .= " WHERE tagsn = $1";
        $Q .= " AND photosn = ANY (SELECT p.photosn $Clauses)";
        QueryArray($Q, $Params);
        $S = "Untagged &quot;$TagName&quot; from all";
        $S .= " photos $KeyDescription.";
        $S = StrNoticeBox($S);
    } else
    {	$S = "No tag specified, so no photos untagged.";
        $S = StrWarningBox($S, "50%");
    }
	return $S;
}

function CellSearchTags($SelectName, $SelectVal, $TagsName, $TagsVal,
                        $Heading=False)
{
    $C = $Heading ? Strong($Heading)."<br />\n" : "";
    $C .= "<label>";
    $C .= HtmlRadioButton($SelectName, "any", ($SelectVal == "any"));
    $C .= " any of:</label><label>";
    $C .= HtmlRadioButton($SelectName, "all", ($SelectVal == "all"));
    $C .= " all of:</label><label>";
    $C .= HtmlRadioButton($SelectName, "none",
                          ($SelectVal == "none"));
    $C .= " none of:</label>";
    $C .= "<br />\n".MenuTagMulti($TagsName, $TagsVal);
    $S = "<td style=\"text-align: center\" title=\"";
    $S .= HintTagSelect."\">$C</td>\n";
    return $S;
}

function ClauseAlbumPhoto($InAlbum, &$KeyDescription)
{
    if ($InAlbum == "either") return "";
    list($Not, $No) = ($InAlbum == "no")
                    ? array("", "no") : array("NOT ", "any");
    $Q = " AND albumphotosn IS {$Not}NULL";
    $KeyDescription .= ",<br /> occurring in {$No} album";
    return $Q;
}

function ClauseBlankDescription($DescIs, &$KeyDescription)
{
    if ($DescIs == "either") return "";
    list($Not, $Non) = ($DescIs == "nonblank")
                     ? array("NOT ", "non") : array("", "");
    $Q = " AND $Not(description IS NULL OR description = '')";
    $KeyDescription .= ",<br /> with {$Non}blank description";
    return $Q;
}

function ClauseHasVideo($HasVideo, &$KeyDescription)
{
    if ($HasVideo == "either") return "";
    list($Not, $No) = ($HasVideo == "no")
                    ? array("", "no ") : array("NOT ", "");
    $Q = " AND vidfilename IS {$Not}NULL";
    $KeyDescription .= ",<br /> with {$No}associated video";
    return $Q;
}

function ClauseFileName($FileNameStr, $MatchNameCase,
                        &$KeyDescription, &$Count, &$Params)
{
    if ($FileNameStr == "") return "";
    $Count++;
    $Compare = $MatchNameCase ? "LIKE" : "ILIKE";
    $Params[] = "%$FileNameStr%";
    $Q = " AND filename $Compare $$Count";
    $K = ",<br /> with file name containing &quot;$FileNameStr&quot;";
    $K .= ($MatchNameCase ? ", match" : ", ignor")."ing case";
    $KeyDescription .= $K;
    return $Q;


    
    /* Parse input string as tokens to match, in any order. Then
       construct query to match tokens in description or albumphoto
       caption, as specified. */
    $Matches = ParsedWords($Words);
    $Compare = $MatchCase ? "LIKE" : "ILIKE";
    $Q = "";
    $K = "";
    $Col = "";
    if ($InDesc) $Col = "COALESCE(p.description, '')";
    if ($InDesc and $InCaption) $Col .= "||";
    if ($InCaption) $Col .= "COALESCE(caption, '')";
    foreach ($Matches as $Match)
    {
        $Count++;
        $Params[] = "%$Match%";
        /* $Q .= " AND $Col $Compare '%$Match%'"; */
        $Q .= " AND $Col $Compare $$Count";
        $K .= " and &quot;$Match&quot;";
    }
    if ($InDesc) $T = "description";
    if ($InDesc and $InCaption) $T .= " or ";
    if ($InCaption) $T .= "caption";
    $K .= ($MatchCase ? ", match" : ", ignor")."ing case";
    $KeyDescription .= ",<br />with $T containing".substr($K, 4);
}

function ClauseLocation($LocationSn, &$KeyDescription, &$Count,
                        &$Params)
{
    if ($LocationSn == "any") return "";
    $Q = " AND p.locationsn ";
    if ($LocationSn == "unknown")
    {
        $Q .= "IS NULL";
        $KeyDescription .= " with no location specified";
    } else
    {
        $Count++;
        $Params[] = $LocationSn;
        $Q .= "= $$Count";
        $KeyDescription .= " at ".Location($LocationSn);
    }
    return $Q;
}

function ClauseNullTime($TimeIs, &$KeyDescription)
{
    if ($TimeIs == "either") return "";
    list($Not, $Un) = ($TimeIs == "set")
                    ? array(" NOT", "") : array("", "un");
    $Q = " AND time IS$Not NULL";
    $KeyDescription .= ",<br /> with date/time {$Un}set";
    return $Q;
}

function ClauseNullTimeAndPlace($TimeIs, $LocationSn,
                                &$KeyDescription, &$Count, &$Params)
{
    $K = "";
    $Q = ClauseLocation($LocationSn, $K, $Count, $Params);
    if ($K > "") $K =  ",<br />taken".$K;
    $Q .= ClauseNullTime($TimeIs, $K);
    $KeyDescription .= $K;
    return $Q;
}

function ClausesActionsForAll($State, &$KeyDescription, &$ClauseOrder,
                              &$Count, &$Params)
{
    // Begins at " FROM photo p, ..."
    list ($WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
          $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
          $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
          $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit, $EndsOnly)
        = UnPackedState($State);
    $Q = ClauseFrom;
    $Q .= ClausesPhotoSearch(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
        $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs,
        $KeyDescription, $Count, $Params);
    $T = ($Sort == "Time") ? "time" : "locname, time";
    $ClauseOrder = " ORDER BY $T, lower(filename)";
    return $Q;
}

function ClausesPhotoSearch(
    $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
    $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
    $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs,
    &$KeyDescription, &$Count, &$Params)
{
    // Return AND clauses for search, and place description of search
    // criteria in $KeyDescription
    $Words = trim($Words);
    $Q = ($Words != "" and ($InDesc or $InCaption))
       ? ClauseText($Words, $InDesc, $InCaption, $MatchCase,
                    $KeyDescription, $Count, $Params)
       : ClauseBlankDescription($DescIs, $KeyDescription);
    $Q .= ClausesTags($Tags, $TagSelect, $Tags2, $TagSelect2,
                      $KeyDescription);
    $Q .= ClauseHasVideo($HasVideo, $KeyDescription);
    $Q .= ClauseAlbumPhoto($InAlbum, $KeyDescription);
    $Q .= ClauseFileName($FileNameStr, $MatchNameCase,
                         $KeyDescription, $Count, $Params);
    $Q .= ($WhenFrom.$WhenTo != "")
       ? ClauseTimeAndPlace($WhenFrom, $WhenTo, $LocationSn,
                            $KeyDescription, $Count, $Params)
       : ClauseNullTimeAndPlace($TimeIs, $LocationSn,
                                $KeyDescription, $Count, $Params);
    $KeyDescription = substr($KeyDescription, 7);
    return $Q;
}

function ClausesTags($Tags, $TagSelect, $Tags2, $TagSelect2,
                     &$KeyDescription)
{
    $K1 = "";
    $K2 = "";
    $Q = ClauseTags($Tags, $TagSelect, $K1);
    $Q .= ClauseTags($Tags2, $TagSelect2, $K2);
    if ($K1 > "" and $K2 > "")
        $K2 = "and ".substr($K2, 8);
    $KeyDescription .= $K1.$K2;
    return $Q;
}

function ClauseTags($Tags, $TagSelect, &$KeyDescription)
{
    // OK for TagSn values to be literals since not user-entered
    if (count($Tags) == 0) return "";
    $Q = "";
    $K = "";
    switch ($TagSelect)
    {
    case "none":
        foreach ($Tags as $TagSn)
        {
            $Q .= ", $TagSn";
            $K .= ", nor &quot;";
            $K .= ColFromSerialNumber("tag", "name", $TagSn)."&quot;";
        }
        $Q = " AND NOT".ClauseTagsApply." IN (".substr($Q, 2)."))";
        $K = substr($K, 5);
        $KeyDescription .= ",<br /> <em>not</em> tagged with$K";
        break;
    case "all":
        foreach ($Tags as $TagSn)
        {
            $Q .= " AND".ClauseTagsApply." = $TagSn)";
            $K .= " and &quot;";
            $K .= ColFromSerialNumber("tag", "name", $TagSn)."&quot;";
        }
        $KeyDescription .= ",<br /> tagged with".substr($K, 4);
        break;
    default:
        foreach ($Tags as $TagSn)
        {
            $Q .= ", $TagSn";
            $K .= " or &quot;";
            $K .= ColFromSerialNumber("tag", "name", $TagSn)."&quot;";
        }
        $Q = " AND".ClauseTagsApply." IN (".substr($Q, 2)."))";
        $KeyDescription .= ",<br /> tagged with".substr($K, 3);
    }
    return $Q;
}

function ClauseText($Words, $InDesc, $InCaption, $MatchCase,
                    &$KeyDescription, &$Count, &$Params)
{
    /* Parse input string as tokens to match, in any order. Then
       construct query to match tokens in description or albumphoto
       caption, as specified. */
    $Matches = ParsedWords($Words);
    $Compare = $MatchCase ? "LIKE" : "ILIKE";
    $Q = "";
    $K = "";
    $Col = "";
    if ($InDesc) $Col = "COALESCE(p.description, '')";
    if ($InDesc and $InCaption) $Col .= "||";
    if ($InCaption) $Col .= "COALESCE(caption, '')";
    foreach ($Matches as $Match)
    {
        $Count++;
        $Params[] = "%$Match%";
        /* $Q .= " AND $Col $Compare '%$Match%'"; */
        $Q .= " AND $Col $Compare $$Count";
        $K .= " and &quot;$Match&quot;";
    }
    if ($InDesc) $T = "description";
    if ($InDesc and $InCaption) $T .= " or ";
    if ($InCaption) $T .= "caption";
    $K .= ($MatchCase ? ", match" : ", ignor")."ing case";
    $KeyDescription .= ",<br />with $T containing".substr($K, 4);
    return $Q;
}

function ClauseTimeAndPlace($WhenFrom, $WhenTo, $LocationSn,
                            &$KeyDescription, &$Count, &$Params)
{
    $K = "";
    $Q = ClauseTime($WhenFrom, $WhenTo, $K, $Count, $Params);
    $Q .= ClauseLocation($LocationSn, $K, $Count, $Params);
    if ($K > "") $K =  ",<br />taken".$K;
    $KeyDescription .= $K;
    return $Q;
}

function ClauseTime($WhenFrom, $WhenTo, &$KeyDescription, &$Count,
                    &$Params)
{
    $Q = "";
    $K = "";
    $Lo = ($WhenFrom > "") ? TimeString($WhenFrom) : NullTimeFlag;
    $Hi = ($WhenTo > "") ? TimeString($WhenTo) : NullTimeFlag ;
    if ($Lo !== NullTimeFlag)
    {
        if ($Hi !== NullTimeFlag)
        {
            // Range
            $Q = ClauseTimeRange($Lo, $Hi, $K, $Count, $Params);
        } else
        {
            // Lower bound
            $Q = ClauseTimeBoundLo($Lo, $Count, $Params);
            $K = " on or after $Lo";
        }
    } elseif ($Hi !== NullTimeFlag)
      {
          // Upper bound
          $Q = ClauseTimeBoundHi($Hi, $Count, $Params);
          $K = " on or before $Hi";
      }
    $KeyDescription .= $K;
    return $Q;
}

function ClauseTimeBoundHi($TimeStamp, &$Count, &$Params)
{
    $Count++;
    $Params[] = $TimeStamp;
    return " AND time <= $$Count";
}

function ClauseTimeBoundLo($TimeStamp, &$Count, &$Params)
{
    $Count++;
    $Params[] = $TimeStamp;
    return " AND time >= $$Count";
}

function ClauseTimeRange($Lo, $Hi, &$K, &$Count, &$Params)
{
    if (strtotime($Lo) < strtotime($Hi))
    {
        $TimeLo = $Lo;
        $TimeHi = $Hi;
    } else
    {
        $TimeLo = $Hi;
        $TimeHi = $Lo;
    }
    $Count++;
    $Params[] = $TimeLo;
    $Q = " AND time BETWEEN $$Count";
    $Count++;
    $Params[] = $TimeHi;
    $Q .= " AND $$Count";
    $K = " between $TimeLo and $TimeHi";
    return $Q;
}

function EndsOnly($Res, $NumRows)
{
    /* Displays first and last photos in series $Res */
    $Plural = ($NumRows > 1) ? "s" : "";
    $S = "Found $NumRows matching photo$Plural:";
    $S = StrWarning($S, NoticeColor);
    $S .= PhotoStats($Res, 0, True);
    if ($NumRows > 1)
        $S .= PhotoStats($Res, $NumRows - 1, False);
    return $S;
}

function IsEndsOnly($State)
{
    /* Given a serialized state array $State whose very last element
     * is $EndsOnly (Boolean), extracts and returns $EndsOnly; */
    if (isset($State))
        list($EndsOnly) = array_slice(UnPackedState($State), -1);
    else
        $EndsOnly = False;
    return $EndsOnly;
}

function MenuTagMulti($VarName, $Selections=array())
{
    // Drop-down menu of available tags, allowing multiple selections
    $Count = count($Selections);
    $Q = "SELECT tagsn, name, notes FROM tag t WHERE NOT EXISTS";
    $Q .= " (SELECT * FROM archivedtag at WHERE at.tagsn = t.tagsn)";
    $Q .= " ORDER BY lower(name)";
    $Res = Query($Q);
    $NumTags = pg_num_rows($Res);
    $ShowPrompt = ($Count == 0
                   or ($Count == 1 and $Selections[0] == "none"));
    $Size = $ShowPrompt ? $NumTags + 1 : $NumTags;
    $Size = min($Size, 10);
    $S = HtmlSelectBegin("$VarName"."[]", "multiple size=\"$Size\"");
    if ($ShowPrompt)
        $S .= HtmlSelectOption("none", PromptMultiTag, True);
    for ($Row=0 ; $Row < $NumTags ; $Row++)
    {
        list($TagSn, $Name, $Notes) = pg_fetch_array($Res, $Row);
        $S .= HtmlSelectOption(
            $TagSn, $Name, in_array($TagSn, $Selections), $Notes);
    }
    $S .= HtmlSelectEnd();
    return $S;
}

function MenuTagsInUse($ResTagsInUse, $NumRows)
{
    // Drop-down menu of tags applied to any photo in gallery
    $Prompt = "&mdash; Select tag to remove &mdash;";
    $S = HtmlSelectBegin("UnTagSn", $Attribs);
    $S .= HtmlSelectOption("none", $Prompt, True);
    for ($Row=0 ; $Row < $NumRows ; $Row++)
    {
	list($TagSn, $Name, $Notes)
	  = pg_fetch_array($ResTagsInUse, $Row);
	$S .= HtmlSelectOption($TagSn, $Name, False, $Notes,
			       $Style);
    }
    $S .= HtmlSelectEnd();
    return $S;
}

function OptionUnTag($ResTagsInUse)
{
    $NumRows = pg_num_rows($ResTagsInUse);
    if ($NumRows > 0)
    {
        $Menu = MenuTagsInUse($ResTagsInUse, $NumRows);
	$S = StrRowBegin();
	$S .= StrCell("Untag selected photos:");
	$C = "<table border=\"0\" width=\"100%\" cellspacing=\"0\"";
	$C .= " cellpadding=\"0\"><tbody>";
	$C .= StrRowBegin();
	$C .= StrCell($Menu);
	$C .= StrCell(HtmlSubmitButton("Untag"), "", "",
		      "text-align: right");
	$C .= StrRowEnd();
	$C .= StrTableEnd();
	$S .= StrCell($C, "", "", BoxStyle);
	$S .= StrRowEnd();
	return $S;
    } else return "";
}

function PageSearchResults(
    $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
    $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
    $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs, $Sort,
    $Limit, $EndsOnly, $SelectAll=False, $EditOk=False,
    &$RangeMarks=False)
{
    /*****************************************************************
      Displays search results page with all widgets and photo
      thumbnails. Calls ImageCells to display thumbnails with
      checkboxes set according to $SelectAll and $RangeMarks.
    *****************************************************************/
    $Res = ResPhotoSearch(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
        $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs,
        $Sort, $KeyDescription);
    $NumRows = pg_num_rows($Res);
    $Plural = ($NumRows > 1) ? "s" : "";
    if ($KeyDescription > "") $KeyDescription = " $KeyDescription";
    $N = "Found $NumRows photo$Plural$KeyDescription.";
    if (! $Res or $NumRows == 0)
        $S = StrNoticeBox("Found no photos$KeyDescription.");
    else
    {
        if (is_numeric($Limit) && $NumRows > $Limit)
        {
            $S = WarnTooManyHits(
                $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
                $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
                $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
                $MatchNameCase, $DescIs, $TimeIs, $Sort, $N);
            $EndsOnly = True;
        } else
        {
            $T = ($Sort == "Time") ? "time" : "place and then time";
            if ($NumRows > 1) $N .= "<br />Sorting by $T.";
            $S = StrNoticeBox($N);
        }
        $State = PackedState(
            array($WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
                  $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
                  $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
                  $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit,
                  $EndsOnly));
        if ($EditOk)
        {
            $S .= StrFormBeginRecurse();
            $S .= HtmlHidden("State",
                             htmlentities($State));
            $S .= HtmlHidden("NumRows", $NumRows);
            if ($EndsOnly)
            {
                $W = "Careful: All $NumRows matching photos are";
                $W .= " selected!";
                $S .= StrWarningBox($W, "50%");
            } else
                $S .= StrSelectionOptions();
            $S .= "<br>\n";
	    $ResTagsInUse = ResTagsInUse(
		$WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
		$InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
		$TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
		$MatchNameCase, $DescIs, $TimeIs, $Sort,
		$KeyDescription);
            $S .= PhotoActionOptions((! $EndsOnly),
				     OptionUnTag($ResTagsInUse));
            $S .= "<br>\n";
        }
        if ($EndsOnly) $S .= EndsOnly($Res, $NumRows);
        else
        {
            if ($SelectAll) $RangeMarks = True;
            $S .= ImageCells($Res, $NumRows, $RangeMarks, "State",
                             $State, $EditOk);
        }
        if ($EditOk) $S .= StrFormEnd();
    }
    $S .= PhotoSearchBox(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
        $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs,
        $Sort, $Limit, $EndsOnly);
    return $S;
}

function ParsedWords($Text)
{
    // Parse input string as tokens to match. Quotes delimit a single
    // token, otherwise tokens are space-delimited words.
    $Matches = array();
    $Match= "";
    $Quoted = False;
    for ($I = 0 ; $I < strlen($Text) ; $I ++)
    {
        $Char = substr($Text, $I, 1);
        switch ($Char)
        {
        case "\"":
            $Quoted = (! $Quoted);
            break;
            ;;
        case " ":
            if ($Quoted) $Match .= $Char;
            else
            {
                $Matches[] = $Match;
                $Match = "";
            }
            break;
            ;;
        default:
            $Match .= $Char;
        }
    }
    if ($Match > "") $Matches[] = $Match;
    return $Matches;
}

function PhotoSearchBox(
    $WhenFrom="", $WhenTo="", $LocationSn=Null, $Words="",
    $InDesc=True, $InCaption=False, $MatchCase=False, $Tags=array(),
    $TagSelect="any", $Tags2=array(), $TagSelect2="any",
    $HasVideo="either", $InAlbum="either", $FileNameStr="",
    $MatchNameCase=False, $DescIs="either", $TimeIs="either",
    $Sort="Time", $Limit=MaxSearchHits, $EndsOnly=False)
{
    /************************************************************
      Form to find photos by time (range), location, or text (in
      description or albumphoto caption).
      Sort results by time or by time, place
    ************************************************************/
    $S = StrFormBeginRecurse();
    $S .= StrTableBegin(0, "", "", BoxStyle);
    $S .= RowSearchTime($WhenFrom, $WhenTo);
    $S .= RowSearchLocation($LocationSn);
    $S .= RowSearchText($Words, $InDesc, $InCaption, $MatchCase);
    $S .= RowSearchTags($Tags, $TagSelect, $Tags2, $TagSelect2);
    $S .= RowSearchHasVideo($HasVideo);
    $S .= RowSearchAlbumPhoto($InAlbum);
    $S .= RowSearchFileName($FileNameStr, $MatchNameCase);
    $S .= RowSearchBlankDescription($DescIs);
    $S .= RowSearchNullTime($TimeIs);
    $S .= RowSeparator();
    $S .= RowSearchLimit($Limit, $EndsOnly);
    $S .= RowSearchSortOrder($Sort);
    $S .= RowSearchSubmitButton();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function ResPhotoSearch(
    $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
    $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
    $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs, $Sort,
    &$KeyDescription)
{
    $KeyDescription = "";
    $Q = "SELECT * FROM";
    $Q .= " (SELECT DISTINCT ON (p.photosn) photosn, foldername,";
    $Q .= " filename, p.locationsn, description, p.width, p.height,";
    $Q .= " thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplay;
    $Q .= ", COALESCE(l.name, '".NullLocationFlag."') AS locname,";
    $Q .= " vidfilename, v.width AS vwidth, v.height AS vheight,";
    $Q .= " time, NULL AS istagged";
    $Q .= ClauseFrom;
    $Count = 0;
    $Params = array();
    $Q .= ClausesPhotoSearch(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
        $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs,
        $KeyDescription, $Count, $Params);
    $Q .= ") AS matches ORDER BY ";
    $Q .= ($Sort == "Time") ? "time" : "locname, time";
    $Q .= ", lower(filename)";
    return QueryArray($Q, $Params);
}

function ResTagsInUse(
    $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
    $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
    $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs, $Sort,
    &$KeyDescription)
{
    $KeyDescription = "";
    $Q = "SELECT DISTINCT ON (name) tp.tagsn, name, notes FROM";
    $Q .= " (SELECT DISTINCT p.photosn";
    $Q .= ClauseFrom;
    $Count = 0;
    $Params = array();
    $Q .= ClausesPhotoSearch(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
        $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs,
        $KeyDescription, $Count, $Params);
    $Q .= ") AS matches, tag_photo tp, tag t";
    $Q .= " WHERE tp.photosn =  matches.photosn";
    $Q .= " AND t.tagsn = tp.tagsn";
    $Q .= " ORDER BY name";
    return QueryArray($Q, $Params);
}

function RowSearchAlbumPhoto($InAlbum)
{
    if (! isset($InAlbum)) $InAlbum = "either";
    $S = StrRowBegin();
    $S .= StrCell(Strong("Is in any album:"));
    $C = "<label>";
    $C .= HtmlRadioButton("InAlbum", "yes", ($InAlbum == "yes"));
    $C .= " yes</label><label>";
    $C .= HtmlRadioButton("InAlbum", "no", ($InAlbum == "no"));
    $C .= " no</label><label>";
    $C .= HtmlRadioButton("InAlbum", "either",
                          ($InAlbum == "either"));
    $C .= " either</label>";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchBlankDescription($DescIs)
{
    if (! isset($DescIs)) $DescIs = "either";
    $S = StrRowBegin();
    $S .= StrCell(Strong("Description is:"));
    $C = "<label>";
    $C .= HtmlRadioButton("DescIs", "blank", ($DescIs == "blank"));
    $C .= " blank</label><label>";
    $C .= HtmlRadioButton("DescIs", "nonblank",
                          ($DescIs == "nonblank"));
    $C .= " nonblank</label><label>";
    $C .= HtmlRadioButton("DescIs", "either",
                          ($DescIs == "either"));
    $C .= " either</label>";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchFileName($FileNameStr, $MatchNameCase)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("File name:"));
    $C = HtmlTextInput("FileNameStr", "20", $FileNameStr);
    $C .= " (partial ok) <label>";
    $C .= HtmlCheckbox("MatchNameCase", $MatchNameCase);
    $C .= " Match case</label>\n";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchHasVideo($HasVideo)
{
    if (! isset($HasVideo)) $HasVideo = "either";
    $S = StrRowBegin();
    $S .= StrCell(Strong("Is video:"));
    $C = "<label>";
    $C .= HtmlRadioButton("HasVideo", "yes", ($HasVideo == "yes"));
    $C .= " yes</label><label>";
    $C .= HtmlRadioButton("HasVideo", "no", ($HasVideo == "no"));
    $C .= " no</label><label>";
    $C .= HtmlRadioButton("HasVideo", "either",
                          ($HasVideo == "either"));
    $C .= " either</label>";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchLimit($Limit, $EndsOnly)
{
    if (! isset($Limit)) $Limit = MaxSearchHits;
    if (! isset($EndsOnly)) $EndsOnly = False;
    $S = StrRowBegin();
    $S .= StrCell(Strong("Limit results:"));
    $C = StrTableBegin("0", "", "", "width: 100%");
    $C .= StrRowBegin();
    $T = "to ".HtmlTextInput("Limit", "5", $Limit)." matches";
    $C .= StrCell($T);
    $T =  "<label>".HtmlCheckbox("EndsOnly", $EndsOnly);
    $T .= " Display only first and last matches</label>";
    $C .= StrCell($T, "", "", "text-align: right");
    $C .= StrRowEnd();
    $C .= StrTableEnd();
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchLocation($LocationSn)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("Where:"));
    $C = MenuLocation($LocationSn, True, False, "(unknown)", False,
                      False, True, True);
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchNullTime($TimeIs)
{
    if (! isset($TimeIs)) $TimeIs = "either";
    $S = StrRowBegin();
    $S .= StrCell(Strong("Date/Time is:"));
    $C = "<label>";
    $C .= HtmlRadioButton("TimeIs", "unset", ($TimeIs == "unset"));
    $C .= " unset</label><label>";
    $C .= HtmlRadioButton("TimeIs", "set",
                          ($TimeIs == "set"));
    $C .= " set</label><label>";
    $C .= HtmlRadioButton("TimeIs", "either",
                          ($TimeIs == "either"));
    $C .= " either</label>";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchSubmitButton()
{
    $S = StrRowBegin();
    $S .= "<td align=\"center\" colspan=\"2\">";
    $S .= StrTableBegin(0, "", "", "width: 100%");
    $S .= StrRowBegin();
    $S .= "<td style=\"width: 25%\"></td>\n";
    $S .= StrCell(HtmlSubmitButton("SearchPhotos", "Search"), "", "",
                  "text-align: center");
    $S .= StrCell(HtmlSubmitButton("ResetForm", "Reset"), "", "",
                  "width: 25%; text-align: right");
    $S .= StrRowEnd();
    $S .= StrTableEnd()."</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowSearchTags($Tags, $TagSelect, $Tags2, $TagSelect2)
{
    if (! isset($TagSelect)) $TagSelect = "any";
    if (! isset($TagSelect2)) $TagSelect2 = "any";
    $S = StrRowBegin();
    $S .= StrCell(Strong("Tagged with:"));
    $S .= "<td>".StrTableBegin(0);
    $S .= StrRowBegin();
    $S .= CellSearchTags("TagSelect", $TagSelect, "Tags", $Tags);
    $S .= CellSearchTags("TagSelect2", $TagSelect2, "Tags2", $Tags2,
                         "and tagged with:");
    $S .= StrRowEnd();
    $S .= StrTableEnd()."</td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowSearchText($Words, $InDesc, $InCaption, $MatchCase)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("Text:"));
    $C = HtmlTextInput("Words", "54", $Words);
    $C .= "<br />\n".Strong("in: ");
    $C .= "<label>".HtmlCheckbox("InDesc", $InDesc);
    $C .= " description</label> <label>";
    $C .= HtmlCheckbox("InCaption", $InCaption);
    $C .= " caption in any album</label> <label>";
    $C .= HtmlCheckbox("MatchCase", $MatchCase);
    $C .= " Match case</label>\n";
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSearchTime($WhenFrom, $WhenTo)
{
    $S = StrRowBegin();
    $S .= StrCell(Strong("When&nbsp;taken:"));
    $C = "between ".HtmlTextInput("WhenFrom", 16, $WhenFrom);
    $C .= " and ".HtmlTextInput("WhenTo", 16, $WhenTo);
    $S .= StrCell($C);
    $S .= StrRowEnd();
    return $S;
}

function RowSeparator()
{
    $S = StrRowBegin();
    $S .= StrCell("<hr />", "2");
    $S .= StrRowEnd();
    return $S;
}

function SelectAll($State, $Selected, $EditOk)
{
    /*****************************************************************
      Sets or clears selection checkboxes on all images according to
      $SelectAll, then redisplays search results page.
    *****************************************************************/
    list ($WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
          $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
          $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
          $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit, $EndsOnly)
        = UnPackedState($State);
    return PageSearchResults(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2,
        $HasVideo, $InAlbum, $FileNameStr, $MatchNameCase, $DescIs,
        $TimeIs, $Sort, $Limit, $EndsOnly, $Selected, $EditOk);
}

function SelectRange($State, $NumThumbs, $FormVars, $EditOk)
{
    /*****************************************************************
      If $NumThumbs > 0, builds an ordered array of selected
      thumbnails which specify starting and stopping points of ranges,
      then calls PageSearchResults to redisplay photo set page with
      checkboxes set for all images in range(s).
    *****************************************************************/
    $RangeMarks = array();
    for ($Row=0 ; $Row < $NumThumbs ; $Row++ )
        if (array_key_exists("Sel$Row", $FormVars)
            && $FormVars["Sel$Row"] == "on")
            $RangeMarks[] = $FormVars["PS$Row"];
    if (count($RangeMarks) > 0)
    {
        list ($WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
              $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
              $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
              $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit,
              $EndsOnly) = UnPackedState($State);
        $T= PageSearchResults(
            $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
            $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
            $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
            $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit,
            $EndsOnly, False, True, $RangeMarks);
        $S = HtmlPara($RangeMarks).$T;
    } else
    {
        $S = HtmlPara("No selection range specified.");
        $S .= SelectAll($State, False, $EditOk);
    }
    return $S;
}

function Untagged($NumRows, $UnTagSn)
{
    $S = "";
    $Count = 0;
    if (is_numeric($UnTagSn))
    {
        $TagName = ColFromSerialNumber("tag", "name", $UnTagSn);
        $FormVars = (FormMethod == "get") ? $_GET : $_POST;
        for ($Row=0 ; $Row < $NumRows ; $Row++ )
        {
            if (array_key_exists("Sel$Row", $FormVars)
                && $FormVars["Sel$Row"] == "on")
            {
                $PhotoSn = $FormVars["PS$Row"];
                UnTagPhoto($UnTagSn, $PhotoSn);
                $S .= ", ".PhotoPointer($PhotoSn);
                $Count ++;
            }
        }
        if ($S > "")
        {
            $S = substr($S, 1);
            $T = ($Count > 1) ? "photos" : "photo" ;
            $T = "Untagged &quot;$TagName&quot; from $Count $T:";
        } else
        {
            $T = "No photos selected; couldn't untag";
            $T .= " &quot;$TagName&quot;.";
        }
        $S = $T.$S;
    } else
        $S = "No tag specified, so no photos untagged.";
    return HtmlPara($S);
}

function WarnTooManyHits(
    $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
    $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2, $HasVideo,
    $InAlbum, $FileNameStr, $MatchNameCase, $DescIs, $TimeIs, $Sort,
    $Report)
{
    $Count = 0;
    $Params = array();
    $Cl .= ClauseFrom.ClausesPhotoSearch(
        $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc, $InCaption,
        $MatchCase, $Tags, $TagSelect, $Tags2, $TagSelect2,
        $HasVideo, $InAlbum, $FileNameStr, $MatchNameCase, $DescIs,
        $TimeIs, $K, $Count, $Params);
    $Q = "SELECT MIN(time), MAX(time) $Cl";
    list($Lo, $Hi) = pg_fetch_array(QueryArray($Q, $Params), 0);
    $TimeLo = ($Lo > "") ? TimeString($Lo) : NullTimeFlag;
    $TimeHi = ($Hi > "") ? TimeString($Hi) : NullTimeFlag;
    $S = StrSubTitle("Too many matches to display; adjust criteria.");
    $S .= HtmlPara($Report);
    $T = "Matching photos were taken between<br />\n";
    $T .= "$TimeLo and $TimeHi";
    if ($LocationSn != "unknown")
        $S .= WarnTooManyHitsLocations($Cl, $T, $Params);
    else $S .= HtmlPara("$T.");
    $Q = "SELECT COUNT(*) $Cl AND time IS NULL";
    $NullTimeCount
        = pg_fetch_result(QueryArray($Q, $Params), 0, "count");
    if ($NullTimeCount > 0)
    {
        $Z = "$NullTimeCount photo";
        $Z .= $NullTimeCount > 1 ? "s have" : " has";
        $Z .= " no date specified.";
        $S .= StrWarningBox($Z, "70%");
    }
    $S = StrWarningBox($S, "50%");
    return $S;
}

function WarnTooManyHitsLocations($Cl, $T, $Params)
{
    $Res =
         QueryArray("SELECT DISTINCT name $Cl ORDER BY name", $Params);
    $NumRows = pg_num_rows($Res);
    if ($NumRows == 1)
    {
        $T .= "<br />\nat location \"";
        $T .= pg_fetch_result($Res, 0, "name")."\"";
        $S = HtmlPara($T);
    } elseif ($NumRows > HitsPerPage)
          $S = HtmlPara("$T<br />\nat $NumRows locations.");
    else
    {
        $S = HtmlPara("$T<br />\nat $NumRows locations:");
        $T = "";
        for ($Row = 0 ; $Row < $NumRows ; $Row ++)
            $T .= pg_fetch_result($Res, $Row, "name")."<br />\n";
        $S .= HtmlPara($T);
    }
    $Q = "SELECT COUNT(*) $Cl AND locationsn IS NULL";
    $NullLocationCount
        = pg_fetch_result(QueryArray($Q, $Params), 0, "count");
    if ($NullLocationCount > 0)
    {
        $T = "$NullLocationCount photos have no location specified.";
        $S .= StrWarningBox($T, "70%");
    }
    return $S;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "AlbumSn",
    "DescIs",
    "Description",
    "EndsOnly",
    "FileNameStr",
    "HasVideo",
    "InAlbum",
    "InCaption",
    "InDesc",
    "Limit",
    "LocationSn",
    "MatchCase",
    "MatchNameCase",
    "NewAlbum",
    "NewTag",
    "NumRows",
    "OtherLoc",
    "SelectAll",
    "Sort",
    "State",
    "Tags",
    "Tags2",
    "TagSelect",
    "TagSelect2",
    "TagSn",
    "TimeIs",
    "UnTagSn",
    "WhenFrom",
    "WhenTo",
    "Words"
);
$GetVarList = array(
    "FetchFull",
    "FetchVideo",
    "FetchWebPic",
    "NumRows",
    "Row",
    "State"
);
$PostVarList = array(
    
);
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
foreach ($PostVarList as $Var)
    if (isset($_POST[$Var])) $$Var = trim($_POST[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = is_string($FormVars[$Var])
                                      ? trim($FormVars[$Var])
                                      : $FormVars[$Var];

if (isset($FetchWebPic)) DownloadFile($FetchWebPic);
elseif (isset($FetchFull)) DownloadFile($FetchFull);
elseif (isset($FetchVideo)) DownloadFile($FetchVideo);
else
{
    $EditOk = UserHasPrivilege(LoggedInUser(), "edit");
    if (isset($_GET["ViewLarge"]))
    {
        list ($WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
              $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
              $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
              $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit,
              $EndsOnly) = UnPackedState($State);
        $Res = ResPhotoSearch(
            $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
            $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
            $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
            $MatchNameCase, $DescIs, $TimeIs, $Sort, $K);
        $S .= StrLargeImageForm($State, $Res, $Row);
    } else
    {
        $Tags = (array_key_exists("Tags", $FormVars))
              ? $FormVars["Tags"] : False;
        $Tags = $Tags ? array_diff($Tags, array("none")) : array();
        $Tags2 = (array_key_exists("Tags2", $FormVars))
               ? $FormVars["Tags2"] : False;
        $Tags2 = $Tags2 ? array_diff($Tags2, array("none")) : array();
        $S = StartOfPage("Search for Photos");
        if (isset($FormVars["SearchPhotos"]))
            $S .= PageSearchResults(
                $WhenFrom, $WhenTo, $LocationSn, $Words, $InDesc,
                $InCaption, $MatchCase, $Tags, $TagSelect, $Tags2,
                $TagSelect2, $HasVideo, $InAlbum, $FileNameStr,
                $MatchNameCase, $DescIs, $TimeIs, $Sort, $Limit,
                $EndsOnly, False, $EditOk);
        elseif (isset($SelectAll) or isset($FormVars["ClearAll"]))
            $S .= SelectAll($State, isset($SelectAll), $EditOk);
        elseif (isset($FormVars["SelectRange"]))
            $S .= SelectRange($State, $NumRows, $FormVars, $EditOk);
        elseif (isset($State) && IsEndsOnly($State) and $EditOk)
            $S .= ActionsForAll($State, $FormVars, $LocationSn,
                                $OtherLoc, $AlbumSn, $NewAlbum,
                                $TagSn, $NewTag, $UnTagSn);
        elseif (isset($FormVars["ApplyLoc"]) and $EditOk)
        {
            $S .= AppliedLocation($NumRows, $LocationSn, $OtherLoc);
            $S .= SelectAll($State, False, $EditOk);
        } elseif (isset($FormVars["ApplyDesc"]) and $EditOk)
          {
              $S .= AppliedDescription($NumRows, $Description);
              $S .= SelectAll($State, False, $EditOk);
          } elseif (isset($FormVars["AddToAlbum"]) and $EditOk)
            {
                $S .= AddedToAlbum($NumRows, $AlbumSn, $NewAlbum);
                $S .= SelectAll($State, False, $EditOk);
            } elseif (isset($FormVars["ApplyTag"]) and $EditOk)
              {
                  $S .= AppliedTag($NumRows, $TagSn, $NewTag);
                  $S .= SelectAll($State, False, $EditOk);
              } elseif (isset($FormVars["Untag"]) and $EditOk)
                {
                    $S .= Untagged($NumRows, $UnTagSn);
                    $S .= SelectAll($State, False, $EditOk);
                } elseif (isset($FormVars["Delete"]) and $EditOk)
                  {
                      $S .= AppliedDeletion($NumRows);
                      $S .= SelectAll($State, False, $EditOk);
                  } else $S .= PhotoSearchBox();
    }
    $S .= "</div></body></html>\n";
    echo $S;
}

?>
