<?php
/*********************************************************************
viewalbum.php
View Album Module for PhotoTrove
______________________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require ("lib_album.php");
require ("lib_albumgen.php");
require_once ("lib_common.php");
require ("lib_filesys.php");
require_once ("lib_layout.php");

function AlbumPage($AlbumSn, $Page)
{
    list ($AlbumName, $Title, $BgColor, $TextColor, $VidBgColor,
          $Author, $Url, $Notes)
        = pg_fetch_array(ResAlbum($AlbumSn), 0);
    $BodyStyle = AlbumBodyStyle($BgColor, $TextColor, $VidBgColor);
    $LinkStyle = AlbumLinkStyle($TextColor);
    $S = HeadAndTitle($Title, "", $BodyStyle, $LinkStyle);
    if (!isset($Page)) $Page = 0;
    $W = "";
    $Res = ResAlbumPage(False, $AlbumSn, $Page);
    if (! $Res)
    {
        $W = WarnPageGone($Page, $AlbumName, $Title);
        $Page = 0;
        $Res = ResAlbumPage(False, $AlbumSn, $Page);
    }
    list ($AlbumPageSn, $Z1, $Z2, $Heading, $TitlePicOrd,
          $TitlePicCaption, $PageText) = pg_fetch_array($Res, 0);
    // Get all pics on page
    $Q = "SELECT foldername, filename, caption, ordinal, p.width,";
    $Q .= " p.height, thumbw, thumbh, v.vidfilename,";
    $Q .= " v.width as vwidth, v.height as vheight";
    $Q .= " FROM albumphoto ap, photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn),";
    $Q .= " photoset s";
    $Q .= " WHERE albumpagesn = $1 AND p.photosn = ap.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY ordinal";
    $Res = Query($Q, $AlbumPageSn);
    $LastPage = LastPage($AlbumSn);
    $NumPics = pg_num_rows($Res);
    if ($Page > 0) $S .= LinkBar($AlbumSn, $Page, $LastPage);
    elseif ($W > "") $S .= $W;
    if ($Heading > "") $S .= "<h2 align=\"center\">$Heading</h2>\n";
    if ($TitlePicOrd > 0 and $NumPics >= $TitlePicOrd)
        $S .= TitlePicPhp($Res, $TitlePicOrd-1, $Page, $AlbumSn,
                          $TitlePicCaption, $VidBgColor);
    if ($PageText > "") $S .= HtmlPara($PageText);
    if ($Page > 0)
        $S .= ThumbnailsPhp($AlbumSn, $Page, $NumPics, $LastPage,
                            $Res);
    else
        $S .= PageZero($AlbumSn, $LastPage, $Author, $Url);
    return $S;
}

function LargePicDisplay($AlbumSn, $Page, $Ordinal)
{
    list ($AlbumName, $Title, $BgColor, $TextColor, $VidBgColor,
          $Author, $Url, $Notes)
        = pg_fetch_array(ResAlbum($AlbumSn), 0);
    $Q = "SELECT foldername, filename, caption, p.width, p.height,";
    $Q .= " vidfilename, v.width as vwidth, v.height as vheight";
    $Q .= " FROM albumpage pg, albumphoto ap, photo p";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn),";
    $Q .= " photoset s";
    $Q .= " WHERE albumsn = $1 AND page = $2";
    $Q .= " AND ap.albumpagesn = pg.albumpagesn";
    $Q .= " AND ordinal = $3 AND p.photosn = ap.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Res = Query($Q, $AlbumSn, $Page, $Ordinal);
    list($FolderName, $FileName, $Caption, $Width, $Height,
         $VidFileName, $VWidth, $VHeight) = pg_fetch_array($Res, 0);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    $IsVideo = ($VidFileName > "");
    $WebPath = "$WebFolder/$FileName";
    $LastPage = LastPage($AlbumSn);
    list($PrevUrl, $NextUrl)
        = AdjacentPics($AlbumSn, $Page, $Ordinal, $LastPage);
    $BodyStyle = AlbumBodyStyle($BgColor, $TextColor, $VidBgColor);
    $LinkStyle = AlbumLinkStyle($TextColor);
    $S = HeadAndTitle($Title, "", $BodyStyle, $LinkStyle);
    if ($IsVideo)
    {
        $OggFile = "$WebFolder/".OggFile($VidFileName);
        list($OggWidth, $OggHeight)
            = array_values(VideoSpecs($OggFile));
        $S .= VideoInPlayer($WebPath, $PrevUrl, $NextUrl, $Caption,
                            $VidFileName, $OggFile, $OggWidth,
                            $OggHeight, $VidBgColor);
    } else
        $S .= StrImageWithMap($Width, $Height, $WebPath, $PrevUrl,
                              $NextUrl, $Caption);
    $S .= LinkBar($AlbumSn, $Page, $LastPage);
    return $S;
}

function ThumbnailLinkPhp($AlbumSn, $Page, $Ordinal, $Caption,
                          $ThumbPath, $ThumbW, $ThumbH, $IsVidThumb)
{
    $Url = LargeImageUrl($AlbumSn, $Page, $Ordinal);
    $Label = HtmlCleanTitle($Caption);
    if ($IsVidThumb) $Label .= " (Video)";
    $S = "<a href=\"$Url\">";
    $S .= "<img src=\"$ThumbPath\" alt=\"$Label\" title=\"$Label\"";
    if ($IsVidThumb) $S .= " class=\"vid\"";
    $S .= " height=\"$ThumbH\" width=\"$ThumbW\" border=\"0\"/>";
    $S .= "</a>\n";
    return $S;
}

function ThumbnailsPhp($AlbumSn, $Page, $NumPics, $LastPage, $Res)
{
    $S = "";
    for ($Row = 0 ; $Row < $NumPics ; $Row++)
    {
        list($FolderName, $FileName, $Caption, $Ordinal, $Width,
             $Height, $ThumbW, $ThumbH, $VidFileName)
            = pg_fetch_array($Res, $Row);
        list ($FullFolder, $WebFolder, $ThumbFolder)
            = ImageFolder ($FolderName);
        $ThumbPath = "$ThumbFolder/$FileName";
        $S .= ThumbnailLinkPhp($AlbumSn, $Page, $Ordinal, $Caption,
                               $ThumbPath, $ThumbW, $ThumbH,
                               ($VidFileName > ""));
    }
    if ($NumPics > 0) $S .= LinkBar($AlbumSn, $Page, $LastPage);
    // It's ok to have a page with just text and no photos
    // else $S .= WarnNoPhotos($Page);
    return $S;
}

function TitlePicPhp($Res, $Row, $Page, $AlbumSn, $TitlePicCaption,
                     $VidBgColor)
{
    list($FolderName, $FileName, $Caption, $Ordinal, $Width, $Height,
         $ThumbW, $ThumbH, $VidFileName, $VWidth, $VHeight)
        = pg_fetch_array($Res, $Row);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder ($FolderName);
    $WebPath = "$WebFolder/$FileName";
    $S = "<div align=\"center\">\n";
    $PrevUrl = PageLinkUrl($AlbumSn, max($Page - 1, 0));
    $NextUrl = ($Page > 0)
             ? LargeImageUrl($AlbumSn, $Page, 1)
             : PageLinkUrl($AlbumSn, 1);
    if ($VidFileName > "")
    {
        ScaleSize($VWidth, $VHeight, TitlePicSize);
        $OggFile = "$WebFolder/".OggFile($VidFileName);
        $S .= VideoInPlayer($WebPath, $PrevUrl, $NextUrl, $Caption,
                            $VidFileName, $OggFile, $VWidth,
                            $VHeight, $VidBgColor, True, False);
    } else
    {
        ScaleSize($Width, $Height, TitlePicSize);
        $S .= StrImageWithMap($Width, $Height, $WebPath, $PrevUrl,
                              $NextUrl, $TitlePicCaption, True,
                              False);
    }
    $S .= "</div>\n";
    return $S;
}

//***************************************************************
session_cache_limiter("");
// Fetch input variables
$FormVarList = array(
    "AlbumSn",
    "Sort"
);
$GetVarList = array(
    "AlbumSn",
    "FetchPic",
    "FetchVid",
    "Ordinal",
    "Page"
);
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (isset($FetchPic)) DownloadFile($FetchPic);
elseif (isset($FetchVid)) DownloadFile($FetchVid);
else
{
    if (! isset($Page)) $Page = 0;
    if (! isset($Sort)) $Sort = "Time";
    if (!isset($AlbumSn))
    {
        $S = StartOfPage("View a Photo Album");
        $S .= FormMenuAlbum($Sort);
    } elseif ($AlbumSn == "none")
      {
          $S = StartOfPage("View a Photo Album");
          $T = "Select an album from the<br>drop-down list";
          $S .= StrWarning($T);
          $S .= FormMenuAlbum($Sort);
      } elseif (isset($_GET["ViewLarge"]))
            $S = LargePicDisplay($AlbumSn, $Page, $Ordinal);
    else
        $S = AlbumPage($AlbumSn, $Page);
    $S .= "</div></body></html>\n";
    echo $S;
}
?>
