<?php
/*********************************************************************
viewset.php
View Photo Set Module for PhotoTrove
______________________________________________________________________
Copyright 2021 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
session_cache_limiter("");
require_once ("lib_common.php");
require ("lib_filesys.php");
require_once ("lib_layout.php");
require ("lib_photoset.php");
require ("local/loc_camera.php");

function AdjacentPics($PhotoSn, $PhotoSetSn)
{
    $Row = TupleNumInPhotoSetBySn($PhotoSn);
    $Res = ResPhotosInSet($PhotoSetSn);
    $NumRows = pg_num_rows($Res);
    $PrevRow = ($Row == 0) ? $Row : $Row - 1;
    $NextRow = ($Row == $NumRows - 1 ) ? $Row : $Row + 1;
    $PrevPicSn = pg_fetch_result($Res, $PrevRow, "photosn");
    $NextPicSn = pg_fetch_result($Res, $NextRow, "photosn");
    return array($PrevPicSn, $NextPicSn);	
}

function PageLargeImage($PhotoSn)
{
    list ($FolderName, $FileName, $PhotoSetSn, $LocationSn,
          $Description, $Width, $Height, $ThumbW, $ThumbH, $Time,
          $LocName, $VidFileName, $VideoSn, $VWidth, $VHeight,
          $Latitude, $Longitude, $Altitude)
        = pg_fetch_array(ResPhoto($PhotoSn), 0);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    list ($PrevPicSn, $NextPicSn)
        = AdjacentPics($PhotoSn, $PhotoSetSn);
    $IsVideo = ($VidFileName > "");
    $HrefFwd = "href=\"".ImageUrl($NextPicSn)."\"";
    $HrefBack = "href=\"".ImageUrl($PrevPicSn)."\"";
    if ($IsVideo)
        $S = StartOfPage("$VidFileName (video)", "", "", True);
    else
    {
        $S = StartOfPage("$FileName (large)", "", "", True);
        $S .= "<map name=\"PicMap\">\n";
        $S .= "<area shape=\"rect\" coords=\"0,0,".PageEdge;
        $S .= ",$Height\" $HrefBack>\n";
        $S .= "<area shape=\"rect\" coords=\"".($Width - PageEdge);
        $S .= ",0,$Width,$Height\" $HrefFwd>\n";
        $S .= "</map>\n";
    }

    $S .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">";
    $S .= "\n<tbody>\n";
    if ($IsVideo)
    {
        $S .= RowWebVid($WebFolder, $VidFileName);
        $S .= RowNavButtons($HrefBack, $HrefFwd);
    } else $S .= RowWebPic($WebFolder, $FileName, $Width, $Height,
                           $Description);
    $S .= RowPicData($FullFolder, $PhotoSn, $FolderName, $FileName,
                     $Description, $Time, $LocName, $VidFileName);
    $S .= StrTableEnd();
    return $S;
}

function PagePhotoSet($PhotoSetSn, $Sort="Time")
{
    $Res = ResPhotosInSet($PhotoSetSn, $Sort);
    $NumRows = pg_num_rows($Res);
    list ($PhotoSetSn, $FolderName, $Nickname, $Notes)
        = pg_fetch_array(ResPhotoSet($PhotoSetSn), 0);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $S = StartOfPage("Review Photo Set &quot;$FolderName&quot;");
    $S .= BoxPhotoSetSelect($PhotoSetSn, $Sort);
    $S .= "<table>\n<tbody>\n";
    $S .= "<tr>";
    $T = Strong("$NumRows photos in set");
    $S .= "<td style=\"width:19%; text-align: center\">$T</td>\n";
    $S .= "<td style=\"width:auto\">";
    $S .= StrBoxBegin("1", "left", TableColor2, "100%");
    $S .= SimpleText($Notes);
    $S .= StrBoxEnd();
    $S .= "</td>\n";
    if (UserHasPrivilege(LoggedInUser(), "edit"))
    {
        $T = StrFormBegin("editset.php");
        $T .= HtmlHidden("PhotoSetSn", $PhotoSetSn);
        $T .= HtmlSubmitButton("Edit", "Edit Set");
        $T .= StrFormEnd();
    } else $T = "&nbsp;";
    $S .= "<td style=\"width:19%; padding:auto;";
    $S .= " text-align: center\">$T</td>\n";
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= ImageCells($Res, $NumRows);
    return $S;
}

function RowPicData($FullFolder, $PhotoSn, $FolderName, $FileName,
                    $Description, $Time, $LocName, $VidFileName)
{
    $IsVideo = ($VidFileName > "");
    $Color = $IsVideo ? HighlightBg : TableColor;
    $S = StrRowBegin($Color);
    $T = HtmlPara($Description);
    $T .= "Taken $Time";
    $T .= " at ${LocName}";
    $G = GeoCoordinates($PhotoSn);
    if ($G !== False) $T .= "<br />$G";
    $T .= "<br />File ".PhotoPointer($PhotoSn);
    /* $T .= AlbumOccurrences($PhotoSn); */
    $S .= StrCell($T, "", "2");
    if (UserHasPrivilege(LoggedInUser(), "edit"))
    {
        $T = StrFormBegin("editset.php");
        $T .= HtmlHidden("PhotoSn", $PhotoSn);
        $T .= HtmlSubmitButton("EditLargeBySn", "Edit");
        $T .= StrFormEnd();
    } else $T = "&nbsp;";
    $S .= "<td align=\"right\">$T<td>\n";
    $S .= StrRowEnd();
    $S .= StrRowBegin($Color);
    $T = BtnGetVideo($FolderName, $VidFileName);
    $T .= BtnGetFullSize($FolderName, $FileName, $IsVideo);
    $T .= StrButton(BtnClose());
    $T = HtmlPara($T);
    $S .= "<td align=\"right\" valign=\"bottom\">$T<td>\n";
    $S .= StrRowEnd();
    return $S;
}

function RowWebPic($WebFolder, $FileName, $Width, $Height,
                   $Description)
{
    // Large image
    $S = StrRowBegin();
    $S .= "<td align=\"center\" colspan=\"2\">\n";
    $T = UrlDownLoad("FetchWebPic", "$WebFolder/$FileName");
    $S .= "<a href=\"$T\">";
    $S .= HtmlImage("$WebFolder/$FileName", $Description, $Height,
                    $Width, "1", $Description, "#PicMap");
    $S .= "</a>";
    $S .= "</td>\n";
    $S .= StrRowEnd();
    return $S;
}

//********************************************************************
// Fetch input variables
$FormVarList = array(
    "PhotoSetSn",
    "PhotoSn",
    "Sort",
    "ViewLarge"
);
$GetVarList = array(
    "PhotoSetSn",
    "PhotoSn",
    "FetchFull",
    "FetchVideo",
    "FetchWebPic",
    "ViewLarge"
);
foreach ($GetVarList as $Var)
    if (isset($_GET[$Var])) $$Var = trim($_GET[$Var]);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);
if (isset($FetchWebPic)) DownloadFile($FetchWebPic);
elseif (isset($FetchFull)) DownloadFile($FetchFull);
elseif (isset($FetchVideo)) DownloadFile($FetchVideo);
else
{
    $S = "";
    if (isset($ViewLarge))
        $S .= PageLargeImage($PhotoSn);
    elseif (! isset($PhotoSetSn))
    {
        $S .= StartOfPage("Review a Photo Set");
        $S .= BoxPhotoSetSelect("", "Time");
    } elseif ($PhotoSetSn == "none")
      {
          $S .= StartOfPage("Review a Photo Set");
          $S .= StrWarning("You must select a photo set");
          $S .= BoxPhotoSetSelect("", "Time");
      } else
        {
            if (! isset($Sort)) $Sort = "Time";
            $S .= PagePhotoSet($PhotoSetSn, $Sort);
        }
    $S .= "</div></body></html>\n";
    echo $S;
}

?>
