<?php
/*********************************************************************
writealbum.php
Write Album Module for PhotoTrove
Generates album in a folder in a format suitable for a CDROM.
______________________________________________________________________
Copyright 2022 Theodore B. Ruegsegger
This file is part of the PhotoTrove web application.

PhotoTrove is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

PhotoTrove is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with PhotoTrove. If not, see <http://www.gnu.org/licenses/>.
______________________________________________________________________

*********************************************************************/
require("lib_album.php");
require("lib_albumgen.php");
require_once("lib_common.php");
require("lib_filesys.php");
require_once("lib_layout.php");
require("local/loc_camera.php");

$T = "COALESCE(to_char(time AT TIME ZONE 'UTC', '";
$T .= DateTimeFormatSQLFull."'), '".NullTimeFlag."')";
define("ClauseTimeForDisplayUTC", $T);

// test whether hard links work
if (@link("winautorun/cdrom.ico", "albums/testhardlink"))
{
    define ("HardLinksOk", True);
    unlink("albums/testhardlink");
} else define ("HardLinksOk", False);

function CreateFolders($UserDir, $AlbumName, $WithFullSize,
                       $Overwrite=False)
{
    if (! file_exists($UserDir)) mkdir($UserDir, 0755);
    if (file_exists("$UserDir/$AlbumName"))
    {
        if ($Overwrite)
        {
            exec("rm -r $UserDir/$AlbumName");
            mkdir("$UserDir/$AlbumName", 0755);
            mkdir("$UserDir/$AlbumName/thumbnails", 0755);
            if ($WithFullSize) 
                mkdir("$UserDir/$AlbumName/fullsize", 0755);
            mkdir("$UserDir/$AlbumName/images", 0755);
            $CreatedNewFolder = True;
        } else $CreatedNewFolder = False;
    } else
    {
        mkdir("$UserDir/$AlbumName", 0755);
        mkdir("$UserDir/$AlbumName/thumbnails", 0755);
        if ($WithFullSize) 
            mkdir("$UserDir/$AlbumName/fullsize", 0755);
        mkdir("$UserDir/$AlbumName/images", 0755);
        $CreatedNewFolder = True;
    }
    return $CreatedNewFolder;
}

function DeleteAlbumCopy($UserId, $AlbumName)
{
    $UserDir = "albums/$UserId";
    exec("rm -r $UserDir/$AlbumName");
    UpdateFolderIndex($UserDir, $AlbumName, "Remove");
    return;
}

function DeletedAlbumCopy($UserId, $AlbumFolder, $AlbumSn, $Sort)
{
    if ($AlbumFolder != "none")
    {
        DeleteAlbumCopy($UserId, $AlbumFolder);
        $Msg = "Deleted copy of $AlbumFolder";
    } else
        $Msg = "No album copy selected, none deleted";
    if (is_numeric($AlbumSn))
    {
        $S = StartWriteAlbumPage($AlbumSn);
        $S .= PageWriteAlbumMain($UserId, $AlbumSn, $Sort, $Msg);
    } else
    {
        $S = StartOfPage("Write a Photo Album to a Folder");
        $S .= StrWarning($Msg, NoticeColor);
        $S .= FormMenuAlbum($Sort);
        $S .= ListWrittenAlbums($UserId);
    }
    return $S;
}

function DownloadAlbum($UserId, $AlbumFolder, $AlbumSn, $Sort)
{
    if ($AlbumFolder != "none")
        DownloadAlbumZip($UserId, $AlbumFolder);
    else
    {
        $Msg = "No album copy selected, nothing downloaded";
        if (is_numeric($AlbumSn))
        {
            $S = StartWriteAlbumPage($AlbumSn);
            $S .= PageWriteAlbumMain($UserId, $AlbumSn, $Sort, $Msg);
        } else
        {
            $S = StartOfPage("Write a Photo Album to a Folder");
            $S .= StrWarning($Msg, NoticeColor);
            $S .= FormMenuAlbum($Sort);
            $S .= ListWrittenAlbums($UserId);
        }
    }
    return $S;
}

function DownloadAlbumZip($UserId, $AlbumName)
{
    $UserDir = "albums/$UserId";
    chdir($UserDir);
    $ZipFile = $AlbumName.".zip";
    shell_exec("zip -r ../../fullsize/.uploads/$ZipFile $AlbumName");
    DownloadFile("../../fullsize/.uploads/$ZipFile");
    unlink("../../fullsize/.uploads/$ZipFile");
    return;
}

function FormPickFolderToDelete($UserDir, $AlbumSn)
{
    // Drop-down menu of folders under albums/$UserId
    $S = StrFormBeginRecurse();
    if ($AlbumSn !== False) $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= StrTableBegin(0);
    $Dirs = Subdirectories($UserDir);
    sort ($Dirs);
    $S .= StrRowBegin();
    $T = HtmlSelectBegin("AlbumFolder");
    $T .= HtmlSelectOption("none", "Delete written copy of an album",
                           True);
    foreach ($Dirs as $Dir) $T .= HtmlSelectOption($Dir, $Dir);
    $T .= HtmlSelectEnd();
    $S .= StrCell($T);
    $S .= StrCell(HtmlSubmitButton("Delete"));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function FormPickFolderToDownload($UserDir, $AlbumSn)
{
    // Drop-down menu of folders under albums/$UserId
    $S = StrFormBeginRecurse();
    if ($AlbumSn !== False) $S .= HtmlHidden("AlbumSn", $AlbumSn);
    $S .= StrTableBegin(0);
    $Dirs = Subdirectories($UserDir);
    sort ($Dirs);
    $S .= StrRowBegin();
    $T = HtmlSelectBegin("AlbumFolder");
    $T .= HtmlSelectOption("none", "Download ZIP file of an album",
                           True);
    foreach ($Dirs as $Dir) $T .= HtmlSelectOption($Dir, $Dir);
    $T .= HtmlSelectEnd();
    $S .= StrCell($T);
    $S .= StrCell(HtmlSubmitButton("Download"));
    $S .= StrRowEnd();
    $S .= StrTableEnd();
    $S .= StrFormEnd();
    return $S;
}

function LargePicPageName($PicName)
{
    return substr($PicName, 0, strrpos($PicName, ".")).".html";
}

function LastOrdinal($AlbumPageSn)
{
    // Highest photo ordinal on album page
    $Q = "SELECT COALESCE(max(ordinal), 0)";
    $Q .= " FROM albumphoto WHERE albumpagesn = $1";
    return pg_fetch_result(Query($Q, $AlbumPageSn), 0, 0);
}

function LinkOrCopy($Source, $Destination, $DeleteGeoTags=False)
{
    // Avoid stripping geotagging from files still in photosets;
    // otherwise, use hard links if filesystem permits it
    if ($DeleteGeoTags)
    {
        copy($Source, $Destination);
        DeleteGPSFromFile($Destination);
    } elseif (HardLinksOk)
        link($Source, $Destination);
    else
        copy($Source, $Destination);
    return;
}

function ListWrittenAlbums($UserId, $AlbumSn=False)
{
    $UserDir = "albums/$UserId";
    if (file_exists($UserDir))
    {
        $T = "<a href=\"albums/$UserId/index.html\"";
        $T .= " target=\"_blank\">View index of $UserId's written";
        $T .= " albums, in separate window</a>"; 
        $S = HtmlPara($T);
        $S .= FormPickFolderToDownload($UserDir, $AlbumSn);
        $S .= FormPickFolderToDelete($UserDir, $AlbumSn);
    } else $S = "";
    return $S;
}

function PageWriteAlbumMain($UserId, $AlbumSn, $Sort, $Msg="")
{
    $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
    $S = ($Msg > "") ? StrWarning($Msg, NoticeColor) : "" ;
    $S .= FormMenuAlbum($Sort, $AlbumSn);
    $T = "<a href=\"viewalbum.php?AlbumSn=$AlbumSn\"";
    $T .= " target=\"_blank\">";
    $T .= "View album \"$AlbumName\" in separate window</a>";
    $S .= HtmlPara($T);
    if (IsContiguousAlbum($AlbumSn))
    {
        $S .= StrFormBeginRecurse();
        $S .= HtmlHidden("AlbumSn", $AlbumSn);
        $S .= HtmlHidden("Sort", $Sort);
        $S .= "<label>".HtmlCheckbox("WithFullSize", True);
        $S .= " Include Full-Size Images "."</label><br />";
        $S .= "<label>".HtmlCheckbox("DeleteGeoTags", True);
        $S .= " Remove Geotagging Metadata "."</label>";
        $S .= HtmlPara(HtmlSubmitButton("Write"));
        $S .= StrFormEnd();
    } else $S .= WarnNotContiguous($AlbumName);
    $S .= ListWrittenAlbums($UserId, $AlbumSn);
    return $S;
}

function PicDataHeaders()
{
    $S = "";
    $Fields = array ("PicName", "FolderName", "FileName", "Location",
                     "Width", "Height", "Thumbw", "Thumbh", "Time",
                     "Caption");
    foreach ($Fields as $Field) $S .= StrHeaderCell($Field);
    $S .= StrRowEnd();
    return $S;
}

function ResPagePics($AlbumSn, $AlbumName, $Page, $TitlePicOrd,
                     $PageDigits, $OrdinalDigits)
{
    $Params = array($AlbumSn, $AlbumName, $Page);
    $Q = "SELECT $2 || '-' ||";
    $Q .= " lpad(CAST($3 AS text), $PageDigits, '0') || '-' ||";
    $Q .= " lpad(CAST(ordinal AS text), $OrdinalDigits, '0') ||";
    $Q .= " SUBSTRING(filename FROM E'\.[^.]*$') AS picname,";
    /* $Q = "SELECT $2 || '-' || $3 || '-' || ordinal ||"; */
    /* $Q .= " SUBSTRING(filename FROM E'\.[^.]*$') AS picname,"; */
    $Q .= " foldername, filename, caption, l.name AS locname,";
    $Q .= " p.width, p.height, thumbw, thumbh, ";
    $Q .= ClauseTimeForDisplayUTC;
    $Q .= ", v.vidfilename,";
    $Q .= "  v.width as vwidth, v.height as vheight";
    $Q .= " FROM albumpage pg, albumphoto ap, photo p";
    $Q .= " LEFT OUTER JOIN location l USING (locationsn)";
    $Q .= " LEFT OUTER JOIN video v ON (v.stillsn = p.photosn),";
    $Q .= " photoset s";
    $Q .= " WHERE albumsn = $1 AND page = CAST ($3 AS INTEGER)";
    $Q .= " AND ap.albumpagesn = pg.albumpagesn";
    if ($Page == 0)
    {
        $Params[] = $TitlePicOrd;
        $Q .= " AND ordinal = $4";
    }
    $Q .= " AND p.photosn = ap.photosn";
    $Q .= " AND s.photosetsn = p.photosetsn";
    $Q .= " ORDER BY ordinal";
    return QueryArray($Q, $Params);
}

function StartWriteAlbumPage($AlbumSn)
{
    $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
    $T = "Write Photo Album &quot;$AlbumName&quot; to a Folder";
    return StartOfPage($T);
}

function ThumbnailLinkHtml($PicName, $Caption, $ThumbPath, $ThumbW,
                           $ThumbH, $IsVidThumb)
{
    $Url = LargePicPageName($PicName);
    $Label = HtmlCleanTitle($Caption);
    $S = "<a href=\"$Url\">";
    $S .= "<img src=\"$ThumbPath\" alt=\"$Label\" title=\"$Label\"";
    if ($IsVidThumb) $S .= " class=\"vid\"";
    $S .= " height=\"$ThumbH\" width=\"$ThumbW\" border=\"0\"/>";

    $S .= "</a>\n";
    return $S;
}

function ThumbnailsHtml($AlbumSn, $Page, $NumPics, $LastPage, $Res)
{
    $S = "";
    for ($Row = 0 ; $Row < $NumPics ; $Row++)
    {
        list($PicName, $FolderName, $FileName, $Caption, $Location,
             $Width, $Height, $ThumbW, $ThumbH, $Time, $VidFileName)
            = pg_fetch_array($Res, $Row);
        list ($FullFolder, $WebFolder, $ThumbFolder)
            = ImageFolder ($FolderName);
        $ThumbPath = "thumbnails/$PicName";
        $S .= ThumbnailLinkHtml($PicName, $Caption, $ThumbPath,
                                $ThumbW, $ThumbH,
                                ($VidFileName > ""));
    }
    if ($NumPics > 0)
        $S .= LinkBar($AlbumSn, $Page, $LastPage, False);
    // It's ok to have a page with just text and no photos
    // else $S .= WarnNoPhotos($Page);
    return $S;
}

function TitlePicHtml($Res, $Row, $AlbumSn, $Page, $TitlePicCaption,
                      $PageDigits, $OrdinalDigits, $VidBgColor)
{
    list($PicName, $FolderName, $FileName, $Caption, $Location,
         $Width, $Height, $ThumbW, $ThumbH, $Time, $VidFileName,
         $VWidth, $VHeight) = pg_fetch_array($Res, $Row);
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);
    $S = "<div align=\"center\">\n";
    $PrevUrl = PageLinkUrl($AlbumSn, max($Page - 1, 0), False);
    $NextUrl = ($Page > 0)
             ? LargeImageUrl($AlbumSn, $Page, 1, False, $PageDigits,
                             $OrdinalDigits)
             : PageLinkUrl($AlbumSn, 1, False);
    if ($VidFileName > "")
    {
        $VidName = VidName($PicName, $VidFileName);
        $OggName = VidName($PicName, OggFile($VidFileName));
        ScaleSize($VWidth, $VHeight, TitlePicSize);
        $S .= VideoInPlayer($PicName, $PrevUrl, $NextUrl, $Caption,
                            $VidName, $OggName, $VWidth, $VHeight,
                            $VidBgColor, False, False);
    } else
    {
        ScaleSize($Width, $Height, TitlePicSize);
        $S .= StrImageWithMap($Width, $Height, $PicName, $PrevUrl,
                              $NextUrl, $TitlePicCaption, False,
                              False);
    }
    $S .= "</div>\n";
    return $S;
}

function UpdateFolderIndex($UserDir, $AlbumName, $Action,
                           $AlbumSn=False)
{
    /*****************************************************************
    Update the album index in $UserDir, either adding a link to
    $AlbumName to the end of the list of links, or deleting
    $AlbumName from the list. May omit $AlbumSn argument unless
    $Action is "Add"
    *****************************************************************/
    $IndexPath = "$UserDir/index.html";
    // Extract links except $AlbumName from existing folder index
    if (file_exists($IndexPath))
    {
        $Cmd = "grep '^<li><a href' $IndexPath";
        $Cmd .= " | grep -v '^<li><a href=\"$AlbumName/index.html'";
        $Links = shell_exec($Cmd);
    } else $Links = "";
    if ($Action == "Add")
    {
        // Append $AlbumName to list of links
        $Title = pg_fetch_result(ResAlbum($AlbumSn), 0, "title");
        $Links .= "<li><a href=\"$AlbumName/index.html\">";
        $Links .= "$Title</a></li>\n";
    }
    // (Re)Build user folder index
    $I = HeadAndTitle("Album List");
    $I .= "<h2 style=\"text-align: center\">Albums in this Set</h2>";
    $I .= "\n";
    $I .= ($Links != "")
       ? "<ul>\n$Links</ul>\n"
       : HtmlPara("No albums found");
    $I .= "</div></body></html>\n";
    $TopIndexFile = fopen($IndexPath, "w");
    fwrite($TopIndexFile, $I);
    fclose($TopIndexFile);
    WriteWindowsAutostart($UserDir);
    return;
}

function VidDataHeaders()
{
    $S = "";
    $Fields = array ("VidName", "FolderName", "VidFileName",
                     "StillFileName", "Width", "Height");
    foreach ($Fields as $Field) $S .= StrHeaderCell($Field);
    $S .= StrRowEnd();
    return $S;
}

function VidName($PicName, $VidFileName)
{
    // Return album-page-specific name for link to
    // $FullFolder/$VidFileName corresponding to $PicName
    $V = substr($PicName, 0, strrpos($PicName, "."));
    $V .= strrchr($VidFileName, ".");
    return $V;
}

function WarnNotContiguous($AlbumName)
{
    $T = "Album &quot;$AlbumName&quot; has missing pages; please";
    $T .= " edit to remove the gaps before publishing.";
    return StrWarningBox($T, "50%");
}

function WriteAlbum($UserId, $AlbumSn, $Sort, $Overwrite,
                    $WithFullSize, $DeleteGeoTags)
{
    $AlbumName = ColFromSerialNumber("album", "name", $AlbumSn);
    $UserDir = "albums/$UserId";
    if (CreateFolders($UserDir, $AlbumName, $WithFullSize,
                      $Overwrite))
    {
        $AlbumPath = "$UserDir/$AlbumName/";
        $AlbumThumbs = $AlbumPath."thumbnails/";
        $AlbumFullSize = $AlbumPath."fullsize/";
        $AlbumSettings = pg_fetch_array(ResAlbum($AlbumSn), 0);
        $LastPage = LastPage($AlbumSn);
        $PicDataFile = fopen($AlbumPath."picdata.tsv", "w");
        $PicDataPage = fopen($AlbumPath."picdata.html", "w");
        $VidDataFile = fopen($AlbumPath."viddata.tsv", "w");
        $VidDataPage = fopen($AlbumPath."viddata.html", "w");
        WriteIcons($AlbumPath);
        WriteWindowsAutostart($AlbumPath);
        for ($Page = 0 ; $Page <= $LastPage ; $Page++)
        {
            WriteAlbumPage(
                $AlbumSn, $Page, $LastPage, $AlbumSettings,
                $AlbumPath, $AlbumThumbs, $AlbumFullSize,
                $PicDataFile, $PicDataPage, $VidDataFile,
                $VidDataPage, $WithFullSize, $DeleteGeoTags);
        }
        fclose($PicDataFile);
        fclose($VidDataFile);
        $H = StrTableEnd();
        $H .= "</div></body></html>\n";
        fwrite($PicDataPage, $H);
        fclose($PicDataPage);
        fwrite($VidDataPage, $H);
        fclose($VidDataPage);

        // Update user folder index (lists all albums)
        UpdateFolderIndex($UserDir, $AlbumName, "Add", $AlbumSn);

        // Confirm completion on screen
        $Msg = "Wrote all files for album $AlbumName";
        $Msg .= " in folder $AlbumPath,<br />";
        if ($WithFullSize)
            $Msg .= "included";
        else
            $Msg .= "omitted";
        $Msg .= " full-size images, ";
        if ($DeleteGeoTags)
            $Msg .= "removed";
        else
            $Msg .= "retained";
        $Msg .= " geotagging metadata, if any.";
        $S = PageWriteAlbumMain($UserId, $AlbumSn, $Sort, $Msg);
    } else
    {
        $S = HtmlPara("Folder &quot;$AlbumName&quot; exists.");
        $S .= StrFormBeginRecurse();
        $S .= HtmlHidden("AlbumSn", $AlbumSn);
        $S .= HtmlHidden("Sort", $Sort);
        $S .= HtmlHidden("WithFullSize", $WithFullSize);
        $S .= HtmlPara(HtmlSubmitButton("Overwrite"));
        $S .= StrFormEnd();
        $S .= HtmlPara("Or select a different album:");
        $S .= FormMenuAlbum($Sort, $AlbumSn);
    }
    return $S;
}

function WriteAlbumPage($AlbumSn, $Page, $LastPage, $AlbumSettings,
                        $AlbumPath, $AlbumThumbs, $AlbumFullSize,
                        $PicDataFile, $PicDataPage, $VidDataFile,
                        $VidDataPage, $WithFullSize, $DeleteGeoTags)
{
    list ($AlbumName, $Title, $BgColor, $TextColor, $VidBgColor,
          $Author, $Url, $Notes) = $AlbumSettings;
    list ($AlbumPageSn, $Z1, $Z2, $Heading, $TitlePicOrd,
          $TitlePicCaption, $PageText)
        = pg_fetch_array(ResAlbumPage(False, $AlbumSn, $Page), 0);
    $PageDigits = floor(log10(max($LastPage, 1))) + 1;
    $LastOrdinal = LastOrdinal($AlbumPageSn);
    $OrdinalDigits = floor(log10(max($LastOrdinal, 1))) + 1;
    $Res = ResPagePics($AlbumSn, $AlbumName, $Page, $TitlePicOrd,
                       $PageDigits, $OrdinalDigits);
    $NumPics = pg_num_rows($Res);
    $BodyStyle = AlbumBodyStyle($BgColor, $TextColor, $VidBgColor);
    $LinkStyle = AlbumLinkStyle($TextColor);
    $HeadAndTitle = HeadAndTitle($Title, "", $BodyStyle, $LinkStyle);
    $P = $HeadAndTitle;
    if ($Page > 0)
        $P .= LinkBar($AlbumSn, $Page, $LastPage, False);
    if ($Heading > "") $P .= "<h2 align=\"center\">$Heading</h2>\n";
    // For page zero we retrieved only the title pic, if any
    if ($Page == 0 and $TitlePicOrd > 0) $TitlePicOrd = 1;
    if ($TitlePicOrd > 0 and $NumPics >= $TitlePicOrd)
        $P .= TitlePicHtml($Res, $TitlePicOrd-1, $AlbumSn, $Page,
                           $TitlePicCaption, $PageDigits,
                           $OrdinalDigits, $VidBgColor);
    if ($PageText > "") $P .= HtmlPara($PageText);
    if ($Page > 0)
    {
        $P .= ThumbnailsHtml($AlbumSn, $Page, $NumPics, $LastPage,
                             $Res);
        $PagePath = $AlbumPath."page$Page.html";
    } else
    {
        $P .= PageZero($AlbumSn, $LastPage, $Author, $Url,
                       False, $WithFullSize);
        $PagePath = $AlbumPath."index.html";
        $BodyStyle = AlbumBodyStyle("#fff", "#000", "#000");
        $H = HeadAndTitle($Title, "", $BodyStyle);
        if ($Heading > "")
            $H .= "<h2 align=\"center\">$Heading</h2>\n";
        $H .= StrTableBegin("1");
        fwrite($PicDataPage, $H.PicDataHeaders());
        fwrite($VidDataPage, $H.VidDataHeaders());
    }
    
    $PageFile = fopen($PagePath, "w");
    fwrite($PageFile, $P);
    fclose($PageFile);

    for ($Row = 0 ; $Row < $NumPics ; $Row++)
        WritePagePics($Res, $Row, $AlbumPath, $AlbumThumbs,
                      $AlbumFullSize, $PicDataFile, $PicDataPage,
                      $VidDataFile, $VidDataPage, $AlbumSn, $Page,
                      $LastPage, $HeadAndTitle, $VidBgColor,
                      $PageDigits, $OrdinalDigits, $WithFullSize,
                      $DeleteGeoTags);
    return;
}

function WriteIcons($AlbumPath)
{
    foreach (array("1leftarrow.png", "1rightarrow.png") as $File)
    {
        // Copy (or hard-link if allowed) the files
        LinkOrCopy("gk/images/$File", "$AlbumPath/images/$File");
    }
    return;
}

function WritePagePics($Res, $Row, $AlbumPath, $AlbumThumbs,
                       $AlbumFullSize, $PicDataFile, $PicDataPage,
                       $VidDataFile, $VidDataPage, $AlbumSn, $Page,
                       $LastPage, $HeadAndTitle, $VidBgColor,
                       $PageDigits, $OrdinalDigits, $WithFullSize,
                       $DeleteGeoTags)
{
    list($PicName, $FolderName, $FileName, $Caption, $Location,
         $Width, $Height, $ThumbW, $ThumbH, $Time, $VidFileName,
         $VWidth, $VHeight) = pg_fetch_array($Res, $Row);
    if ($Time != NullTimeFlag) $Time .= " UTC";
    $IsVideo = ($VidFileName > "");
    list ($FullFolder, $WebFolder, $ThumbFolder)
        = ImageFolder($FolderName);

    // Copy (or hard-link if allowed) the webpics and thumbnails
    LinkOrCopy("$WebFolder/$FileName", $AlbumPath.$PicName,
               $DeleteGeoTags);
    LinkOrCopy("$ThumbFolder/$FileName", $AlbumThumbs.$PicName,
               $DeleteGeoTags);
    // Copy (or hard-link if allowed) full-size images, if option set
    if ($WithFullSize)
        LinkOrCopy("$FullFolder/$FileName", $AlbumFullSize.$PicName,
                   $DeleteGeoTags);
    if ($IsVideo)
    {
        // Copy (or hard-link if allowed) any videos
        $VidName = VidName($PicName, $VidFileName);
        if ($WithFullSize)
            LinkOrCopy("$FullFolder/$VidFileName",
                       $AlbumFullSize.$VidName, $DeleteGeoTags);
        $OggFile = OggFile($VidFileName);
        $OggName = VidName($PicName, $OggFile);
        LinkOrCopy("$WebFolder/$OggFile", $AlbumPath.$OggName,
                   $DeleteGeoTags);
        // Dump vid data in tab-delimited format in $VidDataFile
        $R = $VidName.TAB.$FolderName.TAB.$VidFileName.TAB;
        $R .= $PicName.TAB.$VWidth.TAB.$VHeight."\n";
        fwrite($VidDataFile, $R);
        // Dump vid data as web page (table) in $VidDataPage
        $H = StrRowBegin();
        $H .= StrCell($VidName);
        $H .= StrCell($FolderName);
        $H .= StrCell($VidFileName);
        $H .= StrCell($PicName);
        $H .= StrCell($VWidth);
        $H .= StrCell($VHeight);
        $H .= StrRowEnd();
        fwrite($VidDataPage, $H);
    }
    // Dump pic data in tab-delimited format in $PicDataFile
    $R = $PicName.TAB.$FolderName.TAB.$FileName.TAB.$Location;
    $R .= TAB.$Width.TAB.$Height.TAB.$ThumbW.TAB.$ThumbH;
    $R .= TAB.$Time.TAB;
    $EscapedCaption = str_replace(TAB, '\t', $Caption);
    $EscapedCaption = str_replace("\n", '\r\n', $EscapedCaption);
    $R .= "$EscapedCaption\n";
    fwrite($PicDataFile, $R);

    // Dump pic data as web page (table) in $PicDataPage
    $H = StrRowBegin();
    $H .= StrCell($PicName);
    $H .= StrCell($FolderName);
    $H .= StrCell($FileName);
    $H .= StrCell($Location);
    $H .= StrCell($Width);
    $H .= StrCell($Height);
    $H .= StrCell($ThumbW);
    $H .= StrCell($ThumbH);
    $H .= StrCell($Time);
    $EscapedCaption = str_replace(TAB, '\t', $Caption);
    $EscapedCaption = str_replace("\n", '\r\n', $EscapedCaption);
    $H .= StrCell($EscapedCaption);
    $H .= StrRowEnd();
    fwrite($PicDataPage, $H);

    // Make large image page (html)

    list($PrevUrl, $NextUrl)
        = AdjacentPics($AlbumSn, $Page, $Row+1, $LastPage, False,
                       $PageDigits, $OrdinalDigits);
    $L = $HeadAndTitle;
    if ($IsVideo)
    {
        list($OggWidth, $OggHeight)
            = array_values(VideoSpecs($AlbumPath.$OggName));
        $L .= VideoInPlayer($PicName, $PrevUrl, $NextUrl, $Caption,
                            $VidName, $OggName, $OggWidth, $OggHeight,
                            $VidBgColor, False, $WithFullSize);
    } else
        $L .= StrImageWithMap($Width, $Height, $PicName, $PrevUrl,
                              $NextUrl, $Caption, False,
                              $WithFullSize);
    $L .= LinkBar($AlbumSn, $Page, $LastPage, False);
    $LargePicPage = $AlbumPath.LargePicPageName($PicName);
    $LargePicPageFile = fopen($LargePicPage, "w");
    fwrite($LargePicPageFile, $L);
    fclose($LargePicPageFile);
    return;
}

function WriteWindowsAutostart($RootFolder)
{
    // Places files in a folder so that, if it's the root of a CD and
    // is inserted into a Windows machine, the index.html file will
    // open automatically
    foreach (array("autorun.bat", "autorun.inf", "cdrom.ico")
             as $File)
    {
        // Copy (or hard-link if allowed) the files
        if (! file_exists("$RootFolder/$File"))
            LinkOrCopy("winautorun/$File", "$RootFolder/$File");
    }
    return;
}

//********************************************************************
set_time_limit(0); // Waive the timeout; it takes a while

$UserId = LoggedInUser();

// Fetch input variables
$FormVarList = array(
    "AlbumFolder",
    "AlbumSn",
    "DeleteGeoTags",
    "Sort",
    "WithFullSize"
);
$FormVars = (FormMethod == "get") ? $_GET : $_POST;
foreach ($FormVarList as $Var)
    if (isset($FormVars[$Var])) $$Var = trim($FormVars[$Var]);

if (! isset($Sort)) $Sort = "Time";
if (isset($FormVars["Delete"]))
    $S = DeletedAlbumCopy($UserId, $AlbumFolder, $AlbumSn, $Sort);
elseif (isset($FormVars["Download"]))
    $S = DownloadAlbum($UserId, $AlbumFolder, $AlbumSn, $Sort);
elseif (!isset($AlbumSn))
{
    $S = StartOfPage("Write a Photo Album to a Folder");
    $S .= FormMenuAlbum($Sort);
    $S .= ListWrittenAlbums($UserId);
} elseif ($AlbumSn == "none")
  {
      $S = StartOfPage("Write a Photo Album to a Folder");
      $S .= StrWarning("Select an album from the<br>drop-down list");
      $S .= FormMenuAlbum($Sort);
      $S .= ListWrittenAlbums($UserId);
  } elseif (isset($FormVars["Write"]))
    {
        $S = StartWriteAlbumPage($AlbumSn);
        $S .= WriteAlbum($UserId, $AlbumSn, $Sort, False,
                         $WithFullSize, $DeleteGeoTags);
    } elseif (isset($FormVars["Overwrite"]))
      {
          $S = StartWriteAlbumPage($AlbumSn);
          $S .= WriteAlbum($UserId, $AlbumSn, $Sort, True,
                           $WithFullSize, $DeleteGeoTags);
      } else
        {
            $S = StartWriteAlbumPage($AlbumSn);
            $S .= PageWriteAlbumMain($UserId, $AlbumSn, $Sort);
        }

$S .= "</div></body></html>\n";
echo $S;

?>
